/*
 * @(#)CodeViewer.java  0.2.0 / 2007-12-15
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.demo.codebrowser;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import yu.ac.bg.etf.javaqx.qswing.JQFrame;
import yu.ac.bg.etf.javaqx.qswing.JQSplitPane;
import yu.ac.bg.etf.javaqx.qswing.JQStatusBar;
import yu.ac.bg.etf.javaqx.qswing.JQTree;
import yu.ac.bg.etf.javaqx.qswing.QSwing;
import yu.ac.bg.etf.javaqx.qswing.events.TreeSelectionEvent;
import yu.ac.bg.etf.javaqx.qswing.events.TreeSelectionListener;
import yu.ac.bg.etf.javaqx.qswing.graphics.Font;
import yu.ac.bg.etf.javaqx.qswing.text.PlainDocument;
import yu.ac.bg.etf.javaqx.qswing.tree.DefaultMutableTreeNode;
import yu.ac.bg.etf.javaqx.qswing.tree.DefaultTreeModel;
import yu.ac.bg.etf.javaqx.qswingx.text.code.CodeViewer;
import yu.ac.bg.etf.javaqx.qswingx.text.code.CppDocument;
import yu.ac.bg.etf.javaqx.qswingx.text.code.JavaDocument;

/**
 * Simple Code Viewer.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class CodeBrowser extends JQFrame {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final Set TEXT_FILES;
  private static final Set CPP_FILES;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    TEXT_FILES = new HashSet(Arrays.asList(new String[] {
        ".txt", //$NON-NLS-1$
        ".css", //$NON-NLS-1$
        ".pro", //$NON-NLS-1$
        ".pri", //$NON-NLS-1$
        ".qrc", //$NON-NLS-1$
        ".flex", //$NON-NLS-1$
        ".mf", //$NON-NLS-1$
        ".xml" //$NON-NLS-1$
    }));
    CPP_FILES = new HashSet(Arrays.asList(new String[] {
        ".cpp", //$NON-NLS-1$
        ".c", //$NON-NLS-1$
        ".h" //$NON-NLS-1$
    }));
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final JQStatusBar statusBar;
  private final CodeViewer codeViewer;
  private final PlainDocument plainDocument;
  private CppDocument cppDocument; // NOPMD
  private JavaDocument javaDocument; // NOPMD

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Construcs new CodeViewer object.
   */
  public CodeBrowser() {
    super("JavaQx Code Browser"); //$NON-NLS-1$
    setDefaultCloseOperation(QUIT_ON_CLOSE);
    setSize(800, 540);
    setInsets(4, 4, 1, 4);
    setLocationRelativeTo(null);
    JQSplitPane splitPane = new JQSplitPane();
    codeViewer = new CodeViewer();
    plainDocument = new PlainDocument();
    splitPane.setRightComponent(codeViewer);
    JQTree sourceTree = new JQTree(buildTreeModel());
    sourceTree.setFont(sourceTree.getFont().deriveFont(Font.BOLD));
    sourceTree.addChangeListener(new TreeSelectionListener() {
      public void valueChanged(TreeSelectionEvent event) {
        Object selectedNode = event.getPath().getLastPathComponent();
        if (selectedNode instanceof CodeNode) {
          CodeNode sourceNode = (CodeNode)selectedNode;
          if (sourceNode.isLeaf()) {
            String fileName = sourceNode.getSourceFile().getAbsolutePath();
            String fileExtension = fileName.substring(
                                                    fileName.lastIndexOf('.'));
            fileExtension = fileExtension.toLowerCase(); // NOPMD
            if (".java".equals(fileExtension)) { //$NON-NLS-1$
              loadJavaFile(fileName);
            } else if (CPP_FILES.contains(fileExtension)) {
              loadCppFile(fileName);
            } else if (TEXT_FILES.contains(fileExtension)) { // NOPMD
              loadTextFile(fileName);
            }
          }
        }
      }
    });
    splitPane.setLeftComponent(sourceTree);
    add(splitPane);
    splitPane.setResizeWeight(0.3);
    statusBar = setStatusBar(new JQStatusBar());
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private DefaultTreeModel buildTreeModel() {
    DefaultMutableTreeNode root =
        new DefaultMutableTreeNode("JavaQx Code"); //$NON-NLS-1$
    root.add(new CodeNode("C/C++", "../src/native")); //$NON-NLS-1$ $NON-NLS-2$
    root.add(new CodeNode("Java", "../src/java")); //$NON-NLS-1$ $NON-NLS-2$
    return new DefaultTreeModel(root);
  }

  private void setPlainMessage(String text) {
    codeViewer.setDocument(plainDocument);
    plainDocument.setText(text);
    QSwing.flushEvents();
  }

  private void loadTextFile(String fileName) {
    setPlainMessage("Loading..."); //$NON-NLS-1$
    try {
      File file = new File(fileName);
      plainDocument.readFrom(new FileReader(file));
      statusBar.showMessage(file.getCanonicalPath());
    } catch (IOException ioe) {
      setPlainMessage("Loading failed"); //$NON-NLS-1$
    }
  }

  private void loadCppFile(String fileName) {
    statusBar.clearMessage();
    setPlainMessage("Loading..."); //$NON-NLS-1$
    if (cppDocument == null) {
      cppDocument = new CppDocument();
    }
    try {
      File file = new File(fileName);
      cppDocument.readFrom(new FileReader(file));
      codeViewer.setDocument(cppDocument);
      statusBar.showMessage(file.getCanonicalPath());
    } catch (IOException ioe) {
      setPlainMessage("Loading failed"); //$NON-NLS-1$
    }
  }

  private void loadJavaFile(String fileName) {
    statusBar.clearMessage();
    setPlainMessage("Loading..."); //$NON-NLS-1$
    if (javaDocument == null) {
      javaDocument = new JavaDocument();
    }
    try {
      File file = new File(fileName);
      javaDocument.readFrom(new InputStreamReader(new FileInputStream(file),
                                                  "UTF8")); //$NON-NLS-1$
      codeViewer.setDocument(javaDocument);
      statusBar.showMessage(file.getCanonicalPath());
    } catch (IOException ioe) {
      setPlainMessage("Loading failed"); //$NON-NLS-1$
    }
  }

/*^**************************************************************************^*/
/*- Main Method.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Starts CodeViewer.
   *
   * @param args the application arguments
   */
  public static void main(String[] args) {
    QSwing.initialize(args);
    CodeBrowser codeViewer = new CodeBrowser();
    codeViewer.setVisible(true);
    QSwing.startEventLoop();
  }

}

/*^*****************************************************************************
  File History:

 - 2007-12-15 12:21:48 Slobodan
   Initial version

*****************************************************************************^*/
