/*
 * @(#)DemoModule.java  0.2.0 / 2007-11-20
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.demo.qswingset;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ResourceBundle;

import yu.ac.bg.etf.javaqx.qswing.Icon;
import yu.ac.bg.etf.javaqx.qswing.ImageIcon;
import yu.ac.bg.etf.javaqx.qswing.JQComponent;
import yu.ac.bg.etf.javaqx.qswing.QSwing;
import yu.ac.bg.etf.javaqx.util.Utilities;

/**
 * Demo Module Class.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public abstract class DemoModule {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final String name;
  private final String description;
  private final Icon icon;
  private JQComponent component; // NOPMD
  private String sourceCode; // NOPMD

/*^**************************************************************************^*/
/*- Protected Constructors.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Constructs new DemoModule object.
   *
   * @param name the demo name
   * @param description the demo description
   */
  protected DemoModule(String name, String description) {
    this.name = name;
    this.description = description;
    String demoIconName = "resources/images/" + //$NON-NLS-1$
                          getClassName() + // NOPMD
                          ".png"; //$NON-NLS-1$
    icon = new ImageIcon(loadResource(demoIconName)); // NOPMD
  }

  /**
   * Constructs new DemoModule object.
   *
   * @param name the demo name
   * @param description the demo description
   * @param icon the demo icon
   */
  protected DemoModule(String name, String description, Icon icon) {
    this.name = name;
    this.description = description;
    this.icon = icon;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns demo name.
   *
   * @return demo name
   */
  public String getName() {
    return name;
  }

  /**
   * Returns demo description.
   *
   * @return demo description
   */
  public String getDescription() {
    return description;
  }

  /**
   * Returns demo icon.
   *
   * @return demo icon
   */
  public Icon getIcon() {
    return icon;
  }

  /**
   * Returns demo component.
   *
   * @return demo component
   */
  public JQComponent getComponent() {
    if (component == null) {
      component = buildDemoComponent();
    }
    return component;
  }

  /**
   * Returns demo source code.
   *
   * @return demo source code
   */
  public String getSourceCode() {
    if (sourceCode == null) {
      sourceCode = loadSourceCode();
    }
    return sourceCode;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Builds demo component.
   *
   * @return demo component
   */
  protected abstract JQComponent buildDemoComponent();

  /**
   * Loads demo source code.
   *
   * @return demo source code
   */
  protected String loadSourceCode() {
    try {
      String demoName = getClassName() + ".java"; //$NON-NLS-1$
      return Utilities.toString(loadResourceAsStream(demoName));
    } catch (IOException ex) {
      return "Could not load source code"; //$NON-NLS-1$
    }
  }

  /**
   * Loads the specified resource.
   *
   * @param resourceName the resource name
   *
   * @return the resource url
   */
  protected URL loadResource(String resourceName) {
    return getClass().getResource(resourceName);
  }

  /**
   * Loads the specified resource.
   *
   * @param resourceName the resource name
   *
   * @return the resource url
   */
  protected InputStream loadResourceAsStream(String resourceName) {
    return getClass().getResourceAsStream(resourceName);
  }

  /**
   * Returns text from the main resource bundle.
   *
   * @param text String
   * @return String
   */
  protected String getText(String text) {
    ResourceBundle bundle = ResourceBundle.getBundle(
             "yu/ac/bg/etf/javaqx/demo/qswingset/resources/Text"); //$NON-NLS-1$
    return bundle.getString(text);
  }

  /**
   * Returns the name of the demo class.
   *
   * @return the name of the demo class
   */
  protected String getClassName() {
    Class demoClass = getClass();
    String className = demoClass.getName();
    return className.substring(className.lastIndexOf('.') + 1);
  }

  /**
   * Displays demo as standalone window.
   */
  protected void show() {
    JQComponent component = getComponent();
    component.setName(getName());
    component.setVisible(true);
    QSwing.startEventLoop(true);
  }

}

/*^*****************************************************************************
  File History:

 - 2007-11-20 20:12:35 Slobodan
   Initial version

*****************************************************************************^*/
