/*
 * @(#)JavaQx.java  0.2.0 / 2007-09-11
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx;

import java.io.File;
import java.security.CodeSource;

import yu.ac.bg.etf.javaqx.logging.Logger;

/**
 * Java Qt Extensions.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic <slobodan.vrkacevic@gmail.com>
 * @version 0.2.0
 */
public final class JavaQx {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final String NATIVE_LIBRARY_NAME =
                                           "yu-ac-bg-etf-javaqx"; //$NON-NLS-1$
  private static boolean initialized;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private JavaQx() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if the native library is successfully initialized.
   *
   * @return <tt>true</tt> if the native library is successfully initialized
   */
  public static boolean isInitialized() {
    return initialized;
  }

  /**
   * Initializes the library.
   */
  public static void initialize() {
    if (!initialized) {
      initialized = true;
      try {
        if (loadNativeLibrary() || loadDefaultNativeLibrary()) {
          if (JavaQxConfig.INFO) {
            Logger.logInfo("Native library loaded"); //$NON-NLS-1$
          }
        } else {
          Logger.logError("Could not load native library"); //$NON-NLS-1$
          System.exit(-1); // NOFB
        }
      } catch (Throwable t) {
        Logger.logError("Unexpected failure", t); //$NON-NLS-1$
        System.exit(-1); // NOFB
      }
      _initialize(Logger.class);
      System.setProperty("javaqx.version", //$NON-NLS-1$
                         JavaQxInfo.LIBRARY_VERSION_STRING);
      System.setProperty("javaqx.date", //$NON-NLS-1$
                         JavaQxInfo.LIBRARY_DATE_STRING);
    }
  }

/*^**************************************************************************^*/
/*- Private Static Methods.                                                  -*/
/*^**************************************************************************^*/

  private static boolean loadNativeLibrary() {
    try {
      if (JavaQxConfig.INFO) {
        Logger.logInfo("Loading native library: " + //$NON-NLS-1$
                       System.mapLibraryName(NATIVE_LIBRARY_NAME));
      }
      System.loadLibrary(NATIVE_LIBRARY_NAME);
      return true;
    } catch (UnsatisfiedLinkError ule) {
      if (JavaQxConfig.INFO) {
        Logger.logWarning("Loading native library failed", ule); //$NON-NLS-1$
      }
      return false;
    }
  }

  private static boolean loadDefaultNativeLibrary() {
    try {
      String nativeLibrary = getDefaultNativeLibrary();
      if (JavaQxConfig.INFO) {
        Logger.logInfo("Loading native library: " + //$NON-NLS-1$
                       nativeLibrary);
      }
      System.load(nativeLibrary);
      return true;
    } catch (UnsatisfiedLinkError ule) {
      if (JavaQxConfig.INFO) {
        Logger.logWarning("Loading native library failed", ule); //$NON-NLS-1$
      }
      return false;
    }
  }

  private static String getDefaultNativeLibrary() {
    String nativeLibraryFileName = System.mapLibraryName(NATIVE_LIBRARY_NAME);
    CodeSource codeSource = JavaQx.class.getProtectionDomain().getCodeSource();
    File libraryFile;
    if (codeSource != null) {
      String classLibraryFileName = codeSource.getLocation().getFile();
      File classLibrary = new File(classLibraryFileName);
      classLibrary = classLibrary.getAbsoluteFile();
      if (classLibrary.isFile()) {
        libraryFile = new File(classLibrary.getParentFile(),
                               nativeLibraryFileName);
        if (libraryFile.exists()) {
          return libraryFile.getAbsolutePath();
        }
      }
    }
    libraryFile = new File(nativeLibraryFileName);
    if (libraryFile.exists()) {
      return libraryFile.getAbsolutePath();
    }
    File parentFile = libraryFile.getAbsoluteFile().getParentFile();
    if (parentFile != null) {
      File libDir = new File(parentFile, "lib"); //$NON-NLS-1$
      if (libDir.exists()) {
        libraryFile = new File(libDir, nativeLibraryFileName);
        if (libraryFile.exists()) {
          return libraryFile.getAbsolutePath();
        }
      }
      parentFile = parentFile.getParentFile();
      if (parentFile != null) {
        libDir = new File(parentFile, "lib"); //$NON-NLS-1$
        if (libDir.exists()) {
          libraryFile = new File(libDir, nativeLibraryFileName);
          if (libraryFile.exists()) { // NOPMD
            return libraryFile.getAbsolutePath();
          }
        }
      }
    }
    return System.getProperty("java.io.tmpdir") + //$NON-NLS-1$
           System.mapLibraryName("yu-ac-bg-etf-javaqx"); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Private Static Native Method.                                            -*/
/*^**************************************************************************^*/

  private static native void _initialize(Class loggerClass);

}

/*^*****************************************************************************
  File History:

 - 2007-09-11 17:59:08 Slobodan
   Initial version

*****************************************************************************^*/
