/*
 * @(#)SimpleLogger.java  0.2.0 / 2007-02-12
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.logging.simple;

import java.io.OutputStream;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;

import yu.ac.bg.etf.javaqx.logging.Logger;
import yu.ac.bg.etf.javaqx.logging.LoggingAdapter;

/**
 * Simple logging adapter implementation.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class SimpleLogger implements LoggingAdapter {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final int LOG_METHOD_INDEX = 2;
  private static final String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final PrintStream errorStream;
  private final PrintStream infoStream;
  private final SimpleDateFormat dateFormater;
  private int level;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SimpleLoggingAdapter object.
   */
  public SimpleLogger() {
    this(System.err);
  }

  /**
   * Constructs new SimpleLoggingAdapter object.
   *
   * @param infoStream the info stream
   */
  public SimpleLogger(OutputStream infoStream) {
    this(new PrintStream(infoStream));
  }

  /**
   * Constructs new SimpleLoggingAdapter object.
   *
   * @param errorStream the error stream
   * @param infoStream the info stream
   */
  public SimpleLogger(OutputStream errorStream, OutputStream infoStream) {
    this(new PrintStream(infoStream), new PrintStream(errorStream));
  }

  /**
   * Constructs new SimpleLoggingAdapter object.
   *
   * @param errorStream the error print stream
   */
  public SimpleLogger(PrintStream errorStream) {
    this(errorStream, errorStream);
  }

  /**
   * Constructs new SimpleLoggingAdapter object.
   *
   * @param errorStream the error print stream
   * @param infoStream the info print stream
   */
  public SimpleLogger(PrintStream errorStream, PrintStream infoStream) {
    if (errorStream == null) {
      throw new IllegalArgumentException("Null errorStream"); //$NON-NLS-1$
    }
    this.errorStream = errorStream;
    if (infoStream == null) {
      this.infoStream = errorStream;
    } else {
      this.infoStream = infoStream;
    }
    dateFormater = new SimpleDateFormat(DATE_FORMAT); // NOPMD - default locale
  }

/*^**************************************************************************^*/
/*- LoggingAdapter Interface Implementation.                                 -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String getName() {
    return "Simple Logger"; //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  public boolean isLoggable(int level) {
    return (this.level <= level);
  }

  /**
   * {@inheritDoc}
   */
  public int getLevel() {
    return level;
  }

  /**
   * {@inheritDoc}
   */
  public void setLevel(int level) {
    if (level < Logger.LEVEL_INFO || level > Logger.LEVEL_ERROR) {
      throw new IllegalArgumentException("Invalid level"); //$NON-NLS-1$
    }
    this.level = level;
  }

  /**
   * {@inheritDoc}
   */
  public void log(int level, String message, Throwable t) {
    if (level >= this.level) {
      StringBuffer location = new StringBuffer();
      Throwable traceException = new Throwable(); // NOPMD - fake exception
      StackTraceElement[] stackTrace = traceException.getStackTrace();
      if ((stackTrace != null) && (stackTrace.length > LOG_METHOD_INDEX)) {
        location.append(stackTrace[LOG_METHOD_INDEX]);
      }
      log(level, location.toString(), message, t);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void log(int level, String fileName, int lineNumber, String message,
                  Throwable t) {
    if (level >= this.level) {
      StringBuffer location = new StringBuffer();
      if (fileName != null) {
        location.append(fileName);
        if (lineNumber >= 0) { // NOPMD
          location.append(':');
          location.append(lineNumber);
        }
      }
      log(level, location.toString(), message, t);
    }
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void log(int level, String location, String message, Throwable t) {
    PrintStream ps;
    if (level == Logger.LEVEL_INFO) {
      ps = infoStream;
    } else {
      ps = errorStream;
    }
    ps.print("["); //$NON-NLS-1$
    ps.print(dateFormater.format(new Date()));
    ps.print("]"); //$NON-NLS-1$
    switch (level) {
      case Logger.LEVEL_INFO:
        ps.print("[INFO]"); //$NON-NLS-1$
        break;
      case Logger.LEVEL_WARN:
        ps.print("[WARN]"); //$NON-NLS-1$
        break;
      default:
        ps.print("[ERROR]"); //$NON-NLS-1$
        break;
    }
    if ((location != null) && (location.length() > 0)) {
      ps.print("["); //$NON-NLS-1$
      ps.print(location);
      ps.print("]"); //$NON-NLS-1$
    }
    if (message == null) {
      ps.println();
    } else {
      ps.print(" "); //$NON-NLS-1$
      ps.println(message);
    }
    if (t != null) {
      t.printStackTrace(ps);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2007-02-11 11:41:45 Slobodan
   Initial version

*****************************************************************************^*/
