/*
 * @(#)TestEnvironment.java  0.2.0 / 2007-12-30
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.nunit;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;

/**
 * Test Environment.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class TestEnvironment {

/*^**************************************************************************^*/
/*- Package-Private Static Final Fields.                                     -*/
/*^**************************************************************************^*/

  static final String STRING_JAVA_HOME = "java.home"; //$NON-NLS-1$
  static final String STRING_CLASSPATH = " -classpath "; //$NON-NLS-1$
  static final String STRING_JAVAW = "javaw"; //$NON-NLS-1$
  static final String STRING_JAVA = "java"; //$NON-NLS-1$
  static final String STRING_GIJ = "gij"; //$NON-NLS-1$
  static final String STRING_DOUBLE_QUOTE = "\""; //$NON-NLS-1$
  static final String STRING_SPACE = " "; //$NON-NLS-1$
  static final String STRING_EMPTY = ""; //$NON-NLS-1$
  static final String STRING_NO_JAVA_COMMAND =
                                   "Could not find java command"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static String commandStart;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private TestEnvironment() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Package-Private Static Methods.                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns JVM launch command.
   *
   * @return JVM launch command
   */
  public static String getLaunchCommand() {
    if (commandStart == null) { // NOPMD
      StringBuffer sb = new StringBuffer(getJavaCommand());
      String libraryPath = getLibraryPath().trim();
      String classPath = getClassPath().trim();
      if (libraryPath.length() > 0) {
        sb.append(" -Djava.library.path="); //$NON-NLS-1$
        sb.append(libraryPath);
      }
      if (classPath.length() > 0) {
        sb.append(STRING_CLASSPATH);
        sb.append(classPath);
      }
      sb.append(STRING_SPACE);
      commandStart = sb.toString();
    }
    return commandStart;
  }

  /**
   * Returns java command (command that starts JVM).
   *
   * @return java command
   */
  public static String getJavaCommand() { // NOPMD
    String command = System.getProperty("javaqx.test.jvm"); //$NON-NLS-1$
    if (command != null) {
      return command;
    }
    String javaHome = getJavaHome();
    if (javaHome != null) {
      javaHome = javaHome + File.separator;
      String javaName = System.getProperty("java.vm.name"); //$NON-NLS-1$
      javaName = javaName.toLowerCase(); // NOPMD NOFB
      String javaBin = javaHome + "bin" + File.separator; //$NON-NLS-1$
      if (javaName.startsWith(STRING_JAVA)) {
        command = javaBin + STRING_JAVAW;
        if (verifyJVM(command)) {
          return command;
        }
        command = javaBin + STRING_JAVA;
        if (verifyJVM(command)) {
          return command;
        }
      } else if (javaName.startsWith("gnu")) { //$NON-NLS-1$
        command = javaBin + STRING_GIJ;
        if (verifyJVM(command)) { // NOPMD
          return command;
        }
        command = javaHome + STRING_GIJ;
        if (verifyJVM(command)) { // NOPMD
          return command;
        }
      }
      command = javaHome + STRING_JAVA;
      if (verifyJVM(command)) {
        return command;
      }
    }
    return findJavaCommand();
  }

  /**
   * Returns java home path.
   *
   * @return java home path
   */
  public static String getJavaHome() {
    return System.getProperty(STRING_JAVA_HOME, STRING_EMPTY);
  }

  /**
   * Returns current class path.
   *
   * @return current class path
   */
  public static String getClassPath() {
    String classPath = System.getProperty("java.class.path"); //$NON-NLS-1$
    if (classPath == null) {
      return STRING_EMPTY;
    }
    if (isWinPath(classPath)) {
      classPath = STRING_DOUBLE_QUOTE + classPath + STRING_DOUBLE_QUOTE;
    }
    return classPath;
  }

  /**
   * Returns library class path.
   *
   * @return library class path
   */
  public static String getLibraryPath() {
    String libraryPath = System.getProperty("java.library.path"); //$NON-NLS-1$
    if (libraryPath == null) {
      libraryPath = STRING_EMPTY;
    } else {
      if (isWinPath(libraryPath)) {
        libraryPath = STRING_DOUBLE_QUOTE +
                      libraryPath.replaceAll(STRING_DOUBLE_QUOTE,
                                             STRING_EMPTY) +
                      STRING_DOUBLE_QUOTE;
      }
    }
    return libraryPath;
  }

  /**
   * Executes the specified command.
   *
   * @param command the command
   * @param dir the working directory
   *
   * @throws IOException on an I/O error
   *
   * @return started process
   */
  public static Process execProcess(String command, File dir) throws
                                                                  IOException {
    return Runtime.getRuntime().exec(command, null, dir);
  }

/*^**************************************************************************^*/
/*- Package-Private Static Methods.                                          -*/
/*^**************************************************************************^*/

  static String findJavaCommand() {
    if (verifyJVM(STRING_JAVAW)) {
      return STRING_JAVAW;
    }
    if (verifyJVM(STRING_JAVA)) {
      return STRING_JAVA;
    }
    if (verifyJVM(STRING_GIJ)) {
      return STRING_GIJ;
    }
    if (testJVM(STRING_JAVAW)) {
      return STRING_JAVAW;
    }
    if (testJVM(STRING_JAVA)) {
      return STRING_JAVA;
    }
    if (testJVM(STRING_GIJ)) {
      return STRING_GIJ;
    }
    throw new RuntimeException(STRING_NO_JAVA_COMMAND); // NOPMD
  }

  static boolean verifyJVM(String jvm) {
    BufferedReader isr = null;
    try {
      Process p = execProcess(jvm + STRING_SPACE + STRING_CLASSPATH +
                              getClassPath() + STRING_SPACE +
                              ExecTestSuite.class.getName() +
                              STRING_SPACE + STRING_JAVA_HOME, null);
      if (p.waitFor() == 0) {
        isr = new BufferedReader(new InputStreamReader(p.getInputStream()));
        return getJavaHome().equals(isr.readLine()); // NOFB
      }
    } catch (Throwable t) { // NOPMD
      // empty catch block
    } finally {
      closeReader(isr);
    }
    return false;
  }

  static boolean testJVM(String jvm) {
    try {
      execProcess(jvm + " -version", null); //$NON-NLS-1$
      return true;
    } catch (Throwable t) {
      return false;
    }
  }

  static boolean isWinPath(String path) {
    return (path.indexOf(":\\") > 0); //$NON-NLS-1$ NOFB
  }

  static void closeStream(InputStream is) {
    if (is != null) {
      try {
        is.close();
      } catch (IOException ioe) {
        System.err.println("Error closing stream"); //$NON-NLS-1$ NOPMD
        ioe.printStackTrace(); // NOPMD
      }
    }
  }

  static void closeStream(OutputStream os) {
    if (os != null) {
      try {
        os.close();
      } catch (IOException ioe) {
        System.err.println("Error closing stream"); //$NON-NLS-1$ NOPMD
        ioe.printStackTrace(); // NOPMD
      }
    }
  }

  static void closeReader(Reader r) {
    if (r != null) {
      try {
        r.close();
      } catch (IOException ioe) {
        System.err.println("Error closing reader"); //$NON-NLS-1$ NOPMD
        ioe.printStackTrace(); // NOPMD
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2007-12-30 10:43:02 Slobodan
   Initial version

*****************************************************************************^*/
