/*
 * @(#)ComponentOrientation.java  0.2.0 / 2007-05-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import java.util.Locale;

/**
 * ComponentOrientation Class.
 * <p>
 *   <tt>NOTE: This class is not full compatible with
 *             <em>java.awt.ComponentOrientation</em> class!</tt>
 * </p>
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class ComponentOrientation {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  public static final ComponentOrientation UNKNOWN;
  public static final ComponentOrientation LEFT_TO_RIGHT;
  public static final ComponentOrientation RIGHT_TO_LEFT;

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final int ORIENTATION_UNKNOWN = 1;
  private static final int ORIENTATION_HORIZONTAL = 2;
  private static final int ORIENTATION_LEFT_TO_RIGHT = 4;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    UNKNOWN = new ComponentOrientation(ORIENTATION_UNKNOWN |
                                       ORIENTATION_HORIZONTAL |
                                       ORIENTATION_LEFT_TO_RIGHT);
    LEFT_TO_RIGHT = new ComponentOrientation(ORIENTATION_HORIZONTAL |
                                             ORIENTATION_LEFT_TO_RIGHT);
    RIGHT_TO_LEFT = new ComponentOrientation(ORIENTATION_HORIZONTAL);
  }

/*^**************************************************************************^*/
/*- Package-Private Field.                                                   -*/
/*^**************************************************************************^*/

  final int value;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  private ComponentOrientation(int orientation) {
    this.value = orientation;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if orientation is horizontal.
   *
   * @return <tt>true</tt> if orientation is horizontal
   */
  public boolean isHorizontal() {
    return (value & ORIENTATION_HORIZONTAL) != 0;
  }

  /**
   * Returns <tt>true</tt> if orientation is left-to-right.
   *
   * @return <tt>true</tt> if orientation is left-to-right
   */
  public boolean isLeftToRight() {
    return (value & ORIENTATION_LEFT_TO_RIGHT) != 0;
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Returns component orientation for the specified locale object.
   *
   * @param locale the locale
   *
   * @return component orientation for the specified object
   */
  public static ComponentOrientation getOrientation(Locale locale) {
    if (locale != null) {
      String lang = locale.getLanguage();
      if ("ar".equals(lang) || "fa".equals(lang) || //$NON-NLS-1$ //$NON-NLS-2$
          "iw".equals(lang) || "ur".equals(lang)) { //$NON-NLS-1$ //$NON-NLS-2$
        return RIGHT_TO_LEFT;
      }
      return LEFT_TO_RIGHT;
    }
    return UNKNOWN;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    return value;
  }

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true;
    }
    if ((obj != null) && (obj.getClass() == getClass())) {
      ComponentOrientation co = (ComponentOrientation)obj;
      return co.value == value;
    }
    return false;
  }

}

/*^*****************************************************************************
  File History:

 - 2007-05-04 18:44:29 Slobodan
   Initial version

*****************************************************************************^*/
