/*
 * @(#)Cursor.java  0.2.0 / 2007-10-03
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

/**
 * Cursor Class.
 * <p>
 *   <tt>NOTE: This class is not full compatible with
 *             <em>java.awt.Cursor</em> class!</tt>
 * </p>
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class Cursor {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * The default cursor type.
   */
  public static final int DEFAULT_CURSOR = 0;

  /**
   * The crosshair cursor type.
   */
  public static final int CROSSHAIR_CURSOR = 1;

  /**
   * The text cursor type.
   */
  public static final int TEXT_CURSOR = 2;

  /**
   * The wait cursor type.
   */
  public static final int WAIT_CURSOR = 3;

  /**
   * The south-west-resize cursor type.
   */
  public static final int SW_RESIZE_CURSOR = 4;

  /**
   * The south-east-resize cursor type.
   */
  public static final int SE_RESIZE_CURSOR = 5;

  /**
   * The north-west-resize cursor type.
   */
  public static final int NW_RESIZE_CURSOR = 6;

  /**
   * The north-east-resize cursor type.
   */
  public static final int NE_RESIZE_CURSOR = 7;

  /**
   * The north-resize cursor type.
   */
  public static final int N_RESIZE_CURSOR = 8;

  /**
   * The south-resize cursor type.
   */
  public static final int S_RESIZE_CURSOR = 9;

  /**
   * The west-resize cursor type.
   */
  public static final int W_RESIZE_CURSOR = 10;

  /**
   * The east-resize cursor type.
   */
  public static final int E_RESIZE_CURSOR = 11;

  /**
   * The hand cursor type.
   */
  public static final int HAND_CURSOR = 12;

  /**
   * The move cursor type.
   */
  public static final int MOVE_CURSOR = 13;

  /**
   * The up arrow cursor type.
   */
  public static final int UP_ARROW_CURSOR = 14;

  /**
   * The forbidden cursor type.
   */
  public static final int FORBIDDEN_CURSOR = 15;

  /**
   * The whats this cursor type.
   */
  public static final int WHATS_THIS_CURSOR = 16;

  /**
   * The vertical split cursor type.
   */
  public static final int V_SPLIT_CURSOR = 17;

  /**
   * The horizontal cursor type.
   */
  public static final int H_SPLIT_CURSOR = 18;

  /**
   * The blank cursor type.
   */
  public static final int BLANK_CURSOR = 19;

  /**
   * The blank cursor type.
   */
  public static final int CUSTOM_CURSOR = 20;

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final Cursor[] PREDEFINED_CURSORS;
  private static final String[] CURSOR_NAMES;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    PREDEFINED_CURSORS = new Cursor[20];
    CURSOR_NAMES = new String[] {
        "Default Cursor", //$NON-NLS-1$
        "Crosshair Cursor", //$NON-NLS-1$
        "Text Cursor", //$NON-NLS-1$
        "Wait Cursor", //$NON-NLS-1$
        "Southwest Resize Cursor", //$NON-NLS-1$
        "Southeast Resize Cursor", //$NON-NLS-1$
        "Northwest Resize Cursor", //$NON-NLS-1$
        "Northeast Resize Cursor", //$NON-NLS-1$
        "North Resize Cursor", //$NON-NLS-1$
        "South Resize Cursor", //$NON-NLS-1$
        "West Resize Cursor", //$NON-NLS-1$
        "East Resize Cursor", //$NON-NLS-1$
        "Hand Cursor", //$NON-NLS-1$
        "Move Cursor", //$NON-NLS-1$
        "Up Arrow Cursor", //$NON-NLS-1$
        "Forbidden Cursor", //$NON-NLS-1$
        "Whats This Cursor", //$NON-NLS-1$
        "Vertical Split Cursor", //$NON-NLS-1$
        "Horizontal Split Cursor", //$NON-NLS-1$
        "Blank Cursor" //$NON-NLS-1$
    };
  }

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  final int type;
  final String name;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new Cursor object.
   *
   * @param type the cursor type
   *
   * @throws IllegalArgumentException if the type is invalid
   */
  public Cursor(int type) {
    if ((type < DEFAULT_CURSOR) || (type > BLANK_CURSOR)) {
      throw new IllegalArgumentException("Invalid cursor type"); //$NON-NLS-1$
    }
    this.type = type;
    name = CURSOR_NAMES[type];
  }

/*^**************************************************************************^*/
/*- Package-Private Constructor.                                             -*/
/*^**************************************************************************^*/

  /** @todo  implement custom cursors */

  Cursor(String name) {
    if (name == null) {
      throw new IllegalArgumentException("Null cursor name"); //$NON-NLS-1$
    }
    this.type = CUSTOM_CURSOR;
    this.name = name;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns cursor type.
   *
   * @return cursor type
   */
  public int getType() {
    return type;
  }

  /**
   * Returns cursor name.
   *
   * @return cursor name
   */
  public String getName() {
    return name;
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns predefined cursor.
   *
   * @param type the cursor type
   *
   * @return predefined cursor
   *
   * @throws IllegalArgumentException if the type is invalid
   */
  public static Cursor getPredefinedCursor(int type) {
    if ((type < DEFAULT_CURSOR) || (type > BLANK_CURSOR)) {
      throw new IllegalArgumentException("Invalid cursor type"); //$NON-NLS-1$
    }
    synchronized (PREDEFINED_CURSORS) {
      if (PREDEFINED_CURSORS[type] == null) {
        PREDEFINED_CURSORS[type] = new Cursor(type);
      }
      return PREDEFINED_CURSORS[type];
    }
  }

  /**
   * Returns the system default cursor.
   *
   * @return the system default cursor.
   */
  public static Cursor getDefaultCursor() {
    return getPredefinedCursor(DEFAULT_CURSOR);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    if (type == CUSTOM_CURSOR) {
      return name.hashCode();
    }
    return type;
  }

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true;
    }
    if ((obj != null) && (obj.getClass() == getClass())) {
      Cursor cursor = (Cursor)obj;
      if (type == CUSTOM_CURSOR) {
        return ((cursor.type == CUSTOM_CURSOR) && name.equals(cursor.name));
      }
      return (type == cursor.type);
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public String toString() {
    return (getClass() + "[type=" + type + //$NON-NLS-1$
            ",name=" + name + "]"); //$NON-NLS-1$ //$NON-NLS-2$
  }

}

/*^*****************************************************************************
  File History:

 - 2007-10-03 15:04:37 Slobodan
   Initial version

*****************************************************************************^*/
