/*
 * @(#)JQMainWindow.java  0.2.0 / 2007-05-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import java.util.ArrayList;

import yu.ac.bg.etf.javaqx.qswing.graphics.Dimension;
import yu.ac.bg.etf.javaqx.qswing.graphics.Insets;
import yu.ac.bg.etf.javaqx.qswing.layouts.LayoutManager;

/**
 * Frame Component.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class JQFrame extends JQWindow {

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  JQMenuBar menuBar;
  ArrayList toolBars;
  JQContainer contentPane;
  JQStatusBar statusBar;

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JQFrame object.
   */
  public JQFrame() {
    this(null);
  }

  /**
   * Constructs new JQFrame object.
   *
   * @param title the frame title
   */
  public JQFrame(String title) {
    super(null, title);
    toolBars = new ArrayList();
    contentPane = new ContentPane();
    _setContentPane(_bridge, contentPane._bridge); // NOPMD
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns menu bar.
   *
   * @return menu bar
   */
  public JQMenuBar getMenuBar() {
    return menuBar;
  }

  /**
   * Sets menu bar.
   *
   * @param menuBar the menu bar
   *
   * @return menu bar
   */
  public JQMenuBar setMenuBar(JQMenuBar menuBar) {
    final JQMenuBar oldMenuBar = this.menuBar;
    if (oldMenuBar != menuBar) { // NOPMD
      disposedCheck();
      if (menuBar == null) {
        if (oldMenuBar != null) {
          releaseChild(oldMenuBar);
        }
        _setMenuBar(_bridge, 0);
        this.menuBar = null;
      } else {
        if (menuBar.isDisposed()) {
          throw new IllegalArgumentException(
                                     "Adding disposed menu bar"); //$NON-NLS-1$
        }
        if (menuBar.parent != null) {
          throw new IllegalArgumentException(
                                     "Adding parented menu bar"); //$NON-NLS-1$
        }
        acquireChild(menuBar);
        _setMenuBar(_bridge, menuBar._bridge);
        this.menuBar = menuBar;
      }
    }
    return menuBar;
  }

  /**
   * Returns the number of toolbars.
   *
   * @return the number of toolbars
   */
  public int getToolBarCount() {
    return toolBars.size();
  }

  /**
   * Returns the toolbar at specified index.
   *
   * @param index the index of the toolbar
   *
   * @return the toolbar at specified index
   *
   * @throws IndexOutOfBoundsException if index is out of range
   *         <tt>((index &lt; 0) | (index &gt;= getToolBarCount()))</tt>.
   */
  public JQToolBar getToolBar(int index) {
    return (JQToolBar)toolBars.get(index);
  }

  /**
   * Adds toolbar.
   *
   * @param toolBar the tool bar
   *
   * @return toolbar
   */
  public JQToolBar addToolBar(JQToolBar toolBar) {
    if (toolBar == null) {
      throw new IllegalArgumentException("Null toolbar"); //$NON-NLS-1$
    }
    if (toolBar.isDisposed()) {
      throw new IllegalArgumentException(
                                      "Adding disposed toolBar"); //$NON-NLS-1$
    }
    if (toolBar.parent != null) {
      throw new IllegalArgumentException(
                                      "Adding parented toolBar"); //$NON-NLS-1$
    }
    disposedCheck();
    acquireChild(toolBar);
    _addToolBar(_bridge, toolBar._bridge);
    toolBars.add(toolBar);
    return toolBar;
  }

  /**
   * Removes the specified toolbar.
   *
   * @param toolBar the toolbar to be removed
   */
  public void removeToolBar(JQToolBar toolBar) {
    if ((toolBar != null) && !toolBar.isDisposed() &&
        (toolBar.parent != null)) {
      disposedCheck();
      releaseChild(toolBar);
      _removeToolBar(_bridge, toolBar._bridge);
      toolBars.remove(toolBar);
    }
  }

  /**
   * Returns status bar.
   *
   * @return status bar
   */
  public JQStatusBar getStatusBar() {
    return statusBar;
  }

  /**
   * Sets status bar.
   *
   * @param statusBar the status bar
   *
   * @return status bar
   */
  public JQStatusBar setStatusBar(JQStatusBar statusBar) {
    final JQStatusBar oldStatusBar = this.statusBar;
    if (oldStatusBar != statusBar) { // NOPMD
      disposedCheck();
      if (statusBar == null) {
        if (oldStatusBar != null) {
          releaseChild(oldStatusBar);
        }
        _setStatusBar(_bridge, 0);
        this.statusBar = null;
      } else {
        if (statusBar.isDisposed()) {
          throw new IllegalArgumentException(
                                   "Adding disposed status bar"); //$NON-NLS-1$
        }
        if (statusBar.parent != null) {
          throw new IllegalArgumentException(
                                   "Adding parented status bar"); //$NON-NLS-1$
        }
        acquireChild(statusBar);
        _setStatusBar(_bridge, statusBar._bridge);
        this.statusBar = statusBar;
      }
    }
    return statusBar;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize() {
    if (!preferredSizeSet && (!valid || (preferredSize == null))) {
      disposedCheck();
      preferredSize = _getPreferredSize(_bridge);
      if (insets != null) {
        preferredSize = preferredSize.enlarge(insets.getLeft() +
                                              insets.getRight(),
                                              insets.getTop() +
                                              insets.getBottom());
      }
    }
    return preferredSize;
  }

  /**
   * {@inheritDoc}
   */
  public Insets getInsets() {
    return contentPane.getInsets();
  }

  /**
   * {@inheritDoc}
   */
  public void setInsets(Insets insets) {
    contentPane.setInsets(insets);
  }

  /**
   * {@inheritDoc}
   */
  public void invalidate() {
    contentPane.invalidate();
  }

  /**
   * {@inheritDoc}
   */
  public void validate() {
    contentPane.validate();
  }

  /**
   * {@inheritDoc}
   */
  public void setComponentOrientation(ComponentOrientation orientation) {
    super.setComponentOrientation(orientation);
    if (menuBar != null) {
      menuBar.setComponentOrientation(orientation);
    }
    for (int i = toolBars.size(); --i >= 0;) {
      ((JQToolBar)toolBars.get(i)).setComponentOrientation(orientation);
    }
    contentPane.setComponentOrientation(orientation);
    if (statusBar != null) {
      statusBar.setComponentOrientation(orientation);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void updateLocalization() {
    super.updateLocalization();
    if (title != null) {
      _setTitle(_bridge, localizeString(title));
    }
    if (menuBar != null) {
      menuBar.updateLocalization();
    }
    for (int i = toolBars.size(); --i >= 0;) {
      ((JQToolBar)toolBars.get(i)).updateLocalization();
    }
    contentPane.updateLocalization();
    if (statusBar != null) {
      statusBar.updateLocalization();
    }
  }

  /**
   * {@inheritDoc}
   */
  public void remove(JQComponent component) {
    contentPane.remove(component);
  }

  /**
   * {@inheritDoc}
   */
  public JQComponent remove(int index) {
    return contentPane.remove(index);
  }

  /**
   * {@inheritDoc}
   */
  public JQComponent getComponent(int index) {
    return contentPane.getComponent(index);
  }

  /**
   * {@inheritDoc}
   */
  public int getComponentCount() {
    return contentPane.getComponentCount();
  }

  /**
   * {@inheritDoc}
   */
  public LayoutManager getLayout() {
    return contentPane.getLayout();
  }

  /**
   * {@inheritDoc}
   */
  public void setLayout(LayoutManager layoutManager) {
    contentPane.setLayout(layoutManager);
  }

  /**
   * {@inheritDoc}
   */
  public void doLayout() {
    contentPane.doLayout();
  }

  /**
   * {@inheritDoc}
   */
  protected JQComponent addImpl(JQComponent comp, Object constraints,
                                int index) {
    return contentPane.addImpl(comp, constraints, index);
  }

  void destroyComponent() {
    super.destroyComponent();
    // menu bar, content pane and status bar are automatically destroyed
    for (int i = toolBars.size(); --i >= 0;) {
      ((JQToolBar)toolBars.get(i)).dispose();
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Native Method.                                           -*/
/*^**************************************************************************^*/

  native long _create();

  native void _setMenuBar(long bridge, long menuBarBridge);

  native void _addToolBar(long bridge, long toolBarBridge);

  native void _removeToolBar(long bridge, long toolBarBridge);

  native void _setContentPane(long bridge, long contentPaneBridge);

  native void _setStatusBar(long bridge, long statusBarBridge);

}

/*^*****************************************************************************
  File History:

 - 2007-05-04 18:50:55 Slobodan
   Initial version

*****************************************************************************^*/
