/*
 * @(#)JQLabel.java  0.2.0 / 2007-05-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import yu.ac.bg.etf.javaqx.qswing.properties.JQLabelProperties;

/**
 * Label Component.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class JQLabel extends JQComponent
                     implements QSwingConstants, JQLabelProperties {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * Plain text format (the value is {@value}).
   */
  public static final int PLAIN_TEXT = 0;

  /**
   * Rich text format (the value is {@value}).
   */
  public static final int RICH_TEXT = 1;

  /**
   * Auto text format (the value is {@value}).
   */
  public static final int AUTO_TEXT = 2;

  /**
   * <tt>None</tt> border type (the value is {@value}).
   */
  public static final int BORDER_TYPE_NONE = 0;

  /**
   * <tt>Box</tt> border type (the value is {@value}).
   */
  public static final int BORDER_TYPE_BOX = 1;

  /**
   * <tt>Standard</tt> border type (the value is {@value}).
   */
  public static final int BORDER_TYPE_STANDARD = 2;

  /**
   * <tt>Classic</tt> border type (the value is {@value}).
   */
  public static final int BORDER_TYPE_CLASSIC = 3;

  /**
   * <tt>Styled</tt> border type (the value is {@value}).
   */
  public static final int BORDER_TYPE_STYLED = 6;

  /**
   * <tt>Plain</tt> border (the value is {@value}).
   */
  public static final int BORDER_STYLE_PLAIN = 16;

  /**
   * <tt>Raised</tt> border (the value is {@value}).
   */
  public static final int BORDER_STYLE_RAISED = 32;

  /**
   * <tt>Sunken</tt> border (the value is {@value}).
   */
  public static final int BORDER_STYLE_SUNKEN = 48;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    _initialize();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private String text;
  private int textFormat;
  private boolean wrapText;
  private int horizontalAlignment;
  private int verticalAlignment;
  private QSwingIconSet iconSet;
  private int borderType;
  private int borderStyle;
  private int borderWidth;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JQLabel object.
   */
  public JQLabel() {
    this("", null, LEADING); //$NON-NLS-1$
  }

  /**
   * Constructs new JQLabel object.
   *
   * @param text the text
   */
  public JQLabel(String text) {
    this(text, null, LEADING);
  }

  /**
   * Constructs new JQLabel object.
   *
   * @param icon the icon
   */
  public JQLabel(Icon icon) {
    this("", icon, LEADING); //$NON-NLS-1$
  }

  /**
   * Constructs new JQLabel object.
   *
   * @param text the text
   * @param icon the icon
   */
  public JQLabel(String text, Icon icon) {
    this(text, icon, LEADING);
  }

  /**
   * Constructs new JQLabel object.
   *
   * @param text the text
   * @param horizontalAlignment the horizontal alignment
   */
  public JQLabel(String text, int horizontalAlignment) {
    this(text, null, horizontalAlignment);
  }

  /**
   * Constructs new JQLabel object.
   *
   * @param text the text
   * @param icon the icon
   * @param horizontalAlignment the horizontal alignment
   */
  public JQLabel(String text, Icon icon, int horizontalAlignment) {
    verticalAlignment = CENTER;
    this.horizontalAlignment = horizontalAlignment;
    textFormat = AUTO_TEXT;
    borderStyle = BORDER_STYLE_PLAIN;
    if (text != null) {
      this.text = text;
      _setText(_bridge, text); // NOPMD
    }
    if (icon != null) {
      iconSet = new QSwingIconSet(this);
      _setIcon(_bridge, iconSet); // NOPMD
      iconSet.setIcon(icon, null);
      _setIconSize(_bridge, icon.getIconWidth(), icon.getIconHeight()); // NOPMD
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns text.
   *
   * @return text
   */
  public String getText() {
    return text;
  }

  /**
   * Sets text.
   *
   * @param text the text
   */
  public void setText(String text) {
    final String oldText = this.text;
    if (oldText == null) {
      if (text == null) {
        return;
      }
    } else {
      if (oldText.equals(text)) {
        return;
      }
    }
    disposedCheck();
    _setText(_bridge, text);
    this.text = text;
    invalidate();
    firePropertyChange(PROPERTY_TEXT, oldText, text);
  }

  /**
   * Returns text format.
   *
   * @return text format
   */
  public int getTextFormat() {
    return textFormat;
  }

  /**
   * Sets text format.
   * <p>Valid values are: <tt>PLAIN_TEXT</tt>, <tt>RICH_TEXT</tt> and
   *                      <tt>AUTO_TEXT</tt>.
   * <br/>
   *   Default value is: <tt>AUTO_TEXT</tt>.
   * </p>
   *
   * @param textFormat the new text format
   */
  public void setTextFormat(int textFormat) {
    int oldTextFormat = this.textFormat;
    if (oldTextFormat != textFormat) {
      disposedCheck();
      this.textFormat = checkTextFormatKey(textFormat,
                                           "Invlid text format"); //$NON-NLS-1$
      _setTextFormat(_bridge, textFormat);
      firePropertyChange(PROPERTY_TEXT_FORMAT, oldTextFormat, textFormat);
    }
  }

  /**
   * Returns <tt>true</tt> if the label is wrapping text.
   *
   * @return <tt>true</tt> if the label is wrapping text
   */
  public boolean isWrapText() {
    return wrapText;
  }

  /**
   * Sets text wrap status.
   *
   * @param wrapText if <tt>true</tt> the label will wrap the text
   */
  public void setWrapText(boolean wrapText) {
    boolean oldWrapText = this.wrapText;
    if (oldWrapText != wrapText) {
      disposedCheck();
      _setWordWrap(_bridge, wrapText);
      this.wrapText = wrapText;
      firePropertyChange(PROPERTY_WRAP_TEXT, oldWrapText, wrapText);
    }
  }

  /**
   * Returns horizontal alignment.
   *
   * @return horizontal alignment
   */
  public int getHorizontalAlignment() {
    return horizontalAlignment;
  }

  /**
   * Sets horizontal alignment.
   * <p>Valid values are: <tt>LEFT</tt>, <tt>CENTER</tt>, <tt>RIGHT</tt>,
   *                      <tt>LEADING</tt> and <tt>TRAILING</tt>.
   * <br/>
   *   Default value is: <tt>LEADING</tt>.
   * </p>
   *
   * @param alignment the horizontal alignment
   */
  public void setHorizontalAlignment(int alignment) {
    int oldHorizontalAlignment = horizontalAlignment;
    if (oldHorizontalAlignment != alignment) {
      disposedCheck();
      horizontalAlignment = checkHorizontalKey(alignment,
                              "Invlid horizontal alignment"); //$NON-NLS-1$
      _setAlignment(_bridge, alignment, verticalAlignment);
      firePropertyChange(PROPERTY_HORIZONTAL_ALIGNMENT, oldHorizontalAlignment,
                         alignment);
    }
  }

  /**
   * Returns vertical alignment.
   *
   * @return vertical alignment
   */
  public int getVerticalAlignment() {
    return verticalAlignment;
  }

  /**
   * Sets vertical alignment.
   * <p>Valid values are: <tt>TOP</tt>, <tt>CENTER</tt> and <tt>BOTTOM</tt>.
   * <br/>
   *   Default value is: <tt>CENTER</tt>.
   * </p>
   *
   * @param alignment the vertical alignment
   */
  public void setVerticalAlignment(int alignment) {
    int oldVerticalAlignment = verticalAlignment;
    if (oldVerticalAlignment != alignment) {
      disposedCheck();
      verticalAlignment = checkVerticalKey(alignment,
                            "Invlid vertical alignment"); //$NON-NLS-1$
      _setAlignment(_bridge, horizontalAlignment, alignment);
      firePropertyChange(PROPERTY_VERTICAL_ALIGNMENT, oldVerticalAlignment,
                         alignment);
    }
  }

  /**
   * Returns icon.
   *
   * @return icon
   */
  public Icon getIcon() {
    if (iconSet != null) {
      return iconSet.getIcon();
    }
    return null;
  }

  /**
   * Sets icon.
   *
   * @param icon the icon
   */
  public void setIcon(Icon icon) {
    disposedCheck();
    if (icon == null) {
      if (iconSet != null) {
        iconSet.setIcon(null, PROPERTY_ICON);
        if (iconSet.isEmpty()) {
          _setIcon(_bridge, null);
          iconSet = null;
        }
      }
    } else {
      if (iconSet == null) {
        iconSet = new QSwingIconSet(this);
        _setIcon(_bridge, iconSet);
      }
      iconSet.setIcon(icon, PROPERTY_ICON);
      _setIconSize(_bridge, icon.getIconWidth(), icon.getIconHeight());
    }
  }

  /**
   * Returns disabled icon.
   *
   * @return disabled icon
   */
  public Icon getDisabledIcon() {
    if (iconSet != null) {
      return iconSet.getDisabledIcon();
    }
    return null;
  }

  /**
   * Sets disabled icon.
   *
   * @param icon the disabled icon
   */
  public void setDisabledIcon(Icon icon) {
    disposedCheck();
    if (icon == null) {
      if (iconSet != null) {
        iconSet.setDisabledIcon(null, PROPERTY_DISABLED_ICON);
        if (iconSet.isEmpty()) {
          _setIcon(_bridge, null);
          iconSet = null;
        }
      }
    } else {
      if (iconSet == null) {
        iconSet = new QSwingIconSet(this);
        _setIcon(_bridge, iconSet);
      }
      iconSet.setDisabledIcon(icon, PROPERTY_DISABLED_ICON);
      _setIconSize(_bridge, icon.getIconWidth(), icon.getIconHeight());
    }
  }

  /**
   * Returns border type.
   *
   * @return border type
   */
  public int getBorderType() {
    return borderType;
  }

  /**
   * Sets border type.
   *
   * @param borderType the border type
   */
  public void setBorderType(int borderType) {
    if ((borderType != BORDER_TYPE_NONE) && (borderType != BORDER_TYPE_BOX) &&
        (borderType != BORDER_TYPE_STANDARD) &&
        (borderType != BORDER_TYPE_CLASSIC) &&
        (borderType != BORDER_TYPE_STYLED)) {
      throw new IllegalArgumentException("Invalid border type"); //$NON-NLS-1$
    }
    final int oldBorderType = this.borderType;
    if (oldBorderType != borderType) {
      disposedCheck();
      _setBorderType(_bridge, borderType);
      this.borderType = borderType;
      firePropertyChange(PROPERTY_BORDER_TYPE, oldBorderType, borderType);
    }
  }

  /**
   * Returns border style.
   *
   * @return border style
   */
  public int getBorderStyle() {
    return borderStyle;
  }

  /**
   * Sets border style.
   *
   * @param borderStyle the border style
   */
  public void setBorderStyle(int borderStyle) {
    if ((borderStyle != BORDER_STYLE_PLAIN) &&
        (borderStyle != BORDER_STYLE_RAISED) &&
        (borderStyle != BORDER_STYLE_SUNKEN)) {
      throw new IllegalArgumentException("Invalid border style"); //$NON-NLS-1$
    }
    final int oldBorderStyle = this.borderStyle;
    if (oldBorderStyle != borderStyle) {
      disposedCheck();
      _setBorderStyle(_bridge, borderStyle);
      this.borderStyle = borderStyle;
      firePropertyChange(PROPERTY_BORDER_STYLE, oldBorderStyle, borderStyle);
    }
  }

  /**
   * Returns border width.
   *
   * @return border width
   */
  public int getBorderWidth() {
    return borderWidth;
  }

  /**
   * Sets border width.
   *
   * @param borderWidth int
   */
  public void setBorderWidth(int borderWidth) {
    if (borderWidth < 0) {
      throw new IllegalArgumentException("Invalid border width"); //$NON-NLS-1$
    }
    final int oldBorderWidth = this.borderWidth;
    if (oldBorderWidth != borderWidth) {
      disposedCheck();
      _setBorderWidth(_bridge, borderWidth);
      this.borderWidth = borderWidth;
      firePropertyChange(PROPERTY_BORDER_WIDTH, oldBorderWidth, borderWidth);
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Checks horizontal alignment key.
   *
   * @param key the key
   * @param message the error message
   *
   * @return alignment key
   *
   * @throws IllegalArgumentException if the key is invalid
   */
  protected final int checkHorizontalKey(int key, String message) {
    if ((key == LEFT) || (key == CENTER) || (key == RIGHT) ||
        (key == LEADING) || (key == TRAILING)) {
      return key;
    }
    throw new IllegalArgumentException(message);
  }

  /**
   * Checks vertical alignment key.
   *
   * @param key the key
   * @param message the error message
   *
   * @return alignment key
   *
   * @throws IllegalArgumentException if the key is invalid
   */
  protected final int checkVerticalKey(int key, String message) {
    if ((key == TOP) || (key == CENTER) || (key == BOTTOM)) {
      return key;
    }
    throw new IllegalArgumentException(message);
  }

  /**
   * Checks text format key.
   *
   * @param key the key
   * @param message the error message
   *
   * @return alignment key
   *
   * @throws IllegalArgumentException if the key is invalid
   */
  protected final int checkTextFormatKey(int key, String message) {
    if ((key == PLAIN_TEXT) || (key == RICH_TEXT) || (key == AUTO_TEXT)) {
      return key;
    }
    throw new IllegalArgumentException(message);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void updateLocalization() {
    super.updateLocalization();
    if (text != null) {
      _setText(_bridge, localizeString(text));
    }
  }

  /**
   * {@inheritDoc}
   */
  protected String paramString() {
    StringBuffer sb = new StringBuffer(super.paramString());
    if (text != null) {
      sb.append(",text="); //$NON-NLS-1$
      sb.append(text);
    }
    return sb.toString();
  }

/*^**************************************************************************^*/
/*- Package-Private Native Methods.                                          -*/
/*^**************************************************************************^*/

  native long _create();

  native void _setText(long bridge, String text);

  native void _setTextFormat(long bridge, int textFormat);

  native void _setWordWrap(long bridge, boolean wordWrap);

  native void _setAlignment(long bridge, int ha, int va);

  native void _setIcon(long bridge, QSwingIconSet iconSet);

  native void _setIconSize(long bridge, int width, int height);

  native void _setBorderType(long bridge, int borderType);

  native void _setBorderStyle(long bridge, int borderStyle);

  native void _setBorderWidth(long bridge, int borderWidth);

/*^**************************************************************************^*/
/*- Private Static Native Method.                                            -*/
/*^**************************************************************************^*/

  private static native void _initialize();

}

/*^*****************************************************************************
  File History:

 - 2007-05-04 15:53:56 Slobodan
   Initial version

*****************************************************************************^*/
