/*
 * @(#)JQMenu.java  0.2.0 / 2007-05-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import yu.ac.bg.etf.javaqx.qswing.layouts.LayoutManager;
import yu.ac.bg.etf.javaqx.qswing.properties.JQMenuProperties;

/**
 * Menu Component.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class JQMenu extends JQContainer implements JQMenuProperties {

/** @todo  add Action (on action) */

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private String text;
  private int mnemonic;
  private final boolean autoExclusive;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JQMenu object.
   */
  public JQMenu() {
    this(null, true);
  }

  /**
   * Constructs new JQMenu object.
   *
   * @param text the text for the menu label
   */
  public JQMenu(String text) {
    this(text, true);
  }

  /**
   * Constructs new JQMenu object.
   *
   * @param text the text for the menu label
   * @param autoExclusive <tt>true</tt> if auto exclusive
   */
  public JQMenu(String text, boolean autoExclusive) {
    super(null);
    if (text == null) {
      mnemonic = -1;
    } else {
      this.text = text;
      mnemonic = QSwingUtilities.getMnemonicIndex(text);
      _setText(_bridge, text); // NOPMD
    }
    this.autoExclusive = autoExclusive;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns text.
   *
   * @return text
   */
  public String getText() {
    return text;
  }

  /**
   * Sets text. Char '&' is keyboard mnemonic.
   *
   * @param text the text
   */
  public void setText(String text) {
    String oldText = this.text;
    if (oldText == null) {
      if (text == null) {
        return;
      }
    } else {
      if (oldText.equals(text)) {
        return;
      }
    }
    disposedCheck();
    _setText(_bridge, text);
    this.text = text;
    mnemonic = -1;
    firePropertyChange(PROPERTY_TEXT, oldText, text);
  }

  /**
   * Returns <tt>true</tt> if the keyboard mnemonic has been explicitly set.
   *
   * @return <tt>true</tt> if the keyboard mnemonic has been explicitly set
   */
  public boolean isMnemonicSet() {
    return (mnemonic >= 0);
  }

  /**
   * Returns the keyboard mnemonic.
   *
   * @return the keyboard mnemonic
   */
  public int getMnemonic() {
    return mnemonic;
  }

  /**
   * Sets the keyboard mnemonic.
   *
   * @param mnemonic the keyboard mnemonic
   */
  public void setMnemonic(int mnemonic) {
    int oldMnemonic = this.mnemonic;
    if (oldMnemonic != mnemonic) {
      if (mnemonic == -1) {
        if (text != null) {
          _setText(_bridge, text);
        }
      } else {
        if ((text != null) && (text.length() > mnemonic)) {
          StringBuffer sb = new StringBuffer(text);
          sb.insert(mnemonic, '&');
          _setText(_bridge, sb.toString());
        }
      }
      this.mnemonic = mnemonic;
      firePropertyChange(PROPERTY_MNEMONIC, oldMnemonic, mnemonic);
    }
  }

  /**
   * Adds menu item.
   *
   * @param menuItem the menu item
   *
   * @return the menu item argument
   */
  public JQMenuItem add(JQMenuItem menuItem) {
    super.add(menuItem);
    return menuItem;
  }

  /**
   * Adds menu item.
   *
   * @param menuItem the menu item
   * @param index the menu index
   *
   * @return the menu item argument
   */
  public JQMenuItem add(JQMenuItem menuItem, int index) {
    super.add(menuItem, index);
    return menuItem;
  }

  /**
   * Adds check box menu item.
   *
   * @param menuItem the check box menu item
   *
   * @return the check box menu item argument
   */
  public JQCheckBoxMenuItem add(JQCheckBoxMenuItem menuItem) {
    super.add(menuItem);
    return menuItem;
  }

  /**
   * Adds check box menu item.
   *
   * @param menuItem the check box menu item
   * @param index the check box menu index
   *
   * @return the check box menu item argument
   */
  public JQCheckBoxMenuItem add(JQCheckBoxMenuItem menuItem, int index) {
    super.add(menuItem, index);
    return menuItem;
  }

  /**
   * Adds radio button menu item.
   *
   * @param menuItem the radio button menu item
   *
   * @return the menu radio button item argument
   */
  public JQRadioButtonMenuItem add(JQRadioButtonMenuItem menuItem) {
    return add(menuItem, -1);
  }

  /**
   * Adds radio button menu item.
   *
   * @param menuItem the radio button menu item
   * @param index the radio button menu index
   *
   * @return the radio button menu item argument
   */
  public JQRadioButtonMenuItem add(JQRadioButtonMenuItem menuItem,
                                   int index) { // NOPMD
    super.add(menuItem, index);
    if (autoExclusive) {
      if (index < 0) {
        index = componentCount - 1;
      }
      if (index > 0) {
        JQComponent beforeComponent = components[index - 1];
        if (beforeComponent instanceof JQRadioButtonMenuItem) {
          JQRadioButtonMenuItem beforeRadioButton =
                                        (JQRadioButtonMenuItem)beforeComponent;
          SelectableGroup selectableGroup = beforeRadioButton.
                                                          getSelectableGroup();
          if (selectableGroup == null) {
            selectableGroup = new SelectableGroup();
            selectableGroup.add(beforeRadioButton);
          }
          selectableGroup.add(menuItem);
        }
      }
    }
    return menuItem;
  }

  /**
   * Adds menu.
   *
   * @param menu the menu
   *
   * @return the menu argument
   */
  public JQMenu add(JQMenu menu) {
    super.add(menu);
    return menu;
  }

  /**
   * Adds menu.
   *
   * @param menu the menu
   * @param index the menu index
   *
   * @return the menu argument
   */
  public JQMenu add(JQMenu menu, int index) {
    super.add(menu, index);
    return menu;
  }

  /**
   * Adds separator.
   *
   * @return separator
   */
  public JQSeparator addSeparator() {
    JQSeparator separator = new JQSeparator();
    add(separator);
    return separator;
  }

  /**
   * Adds separator.
   *
   * @param index the separator index
   *
   * @return separator
   */
  public JQSeparator addSeparator(int index) {
    JQSeparator separator = new JQSeparator();
    add(separator, index);
    return separator;
  }

  /**
   * Adds separator.
   *
   * @param text the text
   *
   * @return separator
   */
  public JQSeparator addSeparator(String text) {
    JQSeparator separator = new JQSeparator(text);
    add(separator);
    return separator;
  }

  /**
   * Adds separator.
   *
   * @param text the text
   * @param index the separator index
   *
   * @return separator
   */
  public JQSeparator addSeparator(String text, int index) {
    JQSeparator separator = new JQSeparator(text);
    add(separator, index);
    return separator;
  }

  /**
   * Displays popup menu at specified location.
   *
   * @param x the x coordinate of the location
   * @param y the y coordinate of the location
   */
  public void show(int x, int y) {
    disposedCheck();
    _show(_bridge, x, y);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setLayout(LayoutManager layoutManager) {
    // empty method
  }

  /**
   * {@inheritDoc}
   */
  public void validate() {
    // empty method
  }

  /**
   * {@inheritDoc}
   */
  public void invalidate() {
    // empty method
  }

  /**
   * {@inheritDoc}
   */
  public void revalidate() {
    // empty method
  }

  /**
   * {@inheritDoc}
   */
  public void updateLocalization() {
    super.updateLocalization();
    if (text != null) {
      _setText(_bridge, localizeString(text));
    }
  }

  /**
   * {@inheritDoc}
   */
  protected String paramString() {
    StringBuffer sb = new StringBuffer(super.paramString());
    if (text != null) {
      sb.append(",text="); //$NON-NLS-1$
      sb.append(text);
    }
    return sb.toString();
  }

  void releaseChild(JQComponent child) {
    if (child instanceof JQRadioButtonMenuItem) {
      JQRadioButtonMenuItem radioButtonItem = (JQRadioButtonMenuItem)child;
      SelectableGroup selectableGroup = radioButtonItem.getSelectableGroup();
      if (selectableGroup != null) {
        selectableGroup.remove(radioButtonItem);
      }
    }
    super.releaseChild(child);
  }

/*^**************************************************************************^*/
/*- Package-Private Native Methods.                                          -*/
/*^**************************************************************************^*/

  native long _create();

  native void _setText(long bridge, String text);

  native void _insertChild(long bridge, long childBridge, int index);

  native void _removeChild(long bridge, long childBridge);

  native void _show(long bridge, int x, int y);

}

/*^*****************************************************************************
  File History:

 - 2007-05-04 18:50:47 Slobodan
   Initial version

*****************************************************************************^*/
