/*
 * @(#)JQProgressBar.java  0.2.0 / 2007-05-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import yu.ac.bg.etf.javaqx.qswing.events.ChangeEvent;
import yu.ac.bg.etf.javaqx.qswing.events.ChangeListener;
import yu.ac.bg.etf.javaqx.qswing.properties.JQProgressBarProperties;

/**
 * Progress Bar Component.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class JQProgressBar extends JQComponent
                           implements QSwingConstants, JQProgressBarProperties {

/*^**************************************************************************^*/
/*- Pacakge-Private Static Final Fields.                                     -*/
/*^**************************************************************************^*/

  static final int DEFAULT_MINIMUM = 0;

  static final int DEFAULT_MAXIMUM = 100;

  static final int DEFAULT_ORIENTATION = HORIZONTAL;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean indeterminate;
  private BoundedRangeModel model;
  private int minimum;
  private int maximum;
  private int value;
  private int orientation;
  private String progressString;
  private boolean paintString;
  private ChangeEvent changeEvent;
  private final ChangeListener changeListener;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JQProgressBar object.
   */
  public JQProgressBar() {
    this(DEFAULT_ORIENTATION, DEFAULT_MINIMUM, DEFAULT_MAXIMUM);
  }

  /**
   * Constructs new JQProgressBar object.
   *
   * @param orientation the orientation of the progress bar
   */
  public JQProgressBar(int orientation) {
    this(orientation, DEFAULT_MINIMUM, DEFAULT_MAXIMUM);
  }

  /**
   * Constructs new JQProgressBar object.
   *
   * @param minimum the minimum value of the progress bar
   * @param maximum the maximum value of the progress bar
   */
  public JQProgressBar(int minimum, int maximum) {
    this(DEFAULT_ORIENTATION, minimum, maximum);
  }

  /**
   * Constructs new JQProgressBar object.
   *
   * @param orientation the orientation of the progress bar
   * @param minimum the minimum value of the progress bar
   * @param maximum the maximum value of the progress bar
   *
   * @throws IllegalArgumentException it the orientation is invalid
   */
  public JQProgressBar(int orientation, int minimum, int maximum) {
    if ((orientation < HORIZONTAL) || (orientation > VERTICAL)) {
      throw new IllegalArgumentException("Invalid orientation"); //$NON-NLS-1$
    }
    this.orientation = orientation;
    _setOrientation(_bridge, orientation); // NOPMD
    changeListener = new ModelListener(this);
    model = new DefaultBoundedRangeModel(minimum, 0, minimum, maximum);
    model.addChangeListener(changeListener);
  }

  /**
   * Constructs new JQProgressBar object.
   *
   * @param model the model
   */
  public JQProgressBar(BoundedRangeModel model) {
    changeListener = new ModelListener(this);
    if (model == null) {
      this.model = DefaultBoundedRangeModel.EmptyModel.INSTANCE;
      minimum = DEFAULT_MINIMUM;
      maximum = DEFAULT_MAXIMUM;
      value = DEFAULT_MINIMUM;
    } else {
      this.model = model;
      minimum = model.getMinimum();
      maximum = model.getMaximum();
      value = model.getValue();
      _setModel(_bridge, model.getMinimum(), model.getMaximum(), // NOPMD
                model.getValue());
      model.addChangeListener(changeListener);
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns data model of the progress bar.
   *
   * @return data model of the progress bar
   */
  public BoundedRangeModel getModel() {
    return model;
  }

  /**
   * Sets data model of the progress bar.
   *
   * @param model the data model
   */
  public void setModel(BoundedRangeModel model) { // NOPMD
    final BoundedRangeModel oldModel = this.model;
    if (oldModel != model) { // NOPMD
      if (model == null) {
        if (oldModel == DefaultBoundedRangeModel.EmptyModel.INSTANCE) { // NOPMD
          return;
        }
        model = DefaultBoundedRangeModel.EmptyModel.INSTANCE; // NOPMD
      }
      disposedCheck();
      if (oldModel != null) {
        oldModel.removeChangeListener(changeListener);
      }
      _setModel(_bridge, model.getMinimum(), model.getMaximum(),
                model.getValue());
      this.model = model;
      this.minimum = model.getMinimum();
      this.maximum = model.getMaximum();
      this.value = model.getValue();
      model.addChangeListener(changeListener);
      firePropertyChange(PROPERTY_MODEL, oldModel, model);
    }
  }

  /**
   * Returns current value of the progress bar.
   *
   * @return current value of the progress bar
   */
  public int getValue() {
    return model.getValue();
  }

  /**
   * Sets current value of the progress bar.
   *
   * @param value the current value of the progress bar
   */
  public void setValue(int value) {
    model.setValue(value);
  }

  /**
   * Returns minimum progress value.
   *
   * @return minimum progress value
   */
  public int getMinimum() {
    return model.getMinimum();
  }

  /**
   * Sets minimum progress value.
   *
   * @param minimum the maximum progress value
   */
  public void setMinimum(int minimum) {
    model.setMinimum(minimum);
  }

  /**
   * Returns maximum progress value.
   *
   * @return maximum progress value
   */
  public int getMaximum() {
    return model.getMaximum();
  }

  /**
   * Sets maximum progress value.
   *
   * @param maximum the maximum progress value
   */
  public void setMaximum(int maximum) {
    model.setMaximum(maximum);
  }

  /**
   * Returns progress string.
   *
   * @return progress string
   */
  public String getString() {
    return progressString;
  }

  /**
   * Sets progress string.
   *
   * @param progressString the progress string
   */
  public void setString(String progressString) {
    String oldProgressString = this.progressString;
    if (oldProgressString == null) {
      if (progressString == null) {
        return;
      }
    } else {
      if (oldProgressString.equals(progressString)) {
        return;
      }
    }
    disposedCheck();
    _setProgressString(_bridge, progressString);
    this.progressString = progressString;
    firePropertyChange(PROPERTY_STRING, oldProgressString, progressString);
  }

  /**
   * Returns <tt>true</tt> if the progress string is painted.
   *
   * @return <tt>true</tt> if the progress string is painted
   */
  public boolean isStringPainted() {
    return paintString;
  }

  /**
   * Sets whether of not the progress string is painted.
   *
   * @param paintString <tt>true</tt> if painted
   */
  public void setStringPainted(boolean paintString) {
    final boolean oldPaintString = this.paintString;
    if (oldPaintString != paintString) {
      disposedCheck();
      _setStringPainted(_bridge, paintString);
      this.paintString = paintString;
      firePropertyChange(PROPERTY_STRING_PAINTED, oldPaintString, paintString);
    }
  }

  /**
   * Returns the percent complete. The value is between 0.0 and 1.0.
   *
   * @return the percent complete
   */
  public double getPercentComplete() {
    int minimum = getMinimum();
    int maximum = getMaximum();
    int span = maximum - minimum;
    if (span == 0) {
      return 1.0;
    }
    return (double)(getValue() - minimum) / span;
  }

  /**
   * Returns orientation of the progress bar.
   *
   * @return orientation of the progress bar
   */
  public int getOrientation() {
    return orientation;
  }

  /**
   * Sets orientation.
   *
   * @param orientation the orientation
   */
  public void setOrientation(int orientation) {
    if ((orientation < HORIZONTAL) || (orientation > VERTICAL)) {
      throw new IllegalArgumentException("Invalid orientation"); //$NON-NLS-1$
    }
    final int oldOrientation = this.orientation;
    if (oldOrientation != orientation) {
      disposedCheck();
      _setOrientation(_bridge, orientation);
      this.orientation = orientation;
      firePropertyChange(PROPERTY_ORIENTATION, oldOrientation, orientation);
    }
  }

  /**
   * Returns <tt>true</tt> if the progress bar is indeterminate.
   *
   * @return <tt>true</tt> if the progress bar is indeterminate
   */
  public boolean isIndeterminate() {
    return indeterminate;
  }

  /**
   * Sets whether or not the progress bar is indeterminate.
   *
   * @param indeterminate <tt>true</tt> if indeterminate
   */
  public void setIndeterminate(boolean indeterminate) {
    final boolean oldIndeterminate = this.indeterminate;
    if (oldIndeterminate != indeterminate) {
      disposedCheck();
      if (indeterminate) {
        _setModel(_bridge, 0, 0, 0);
      } else {
        final BoundedRangeModel model = this.model;
        _setModel(_bridge, model.getMinimum(), model.getMaximum(),
                  model.getValue());
      }
      this.indeterminate = indeterminate;
      firePropertyChange(PROPERTY_INDETERMINATE, oldIndeterminate,
                         indeterminate);
    }
  }

  /**
   * Adds change listener.
   *
   * @param listener the change listener
   */
  public void addChangeListener(ChangeListener listener) {
    listenerList.add(ChangeListener.class, listener);
  }

  /**
   * Removes change listener.
   *
   * @param listener the component listener
   */
  public void removeChangeListener(ChangeListener listener) {
    listenerList.remove(ChangeListener.class, listener);
  }

  /**
   * Returns change listeners.
   *
   * @return change listeners
   */
  public ChangeListener[] getChangeListeners() {
    int count = listenerList.getListenerCount(ChangeListener.class);
    ChangeListener[] listeners = new ChangeListener[count];
    if (count > 0) {
      System.arraycopy(listenerList.getListenersArray(ChangeListener.class),
                       0, listeners,  0, count);
    }
    return listeners;
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Fires state changed event.
   */
  protected void fireStateChanged() {
    int count = listenerList.getListenerCount(ChangeListener.class);
    if (count > 0) {
      ChangeListener[] listeners = (ChangeListener[])listenerList.
                                       getListenersArray(ChangeListener.class);
      ChangeEvent changeEvent = this.changeEvent;
      if (changeEvent == null) {
        changeEvent = new ChangeEvent(this);
        this.changeEvent = changeEvent;
      }
      for (int i = count; --i >= 0;) {
        listeners[i].stateChanged(changeEvent);
      }
    }
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void updateLocalization() {
    super.updateLocalization();
    if (progressString != null) {
      _setProgressString(_bridge, localizeString(progressString));
    }
  }

  /**
   * {@inheritDoc}
   */
  protected String paramString() {
    StringBuffer sb = new StringBuffer(80);
    sb.append(super.paramString());
    sb.append(",orientation="); //$NON-NLS-1$
    if (orientation == HORIZONTAL) {
      sb.append("HORIZONTAL"); //$NON-NLS-1$
    } else {
      sb.append("VERTICAL"); //$NON-NLS-1$
    }
    sb.append(",progressString="); //$NON-NLS-1$
    sb.append(progressString);
    sb.append(",paintString="); //$NON-NLS-1$
    sb.append(paintString);
    sb.append(",indeterminate="); //$NON-NLS-1$
    sb.append(indeterminate);
    return sb.toString();
  }

  void componentDestroyed() {
    super.componentDestroyed();
    model.removeChangeListener(changeListener);
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  void processChangeEvent() {
    final BoundedRangeModel model = this.model;
    if (!isIndeterminate()) {
      final int minimum = model.getMinimum();
      if (this.minimum != minimum) {
        _setMinimum(_bridge, minimum);
        this.minimum = minimum;
      }
      final int maximum = model.getMaximum();
      if (this.maximum != maximum) {
        _setMaximum(_bridge, maximum);
        this.maximum = maximum;
      }
      final int value = model.getValue();
      if (this.value != value) {
        _setValue(_bridge, value);
        this.value = value;
      }
      fireStateChanged();
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Native Methods.                                          -*/
/*^**************************************************************************^*/

  native long _create();

  native void _setModel(long bridge, int minimum, int maximum, int value);

  native void _setValue(long bridge, int value);

  native void _setMinimum(long bridge, int minimum);

  native void _setMaximum(long bridge, int maximum);

  native void _setOrientation(long bridge, int orientation);

  native void _setProgressString(long bridge, String progressString);

  native void _setStringPainted(long bridge, boolean paintString);

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Model Change Listener.
   */
  static class ModelListener implements ChangeListener {
    final JQProgressBar progressBar;
    public ModelListener(JQProgressBar progressBar) {
      this.progressBar = progressBar;
    }
    public void stateChanged(ChangeEvent e) {
      progressBar.processChangeEvent();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2007-05-04 18:54:29 Slobodan
   Initial version

*****************************************************************************^*/
