/*
 * @(#)JQSplitPane.java  0.2.0 / 2007-12-03
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import yu.ac.bg.etf.javaqx.qswing.properties.JQSplitPaneProperties;

/**
 * Split Pane Component.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class JQSplitPane extends JQComponent
                         implements QSwingConstants, JQSplitPaneProperties {

/*^**************************************************************************^*/
/*- Package-Private Field.                                                   -*/
/*^**************************************************************************^*/

 static final int DEFAULT_ORIENTATION = HORIZONTAL;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    _initialize();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int orientation;
  private boolean continuousLayout;
  private double resizeWeight;
  private boolean gatherDividerSize;
  private int dividerSize;
  private int dividerLocation;
  private boolean oneTouchExpandable;
  private JQComponent leftComponent;
  private JQComponent rightComponent;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JQSplitPane object.
   */
  public JQSplitPane() {
    this(DEFAULT_ORIENTATION, true, null, null);
  }

  /**
   * Constructs new JQSplitPane object.
   *
   * @param orientation the orientation
   */
  public JQSplitPane(int orientation) {
    this(orientation, true, null, null);
  }

  /**
   * Constructs new JQSplitPane object.
   *
   * @param orientation the orientation
   * @param continuousLayout <tt>true</tt> if resize of split pane's components
   *                         is continuous
   */
  public JQSplitPane(int orientation,  boolean continuousLayout) {
    this(orientation, continuousLayout, null, null);
  }

  /**
   * Constructs new JQSplitPane object.
   *
   * @param orientation the orientation
   * @param leftComponent the left component
   * @param rightComponent the right component
   */
  public JQSplitPane(int orientation, JQComponent leftComponent,
                     JQComponent rightComponent) {
    this(orientation, true, leftComponent, rightComponent);
  }

  /**
   * Constructs new JQSplitPane object.
   *
   * @param leftComponent the left component
   * @param rightComponent the right component
   */
  public JQSplitPane(JQComponent leftComponent, JQComponent rightComponent) {
    this(DEFAULT_ORIENTATION, true, leftComponent, rightComponent);
  }

  /**
   * Constructs new JQSplitPane object.
   *
   * @param orientation the orientation
   * @param continuousLayout <tt>true</tt> if resize of split pane's components
   *                         is continuous
   * @param leftComponent the left component
   * @param rightComponent the right component
   */
  public JQSplitPane(int orientation,  boolean continuousLayout,
                     JQComponent leftComponent, JQComponent rightComponent) {
    if ((orientation < HORIZONTAL) || (orientation > VERTICAL)) {
      throw new IllegalArgumentException("Invalid orientation"); //$NON-NLS-1$
    }
    if (orientation != DEFAULT_ORIENTATION) {
      _setOrientation(_bridge, VERTICAL); // NOPMD
    }
    this.orientation = orientation;
    gatherDividerSize = true;
    this.continuousLayout = continuousLayout;
    if (leftComponent != null) {
      if (leftComponent.isDisposed()) {
        throw new IllegalArgumentException(
                                      "Left component disposed"); //$NON-NLS-1$
      }
      acquireComponent(leftComponent);
      _setLeftComponent(_bridge, leftComponent._bridge); // NOPMD
    }
    if (rightComponent != null) {
      if (rightComponent.isDisposed()) {
        throw new IllegalArgumentException(
                                     "Right component disposed"); //$NON-NLS-1$
      }
      acquireComponent(leftComponent);
      _setRightComponent(_bridge, rightComponent._bridge); // NOPMD
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns orientation of the split pane.
   *
   * @return orientation of the split pane
   */
  public int getOrientation() {
    return orientation;
  }

  /**
   * Sets orientation.
   *
   * @param orientation the orientation
   */
  public void setOrientation(int orientation) {
    final int oldOrientation = this.orientation;
    if (oldOrientation != orientation) {
      if ((orientation < HORIZONTAL) || (orientation > VERTICAL)) {
        throw new IllegalArgumentException("Invalid orientation"); //$NON-NLS-1$
      }
      disposedCheck();
      _setOrientation(_bridge, orientation);
      this.orientation = orientation;
      firePropertyChange(PROPERTY_ORIENTATION, oldOrientation, orientation);
    }
  }

  /**
   * Returns <tt>true</tt> if the resize of split pane's components is
   * continuous.
   *
   * @return <tt>true</tt> if the resize of split pane's components is
   *         continuous
   */
  public boolean isContinuousLayout() {
    return continuousLayout;
  }

  /**
   * Sets whether or not the resize of split pane's components is continuous.
   *
   * @param continuousLayout <tt>true</tt> if continuous
   */
  public void setContinuousLayout(boolean continuousLayout) {
    final boolean oldContinuousLayout = this.continuousLayout;
    if (oldContinuousLayout != continuousLayout) {
      disposedCheck();
      _setContinuousLayout(_bridge, continuousLayout);
      this.continuousLayout = continuousLayout;
      firePropertyChange(PROPERTY_CONTINUOUS_LAYOUT, oldContinuousLayout,
                         continuousLayout);
    }
  }

  /**
   * Returns the resize weight.
   *
   * @return the resize weight
   */
  public double getResizeWeight() {
    return resizeWeight;
  }

  /**
   * Sets resize weight.
   *
   * @param resizeWeight the resize weight
   */
  public void setResizeWeight(double resizeWeight) {
    if ((resizeWeight < 0) || (resizeWeight > 1)) {
      throw new IllegalArgumentException(
                        "Resize weight must be between 0 and 1"); //$NON-NLS-1$
    }
    final double oldResizeWeight = this.resizeWeight;
    if (Double.doubleToRawLongBits(oldResizeWeight) !=
        Double.doubleToRawLongBits(resizeWeight)) {
      disposedCheck();
      _setResizeWeight(_bridge, resizeWeight);
      this.resizeWeight = resizeWeight;
      firePropertyChange(PROPERTY_RESIZE_WEIGHT, oldResizeWeight, resizeWeight);
    }
  }

  /**
   * Returns divider size.
   *
   * @return divider size
   */
  public int getDividerSize() {
    if (gatherDividerSize) {
      dividerSize = _getDividerSize(_bridge);
      gatherDividerSize = false;
    }
    return dividerSize;
  }

  /**
   * Sets divider size.
   *
   * @param dividerSize the divider size
   */
  public void setDividerSize(int dividerSize) {
    if (dividerSize <= 0) {
      throw new IllegalArgumentException("Invalid divider size"); //$NON-NLS-1$
    }
    final int oldDividerSize = this.dividerSize;
    if (oldDividerSize != dividerSize) {
      disposedCheck();
      _setDividerSize(_bridge, dividerSize);
      this.dividerSize = dividerSize;
      firePropertyChange(PROPERTY_DIVIDER_SIZE, oldDividerSize, dividerSize);
    }
  }

  /**
   * Returns divider location.
   *
   * @return divider location
   */
  public int getDividerLocation() {
    return dividerLocation;
  }

  /**
   * Sets divider location.
   *
   * @param dividerLocation the divider location
   */
  public void setDividerLocation(int dividerLocation) {
    disposedCheck();
    _setDividerLocation(_bridge, dividerLocation);
  }

  /**
   * Returns <tt>true</tt> if the components of the split pane can be quickly
   * expanded/collapsed.
   *
   * @return <tt>true</tt> if the components of the split pane can be quickly
   * expanded/collapsed.
   */
  public boolean isOneTouchExpandable() {
    return oneTouchExpandable;
  }

  /**
   * Sets whether or not the components of the split pane can be quickly
   * expanded/collapsed.
   *
   * @param oneTouchExpandable <tt>true</tt> if expandable
   */
  public void setOneTouchExpandable(boolean oneTouchExpandable) {
    final boolean oldOneTouchExpandable = this.oneTouchExpandable;
    if (oldOneTouchExpandable != oneTouchExpandable) {
      disposedCheck();
      _setOneTouchExpandable(_bridge, oneTouchExpandable);
      this.oneTouchExpandable = oneTouchExpandable;
      firePropertyChange(PROPERTY_ONE_TOUCH_EXPANDABLE, oldOneTouchExpandable,
                         oneTouchExpandable);
    }
  }

  /**
   * Returns left component.
   *
   * @return left component
   */
  public JQComponent getLeftComponent() {
    return leftComponent;
  }

  /**
   * Sets left component.
   *
   * @param leftComponent the left component
   *
   * @return old left component
   */
  public JQComponent setLeftComponent(JQComponent leftComponent) {
    JQComponent oldLeftComponent = this.leftComponent;
    if (oldLeftComponent != leftComponent) { // NOPMD
      disposedCheck();
      releaseComponent(oldLeftComponent);
      if (leftComponent != null) {
        if (leftComponent.isDisposed()) { // NOPMD
          throw new IllegalArgumentException(
                                      "Left component disposed"); //$NON-NLS-1$
        }
        acquireComponent(leftComponent);
        _setLeftComponent(_bridge, leftComponent._bridge);
      }
      this.leftComponent = leftComponent;
    }
    return oldLeftComponent;
  }

  /**
   * Returns right component.
   *
   * @return right component
   */
  public JQComponent getRightComponent() {
    return rightComponent;
  }

  /**
   * Sets right component.
   *
   * @param rightComponent the right component
   */
  public void setRightComponent(JQComponent rightComponent) {
    JQComponent oldRightComponent = this.rightComponent;
    if (oldRightComponent != rightComponent) { // NOPMD
      disposedCheck();
      releaseComponent(oldRightComponent);
      if (rightComponent != null) {
        if (rightComponent.isDisposed()) { // NOPMD
          throw new IllegalArgumentException(
                                     "Right component disposed"); //$NON-NLS-1$
        }
        acquireComponent(rightComponent);
        _setRightComponent(_bridge, rightComponent._bridge);
      }
      this.rightComponent = rightComponent;
    }
  }

  /**
   * Returns top component.
   *
   * @return top component
   */
  public JQComponent getTopComponent() {
    return leftComponent;
  }

  /**
   * Sets top component.
   *
   * @param topComponent top component
   */
  public void setTopComponent(JQComponent topComponent) {
    setLeftComponent(topComponent);
  }

  /**
   * Returns bottom component.
   *
   * @return bottom component
   */
  public JQComponent getBottomComponent() {
    return rightComponent;
  }

  /**
   * Sets bottom component.
   *
   * @param bottomComponent bottom component
   */
  public void setBottomComponent(JQComponent bottomComponent) {
    setRightComponent(bottomComponent);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public JQComponent[] getOwnedComponents() {
    if ((leftComponent != null) && (rightComponent != null)) { // NOPMD
      return new JQComponent[] {leftComponent, rightComponent};
    } else  if (leftComponent != null) { // NOPMD
      return new JQComponent[] {leftComponent};
    } else  if (rightComponent != null) {
      return new JQComponent[] {rightComponent};
    }
    return EMPTY_COMPONENT_ARRAY; // NOFB NOPMD
  }

  void releaseComponent(JQComponent component) {
    if (component != null) {
      if (leftComponent == component) { // NOPMD
        _setLeftComponent(_bridge, 0);
        leftComponent = null;
        component.owner = null;
      } else if (rightComponent == component) { // NOPMD
        _setRightComponent(_bridge, 0);
        rightComponent = null;
        component.owner = null;
      }
    }
  }

  /*!
   * @javaqx.native
   *    overridden="true"
   */
  void _styleChanged() {
    super._styleChanged();
    gatherDividerSize = true;
  }

/*^**************************************************************************^*/
/*- Native Callback.                                                         -*/
/*^**************************************************************************^*/

  void _dividerLocationChanged(int dividerLocation) {
    final int oldDividerLocation = this.dividerLocation;
    if (oldDividerLocation != dividerLocation) {
      this.dividerLocation = dividerLocation;
      firePropertyChange(PROPERTY_DIVIDER_LOCATION, oldDividerLocation,
                         dividerLocation);
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Native Methods.                                          -*/
/*^**************************************************************************^*/

  native long _create();

  native void _setLeftComponent(long bridge, long componentBridge);

  native void _setRightComponent(long bridge, long componentBridge);

  native void _setOrientation(long bridge, int orientation);

  native void _setContinuousLayout(long bridge, boolean continuousLayout);

  native void _setResizeWeight(long bridge, double resizeWeight);

  native int _getDividerSize(long bridge);

  native void _setDividerSize(long bridge, int dividerSize);

  native void _setDividerLocation(long bridge, int dividerLocation);

  native void _setOneTouchExpandable(long bridge, boolean oneTouchExpandable);

/*^**************************************************************************^*/
/*- Private Static Native Method.                                            -*/
/*^**************************************************************************^*/

  private static native void _initialize();

}

/*^*****************************************************************************
  File History:

 - 2007-12-03 22:59:05 Slobodan
   Initial version

*****************************************************************************^*/
