/*
 * @(#)JQTabbedPane.java  0.2.0 / 2007-05-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import java.util.ArrayList;

import yu.ac.bg.etf.javaqx.qswing.layouts.LayoutManager;
import yu.ac.bg.etf.javaqx.qswing.properties.JQTabbedPaneProperties;

/**
 * TabbedPane Component.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class JQTabbedPane extends JQContainer
                          implements QSwingConstants, JQTabbedPaneProperties {

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    _initialize();
  }

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  private ArrayList pages;
  private int tabPlacement;
  private int selectedIndex;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JQTabbedPane object.
   */
  public JQTabbedPane() {
    super(null);
    pages = new ArrayList(3);
    tabPlacement = TOP;
    selectedIndex = -1;
  }

  /**
   * Constructs new JQTabbedPane object.
   *
   * @param tabPlacement the tab placement
   */
  public JQTabbedPane(int tabPlacement) {
    this();
    if ((tabPlacement < TOP) || (tabPlacement > RIGHT)) {
      throw new IllegalArgumentException("Invalid tab placement"); //$NON-NLS-1$
    }
    this.tabPlacement = tabPlacement;
    _setTabPlacement(_bridge, tabPlacement); // NOPMD
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Adds new tab.
   *
   * @param title the tab title
   * @param component the tab component
   *
   * @return the component argument
   */
  public JQComponent addTab(String title, JQComponent component) {
    return insertTab(title, null, component, null, -1);
  }

  /**
   * Adds new tab.
   *
   * @param title the tab title
   * @param icon the tab icon
   * @param component the tab component
   *
   * @return the component argument
   */
  public JQComponent addTab(String title, Icon icon, JQComponent component) {
    return insertTab(title, icon, component, null, -1);
  }

  /**
   * Adds new tab.
   *
   * @param title the tab title
   * @param icon the tab icon
   * @param component the tab component
   * @param tooltip the tab tooltip
   *
   * @return the component argument
   */
  public JQComponent addTab(String title, Icon icon, JQComponent component,
                            String tooltip) {
    return insertTab(title, icon, component, tooltip, -1);
  }

  /**
   * Inserts new tab at specified position.
   *
   * @param title the tab title
   * @param icon the tab icon
   * @param component the tab component
   * @param tooltip the tab tooltip
   * @param index the index
   *
   * @return the component argument
   */
  public JQComponent insertTab(String title, Icon icon, // NOPMD
                               JQComponent component, String tooltip, // NOPMD
                               int index) { // NOPMD
    if (component != null) {
      if (title == null) {
        title = component.getName(); // NOPMD
      }
      addImpl(component, title, index);
      Page page = new Page(title, tooltip);
      if ((index < 0) || (index >= pages.size())) {
        index = pages.size();
        pages.add(page);
      } else {
        pages.add(index, page);
      }
      if (tooltip == null) {
        tooltip = component.getToolTipText(); // NOPMD
        page.toolTipText = tooltip;
      }
      if (tooltip != null) {
        _setTabToolTipText(_bridge, index, tooltip);
      }
      String whatsThisText = component.getWhatsThisText();
      if (whatsThisText != null) {
        page.whatsThisText = whatsThisText;
        _setTabWhatsThisText(_bridge, index, whatsThisText);
      }
      if (icon != null) {
        QSwingIconSet iconSet = new QSwingIconSet(this);
        page.iconSet = iconSet;
        _setTabIcon(_bridge, index, iconSet);
        iconSet.setIcon(icon, null);
      }
    }
    return component;
  }

  /**
   * Returns tab placement.
   *
   * @return tab placement
   */
  public int getTabPlacement() {
    return tabPlacement;
  }

  /**
   * Sets tab placement.
   *
   * @param tabPlacement the tab placement
   */
  public void setTabPlacement(int tabPlacement) {
    if ((tabPlacement < TOP) || (tabPlacement > RIGHT)) {
      throw new IllegalArgumentException("Invalid tab placement"); //$NON-NLS-1$
    }
    final int oldTabPlacement = this.tabPlacement;
    if (oldTabPlacement != tabPlacement) {
      disposedCheck();
      this.tabPlacement = tabPlacement;
      _setTabPlacement(_bridge, tabPlacement);
      invalidate();
      firePropertyChange(PROPERTY_TAB_PLACEMENT, oldTabPlacement, tabPlacement);
    }
  }

  /**
   * Returns index of the selected component.
   *
   * @return the selected component
   */
  public int getSelectedIndex() {
    return selectedIndex;
  }

  /**
   * Sets index of the selected component.
   *
   * @param selectedIndex the selected index
   */
  public void setSelectedIndex(int selectedIndex) {
    if (this.selectedIndex != selectedIndex) {
      disposedCheck();
      _setCurrentIndex(_bridge, selectedIndex);
    }
  }

  /**
   * Returns currently selected component.
   *
   * @return currently selected component
   */
  public JQComponent getSelectedComponent() {
    if (selectedIndex >= 0) {
      return getComponent(selectedIndex);
    }
    return null;
  }

  /**
   * Sets currently selected component.
   *
   * @param component the component to be selected
   */
  public void setSelectedComponent(JQComponent component) {
    int index = getComponentIndex(component);
    if (index == -1) {
      throw new IllegalArgumentException("Invalid component"); //$NON-NLS-1$
    }
    setSelectedIndex(index);
  }

  /**
   * Returns <tt>true</tt> if the tab at specified index is enabled.
   *
   * @param index the component index
   *
   * @return <tt>true</tt> if the tab at specified index is enabled
   */
  public boolean isEnabledAt(int index) {
    return getPage(index).enabled;
  }

  /**
   * Enables/disables the component at specified index.
   *
   * @param index the component index
   * @param enabled <tt>true</tt> if enabled
   */
  public void setEnabledAt(int index, boolean enabled) {
    Page page = getPage(index);
    if (page.enabled != enabled) {
      disposedCheck();
      page.enabled = enabled;
      _setTabEnabled(_bridge, index, enabled);
    }
  }

  /**
   * Returns the title of the tab at specified index.
   *
   * @param index the component index
   *
   * @return the title of the tab at specified index
   */
  public String getTitleAt(int index) {
    return getPage(index).titleText;
  }

  /**
   * Sets title of the tab at specified index.
   *
   * @param index the tab index
   * @param title the title
   */
  public void setTitleAt(int index, String title) {
    Page page = getPage(index);
    final String oldTitle = page.titleText;
    if (oldTitle == null) {
      if (title == null) {
        return;
      }
    } else {
      if (oldTitle.equals(title)) {
        return;
      }
    }
    disposedCheck();
    _setTabTitleText(_bridge, index, title);
    page.titleText = title;
    firePropertyChange(PROPERTY_INDEX_FOR_TITLE, -1, index);
  }

  /**
   * Returns the tool tip text of the tab at specified index.
   *
   * @param index the tab index
   *
   * @return the tool tip text  of the tab at specified index
   */
  public String getToolTipTextAt(int index) {
    return getPage(index).toolTipText;
  }

  /**
   * Sets tool tip text of the tab at specified index.
   *
   * @param index the tab index
   * @param toolTipText the tool tip text
   */
  public void setToolTipTextAt(int index, String toolTipText) {
    Page page = getPage(index);
    final String oldToolTipText = page.titleText;
    if (oldToolTipText == null) {
      if (toolTipText == null) {
        return;
      }
    } else {
      if (oldToolTipText.equals(toolTipText)) {
        return;
      }
    }
    disposedCheck();
    _setTabToolTipText(_bridge, index, toolTipText);
    page.toolTipText = toolTipText;
  }

  /**
   * Returns the what's this text of the tab at specified index.
   *
   * @param index the tab index
   *
   * @return the what's this text of the tab at specified index
   */
  public String getWhatsThisText(int index) {
    return getPage(index).whatsThisText;
  }

  /**
   * Sets what's this text of the tab at specified index.
   *
   * @param index the tab index
   * @param whatsThisText the what's this text
   */
  public void setWhatsThisText(int index, String whatsThisText) {
    Page page = getPage(index);
    final String oldWhatsThisText = page.whatsThisText;
    if (oldWhatsThisText == null) {
      if (whatsThisText == null) {
        return;
      }
    } else {
      if (oldWhatsThisText.equals(whatsThisText)) {
        return;
      }
    }
    disposedCheck();
    _setTabWhatsThisText(_bridge, index, toolTipText);
    page.whatsThisText = whatsThisText;
  }

  /**
   * Returns icon of the specified tab.
   *
   * @param index the tab index
   *
   * @return icon of the specified tab
   */
  public Icon getIconAt(int index) {
    QSwingIconSet iconSet = getPage(index).iconSet;
    if (iconSet != null) {
      return iconSet.getIcon();
    }
    return null;
  }

  /**
   * Sets icon of the specified tab.
   *
   * @param index the tab index
   * @param icon the icon
   */
  public void setIconAt(int index, Icon icon) {
    Page page = getPage(index);
    QSwingIconSet iconSet = page.iconSet;
    disposedCheck();
    if (icon == null) {
      if (iconSet != null) {
        iconSet.setIcon(null, null);
        if (iconSet.isEmpty()) {
          _setTabIcon(_bridge, index, null);
          page.iconSet = null;
        }
      }
    } else {
      if (iconSet == null) {
        iconSet = new QSwingIconSet(this);
        page.iconSet = iconSet;
        _setTabIcon(_bridge, index, iconSet);
      }
      iconSet.setIcon(icon, null);
    }
  }

  /**
   * Returns disabled icon of the specified tab.
   *
   * @param index the tab index
   *
   * @return disabled icon of the specified tab
   */
  public Icon getDisabledIconAt(int index) {
    QSwingIconSet iconSet = getPage(index).iconSet;
    if (iconSet != null) {
      return iconSet.getDisabledIcon();
    }
    return null;
  }

  /**
   * Sets disabled icon of the specified tab.
   *
   * @param index the tab index
   * @param icon the Disabled icon
   */
  public void setDisabledAt(int index, Icon icon) {
    Page page = getPage(index);
    QSwingIconSet iconSet = page.iconSet;
    disposedCheck();
    if (icon == null) {
      if (iconSet != null) {
        iconSet.setDisabledIcon(null, null);
        if (iconSet.isEmpty()) {
          _setTabIcon(_bridge, index, null);
          page.iconSet = null;
        }
      }
    } else {
      if (iconSet == null) {
        iconSet = new QSwingIconSet(this);
        page.iconSet = iconSet;
        _setTabIcon(_bridge, index, iconSet);
      }
      iconSet.setDisabledIcon(icon, null);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setLayout(LayoutManager layoutManager) {
    // empty method
  }

  /**
   * {@inheritDoc}
   */
  public void invalidate() {
    if (valid) {
      valid = false;
      if (visible) {
        disposedCheck();
        _validate(_bridge);
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public void validate() {
    if (!valid) {
      disposedCheck();
      valid = true;
      final JQComponent[] components = this.components;
      for (int i = componentCount; --i >= 0;) {
        components[i].validate();
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public JQComponent add(JQComponent component) {
    if (component != null) {
      return insertTab(component.getName(), null, component, null, -1);
    }
    return null;
  }

  /**
   * {@inheritDoc}
   */
  public JQComponent add(JQComponent component, int index) {
    if (component != null) {
      return insertTab(component.getName(), null, component, null, index);
    }
    return null;
  }

  /**
   * {@inheritDoc}
   */
  public JQComponent add(JQComponent component, Object constraints) {
    if (constraints instanceof String) {
      return insertTab((String)constraints, null, component, null, -1);
    } else if (constraints instanceof Icon) {
      return insertTab(null, (Icon)constraints, component, null, -1);
    } else if (component != null) {
      return insertTab(component.getName(), null, component, null, -1);
    }
    return null;
  }

  /**
   * {@inheritDoc}
   */
  public JQComponent add(JQComponent component, Object constraints, int index) {
    Icon icon = null;
    if (constraints instanceof Icon) {
      icon = (Icon)constraints;
    }
    String title = null;
    if (constraints instanceof String) {
      title = (String)constraints;
    }
    return insertTab(title, icon, component, null, index);
  }

  /**
   * {@inheritDoc}
   */
  protected String paramString() {
    StringBuffer sb = new StringBuffer(80);
    sb.append(super.paramString());
    sb.append(",tabPlacement="); //$NON-NLS-1$
    switch (tabPlacement) { // NOPMD
      case TOP:
        sb.append("TOP"); //$NON-NLS-1$
        break;
      case LEFT:
        sb.append("LEFT"); //$NON-NLS-1$
        break;
      case BOTTOM:
        sb.append("BOTTOM"); //$NON-NLS-1$
        break;
      case RIGHT:
        sb.append("RIGHT"); //$NON-NLS-1$
        break;
    }
    return super.paramString();
  }

  void insertChild(JQComponent child, Object constraints, int index) {
    if (constraints == null) {
      _insertTab(_bridge, child._bridge, index, null);
    } else {
      _insertTab(_bridge, child._bridge, index, constraints.toString());
    }
  }

/*^**************************************************************************^*/
/*- Pacakge-Private Method.                                                  -*/
/*^**************************************************************************^*/

  Page getPage(int index) {
    if ((index < 0) || (index >= pages.size())) {
      throw new IllegalArgumentException(
                                      "Invalid component index"); //$NON-NLS-1$
    }
    return (Page)pages.get(index);
  }

/*^**************************************************************************^*/
/*- Native Callback.                                                         -*/
/*^**************************************************************************^*/

  void _currentChanged(int selectedIndex) {
    final int oldSelectedIndex = this.selectedIndex;
    if (oldSelectedIndex != selectedIndex) {
      this.selectedIndex = selectedIndex;
      firePropertyChange(PROPERTY_SELECTED_INDEX, oldSelectedIndex,
                         selectedIndex);
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Native Methods.                                          -*/
/*^**************************************************************************^*/

  native long _create();

  native void _insertTab(long bridge, long tabBridge, int index, String title);

  native void _removeChild(long bridge, long childBridge);

  native void _setTabPlacement(long bridge, int tabPlacement);

  native void _setTabEnabled(long bridge, int index, boolean enabled);

  native void _setTabTitleText(long bridge, int index, String titleText);

  native void _setTabToolTipText(long bridge, int index, String toolTipText);

  native void _setTabWhatsThisText(long bridge, int index,
                                   String whatsThisText);

  native void _setTabIcon(long bridge, int index, QSwingIconSet iconSet);

  native void _setCurrentIndex(long bridge, int index);

/*^**************************************************************************^*/
/*- Private Static Native Method.                                            -*/
/*^**************************************************************************^*/

  private static native void _initialize();

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Tab page class.
   */
  static final class Page {
    boolean enabled;
    String titleText;
    String toolTipText;
    String whatsThisText;
    QSwingIconSet iconSet;
    public Page(String titleText, String toolTipText) {
      enabled = true;
      this.titleText = titleText;
      this.toolTipText = toolTipText;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2007-05-04 18:53:51 Slobodan
   Initial version

*****************************************************************************^*/
