/*
 * @(#)JQTextField.java  0.2.0 / 2007-05-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import yu.ac.bg.etf.javaqx.qswing.events.ActionEvent;
import yu.ac.bg.etf.javaqx.qswing.events.ActionListener;
import yu.ac.bg.etf.javaqx.qswing.events.QSwingEvent;
import yu.ac.bg.etf.javaqx.qswing.properties.JQTextFieldProperties;

/**
 * Text Field Component.
 * <p>
 *   <tt>NOTE: This class is not compatible with
 *             <em>javax.swing.JTextPane</em> class!</tt>
 * </p>
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class JQTextField extends JQComponent
                         implements QSwingConstants, JQTextFieldProperties {

/** @todo Extend JQTextComponent class. */

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    _initialize();
  }

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  String text; /** @todo  temp - use text document */
  String actionCommand;
  int horizontalAlignment;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JQTextField object.
   */
  public JQTextField() {
    // empty constructor body
  }

  /**
   * Constructs new JQTextField object.
   *
   * @param text the text
   */
  public JQTextField(String text) {
    if (text != null) {
      this.text = text;
      _setText(_bridge, text); // NOPMD
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns text.
   *
   * @return text
   */
  public String getText() {
    disposedCheck();
    return _getText(_bridge);
  }

  /**
   * Sets text.
   *
   * @param text the text
   */
  public void setText(String text) {
    disposedCheck();
    _setText(_bridge, text);
  }

  /**
   * Returns horizontal alignment.
   *
   * @return horizontal alignment
   */
  public int getHorizontalAlignment() {
    return horizontalAlignment;
  }

  /**
   * Sets horizontal alignment.
   * <p>Valid values are: <tt>LEFT</tt>, <tt>CENTER</tt>, <tt>RIGHT</tt>,
   *                      <tt>LEADING</tt> and <tt>TRAILING</tt>.
   * <br/>
   *   Default value is: <tt>LEADING</tt>.
   * </p>
   *
   * @param alignment the horizontal alignment
   */
  public void setHorizontalAlignment(int alignment) {
    int oldHorizontalAlignment = horizontalAlignment;
    if (oldHorizontalAlignment != alignment) {
      disposedCheck();
      horizontalAlignment = checkHorizontalKey(alignment,
                                   "Invlid horizontal alignment"); //$NON-NLS-1$
      _setHorizontalAlignment(_bridge, alignment);
      firePropertyChange(PROPERTY_HORIZONTAL_ALIGNMENT, oldHorizontalAlignment,
                         alignment);
    }
  }

  /**
   * Returns action command.
   *
   * @return action command
   */
  public String getActionCommand() {
    if (actionCommand == null) {
      return getText();
    }
    return actionCommand;
  }

  /**
   * Sets action command for this button.
   *
   * @param actionCommand the action command
   */
  public void setActionCommand(String actionCommand) {
    final String oldActionCommand = this.actionCommand;
    if (oldActionCommand == null) {
      if (actionCommand == null) {
        return;
      }
    } else {
      if (oldActionCommand.equals(actionCommand)) {
        return;
      }
    }
    this.actionCommand = actionCommand;
    firePropertyChange(PROPERTY_ACTION_COMMAND, oldActionCommand,
                       actionCommand);
  }

  /**
   * Adds action listener.
   *
   * @param listener the action listener
   */
  public void addActionListener(ActionListener listener) {
    if (listener != null) {
      enableEvents(QSwingEvent.ACTION_EVENT_MASK);
      listenerList.add(ActionListener.class, listener);
    }
  }

  /**
   * Remove action listener.
   *
   * @param listener the action listener
   */
  public void removeActionListener(ActionListener listener) {
    if (listener != null) {
      listenerList.remove(ActionListener.class, listener);
      if (listenerList.getListenerCount(ActionListener.class) == 0) {
        disableEvents(QSwingEvent.ACTION_EVENT_MASK);
      }
    }
  }

  /**
   * Returns all action listeners.
   *
   * @return the action listener array
   */
  public ActionListener[] getActionListeners() {
    int count = listenerList.getListenerCount(ActionListener.class);
    ActionListener[] listeners = new ActionListener[count];
    if (count > 0) {
      System.arraycopy(listenerList.getListenersArray(ActionListener.class), 0,
                       listeners,  0, count);
    }
    return listeners;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Checks horizontal alignment key.
   *
   * @param key the key
   * @param message the error message
   *
   * @return alignment key
   *
   * @throws IllegalArgumentException if the key is invalid
   */
  protected final int checkHorizontalKey(int key, String message) {
    if ((key == LEFT) || (key == CENTER) || (key == RIGHT) ||
        (key == LEADING) || (key == TRAILING)) {
      return key;
    }
    throw new IllegalArgumentException(message);
  }

  /**
   * Fires action event.
   *
   * @param e the action event
   */
  protected void fireActionPerformed(ActionEvent e) {
    int count = listenerList.getListenerCount(ActionListener.class);
    if (count > 0) {
      ActionListener[] listeners = (ActionListener[])listenerList.
                                       getListenersArray(ActionListener.class);
      for (int i = count; --i >= 0;) {
        listeners[i].actionPerformed(e);
      }
    }
  }

/*^**************************************************************************^*/
/*- Native Callbacks.                                                        -*/
/*^**************************************************************************^*/

  void _returnPressed(int modifiers) {
    fireActionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
                                        getActionCommand(),
                                        System.currentTimeMillis(), modifiers));
  }

/*^**************************************************************************^*/
/*- Package-Private Native Methods.                                          -*/
/*^**************************************************************************^*/

  native long _create();

  native String _getText(long bridge);

  native void _setText(long bridge, String text);

  native void _setHorizontalAlignment(long bridge, int alignment);

/*^**************************************************************************^*/
/*- Private Static Native Method.                                            -*/
/*^**************************************************************************^*/

  private static native void _initialize();

}

/*^*****************************************************************************
  File History:

 - 2007-05-04 18:57:03 Slobodan
   Initial version

*****************************************************************************^*/
