/*
 * @(#)JQTimer.java  0.2.0 / 2007-05-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import yu.ac.bg.etf.javaqx.core.events.EventListenerList;
import yu.ac.bg.etf.javaqx.qswing.events.ActionEvent;
import yu.ac.bg.etf.javaqx.qswing.events.ActionListener;

/**
 * Timer Class.
 * <p>
 *   <tt>NOTE: JQTimer object is garbage collectable - there are no hidden
 *             references on it!</tt>
 * </p>
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class JQTimer {

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    QSwing.initialize();
    _initialize();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private long _bridge;
  private int delay;
  private int initialDelay;
  private boolean repeats;
  private boolean running;

/*^**************************************************************************^*/
/*- Protected Field.                                                         -*/
/*^**************************************************************************^*/

  protected final EventListenerList listenerList;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JQTimer object.
   *
   * @param delay the number of milliseconds between action events
   * @param initialDelay the initial delay
   * @param listener the listener
   */
  public JQTimer(int delay, int initialDelay, ActionListener listener) {
    this.delay = delay;
    this.initialDelay = initialDelay;
    repeats = true;
    listenerList = new EventListenerList();
    if (listener != null) {
      addActionListener(listener); // NOPMD
    }
  }

  /**
   * Constructs new JQTimer object.
   *
   * @param delay the number of milliseconds between action events
   * @param listener the listener
   */
  public JQTimer(int delay, ActionListener listener) {
    this(delay, delay, listener);
  }

  /**
   * Constructs new JQTimer object.
   *
   * @param delay the number of milliseconds between action events
   */
  public JQTimer(int delay) {
    this(delay, null);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Adds action listener.
   *
   * @param listener the action listener
   */
  public void addActionListener(ActionListener listener) {
    listenerList.add(ActionListener.class, listener);
  }

  /**
   * Removes action listener.
   *
   * @param listener the action listener
   */
  public void removeActionListener(ActionListener listener) {
    listenerList.remove(ActionListener.class, listener);
  }

  /**
   * Returns all action listeners.
   *
   * @return the action listener array
   */
  public ActionListener[] getActionListeners() {
    int count = listenerList.getListenerCount(ActionListener.class);
    ActionListener[] listeners = new ActionListener[count];
    if (count > 0) {
      System.arraycopy(listenerList.getListenersArray(ActionListener.class), 0,
                       listeners,  0, count);
    }
    return listeners;
  }

  /**
   * Returns delay.
   *
   * @return delay
   */
  public int getDelay() {
    return delay;
  }

  /**
   * Sets delay.
   *
   * @param delay the delay
   *
   * @throws IllegalArgumentException if (delay &lt; 0)
   */
  public void setDelay(int delay) {
    if (delay < 0) {
      throw new IllegalArgumentException("Invalid delay: " //$NON-NLS-1$
                                         + delay);
    }
    if (this.delay != delay) {
      this.delay = delay;
      synchronized (this) {
        if (running) {
          _setInterval(_bridge, delay);
        }
      }
    }
  }

  /**
   * Return initial delay.
   *
   * @return initial delay
   */
  public int getInitialDelay() {
    return initialDelay;
  }

  /**
   * Sets delay.
   *
   * @param initialDelay the initial delay
   *
   * @throws IllegalArgumentException if (delay &lt; 0)
   */
  public void setInitialDelay(int initialDelay) {
    if (initialDelay < 0) {
      throw new IllegalArgumentException("Invalid initial delay: " //$NON-NLS-1$
                                         + initialDelay);
    }
    this.initialDelay = initialDelay;
  }

  /**
   * Returns <tt>true</tt> if the timer is sending an action event to its
   * listeners multiple times.
   *
   * @return  <tt>true</tt> if the timer is sending an action event to its
   * listeners multiple times
   */
  public boolean isRepeats() {
    return repeats;
  }

  /**
   * If repeats is <tt>false</tt> the timer will send only one action event to
   * its listeners.
   *
   * @param repeats the repeats flag
   */
  public void setRepeats(boolean repeats) {
    if (this.repeats != repeats) {
      this.repeats = repeats;
      synchronized (this) {
        if (running) {
          _setSingleShot(_bridge, !repeats);
        }
      }
    }
  }

  /**
   * Returns <tt>true</tt> if the timer is running.
   *
   * @return <tt>true</tt> if the timer is running
   */
  public boolean isRunning() {
    synchronized (this) {
      return running;
    }
  }

  /**
   * Starts the timer.
   *
   * @param initialDelay the initial delay
   */
  public void start(int initialDelay) {
    synchronized (this) {
      if (!running) {
        running = true;
        this.initialDelay = initialDelay;
        _bridge = _create(delay, initialDelay, !repeats);
      }
    }
  }

  /**
   * Starts the timer.
   */
  public void start() {
    start(initialDelay);
  }

  /**
   * Stops the timer.
   */
  public void stop() {
    synchronized (this) {
      if (running) {
        dispose();
        running = false;
      }
    }
  }

  /**
   * Restarts the timer.
   */
  public void restart() {
    stop();
    start();
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Fires action event.
   *
   * @param e the action event
   */
  protected void fireActionPerformed(ActionEvent e) {
    int count = listenerList.getListenerCount(ActionListener.class);
    if (count > 0) {
      ActionListener[] listeners = (ActionListener[])listenerList.
                                       getListenersArray(ActionListener.class);
      for (int i = count - 1; i >= 0; i--) {
        listeners[i].actionPerformed(e);
      }
    }
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void finalize() {
    dispose(); // NOPMD NOFB
  }

/*^**************************************************************************^*/
/*- Package-Private Method.                                                  -*/
/*^**************************************************************************^*/

  void _timeout() {
    fireActionPerformed(new ActionEvent(this));
    if (!repeats) {
      stop();
    }
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void dispose() {
    if (_bridge != 0) {
      _destroy(_bridge);
      _bridge = 0;
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Native Methods.                                          -*/
/*^**************************************************************************^*/

  native long _create(int delay, int initialDelay, boolean singleShot);

  native void _destroy(long bridge);

  native void _setInterval(long bridge, int interval);

  native void _setSingleShot(long bridge, boolean singleShot);

/*^**************************************************************************^*/
/*- Private Static Native Method.                                            -*/
/*^**************************************************************************^*/

  private static native void _initialize();

}

/*^*****************************************************************************
  File History:

 - 2007-05-04 18:57:16 Slobodan
   Initial version

*****************************************************************************^*/
