/*
 * @(#)JQToolBar.java  0.2.0 / 2007-05-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import yu.ac.bg.etf.javaqx.qswing.graphics.Dimension;
import yu.ac.bg.etf.javaqx.qswing.layouts.LayoutManager;
import yu.ac.bg.etf.javaqx.qswing.properties.JQToolbarProperties;

/**
 * ToolBar Component.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class JQToolBar extends JQContainer
                       implements QSwingConstants, JQToolbarProperties {

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    _initialize();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean floatable;
  private int orientation;
  private Dimension iconSize;
  private Dimension defaultIconSize;
  private boolean autoExclusive;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JQToolBar object.
   */
  public JQToolBar() {
    this(null, true);
  }

  /**
   * Constructs new JQToolBar object.
   *
   * @param title the tool bar title
   */
  public JQToolBar(String title) {
    this(title, true);
  }

  /**
   * Constructs new JQToolBar object.
   *
   * @param title the tool bar title
   * @param autoExclusive <tt>true</tt> if auto exclusive
   */
  public JQToolBar(String title, boolean autoExclusive) {
    super(null);
    floatable = true;
    if (title != null) {
      _setName(_bridge, title); // NOPMD
      name = title;
    }
    this.autoExclusive = autoExclusive;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Adds tool button item.
   *
   * @param toolButton the tool button
   *
   * @return the tool button argument
   */
  public JQToolButton add(JQToolButton toolButton) {
    addImpl(toolButton, null, -1);
    return toolButton;
  }

  /**
   * Adds tool button item.
   *
   * @param toolButton the tool button
   * @param index the tool button index
   *
   * @return the tool button argument
   */
  public JQToolButton add(JQToolButton toolButton, int index) {
    addImpl(toolButton, null, index);
    return toolButton;
  }

  /**
   * Adds toggle tool button item.
   *
   * @param toolButton the toggle tool button
   *
   * @return the toggle tool button argument
   */
  public JQToggleToolButton add(JQToggleToolButton toolButton) {
    return add(toolButton, -1);
  }

  /**
   * Adds toggle tool button item.
   *
   * @param toolButton the toggle tool button
   * @param index the toggle tool button index
   *
   * @return the toggle tool button argument
   */
  public JQToggleToolButton add(JQToggleToolButton toolButton,
                                int index) { // NOPMD
    super.add(toolButton, index);
    if (autoExclusive) {
      if (index < 0) {
        index = componentCount - 1;
      }
      if (index > 0) {
        JQComponent beforeComponent = components[index - 1];
        if (beforeComponent instanceof JQToggleToolButton) { // NOPMD
          JQToggleToolButton beforeButton =
                                          (JQToggleToolButton)beforeComponent;
          SelectableGroup selectableGroup = beforeButton.getSelectableGroup();
          if (selectableGroup == null) {
            selectableGroup = new SelectableGroup();
            selectableGroup.add(beforeButton);
          }
          selectableGroup.add(toolButton);
        }
      }
    }
    return toolButton;
  }

  /**
   * Adds separator.
   *
   * @return separator
   */
  public JQSeparator addSeparator() {
    JQSeparator separator = new JQSeparator();
    add(separator);
    return separator;
  }

  /**
   * Adds separator.
   *
   * @param index the separator index
   *
   * @return separator
   */
  public JQSeparator addSeparator(int index) {
    JQSeparator separator = new JQSeparator();
    add(separator, index);
    return separator;
  }

  /**
   * Returns floatable property.
   *
   * @return floatable property
   */
  public boolean isFloatable() {
    return floatable;
  }

  /**
   * Sets floatable property.
   *
   * @param floatable the floatable property
   */
  public void setFloatable(boolean floatable) {
    final boolean oldFloatable = this.floatable;
    if (oldFloatable != floatable) {
      disposedCheck();
      _setFloatable(_bridge, floatable);
      this.floatable = floatable;
      firePropertyChange(PROPERTY_FLOATABLE, oldFloatable, floatable);
    }
  }

  /**
   * Returns orientation of the toolbar.
   *
   * @return orientation of the toolbar
   */
  public int getOrientation() {
    return orientation;
  }

  /**
   * Sets orientation.
   *
   * @param orientation the orientation
   */
  public void setOrientation(int orientation) {
    if ((orientation < HORIZONTAL) || (orientation > VERTICAL)) {
      throw new IllegalArgumentException("Invalid orientation"); //$NON-NLS-1$
    }
    disposedCheck();
    _setOrientation(_bridge, orientation);
  }

  /**
   * Returns default size of the tool bar icons.
   *
   * @return default size of the tool bar icons
   */
  public Dimension getDefaultIconSize() {
    return defaultIconSize;
  }

  /**
   * Sets default size of the tool bar icons.
   *
   * @param defaultIconSize the default size
   */
  public void setDefaultIconSize(Dimension defaultIconSize) {
    if (defaultIconSize == null) {
      throw new IllegalArgumentException("Null defaultIconSize"); //$NON-NLS-1$
    }
    disposedCheck();
    this.defaultIconSize = defaultIconSize;
    _setIconSize(_bridge, defaultIconSize.getWidth(),
                 defaultIconSize.getHeight());
  }

  /**
   * Sets default size of the tool bar icons.
   *
   * @param width the default width
   * @param height the height size
   */
  public void setDefaultIconSize(int width, int height) {
    setDefaultIconSize(new Dimension(width, height));
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setLayout(LayoutManager layoutManager) {
    // empty method
  }

  /**
   * {@inheritDoc}
   */
  public void validate() {
    // empty method
  }

  /**
   * {@inheritDoc}
   */
  public void invalidate() {
    // empty method
  }

  /**
   * {@inheritDoc}
   */
  public void revalidate() {
    // empty method
  }

  /**
   * {@inheritDoc}
   */
  protected String paramString() {
    StringBuffer sb = new StringBuffer(80);
    sb.append(super.paramString());
    sb.append(",orientation="); //$NON-NLS-1$
    if (orientation == HORIZONTAL) {
      sb.append("HORIZONTAL"); //$NON-NLS-1$
    } else {
      sb.append("VERTICAL"); //$NON-NLS-1$
    }
    if (floatable) {
      sb.append(",floatable"); //$NON-NLS-1$
    }
    return sb.toString();
  }

  void acquireChild(JQComponent child) {
    if (defaultIconSize == null) {
      Icon icon = null;
      if (child instanceof JQToolButton) {
        JQToolButton toolButton = (JQToolButton)child;
        icon = toolButton.getIcon();
      } else if (child instanceof JQToggleToolButton) {
        JQToggleToolButton toggleToolButton = (JQToggleToolButton)child;
        icon = toggleToolButton.getIcon();
      }
      if (icon != null) {
        int iconWidth = icon.getIconWidth();
        int iconHeight = icon.getIconHeight();
        if (iconSize == null) {
          iconSize = new Dimension(iconWidth, iconHeight);
          _setIconSize(_bridge, iconWidth, iconHeight);
        } else {
          int oldIconWidth = iconSize.getWidth();
          int oldIconHeight = iconSize.getHeight();
          if (iconWidth < oldIconWidth) {
            iconWidth = oldIconWidth;
          }
          if (iconHeight < oldIconHeight) {
            iconHeight = oldIconHeight;
          }
          if ((iconWidth != oldIconWidth) || (iconHeight != oldIconHeight)) {
            iconSize = new Dimension(iconWidth, iconHeight);
            _setIconSize(_bridge, iconWidth, iconHeight);
          }
        }
      }
    }
    super.acquireChild(child);
  }

  void releaseChild(JQComponent child) {
    if (child instanceof JQToggleToolButton) {
      JQToggleToolButton toggleToolButton = (JQToggleToolButton)child;
      SelectableGroup selectableGroup = toggleToolButton.getSelectableGroup();
      if (selectableGroup != null) {
        selectableGroup.remove(toggleToolButton);
      }
    }
    super.releaseChild(child);
  }

/*^**************************************************************************^*/
/*- Native Callback.                                                         -*/
/*^**************************************************************************^*/

  void _orientationChanged(int orientation) {
    final int oldOrientation = this.orientation;
    if (oldOrientation != orientation) {
      this.orientation = orientation;
      firePropertyChange(PROPERTY_ORIENTATION, oldOrientation, orientation);
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Native Methods.                                          -*/
/*^**************************************************************************^*/

  native long _create();

  native void _setName(long bridge, String name);

  native void _insertChild(long bridge, long childBridge, int index);

  native void _removeChild(long bridge, long childBridge);

  native void _setFloatable(long bridge, boolean floatable);

  native void _setOrientation(long bridge, int orientation);

  native void _setIconSize(long bridge, int width, int height);

/*^**************************************************************************^*/
/*- Private Static Native Method.                                            -*/
/*^**************************************************************************^*/

  private static native void _initialize();

}

/*^*****************************************************************************
  File History:

 - 2007-05-04 18:56:51 Slobodan
   Initial version

*****************************************************************************^*/
