/*
 * @(#)QSwingImage.java  0.2.0 / 2007-05-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Arrays;

import yu.ac.bg.etf.javaqx.qswing.graphics.Image;
import yu.ac.bg.etf.javaqx.qswing.graphics.ImageConsumer;
import yu.ac.bg.etf.javaqx.qswing.graphics.ImageProducer;
import yu.ac.bg.etf.javaqx.util.Utilities;

/**
 * QSwing Image.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
class QSwingImage extends Image {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static Image emptyImage;

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  int imageReferences;
  QSwingImageProducer imageProducer; // NOPMD
  byte[] data;
  int hashCode; // NOPMD

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  long _imagePointer;
  int _width;
  int _height;
  String _format;

/*^**************************************************************************^*/
/*- Package-Private Constructors.                                            -*/
/*^**************************************************************************^*/

  QSwingImage() {
    setImagePointer(_create()); // NOPMD
  }

  QSwingImage(InputStream is) throws IOException {
    this(Utilities.toByteArray(is));
  }

  QSwingImage(String fileName) throws IOException {
    this(new FileInputStream(fileName)); // NOPMD
  }

  QSwingImage(File file) throws IOException {
    this(new FileInputStream(file));
  }

  QSwingImage(URL location) throws IOException {
    this(location.openStream()); // NOPMD
  }

  QSwingImage(byte[] imageData) throws IOException {
    this(imageData, imageData.length);
  }

  QSwingImage(byte[] imageData, int length) throws IOException {
    if ((imageData == null) || (length > imageData.length)) {
      throw new IllegalArgumentException();
    }
    setImagePointer(_create(imageData, length)); // NOPMD
    if (_imagePointer == 0) {
      throw new IOException("QSwingImage creation failed"); //$NON-NLS-1$
    }
    data = new byte[length];
    System.arraycopy(imageData, 0, data, 0, length);
    _format = getFormat(imageData);
  }

  QSwingImage(long imagePointer, byte[] data, int width, int height) { // NOPMD
    _imagePointer = imagePointer;
    this.data = data;
    _width = width;
    _height = height;
    _format = getFormat(data);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true;
    }
    if ((obj != null) && (obj.getClass() == QSwingImage.class)) {
      QSwingImage image = (QSwingImage)obj;
      return Arrays.equals(image.data, data);
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    if (hashCode == 0) {
      int hashCode = 419;
      byte[] data = this.data;
      for (int i = data.length; --i >= 0;) {
        hashCode = 67 * hashCode + data[i];
      }
      this.hashCode = hashCode;
    }
    return hashCode;
  }

  /**
   * {@inheritDoc}
   */
  public int getWidth() {
    return _width;
  }

  /**
   * {@inheritDoc}
   */
  public int getHeight() {
    return _height;
  }

  /**
   * {@inheritDoc}
   */
  public ImageProducer getSource() {
    synchronized (data) {
      if (imageProducer == null) {
        imageProducer = new QSwingImageProducer();
      }
      return imageProducer;
    }
  }

  /**
   * {@inheritDoc}
   */
  public Image getScaledInstance(int width, int height, int hints) {
    if ((hints != SCALE_FAST) && (hints != SCALE_SMOOTH)) {
      throw new IllegalArgumentException("Invalid hints"); //$NON-NLS-1$
    }
    long imagePointer = acquireImagePointer();
    Image scaledImage = _createScaled(imagePointer, width, height, hints,
                                      _format);
    releaseImagePointer();
    return scaledImage;
  }

  /**
   * {@inheritDoc}
   */
  public void flush() {
    synchronized (data) {
      if ((_imagePointer != 0) && (imageReferences == 0)) {
        _destroy(_imagePointer);
        _imagePointer = 0;
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void finalize() {
    synchronized (data) { // NOPMD
      if (_imagePointer != 0) {
        _destroy(_imagePointer);
        _imagePointer = 0;
      }
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  long acquireImagePointer() {
    synchronized (data) {
      imageReferences++;
      if (_imagePointer == 0) {
        setImagePointer(_create(data, data.length));
      }
      return _imagePointer;
    }
  }

  void releaseImagePointer() {
    synchronized (data) {
      imageReferences--;
      if ((imageReferences == 0) && (_imagePointer != 0)) {
        _destroy(_imagePointer);
        _imagePointer = 0;
      }
    }
  }

  long getImagePointer() {
    synchronized (data) {
      if (_imagePointer == 0) {
        setImagePointer(_create(data, data.length));
      }
      return _imagePointer;
    }
  }

  void setImagePointer(long imagePointer) {
    if (imagePointer == 0) {
      throw new OutOfMemoryError("QSwingImage creation failed"); //$NON-NLS-1$
    }
    _imagePointer = imagePointer;
  }

/*^**************************************************************************^*/
/*- Package-Private Static Method.                                           -*/
/*^**************************************************************************^*/

  static String getFormat(byte[] imageData) {
    // simple image format detection
    if ((imageData != null) && (imageData.length > 3)) {
      if ((imageData[1] == 0x50) && (imageData[2] == 0x4E) &&
          (imageData[3] == 0x47)) {
        return "PNG"; //$NON-NLS-1$
      }
      if ((imageData[0] == (byte)0xFF) && (imageData[1] == (byte)0xD8)) {
        return "JPG"; //$NON-NLS-1$
      }
    }
    return "BMP"; //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Package-Private Static Methods.                                          -*/
/*^**************************************************************************^*/

  static Image getEmptyImage() {
    if (emptyImage == null) { // NOPMD - thread safe singleton
      emptyImage = new QSwingEmptyImage();
    }
    return emptyImage;
  }

/*^**************************************************************************^*/
/*- Package-Private Native Methods.                                          -*/
/*^**************************************************************************^*/

  native long _create();

  native long _create(byte[] imageData, int lenght);

  native Image _createScaled(long imagePointer, int width, int height,
                             int hints, String format);

  native void _destroy(long imagePointer);

  /*
   * @todo  add native method byte[] _getData(long imagePointer), and remove
   *        caching of large images
   */

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * QSwing Image Producer.
   */
  final class QSwingImageProducer implements ImageProducer {
    public void startProduction(ImageConsumer ic) {
      ic.setPixels(data, data.length);
    }
  }

  /**
   * QSwing Image Consumer.
   */
  static final class QSwingImageConsumer implements ImageConsumer {
    byte[] pixels;
    int length;
    public void setPixels(byte[] pixels, int length) { // NOPMD
      this.pixels = pixels;
      this.length = length;
    }
  }

  /**
   * Empty Image Class.
   */
  static final class QSwingEmptyImage extends QSwingImage {
    public boolean equals(Object obj) {
      return (obj == this);
    }
    public int hashCode() {
      return (int)_imagePointer;
    }
    public int getWidth() {
      return 0;
    }
    public int getHeight() {
      return 0;
    }
    public Image getScaledInstance(int width, int height, int hints) {
      return this;
    }
    public void flush() {
      // empty method
    }
    protected void finalize() {
      if (_imagePointer != 0) { // NOPMD
        _destroy(_imagePointer);
      }
    }
    long acquireImagePointer() {
      return _imagePointer;
    }
    long getImagePointer() {
      return _imagePointer;
    }
    void releaseImagePointer() {
      // empty method
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2007-05-04 19:03:20 Slobodan
   Initial version

*****************************************************************************^*/
