/*
 * @(#)QSwingUtilities.java  0.2.0 / 2007-11-10
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import java.lang.reflect.InvocationTargetException;

/**
 * QSwing Utilities.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class QSwingUtilities {

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    QSwing.initialize();
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private QSwingUtilities() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Invokes runnable target from the  GUI loop.
   *
   * @param target the runnable target
   *
   * @throws IllegalArgumentException if the target is <tt>null</tt>
   */
  public static void invokeLater(Runnable target) {
    if (target == null) {
      throw new IllegalArgumentException("Null target"); //$NON-NLS-1$
    }
    _invokeLater(target);
  }

  /**
   * Invokes runnable target from the GUI loop and waits until finish.
   * <p>
   *   <tt>NOTE: Do not call this method form QSwing thread.</tt>
   * </p>
   *
   * @param target the runnable target
   *
   * @throws IllegalArgumentException if the target is <tt>null</tt>
   * @throws Error if the method is called from the QSwing thread.
   * @throws InvocationTargetException if an exception is thrown while running
   *                                   <tt>target</tt>
   */
  public static void invokeAndWait(Runnable target) throws
                                                     InvocationTargetException {
    if (target == null) {
      throw new IllegalArgumentException("Null target"); //$NON-NLS-1$
    }
    if (QSwing.isQSwingThread()) {
      throw new Error("Method invokeSafeAndWait() called " + //$NON-NLS-1$ NOPMD
                      "from the QSwing thread"); //$NON-NLS-1$
    }
    _invokeAndWait(target);
  }

  /**
   * Invokes "safe" runnable target from the GUI loop and waits until finish.
   * Exceptions are ignored and printed on the standard error stream. "Safe"
   * runnable target doesn't throw exceptions.
   * <p>
   *   <tt>NOTE: Do not call this method form QSwing thread.</tt>
   * </p>
   *
   * @param safeTarget the runnable target
   *
   * @throws IllegalArgumentException if the target is <tt>null</tt>
   * @throws Error if the method is called from the QSwing thread.
   */
  public static void invokeSafeAndWait(Runnable safeTarget) {
    if (safeTarget == null) {
      throw new IllegalArgumentException("Null target"); //$NON-NLS-1$
    }
    if (QSwing.isQSwingThread()) {
      throw new Error("Method invokeSafeAndWait() called " + //$NON-NLS-1$ NOPMD
                      "from the QSwing thread"); //$NON-NLS-1$
    }
    _invokeSafeAndWait(safeTarget);
  }

  /**
   * Returns the index of mnemonic '&'.
   *
   * @param text the text
   *
   * @return mnemonic index
   */
  public static int getMnemonicIndex(String text) {
    if (text != null) {
      int lastChar = text.length();
      --lastChar;
      for (int i = 0; i < lastChar;) {
        if (text.charAt(i) == '&') {
          ++i;
          if (text.charAt(i) != '&') { // NOPMD
            return --i;
          }
        }
        ++i;
      }
    }
    return -1;
  }

/*^**************************************************************************^*/
/*- Private Static Native Methods.                                           -*/
/*^**************************************************************************^*/

  private static native void _invokeLater(Runnable target);

  private static native void _invokeAndWait(Runnable target) throws
                                                      InvocationTargetException;

  private static native void _invokeSafeAndWait(Runnable safeTarget);

}

/*^*****************************************************************************
  File History:

 - 2007-11-10 19:26:28 Slobodan
   Initial version

*****************************************************************************^*/
