/*
 * @(#)StyleSheet.java  0.2.0 / 2007-09-26
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.WeakReference;

import yu.ac.bg.etf.javaqx.util.Utilities;

/**
 * Style Sheet.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class StyleSheet {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private String text;
  private boolean valid;
  private boolean converted;
  private WeakReference textCache;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new StyleSheet object.
   *
   * @param styleSheetStream the style sheet input stream
   *
   * @throws IOException on an I/O error
   */
  public StyleSheet(InputStream styleSheetStream) throws IOException {
    this(Utilities.toString(styleSheetStream));
  }

  /**
   * Constructs new StyleSheet object.
   *
   * @param styleSheetFile the style sheet file
   *
   * @throws IOException on an I/O error
   */
  public StyleSheet(File styleSheetFile) throws IOException {
    this(Utilities.toString(new FileReader(styleSheetFile)));
  }

  /**
   * Constructs new StyleSheet object.
   *
   * @param styleSheetText the style sheet text
   */
  public StyleSheet(String styleSheetText) {
    if (styleSheetText == null) {
      throw new IllegalArgumentException("Null style sheet"); //$NON-NLS-1$
    }
    text = styleSheetText;
  }


/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if the style sheet is valid.
   *
   * @return <tt>true</tt> if the style sheet is valid
   */
  public final boolean isValid() {
    if (!converted) {
      makeCache();
    }
    return valid;
  }

  /**
   * Returns style sheet text.
   *
   * @return style sheet text
   */
  public final String getText() {
    return text;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true;
    }
    if ((obj != null) && (obj.getClass() == getClass())) {
      return text.equals(((StyleSheet)obj).text);
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    return text.hashCode();
  }

/*^**************************************************************************^*/
/*- Pacakge-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  String getStyleSheetText() {
    if (!converted) {
      makeCache();
    }
    if (valid) {
      String cachedText = (String)textCache.get();
      if (cachedText == null) {
        cachedText = convertStyleSheet(text);
        textCache = new WeakReference(cachedText);
      }
      return cachedText;
    }
    return null;
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void makeCache() {
    String cachedText = convertStyleSheet(text);
    if (cachedText == null) {
      valid = false;
    } else {
      textCache = new WeakReference(cachedText);
      valid = true;
    }
    converted = true;
  }

  private String convertStyleSheet(String styleSheetText) {
    /** @todo  implement better converter */
    /** @todo  add tooltip support */
    int textLen = styleSheetText.length();
    StringBuffer sb = new StringBuffer(textLen << 1);
    StringBuffer selector = new StringBuffer();
    int state = 0;
    boolean insideBlock = false;
    for (int i = 0; i < textLen; i++) {
      char c = styleSheetText.charAt(i);
      switch (state) { // NOPMD
        case 0: // selector state
          if (c == '/') {
            state = 101;
          } else if (c == '{') {
            sb.append(c);
            state = 201;
          } else if (Character.isJavaIdentifierStart(c)) {
            selector.setLength(0);
            selector.append("QSwing_"); //$NON-NLS-1$
            selector.append(c);
            state = 1;
          } else {
            sb.append(c);
          }
          break;
        case 1:
          if (Character.isJavaIdentifierPart(c)) {
            selector.append(c);
            break;
          } else if (c == '/') {
            state = 101;
          } else if (c == '{') {
            state = 201;
          } else if (c == ':') {
            state = 2;
          } else {
            state = 0;
          }
          sb.append("QWidget["); //$NON-NLS-1$
          sb.append(selector);
          sb.append(']');
          sb.append(c);
          break;
        case 2:
          if ((c != ':') && !Character.isJavaIdentifierPart(c)) {
            state = 0;
          }
          sb.append(c);
          break;
        case 101: // comment state
          if (c != '*') {
            return null;
          }
          state = 102;
          break;
        case 102:
          if (c == '*') {
            state = 103;
          }
          break;
        case 103:
          if (c != '/') { // NOPMD
            state = 102;
          } else if (insideBlock) {
            state = 201;
          } else {
            state = 0;
          }
          break;
        case 201: // block state
          if (c == '/') {
            state = 101;
            break;
          }
          if (c == '(') {
            state = 202;
          } else if (c == '}') {
            state = 0;
          }
          sb.append(c);
          break;
        case 202: // url state
          if (c == ')') {
            state = 201;
          }
          sb.append(c);
          break;
      }
    }
    return sb.toString();
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-26 16:03:00 Slobodan
   Initial version

*****************************************************************************^*/
