/*
 * @(#)ContextEvent.java  0.2.0 / 2007-11-13
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.events;

import yu.ac.bg.etf.javaqx.qswing.JQComponent;
import yu.ac.bg.etf.javaqx.qswing.graphics.Point;

/**
 * Context Event (<i>right click</i> event).
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class ContextMenuEvent extends InputEvent {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Mouse caused reason id (the value is {@value}).
   */
  public static final int REASON_MOUSE = 0;

  /**
   * Mouse caused reason id (the value is {@value}).
   */
  public static final int REASON_KEYBOARD = 1;

  /**
   * Other (unknown) reason id (the value is {@value}).
   */
  public static final int REASON_OTHER = 2;

  /**
   * Other (unknown) reason id (the value is {@value}).
   */
  public static final int CONTEXT_MENU_TRIGGERED = 10001;

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = -6777628913661501237L;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final int x;
  private final int y;
  private final int globalX;
  private final int globalY;
  private final int reason;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MouseEvent object.
   *
   * @param source the source component
   * @param when the time the event occurred
   * @param modifiers the modifiers
   * @param reason the reason
   * @param x the x coordinate of the mouse point
   * @param y the y coordinate of the mouse point
   * @param globalX the global x coordinate of the mouse point
   * @param globalY the global y coordinate of the mouse point
   *
   * @throws IllegalArgumentException if the source is <tt>null</tt>
   */
  public ContextMenuEvent(JQComponent source, long when, int reason,
                          int modifiers, int x, int y, int globalX,
                          int globalY) {
    super(source, CONTEXT_MENU_TRIGGERED, when, modifiers);
    this.x = x;
    this.y = y;
    this.globalX = globalX;
    this.globalY = globalY;
    this.reason = reason;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the x coordinate of the mouse position relative to the source
   * component.
   *
   * @return the x coordinate
   */
  public int getX() {
    return x;
  }

  /**
   * Returns the y coordinate of the mouse position relative to the source
   * component.
   *
   * @return the y coordinate
   */
  public int getY() {
    return y;
  }

  /**
   * Returns point of the mouse pointer relative to the source component.
   *
   * @return point of the mouse pointer
   */
  public Point getPoint() {
    return new Point(x, y);
  }

  /**
   * Returns the x coordinate of the global mouse position.
   *
   * @return the x coordinate
   */
  public int getGlobalX() {
    return globalX;
  }

  /**
   * Returns the y coordinate of the global mouse position.
   *
   * @return the y coordinate
   */
  public int getGlobalY() {
    return globalY;
  }

  /**
   * Returns global mouse position.
   *
   * @return point of the mouse pointer
   */
  public Point getGlobalPoint() {
    return new Point(globalX, globalY);
  }

  /**
   * Returns reason why the event was sent.
   *
   * @return reason why the event was sent
   */
  public int getReason() {
    return reason;
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String paramString() {
    StringBuffer sb = new StringBuffer(80);
    switch (getReason()) {
      case REASON_MOUSE:
        sb.append("REASON_MOUSE"); //$NON-NLS-1$
        break;
      case REASON_KEYBOARD:
        sb.append("REASON_KEYBOARD"); //$NON-NLS-1$
        break;
      case REASON_OTHER:
        sb.append("REASON_OTHER"); //$NON-NLS-1$
        break;
      default:
        sb.append("UNKNOWN"); //$NON-NLS-1$
        break;
    }
    sb.append(",("); //$NON-NLS-1$
    sb.append(getGlobalX());
    sb.append(',');
    sb.append(getGlobalY());
    sb.append(')');
    int modifiers = getModifiers();
    if (modifiers != 0) {
      sb.append(",modifiers="); //$NON-NLS-1$
      sb.append(getModifiersText(modifiers));
    }
    return sb.toString();
  }

}

/*^*****************************************************************************
  File History:

 - 2007-11-13 19:37:13 Slobodan
   Initial version

*****************************************************************************^*/
