/*
 * @(#)InputEvent.java  0.2.0 / 2007-09-26
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.events;

import yu.ac.bg.etf.javaqx.qswing.JQComponent;

/**
 * Input Event.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class InputEvent extends ComponentEvent {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * The shift modifier mask.
   */
  public static final int SHIFT_MASK = 1 << 0;

  /**
   * The control modifier mask.
   */
  public static final int CTRL_MASK = 1 << 1;

  /**
   * The meta modifier mask.
   */
  public static final int META_MASK = 1 << 2;

  /**
   * The alt modifier mask.
   */
  public static final int ALT_MASK = 1 << 3;

  /**
   * The button1 modifier mask.
   */
  public static final int BUTTON1_MASK = 1 << 4;

  /**
   * The button2 modifier mask.
   */
  public static final int BUTTON2_MASK = 1 << 5;

  /**
   * The button3 modifier mask.
   */
  public static final int BUTTON3_MASK = 1 << 6;

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = 675478433546059330L;

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  final long when;
  final int modifiers;

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Constructs new InputEvent object.
   *
   * @param source the event source
   * @param id the event id
   * @param when the time the event occurred
   * @param modifiers the modifiers
   *
   * @throws IllegalArgumentException if the source is <tt>null</tt>
   */
  protected InputEvent(JQComponent source, int id, long when, int modifiers) {
    super(source, id);
    this.when = when;
    this.modifiers = modifiers;
  }

  /**
   * Constructs new InputEvent object.
   *
   * @param source the event source
   * @param id the event id
   * @param modifiers the modifiers
   *
   * @throws IllegalArgumentException if the source is <tt>null</tt>
   */
  protected InputEvent(JQComponent source, int id, int modifiers) {
    this(source, id, 0, modifiers);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the timestamp of when this event occurred.
   *
   * @return the timestamp of when this event occurred
   */
  public long getWhen() {
    return when;
  }

  /**
   * Returns the modifiers.
   *
   * @return the modifiers
   */
  public int getModifiers() {
    return modifiers;
  }

  /**
   * Returns <tt>true</tt> if shift modifier is down.
   *
   * @return <tt>true</tt> if shift modifier is down
   */
  public boolean isShiftDown() {
    return (modifiers & SHIFT_MASK) != 0;
  }

  /**
   * Returns <tt>true</tt> if control modifier is down.
   *
   * @return <tt>true</tt> if control modifier is down
   */
  public boolean isControlDown() {
    return (modifiers & CTRL_MASK) != 0;
  }

  /**
   * Returns <tt>true</tt> if meta modifier is down.
   *
   * @return <tt>true</tt> if meta modifier is down
   */
  public boolean isMetaDown() {
    return (modifiers & META_MASK) != 0;
  }

  /**
   * Returns <tt>true</tt> if alt modifier is down.
   *
   * @return <tt>true</tt> if alt modifier is down
   */
  public boolean isAltDown() {
    return (modifiers & ALT_MASK) != 0;
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Returns a string representation of the modifiers.
   *
   * @param modifiers the modifiers
   *
   * @return a string representation of the modifiers
   */
  public static String getModifiersText(int modifiers) { // NOPMD
    StringBuffer sb = new StringBuffer(50);
    if ((modifiers & ALT_MASK) != 0) {
      sb.append("Alt+"); //$NON-NLS-1$
    }
    if ((modifiers & META_MASK) != 0) {
      sb.append("Meta+"); //$NON-NLS-1$
    }
    if ((modifiers & CTRL_MASK) != 0) {
      sb.append("Ctrl+"); //$NON-NLS-1$
    }
    if ((modifiers & SHIFT_MASK) != 0) {
      sb.append("Shift+"); //$NON-NLS-1$
    }
    if ((modifiers & BUTTON1_MASK) != 0) {
      sb.append("Button1+"); //$NON-NLS-1$
    }
    if ((modifiers & BUTTON2_MASK) != 0) {
      sb.append("Button2+"); //$NON-NLS-1$
    }
    if ((modifiers & BUTTON3_MASK) != 0) {
      sb.append("Button3+"); //$NON-NLS-1$
    }
    if (sb.length() > 0) {
      sb.setLength(sb.length() - 1); // NOPMD
    }
    return sb.toString();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean isConsumed() {
    return consumed;
  }

  /**
   * {@inheritDoc}
   */
  public void consume() {
    consumed = true;
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-26 13:57:05 Slobodan
   Initial version

*****************************************************************************^*/
