/*
 * @(#)MouseEvent.java  0.2.0 / 2007-09-26
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.events;

import yu.ac.bg.etf.javaqx.qswing.JQComponent;
import yu.ac.bg.etf.javaqx.qswing.graphics.Point;

/**
 * Mouse Event.
 * <p>
 *   <tt>NOTE: This class is not full compatible with
 *             <em>java.awt.event.MouseEvent</em> class!</tt>
 * </p>
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class MouseEvent extends InputEvent {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Mouse clicked id.
   */
  public static final int MOUSE_CLICKED = 500;

  /**
   * Mouse pressed id.
   */
  public static final int MOUSE_PRESSED = 501;

  /**
   * Mouse released id.
   */
  public static final int MOUSE_RELEASED = 502;

  /**
   * Mouse moved id.
   */
  public static final int MOUSE_MOVED = 503;

  /**
   * Mouse entered id.
   */
  public static final int MOUSE_ENTERED = 504;

  /**
   * Mouse exited id.
   */
  public static final int MOUSE_EXITED = 505;

  /**
   * Mouse dragged id.
   */
  public static final int MOUSE_DRAGGED = 506;

  /**
   * Mouse wheel id.
   */
  public static final int MOUSE_WHEEL = 507;

  /**
   * Indicates that no button changed state.
   */
  public static final int NOBUTTON = 0;

  /**
   * Indicates that button 1 changed state.
   */
  public static final int BUTTON1 = 1;

  /**
   * Indicates that button 2 changed state.
   */
  public static final int BUTTON2 = 2;

  /**
   * Indicates button 3 changed state.
   */
  public static final int BUTTON3 = 3;

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = 7098657477558637427L;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int x;
  private int y;
  private int clickCount;
  private int button;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MouseEvent object.
   *
   * @param source the source component
   * @param id the event id
   * @param when the time the event occurred
   * @param modifiers the modifiers
   * @param x the x coordinate of the mouse point
   * @param y the y coordinate of the mouse point
   * @param clickCount the click count
   * @param button the mouse button
   *
   * @throws IllegalArgumentException if the source is <tt>null</tt>
   */
  public MouseEvent(JQComponent source, int id, long when, int modifiers, int x,
                    int y, int clickCount, int button) {
    super(source, id, when, modifiers);
    this.x = x;
    this.y = y;
    this.clickCount = clickCount;
    this.button = button;
  }

  /**
   * Constructs new MouseEvent object.
   *
   * @param source the source component
   * @param id the event id
   * @param when the time the event occurred
   * @param modifiers the modifiers
   * @param x the x coordinate of the mouse point
   * @param y the y coordinate of the mouse point
   * @param clickCount the click count
   *
   * @throws IllegalArgumentException if the source is <tt>null</tt>
   */
  public MouseEvent(JQComponent source, int id, long when, int modifiers, int x,
                    int y, int clickCount) {
    this(source, id, when, modifiers, x, y, clickCount, NOBUTTON);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the x coordinate of the mouse position relative to the source
   * component.
   *
   * @return the x coordinate
   */
  public int getX() {
    return x;
  }

  /**
   * Returns the y coordinate of the mouse position relative to the source
   * component.
   *
   * @return the y coordinate
   */
  public int getY() {
    return y;
  }

  /**
   * Returns point of the mouse pointer relative to the source component.
   *
   * @return point of the mouse pointer
   */
  public Point getPoint() {
    return new Point(x, y);
  }

  /**
   * Returns click count.
   *
   * @return click count
   */
  public int getClickCount()  {
    return clickCount;
  }

  /**
   * Returns the button that changed state.
   *
   * @return the button that changed state
   */
  public int getButton() {
    return button;
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String paramString() {
    StringBuffer sb = new StringBuffer(80);
    switch (getID()) {
      case MOUSE_PRESSED:
        sb.append("MOUSE_PRESSED"); //$NON-NLS-1$
        break;
      case MOUSE_RELEASED:
        sb.append("MOUSE_RELEASED"); //$NON-NLS-1$
        break;
      case MOUSE_CLICKED:
        sb.append("MOUSE_CLICKED"); //$NON-NLS-1$
        break;
      case MOUSE_ENTERED:
        sb.append("MOUSE_ENTERED"); //$NON-NLS-1$
        break;
      case MOUSE_EXITED:
        sb.append("MOUSE_EXITED"); //$NON-NLS-1$
        break;
      case MOUSE_MOVED:
        sb.append("MOUSE_MOVED"); //$NON-NLS-1$
        break;
      case MOUSE_DRAGGED:
        sb.append("MOUSE_DRAGGED"); //$NON-NLS-1$
        break;
      case MOUSE_WHEEL:
        sb.append("MOUSE_WHEEL"); //$NON-NLS-1$
        break;
       default:
        sb.append("unknown type"); //$NON-NLS-1$
    }
    sb.append(",("); //$NON-NLS-1$
    sb.append(getX());
    sb.append(',');
    sb.append(getY());
    sb.append("),button="); //$NON-NLS-1$
    sb.append(getButton());
    int modifiers = getModifiers();
    if (modifiers != 0) {
      sb.append(",modifiers="); //$NON-NLS-1$
      sb.append(getModifiersText(modifiers));
    }
    sb.append(",clickCount="); //$NON-NLS-1$
    sb.append(getClickCount());
    return sb.toString();
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-26 14:14:42 Slobodan
   Initial version

*****************************************************************************^*/
