/*
 * @(#)WindowEvent.java  0.2.0 / 2007-09-26
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.events;

import yu.ac.bg.etf.javaqx.qswing.JQWindow;
import yu.ac.bg.etf.javaqx.qswing.JQWindowConstants;

/**
 * Window Event.
 * <p>
 *   <tt>NOTE: This class is not full compatible with
 *             <em>java.awt.event.WindowEvent</em> class!</tt>
 * </p>
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class WindowEvent extends ComponentEvent {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * The window opened id.
   */
  public static final int WINDOW_OPENED = 200;

  /**
   * The window closing id.
   */
  public static final int WINDOW_CLOSING = 201;

  /**
   * The window closed id.
   */
  public static final int WINDOW_CLOSED = 202;

  /**
   * The window activated id.
   */
  public static final int WINDOW_ACTIVATED = 205;

  /**
   * The window deactivated id.
   */
  public static final int WINDOW_DEACTIVATED = 206;

  /**
   * The window focus gained id.
   */
  public static final int WINDOW_GAINED_FOCUS = 207;

  /**
   * The window focus lost id.
   */
  public static final int WINDOW_LOST_FOCUS = 208;

  /**
   * The window state changed id.
   */
  public static final int WINDOW_STATE_CHANGED = 209;

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = -3330116936677137106L;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int oldState;
  private int newState;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new WindowEvent object.
   *
   * @param source the source of the event
   * @param id the event type
   * @param oldState the old state of the window
   * @param newState the new state of the window
   *
   * @throws IllegalArgumentException if the source is <tt>null</tt>
   */
  public WindowEvent(JQWindow source, int id, int oldState, int newState) {
    super(source, id);
    this.oldState = oldState;
    this.newState = newState;
  }

  /**
   * Constructs new WindowEvent object.
   *
   * @param source the source of the event
   * @param id the event type
   *
   * @throws IllegalArgumentException if the source is <tt>null</tt>
   */
  public WindowEvent(JQWindow source, int id) {
    this(source, id, 0, 0);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the source window of the event.
   *
   * @return the source window of the event
   */
  public JQWindow getWindow() {
    return (JQWindow)source;
  }

  /**
   * Returns old window state.
   *
   * @return old window state
   */
  public int getOldState() {
    return oldState;
  }

  /**
   * Returns new window state.
   *
   * @return new window state
   */
  public int getNewState() {
    return newState;
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Returns string representation of the window state.
   *
   * @param state the window state
   *
   * @return string representation of the window state
   */
  protected String stateString(int state) {
    switch (state) {
      case JQWindowConstants.NORMAL:
        return "NORMAL"; //$NON-NLS-1$
      case JQWindowConstants.ICONIFIED:
        return "ICONIFIED"; //$NON-NLS-1$
      case JQWindowConstants.MAXIMIZED:
        return "MAXIMIZED"; //$NON-NLS-1$
      case JQWindowConstants.FULL_SCREEN:
        return "FULL_SCREEN"; //$NON-NLS-1$
      default:
        return "UNKNOWN"; //$NON-NLS-1$
    }
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String paramString() {
    String paramString;
    switch (id) {
      case WINDOW_OPENED:
        paramString = "WINDOW_OPENED"; //$NON-NLS-1$
        break;
      case WINDOW_CLOSING:
        paramString = "WINDOW_CLOSING"; //$NON-NLS-1$
        break;
      case WINDOW_CLOSED:
        paramString = "WINDOW_CLOSED"; //$NON-NLS-1$
        break;
      case WINDOW_ACTIVATED:
        paramString = "WINDOW_ACTIVATED"; //$NON-NLS-1$
        break;
      case WINDOW_DEACTIVATED:
        paramString = "WINDOW_DEACTIVATED"; //$NON-NLS-1$
        break;
      case WINDOW_GAINED_FOCUS:
        paramString = "WINDOW_GAINED_FOCUS"; //$NON-NLS-1$
        break;
      case WINDOW_LOST_FOCUS:
        paramString = "WINDOW_LOST_FOCUS"; //$NON-NLS-1$
        break;
      case WINDOW_STATE_CHANGED:
        paramString = "WINDOW_STATE_CHANGED"; //$NON-NLS-1$
        break;
      default:
        paramString = "unknown type"; //$NON-NLS-1$
    }
    return paramString + ",oldState=" +  //$NON-NLS-1$
           stateString(getOldState()) + ",newState=" + //$NON-NLS-1$
           stateString(getNewState());
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-26 15:15:56 Slobodan
   Initial version

*****************************************************************************^*/
