/*
 * @(#)CubicCurve.java  0.2.0 / 2007-10-01
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.graphics;

import java.util.NoSuchElementException;

/**
 * Cubic Curve Shape.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class CubicCurve implements Shape {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final int x1;
  private final int y1;
  private final int ctrlx1;
  private final int ctrly1;
  private final int ctrlx2;
  private final int ctrly2;
  private final int x2;
  private final int y2;
  private Rectangle bounds; // NOPMD
  private int hashCode; // NOPMD

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new CubicCurve object.
   */
  public CubicCurve() {
    x1 = 0;
    y1 = 0;
    ctrlx1 = 0;
    ctrly1 = 0;
    ctrlx2 = 0;
    ctrly2 = 0;
    x2 = 0;
    y2 = 0;
  }

  /**
   * Constructs new CubicCurve object.
   *
   * @param x1 the x coordinate of the curve's start point
   * @param y1 the y coordinate of the curve's start point
   * @param ctrlx1 the x coordinate of the curve's first control point
   * @param ctrly1 the y coordinate of the curve's first control point
   * @param ctrlx2 the x coordinate of the curve's second control point
   * @param ctrly2 the y coordinate of the curve's second control point
   * @param x2 the x coordinate of the curve's end point
   * @param y2 the y coordinate of the curve's end point
   */
  public CubicCurve(int x1, int y1, int ctrlx1, int ctrly1, int ctrlx2,
                    int ctrly2, int x2, int y2) {
    this.x1 = x1;
    this.y1 = y1;
    this.ctrlx1 = ctrlx1;
    this.ctrly1 = ctrly1;
    this.ctrlx2 = ctrlx2;
    this.ctrly2 = ctrly2;
    this.x2 = x2;
    this.y2 = y2;
  }

  /**
   * Constructs new CubicCurve object.
   *
   * @param p1 the curve's start point
   * @param ctrl1 the curve's first control point
   * @param ctrl2 the curve's second control point
   * @param p2 the curve's end point
   */
  public CubicCurve(Point p1, Point ctrl1, Point ctrl2, Point p2) {
    x1 = p1.getX();
    y1 = p1.getY();
    ctrlx1 = ctrl1.getX();
    ctrly1 = ctrl1.getY();
    ctrlx2 = ctrl2.getX();
    ctrly2 = ctrl2.getY();
    x2 = p2.getX();
    y2 = p2.getY();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the x coordinate of the curve's start point.
   *
   * @return the x coordinate of the curve's start point
   */
  public int getX1() {
    return x1;
  }

  /**
   * Returns the y coordinate of the curve's start point.
   *
   * @return the y coordinate of the curve's start point
   */
  public int getY1() {
    return y1;
  }

  /**
   * Returns the x coordinate of the curve's first control point.
   *
   * @return the x coordinate of the curve's first control point
   */
  public int getCtrlX1() {
    return ctrlx1;
  }

  /**
   * Returns the y coordinate of the curve's first control point.
   *
   * @return the y coordinate of the curve's first control point
   */
  public int getCtrlY1() {
    return ctrly1;
  }

  /**
   * Returns the x coordinate of the curve's second control point.
   *
   * @return the x coordinate of the curve's second control point
   */
  public int getCtrlX2() {
    return ctrlx2;
  }

  /**
   * Returns the y coordinate of the curve's second control point.
   *
   * @return the y coordinate of the curve's second control point
   */
  public int getCtrlY2() {
    return ctrly2;
  }

  /**
   * Returns the x coordinate of the curve's end point.
   *
   * @return the x coordinate of the curve's end point
   */
  public int getX2() {
    return x2;
  }

  /**
   * Returns the y coordinate of the curve's end point.
   *
   * @return the y coordinate of the curve's end point
   */
  public int getY2() {
    return y2;
  }

/*^**************************************************************************^*/
/*- Shape Interface Implementation.                                          -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean contains(int x, int y) {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public Rectangle getBounds() {
    if (bounds == null) {
      int minx = min4(x1, x2, ctrlx1, ctrlx2);
      int miny = min4(y1, y2, ctrly1, ctrly2);
      int maxx = max4(x1, x2, ctrlx1, ctrlx2);
      int maxy = max4(y1, y2, ctrly1, ctrly2);
      bounds = new Rectangle(minx, miny, maxx - minx, maxy - miny);
    }
    return bounds;
  }

  /**
   * {@inheritDoc}
   */
  public PathIterator getPathIterator() {
    return new CubicPathIterator(this);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true;
    }
    if ((obj != null) && (obj.getClass() == getClass())) {
      CubicCurve c = (CubicCurve)obj;
      return ((c.getX1() == getX1()) && (c.getY1() == getY1()) &&
              (c.getCtrlX1() == getCtrlX1()) &&
              (c.getCtrlY1() == getCtrlY1()) &&
              (c.getCtrlX2() == getCtrlX2()) &&
              (c.getCtrlY2() == getCtrlY2()) &&
              (c.getX2() == getX2()) && (c.getY2() == getY2()));
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    if (hashCode == 0) {
      int hashCode = 23;
      hashCode = 127 * hashCode + getX1();
      hashCode = 127 * hashCode + getY1();
      hashCode = 127 * hashCode + getCtrlX1();
      hashCode = 127 * hashCode + getCtrlY1();
      hashCode = 127 * hashCode + getCtrlX2();
      hashCode = 127 * hashCode + getCtrlY2();
      hashCode = 127 * hashCode + getX2();
      hashCode = 127 * hashCode + getY2();
      this.hashCode = hashCode;
    }
    return hashCode;
  }

  /**
   * {@inheritDoc}
   */
  public String toString() {
    return getClass().getName() + "[x1=" + getX1() + //$NON-NLS-1$
           ",y1=" + getY1() + ",ctrlx1=" +  //$NON-NLS-1$ //$NON-NLS-2$
           getCtrlX1() + ",ctrly1=" + getCtrlY1() + //$NON-NLS-1$
           ",ctrlx2=" + getCtrlX2() + ",ctrly2=" + //$NON-NLS-1$ //$NON-NLS-2$
           getCtrlY2() + ",x2=" + getX2() + ",y2=" + //$NON-NLS-1$ //$NON-NLS-2$
           getY2() + "]"; //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Protected Inner Class.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Cubic Path Iterator.
   */
  protected static class CubicPathIterator implements PathIterator {

    private final CubicCurve cubic;
    private int iteration;

    /**
     * Constructs new CubicPathIterator object.
     *
     * @param cubic the cubic curve
     */
    public CubicPathIterator(CubicCurve cubic) {
      this.cubic = cubic;
    }

    /**
     * {@inheritDoc}
     */
    public int getWindingRule() {
      return WIND_NON_ZERO;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isDone() {
      return (iteration > 1);
    }

    /**
     * {@inheritDoc}
     */
    public void next() {
      iteration++;
    }

    /**
     * {@inheritDoc}
     */
    public int currentSegment(int[] coords) {
      switch (iteration) {
        case 0:
          coords[0] = cubic.getX1();
          coords[1] = cubic.getY1();
          return SEG_MOVETO;
        case 1:
          coords[0] = cubic.getCtrlX1();
          coords[1] = cubic.getCtrlY1();
          coords[2] = cubic.getCtrlX2();
          coords[3] = cubic.getCtrlY2();
          coords[4] = cubic.getX2();
          coords[5] = cubic.getY2();
          return SEG_CUBICTO;
        default:
          throw new NoSuchElementException(
                      "Cubic Path Iterator out of bounds"); //$NON-NLS-1$
      }
    }

  }

/*^**************************************************************************^*/
/*- Private Static Methods.                                                  -*/
/*^**************************************************************************^*/

  private static int min4(int i1, int i2, int i3, int i4) {
    int min = i1;
    if (i2 < min) {
      min = i2;
    }
    if (i3 < min) {
      min = i3;
    }
    if (i4 < min) {
      min = i4;
    }
    return min;
  }

  private static int max4(int i1, int i2, int i3, int i4) {
    int max = i1;
    if (i2 > max) {
      max = i2;
    }
    if (i3 > max) {
      max = i3;
    }
    if (i4 > max) {
      max = i4;
    }
    return max;
  }

}

/*^*****************************************************************************
  File History:

 - 2007-10-01 13:43:39 Slobodan
   Initial version

*****************************************************************************^*/
