/*
 * @(#)Font.java  0.2.0 / 2007-09-07
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.graphics;

import yu.ac.bg.etf.javaqx.core.Disposable;
import yu.ac.bg.etf.javaqx.qswing.QSwing;
import yu.ac.bg.etf.javaqx.util.CommonConstants;

/**
 * Font Class.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class Font implements Disposable {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * The plain style constant.
   */
  public static final int PLAIN = 0;

  /**
   * The bold style constant.
   */
  public static final int BOLD = 1;

  /**
   * The italic style constant.
   */
  public static final int ITALIC = 2;

  /**
   * The underline style constant.
   */
  public static final int UNDERLINE = 4;

  /**
   * The overline style constant.
   */
  public static final int OVERLINE = 8;

  /**
   * The strikeout style constant.
   */
  public static final int STRIKEOUT = 16;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    QSwing.initialize();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private long _fontPointer;
  private long _fontMetricsPointer;
  private int fontMetricsReferences;
  private final Object lock;
  private final String name;
  private final int style;
  private final int size;
  private int hashCode; // NOPMD

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new Font object.
   *
   * @param name the font name
   * @param style the font style
   * @param size the font size
   */
  public Font(String name, int style, int size) {
    lock = new Object();
    if (name == null) {
      this.name = _getDefaultFontName();
    } else {
      this.name = name;
    }
    this.style = style;
    this.size = size;
  }

/*^**************************************************************************^*/
/*- Package-Private Constructor.                                             -*/
/*^**************************************************************************^*/

  Font(String name, int style, int size, long fontPointer) {
    lock = new Object();
    this.name = name;
    this.style = style;
    this.size = size;
    _fontPointer = fontPointer;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns font name.
   *
   * @return font name
   */
  public String getName() {
    return name;
  }

  /**
   * Returns font style.
   *
   * @return font style
   */
  public int getStyle() {
    return style;
  }

  /**
   * Returns font size.
   *
   * @return font size
   */
  public int getSize() {
    return size;
  }

  /**
   * Returns <tt>true</tt> if font is plane.
   *
   * @return <tt>true</tt> if font is plane
   */
  public boolean isPlain() {
    return (style == 0);
  }

  /**
   * Returns <tt>true</tt> if font is bold.
   *
   * @return <tt>true</tt> if font is bold
   */
  public boolean isBold() {
    return ((style & BOLD) != 0);
  }

  /**
   * Returns <tt>true</tt> if font is italic.
   *
   * @return <tt>true</tt> if font is italic
   */
  public boolean isItalic() {
    return ((style & ITALIC) != 0);
  }

  /**
   * Returns <tt>true</tt> if font is underline.
   *
   * @return <tt>true</tt> if font is underline
   */
  public boolean isUnderline() {
    return ((style & UNDERLINE) != 0);
  }

  /**
   * Returns <tt>true</tt> if font is overline.
   *
   * @return <tt>true</tt> if font is overline
   */
  public boolean isOverline() {
    return ((style & OVERLINE) != 0);
  }

  /**
   * Returns <tt>true</tt> if font is overline.
   *
   * @return <tt>true</tt> if font is overline
   */
  public boolean isStrikeOut() {
    return ((style & STRIKEOUT) != 0);
  }

  /**
   * Constructs new Font object using this font and specified style.
   *
   * @param style font style
   *
   * @return derived font
   */
  public Font deriveFont(int style) {
    return new Font(name, style, size);
  }

/*^**************************************************************************^*/
/*- Disposable Interface Implementation.                                     -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public final boolean isDisposed() {
    synchronized (lock) {
      return (_fontPointer != 0);
    }
  }

  /**
   * {@inheritDoc}
   */
  public final void dispose() {
    synchronized (lock) {
      if ((_fontPointer != 0) && (fontMetricsReferences == 0)) {
        _destroy(_fontPointer);
        _fontPointer = 0;
      }
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true;
    }
    if ((obj != null) && (obj.getClass() == getClass())) {
      Font font = (Font)obj;
      return (getName().equals(font.getName()) &&
              (getStyle() == font.getStyle()) && (getSize() == font.getSize()));
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    if (hashCode == 0) {
      int hashCode = 3;
      hashCode = 23 * hashCode + getName().hashCode();
      hashCode = 23 * hashCode + getStyle();
      hashCode = 23 * hashCode + getSize();
      this.hashCode = hashCode;
    }
    return hashCode;
  }

  /**
   * {@inheritDoc}
   */
  public String toString() {
    StringBuffer sb = new StringBuffer(getClass().getName());
    sb.append("[name=\""); //$NON-NLS-1$
    sb.append(getName());
    sb.append("\",style=["); //$NON-NLS-1$
    if (isPlain()) {
      sb.append("plain"); //$NON-NLS-1$
    } else {
      boolean appendComma = false;
      if (isBold()) {
        sb.append("bold"); //$NON-NLS-1$
        appendComma = true;
      }
      if (isItalic()) {
        if (appendComma) {
          sb.append(CommonConstants.STRING_COMMA);
        }
        sb.append("italic"); //$NON-NLS-1$
        appendComma = true;
      }
      if (isUnderline()) {
        if (appendComma) {
          sb.append(',');
        }
        sb.append("underline"); //$NON-NLS-1$
        appendComma = true;
      }
      if (isOverline()) {
        if (appendComma) {
          sb.append(CommonConstants.STRING_COMMA);
        }
        sb.append("overline"); //$NON-NLS-1$
        appendComma = true;
      }
      if (isStrikeOut()) {
        if (appendComma) {
          sb.append(CommonConstants.STRING_COMMA);
        }
        sb.append("strikeout"); //$NON-NLS-1$
      }
    }
    sb.append("],size="); //$NON-NLS-1$
    sb.append(getSize());
    sb.append(']');
    return sb.toString();
  }

  /**
   * {@inheritDoc}
   */
  protected final void finalize() {
    synchronized (lock) { // NOPMD
      if (_fontPointer != 0) {
        _destroy(_fontPointer);
        _fontPointer = 0;
      }
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  long acquireFontMetricsPointer() {
    synchronized (lock) {
      fontMetricsReferences++;
      if (_fontMetricsPointer == 0) {
        if (_fontPointer == 0) {
          _fontPointer = _create(name, style, size);
          if (_fontPointer == 0) {
            throw new OutOfMemoryError("Font creation failed"); //$NON-NLS-1$
          }
        }
        _fontMetricsPointer = _createFontMetrics(_fontPointer);
        if (_fontMetricsPointer == 0) {
          throw new OutOfMemoryError(
                                  "FontMetrics creation failed"); //$NON-NLS-1$
        }
      }
      return _fontMetricsPointer;
    }
  }

  void releaseFontMetricsPointer() {
    synchronized (lock) {
      fontMetricsReferences--;
      if ((fontMetricsReferences == 0) && (_fontMetricsPointer != 0)) {
        _destroyFontMetrics(_fontMetricsPointer);
        _fontMetricsPointer = 0;
      }
    }
  }

/*^**************************************************************************^*/
/*- Native Callback.                                                         -*/
/*^**************************************************************************^*/

  long _getFontPointer() { // synchronized by the native code
    if (_fontPointer == 0) {
      _fontPointer = _create(name, style, size);
      if (_fontPointer == 0) {
        throw new OutOfMemoryError("Font creation failed"); //$NON-NLS-1$
      }
    }
    return _fontPointer;
  }

/*^**************************************************************************^*/
/*- Package-Private Native Methods.                                          -*/
/*^**************************************************************************^*/

  native long _create(String name, int style, int size);

  native long _createFontMetrics(long fontPonter);

  native void _destroy(long fontPonter);

  native void _destroyFontMetrics(long fontMetricsPointer);

/*^**************************************************************************^*/
/*- Pacakge-Private Native Method.                                           -*/
/*^**************************************************************************^*/

  static native String _getDefaultFontName();

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 13:49:31 Slobodan
   Initial version

*****************************************************************************^*/
