/*
 * @(#)FontMetrics.java  0.2.0 / 2007-10-01
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.graphics;

import yu.ac.bg.etf.javaqx.core.Disposable;
import yu.ac.bg.etf.javaqx.qswing.QSwing;

/**
 * Font Metrics.
 *
 * @javaqx.native
 *    generate="header"
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public class FontMetrics implements Disposable {

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    QSwing.initialize();
    _initialize();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final Object lock;
  private boolean disposed;
  private final Font font;
  private long _fontMetricsPointer;

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  int ascent;
  int descent;
  int height;
  int leading;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new FontMetrics for the specified font.
   *
   * @param font the specified font
   *
   * @throws IllegalArgumentException if the font is <tt>null</tt>
   * @throws RuntimeException if native font metrics creation failed
   */
  public FontMetrics(Font font) {
    if (font == null) {
      throw new IllegalArgumentException("Null font"); //$NON-NLS-1$
    }
    lock = new Object();
    this.font = font;
    _fontMetricsPointer = font.acquireFontMetricsPointer();
    _getMetrics(_fontMetricsPointer); // NOPMD
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns font.
   *
   * @return font
   */
  public Font getFont() {
    return font;
  }

  /**
   * Returns width of the specified string.
   *
   * @param str the string
   *
   * @return width of the specified string
   */
  public int stringWidth(String str) {
    synchronized (lock) {
      if (disposed) {
        _fontMetricsPointer = font.acquireFontMetricsPointer();
        disposed = false;
      }
      return _stringWidth(_fontMetricsPointer, str);
    }
  }

  /**
   * Returns font ascent.
   *
   * @return font ascent
   */
  public int getAscent() {
    return ascent;
  }

  /**
   * Returns font descent.
   *
   * @return font descent
   */
  public int getDescent() {
    return descent;
  }

  /**
   * Returns font height.
   *
   * @return font height
   */
  public int getHeight() {
    return height;
  }

  /**
   * Returns font leading.
   *
   * @return font leading
   */
  public int getLeading() {
    return leading;
  }

/*^**************************************************************************^*/
/*- Disposable Interface Implementation.                                     -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public final boolean isDisposed() {
    synchronized (lock) {
      return disposed;
    }
  }

  /**
   * {@inheritDoc}
   */
  public final void dispose() {
    synchronized (lock) {
      if (!disposed) {
        font.releaseFontMetricsPointer();
        disposed = true;
      }
    }
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected final void finalize() {
    dispose(); // NOPMD
  }

/*^**************************************************************************^*/
/*- Package-Private Native Methods.                                          -*/
/*^**************************************************************************^*/

  native void _getMetrics(long fontMetricsPointer);

  native int _stringWidth(long fontMetricsPointer, String str);

/*^**************************************************************************^*/
/*- Private Static Native Method.                                            -*/
/*^**************************************************************************^*/

  private static native void _initialize();

}

/*^*****************************************************************************
  File History:

 - 2007-10-01 15:44:27 Slobodan
   Initial version

*****************************************************************************^*/
