/*
 * @(#)BoundedSize.java  0.2.0 / 2007-11-09
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout;

import yu.ac.bg.etf.javaqx.qswing.JQContainer;

import java.util.List;

/**
 * Describes sizes that provide lower and upper bounds as used by the JGoodies
 * FormLayout.
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.9 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class BoundedSize implements Size {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Holds the base size.
   */
  private final Size basis;

  /**
   * Holds an optional lower bound.
   */
  private final Size lowerBound;

  /**
   * Holds an optional upper bound.
   */
  private final Size upperBound;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs a BoundedSize for the given basis using the specified lower and
   * upper bounds.<p>
   *
   * @todo Consider throwing an IllegalArgumentException, if the lower bound
   *        and upper bound are both <tt>null</tt>.
   *
   * @param basis the base size
   * @param lowerBound the lower bound size
   * @param upperBound the upper bound size
   *
   * @throws IllegalArgumentException if the basis is null
   */
  public BoundedSize(Size basis, Size lowerBound, Size upperBound) {
    if (basis == null) {
      throw new IllegalArgumentException(
                "The basis of a bounded size must not be null."); //$NON-NLS-1$
    }
    this.basis = basis;
    this.lowerBound = lowerBound;
    this.upperBound = upperBound;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the base size, which is not-<code>null</code>.
   *
   * @return the base size
   */
  public Size getBasis() {
    return basis;
  }

  /**
   * Returns the optional lower bound.
   *
   * @return the optional lower bound
   */
  public Size getLowerBound() {
    return lowerBound;
  }

  /**
   * Returns the optional upper bound.
   *
   * @return the optional upper bound
   */
  public Size getUpperBound() {
    return upperBound;
  }

/*^**************************************************************************^*/
/*- Size Interface Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * Returns this size as pixel size. Neither requires the component list nor
   * the specified measures. Honors the lower and upper bound.<p>
   *
   * Invoked by <code>FormSpec</code> to determine the size of a column or row.
   *
   * @param container the layout container
   * @param components the list of components to measure
   * @param minMeasure the measure used to determine the minimum size
   * @param prefMeasure the measure used to determine the preferred size
   * @param defaultMeasure  the measure used to determine the default size
   *
   * @return the maximum size in pixels
   */
  public int maximumSize(JQContainer container, List components,
                         FormLayout.Measure minMeasure,
                         FormLayout.Measure prefMeasure,
                         FormLayout.Measure defaultMeasure) {
    int size = basis.maximumSize(container, components, minMeasure, prefMeasure,
                                 defaultMeasure);
    if (lowerBound != null) {
      size = Math.max(size, lowerBound.maximumSize(container, components,
                                                   minMeasure, prefMeasure,
                                                   defaultMeasure));
    }
    if (upperBound != null) {
      size = Math.min(size, upperBound.maximumSize(container, components,
                                                   minMeasure, prefMeasure,
                                                   defaultMeasure));
    }
    return size;
  }

  /**
   * Describes if this Size can be compressed, if container space gets scarce.
   * Used by the FormLayout size computations in <code>#compressedSizes</code>
   * to check whether a column or row can be compressed or not.<p>
   *
   * BoundedSizes are compressible if the base Size is compressible.
   *
   * @return <code>true</code> if and only if the basis is compressible
   */
  public boolean compressible() {
    return getBasis().compressible();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Indicates whether some other BoundedSize is "equal to" this one.
   *
   * @param object the object with which to compare
   *
   * @return <tt>true</tt> if this object is the same as the object argument,
   *         <tt>false</tt> otherwise.
   */
  public boolean equals(Object object) {
    if (this == object) {
      return true;
    }
    if (!(object instanceof BoundedSize)) {
      return false;
    }
    BoundedSize size = (BoundedSize) object;
    return basis.equals(size.basis) &&
           (((lowerBound == null) && (size.lowerBound == null)) ||
            ((lowerBound != null) && lowerBound.equals(size.lowerBound))) &&
           (((upperBound == null) && (size.upperBound == null)) ||
            (upperBound != null && upperBound.equals(size.upperBound)));
  }

  /**
   * Returns a hash code value for the object. This method is supported for the
   * benefit of hashtables such as those provided by
   * <tt>java.util.Hashtable</tt>.
   *
   * @return  a hash code value for this object.
   */
  public int hashCode() {
    int hashValue = basis.hashCode();
    if (lowerBound != null) {
      hashValue = hashValue * 37 + lowerBound.hashCode();
    }
    if (upperBound != null) {
      hashValue = hashValue * 37 + upperBound.hashCode();
    }
    return hashValue;
  }

  /**
   * Returns a string representation of this size object.
   *
   * <strong>Note:</strong> The string representation may change at any time.
   * It is strongly recommended to not use this string for parsing purposes.
   *
   * @return  a string representation of the constant size
   */
  public String toString() {
    if (lowerBound != null) { // NOPMD
      if (upperBound == null) {
        return "max(" + basis + ';' + lowerBound + ')'; //$NON-NLS-1$
      }
      return "max(" + lowerBound + ";min(" + //$NON-NLS-1$ //$NON-NLS-2$
             basis + ';' + upperBound + "))"; //$NON-NLS-1$
    } else if (upperBound != null) { // NOPMD
      return "min(" + basis + ';' + upperBound + ')'; //$NON-NLS-1$
    } else {
      return "bounded(" +  basis + ')'; //$NON-NLS-1$
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-11-09 15:21:33 Slobodan
   QSwing port

*****************************************************************************^*/
