/*
 * @(#)CellConstraints.java  0.1.0 / 2007-11-09
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout;

import java.util.Locale;
import java.util.StringTokenizer;

import yu.ac.bg.etf.javaqx.qswing.JQComponent;
import yu.ac.bg.etf.javaqx.qswing.graphics.Insets;
import yu.ac.bg.etf.javaqx.qswing.graphics.Rectangle;

/**
 * Defines constraints for components that are layed out with the FormLayout.
 * Defines the components display area: grid&nbsp;x, grid&nbsp;y,
 * grid width (column span), grid height (row span), horizontal alignment
 * and vertical alignment.<p>
 *
 * Most methods return <em>this</em> object to enable method chaining.<p>
 *
 * You can set optional insets in a constructor. This is useful if you
 * need to use a pixel-size insets to align perceived component bounds
 * with pixel data, for example an icon. Anyway, this is rarely used.
 * The insets don't affect the size computation for columns and rows.
 * I consider renaming the insets to offsets to better indicate the
 * motivation for this option.<p>
 *
 * <strong>Examples</strong>:<br>
 * The following cell constraints locate a component in the third
 * column of the fifth row; column and row span are 1; the component
 * will be aligned with the column's right-hand side and the row's
 * bottom.
 * <pre>
 * CellConstraints cc = new CellConstraints();
 * cc.xy  (3, 5);
 * cc.xy  (3, 5, CellConstraints.RIGHT, CellConstraints.BOTTOM);
 * cc.xy  (3, 5, "right, bottom");
 *
 * cc.xyw (3, 5, 1);
 * cc.xyw (3, 5, 1, CellConstraints.RIGHT, CellConstraints.BOTTOM);
 * cc.xyw (3, 5, 1, "right, bottom");
 *
 * cc.xywh(3, 5, 1, 1);
 * cc.xywh(3, 5, 1, 1, CellConstraints.RIGHT, CellConstraints.BOTTOM);
 * cc.xywh(3, 5, 1, 1, "right, bottom");
 * </pre>
 * See also the examples in the {@link FormLayout} class comment.<p>
 *
 * <strong>Note:</strong> The new method sets <code>#rc</code>,
 * <code>#rcw</code>, and <code>#rchw</code> are experimental and
 * maybe removed from the 1.1 final.<p>
 *
 * @todo Remove the above comment in the 1.1 final.<p>
 *
 * @todo Explain in the JavaDocs that the insets are actually offsets.
 * And describe that these offsets are not taken into account when
 * FormLayout computes the column and row sizes.<p>
 *
 * @todo Rename the inset to offsets.<p>
 *
 * @todo In the Forms 1.0.x invisible components are not taken into account
 * when the FormLayout lays out the container. Add an optional setting for
 * this on both the container-level and component-level. So one can specify
 * that invisible components shall be taken into account, but may exclude
 * individual components. Or the other way round, exclude invisible components,
 * and include individual components. The API of both the FormLayout and
 * CellConstraints classes shall be extended to support this option.
 * This feature is planned for the Forms version 1.1 and is described in
 * <a href="https://forms.dev.java.net/issues/show_bug.cgi?id=28">issue #28</a>
 * of the Forms' issue tracker where you can track the progress.
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.4 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class CellConstraints implements Cloneable {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Use the column's or row's default alignment.
   */
  public static final Alignment DEFAULT = new Alignment("default", //$NON-NLS-1$
                                                        Alignment.BOTH);

  /**
   * Fill the cell either horizontally or vertically.
   */
  public static final Alignment FILL = new Alignment("fill", //$NON-NLS-1$
                                                     Alignment.BOTH);

  /**
   * Put the component in the left.
   */
  public static final Alignment LEFT = new Alignment("left",  //$NON-NLS-1$
                                                     Alignment.HORIZONTAL);

  /**
   * Put the component in the right.
   */
  public static final Alignment RIGHT = new Alignment("right",  //$NON-NLS-1$
                                                      Alignment.HORIZONTAL);

  /**
   * Put the component in the center.
   */
  public static final Alignment CENTER = new Alignment("center",  //$NON-NLS-1$
                                                       Alignment.BOTH);

  /**
   * Put the component in the top.
   */
  public static final Alignment TOP = new Alignment("top",  //$NON-NLS-1$
                                                    Alignment.VERTICAL);

  /**
   * Put the component in the bottom.
   */
  public static final Alignment BOTTOM = new Alignment("bottom",  //$NON-NLS-1$
                                                       Alignment.VERTICAL);

/*^**************************************************************************^*/
/*- Public Fields.                                                           -*/
/*^**************************************************************************^*/

// CS:OFF
  /**
   * Describes the component's horizontal grid origin (starts at 1).
   */
  public int gridX;

  /**
   * Describes the component's vertical grid origin (starts at 1).
   */
  public int gridY;

  /**
   * Describes the component's horizontal grid extend (number of cells).
   */
  public int gridWidth;

  /**
   * Describes the component's vertical grid extent (number of cells).
   */
  public int gridHeight;

  /**
   * Describes the component's horizontal alignment.
   */
  public Alignment hAlign;

  /**
   * Describes the component's vertical alignment.
   */
  public Alignment vAlign;

  /**
   * Describes the component's <code>Insets</code> in it's display area.
   */
  public Insets insets;
// CS:ON

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs a default instance of <code>CellConstraints</code>.
   */
  public CellConstraints() {
    this(1, 1);
  }

  /**
   * Constructs an instance of <code>CellConstraints</code> for the given
   * cell position.<p>
   *
   * <strong>Examples:</strong><pre>
   * new CellConstraints(1, 3);
   * new CellConstraints(1, 3);
   * </pre>
   *
   * @param gridX the component's horizontal grid origin
   * @param gridY the component's vertical grid origin
   */
  public CellConstraints(int gridX, int gridY) {
    this(gridX, gridY, 1, 1);
  }

  /**
   * Constructs an instance of <code>CellConstraints</code> for the given cell
   * position, anchor, and fill.<p>
   *
   * <strong>Examples:</strong><pre>
   * new CellConstraints(1, 3, CellConstraints.LEFT,   CellConstraints.BOTTOM);
   * new CellConstraints(1, 3, CellConstraints.CENTER, CellConstraints.FILL);
   * </pre>
   *
   * @param gridX the component's horizontal grid origin
   * @param gridY the component's vertical grid origin
   * @param hAlign the component's horizontal alignment
   * @param vAlign the component's vertical alignment
   */
  public CellConstraints(int gridX, int gridY, Alignment hAlign,
                         Alignment vAlign) {
    this(gridX, gridY, 1, 1, hAlign, vAlign, Insets.ZERO_INSEST);
  }

  /**
   * Constructs an instance of <code>CellConstraints</code> for the given cell
   * position and size.<p>
   *
   * <strong>Examples:</strong><pre>
   * new CellConstraints(1, 3, 2, 1);
   * new CellConstraints(1, 3, 7, 3);
   * </pre>
   *
   * @param gridX the component's horizontal grid origin
   * @param gridY the component's vertical grid origin
   * @param gridWidth the component's horizontal extent
   * @param gridHeight the component's vertical extent
   */
  public CellConstraints(int gridX, int gridY, int gridWidth, int gridHeight) {
    this(gridX, gridY, gridWidth, gridHeight, DEFAULT, DEFAULT);
  }

  /**
   * Constructs an instance of <code>CellConstraints</code> for the given
   * cell position and size, anchor, and fill.<p>
   *
   * <strong>Examples:</strong><pre>
   * new CellConstraints(1, 3, 2, 1, CellConstraints.LEFT,
   *                     CellConstraints.BOTTOM);
   * new CellConstraints(1, 3, 7, 3, CellConstraints.CENTER,
   *                     CellConstraints.FILL);
   * </pre>
   *
   * @param gridX the component's horizontal grid origin
   * @param gridY the component's vertical grid origin
   * @param gridWidth the component's horizontal extent
   * @param gridHeight the component's vertical extent
   * @param hAlign the component's horizontal alignment
   * @param vAlign the component's vertical alignment
   */
  public CellConstraints(int gridX, int gridY, int gridWidth, int gridHeight,
                          Alignment hAlign, Alignment vAlign) {
    this(gridX, gridY, gridWidth, gridHeight, hAlign, vAlign,
         Insets.ZERO_INSEST);
  }

  /**
   * Constructs an instance of <code>CellConstraints</code> for the complete set
   * of available properties.<p>
   *
   * <strong>Examples:</strong><pre>
   * new CellConstraints(1, 3, 2, 1, CellConstraints.LEFT,
   *                     CellConstraints.BOTTOM, new Insets(0, 1, 0, 3));
   * new CellConstraints(1, 3, 7, 3, CellConstraints.CENTER,
   *                     CellConstraints.FILL, new Insets(0, 1, 0, 0));
   * </pre>
   *
   * @param gridX the component's horizontal grid origin
   * @param gridY the component's vertical grid origin
   * @param gridWidth the component's horizontal extent
   * @param gridHeight the component's vertical extent
   * @param hAlign the component's horizontal alignment
   * @param vAlign the component's vertical alignment
   * @param insets the component's display area <code>Insets</code>
   *
   * @throws IndexOutOfBoundsException if the grid origin or extent is negative
   * @throws IllegalArgumentException if the horizontal or vertical alignment is
   *                                   <tt>null</tt> or invalid
   */
  public CellConstraints(int gridX, int gridY, int gridWidth, int gridHeight,
                         Alignment hAlign, Alignment vAlign, Insets insets) {
    if (gridX <= 0) {
      throw new IndexOutOfBoundsException(
          "The grid x must be a positive number."); //$NON-NLS-1$
    }
    if (gridY <= 0) {
      throw new IndexOutOfBoundsException(
          "The grid y must be a positive number."); //$NON-NLS-1$
    }
    if (gridWidth <= 0) {
      throw new IndexOutOfBoundsException(
          "The grid width must be a positive number."); //$NON-NLS-1$
    }
    if (gridHeight <= 0) {
      throw new IndexOutOfBoundsException(
          "The grid height must be a positive number."); //$NON-NLS-1$
    }
    if (hAlign == null) {
      throw new IllegalArgumentException(
          "The horizontal alignment must not be null."); //$NON-NLS-1$
    }
    if (vAlign == null) {
      throw new IllegalArgumentException(
          "The vertical alignment must not be null."); //$NON-NLS-1$
    }
    this.gridX = gridX;
    this.gridY = gridY;
    this.gridWidth = gridWidth;
    this.gridHeight = gridHeight;
    this.hAlign = hAlign;
    this.vAlign = vAlign;
    this.insets = insets;
    ensureValidOrientations(hAlign, vAlign);
  }

  /**
   * Constructs an instance of <code>CellConstraints</code> from the given
   * encoded string properties.<p>
   *
   * <strong>Examples:</strong><pre>
   * new CellConstraints("1, 3");
   * new CellConstraints("1, 3, left, bottom");
   * new CellConstraints("1, 3, 2, 1, left, bottom");
   * new CellConstraints("1, 3, 2, 1, l, b");
   * </pre>
   *
   * @param encodedConstraints the constraints encoded as string
   */
  public CellConstraints(String encodedConstraints) {
    initFromConstraints(encodedConstraints);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Sets column and row origins; sets width and height to 1; uses the default
   * alignments.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xy(1, 1);
   * cc.xy(1, 3);
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   *
   * @return this
   */
  public CellConstraints xy(int col, int row) { // NOPMD
    return xywh(col, row, 1, 1);
  }

  /**
   * Sets column and row origins; sets width and height to 1; decodes horizontal
   * and vertical alignments from the given string.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xy(1, 3, "left, bottom");
   * cc.xy(1, 3, "l, b");
   * cc.xy(1, 3, "center, fill");
   * cc.xy(1, 3, "c, f");
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   * @param encodedAlignments describes the horizontal and vertical alignments
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints xy(int col, int row, // NOPMD
                            String encodedAlignments) {
    return xywh(col, row, 1, 1, encodedAlignments);
  }

  /**
   * Sets the column and row origins; sets width and height to 1; set horizontal
   * and vertical alignment using the specified objects.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xy(1, 3, CellConstraints.LEFT, CellConstraints.BOTTOM);
   * cc.xy(1, 3, CellConstraints.CENTER, CellConstraints.FILL);
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   * @param colAlign horizontal component alignment
   * @param rowAlign vertical component alignment
   *
   * @return this
   */
  public CellConstraints xy(int col, int row, Alignment colAlign, // NOPMD
                            Alignment rowAlign) {
    return xywh(col, row, 1, 1, colAlign, rowAlign);
  }

  /**
   * Sets the column, row, width, and height; uses a height (row span) of 1 and
   * the horizontal and vertical default alignments.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xyw(1, 3, 7);
   * cc.xyw(1, 3, 2);
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   * @param colSpan the column span or grid width
   *
   * @return this
   */
  public CellConstraints xyw(int col, int row, int colSpan) {
    return xywh(col, row, colSpan, 1, DEFAULT, DEFAULT);
  }

  /**
   * Sets the column, row, width, and height; decodes the horizontal and
   * vertical alignments from the given string. The row span (height) is set to
   * 1.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xyw(1, 3, 7, "left, bottom");
   * cc.xyw(1, 3, 7, "l, b");
   * cc.xyw(1, 3, 2, "center, fill");
   * cc.xyw(1, 3, 2, "c, f");
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   * @param colSpan the column span or grid width
   * @param encodedAlignments describes the horizontal and vertical alignments
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints xyw(int col, int row, int colSpan,
                             String encodedAlignments) {
    return xywh(col, row, colSpan, 1, encodedAlignments);
  }

  /**
   * Sets the column, row, width, and height; sets the horizontal and vertical
   * alignment using the specified alignment objects. The row span (height) is
   * set to 1.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xyw(1, 3, 2, CellConstraints.LEFT,   CellConstraints.BOTTOM);
   * cc.xyw(1, 3, 7, CellConstraints.CENTER, CellConstraints.FILL);
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   * @param colSpan the column span or grid width
   * @param colAlign horizontal component alignment
   * @param rowAlign vertical component alignment
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints xyw(int col, int row, int colSpan, Alignment colAlign,
                             Alignment rowAlign) {
    return xywh(col, row, colSpan, 1, colAlign, rowAlign);
  }

  /**
   * Sets the column, row, width, and height; uses a width (column span) of 1
   * and the horizontal and vertical default alignments.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xyh(1, 3, 7);
   * cc.xyh(1, 3, 2);
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   * @param rowSpan the row span or grid height
   *
   * @return this
   */
  public CellConstraints xyh(int col, int row, int rowSpan) {
    return xywh(col, row, 1, rowSpan, DEFAULT, DEFAULT);
  }

  /**
   * Sets the column, row, width, and height; decodes the horizontal and
   * vertical alignments from the given string. The column span (width) is set
   * to 1.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xyh(1, 3, 7, "left, bottom");
   * cc.xyh(1, 3, 7, "l, b");
   * cc.xyh(1, 3, 2, "center, fill");
   * cc.xyh(1, 3, 2, "c, f");
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   * @param rowSpan the row span or grid height
   * @param encodedAlignments describes the horizontal and vertical alignments
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints xyh(int col, int row, int rowSpan,
                             String encodedAlignments) {
    return xywh(col, row, 1, rowSpan, encodedAlignments);
  }

  /**
   * Sets the column, row, width, and height; sets the horizontal and vertical
   * alignment using the specified alignment objects. The column span (width) is
   * set to 1.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xyh(1, 3, 2, CellConstraints.LEFT,   CellConstraints.BOTTOM);
   * cc.xyh(1, 3, 7, CellConstraints.CENTER, CellConstraints.FILL);
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   * @param rowSpan the row span or grid height
   * @param colAlign horizontal component alignment
   * @param rowAlign vertical component alignment
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints xyh(int col, int row, int rowSpan, Alignment colAlign,
                             Alignment rowAlign) {
    return xywh(col, row, 1, rowSpan, colAlign, rowAlign);
  }

  /**
   * Sets the column, row, width, and height; uses default alignments.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xywh(1, 3, 2, 1);
   * cc.xywh(1, 3, 7, 3);
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   * @param colSpan the column span or grid width
   * @param rowSpan the row span or grid height
   *
   * @return this
   */
  public CellConstraints xywh(int col, int row, int colSpan, int rowSpan) {
    return xywh(col, row, colSpan, rowSpan, DEFAULT, DEFAULT);
  }

  /**
   * Sets the column, row, width, and height;
   * decodes the horizontal and vertical alignments from the given string.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xywh(1, 3, 2, 1, "left, bottom");
   * cc.xywh(1, 3, 2, 1, "l, b");
   * cc.xywh(1, 3, 7, 3, "center, fill");
   * cc.xywh(1, 3, 7, 3, "c, f");
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   * @param colSpan the column span or grid width
   * @param rowSpan the row span or grid height
   * @param encodedAlignments describes the horizontal and vertical alignments
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints xywh(int col, int row, int colSpan, int rowSpan,
                              String encodedAlignments) {
    CellConstraints result = xywh(col, row, colSpan, rowSpan);
    result.setAlignments(encodedAlignments, true);
    return result;
  }

  /**
   * Sets the column, row, width, and height; sets the horizontal and vertical
   * alignment using the specified alignment objects.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.xywh(1, 3, 2, 1, CellConstraints.LEFT, CellConstraints.BOTTOM);
   * cc.xywh(1, 3, 7, 3, CellConstraints.CENTER, CellConstraints.FILL);
   * </pre>
   *
   * @param col the new column index
   * @param row the new row index
   * @param colSpan the column span or grid width
   * @param rowSpan the row span or grid height
   * @param colAlign horizontal component alignment
   * @param rowAlign vertical component alignment
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints xywh(int col, int row, int colSpan, int rowSpan,
                              Alignment colAlign, Alignment rowAlign) {
    this.gridX = col;
    this.gridY = row;
    this.gridWidth = colSpan;
    this.gridHeight = rowSpan;
    this.hAlign = colAlign;
    this.vAlign = rowAlign;
    ensureValidOrientations(hAlign, vAlign);
    return this;
  }

  /**
   * Sets row and column origins; sets height and width to 1;
   * uses the default alignments.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.rc(1, 1);
   * cc.rc(3, 1);
   * </pre>
   *
   * @param row the new row index
   * @param col the new column index
   *
   * @return this
   */
  public CellConstraints rc(int row, int col) { // NOPMD
    return rchw(row, col, 1, 1);
  }

  /**
   * Sets row and column origins; sets height and width to 1; decodes vertical
   * and horizontal alignments from the given string.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.rc(3, 1, "bottom, left");
   * cc.rc(3, 1, "b, l");
   * cc.rc(3, 1, "fill, center");
   * cc.rc(3, 1, "f, c");
   * </pre>
   *
   * @param row the new row index
   * @param col the new column index
   * @param encodedAlignments  describes the vertical and horizontal alignments
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints rc(int row, int col, // NOPMD
                            String encodedAlignments) {
    return rchw(row, col, 1, 1, encodedAlignments);
  }

  /**
   * Sets the row and column origins; sets width and height to 1;
   * set horizontal and vertical alignment using the specified objects.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.rc(3, 1, CellConstraints.BOTTOM, CellConstraints.LEFT);
   * cc.rc(3, 1, CellConstraints.FILL,   CellConstraints.CENTER);
   * </pre>
   *
   * @param row the new row index
   * @param col the new column index
   * @param rowAlign vertical component alignment
   * @param colAlign horizontal component alignment
   *
   * @return this
   */
  public CellConstraints rc(int row, int col, Alignment rowAlign, // NOPMD
                            Alignment colAlign) {
    return rchw(row, col, 1, 1, rowAlign, colAlign);
  }

  /**
   * Sets the row, column, height, and width; uses a height (row span) of 1 and
   * the vertical and horizontal default alignments.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.rcw(3, 1, 7);
   * cc.rcw(3, 1, 2);
   * </pre>
   *
   * @param row the new row index
   * @param col the new column index
   * @param colSpan the column span or grid width
   *
   * @return this
   */
  public CellConstraints rcw(int row, int col, int colSpan) {
    return rchw(row, col, 1, colSpan, DEFAULT, DEFAULT);
  }

  /**
   * Sets the row, column, height, and width; decodes the vertical and
   * horizontal alignments from the given string. The row span (height) is set
   * to 1.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.rcw(3, 1, 7, "bottom, left");
   * cc.rcw(3, 1, 7, "b, l");
   * cc.rcw(3, 1, 2, "fill, center");
   * cc.rcw(3, 1, 2, "f, c");
   * </pre>
   *
   * @param row the new row index
   * @param col the new column index
   * @param colSpan the column span or grid width
   * @param encodedAlignments describes the vertical and horizontal alignments
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints rcw(int row, int col, int colSpan,
                             String encodedAlignments) {
    return rchw(row, col, 1, colSpan, encodedAlignments);
  }

  /**
   * Sets the row, column, height, and width; sets the vertical and
   * horizontalalignment using the specified alignment objects. The row span
   * (height) is set to 1.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.rcw(3, 1, 2, CellConstraints.BOTTOM, CellConstraints.LEFT);
   * cc.rcw(3, 1, 7, CellConstraints.FILL,   CellConstraints.CENTER);
   * </pre>
   *
   * @param row the new row index
   * @param col the new column index
   * @param colSpan the column span or grid width
   * @param rowAlign vertical component alignment
   * @param colAlign horizontal component alignment
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints rcw(int row, int col, int colSpan, Alignment rowAlign,
                             Alignment colAlign) {
    return rchw(row, col, 1, colSpan, rowAlign, colAlign);
  }

  /**
   * Sets the row, column, height, and width; uses default alignments.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.rchw(1, 3, 2, 1);
   * cc.rchw(1, 3, 7, 3);
   * </pre>
   *
   * @param row the new row index
   * @param col the new column index
   * @param rowSpan the row span or grid height
   * @param colSpan the column span or grid width
   *
   * @return this
   */
  public CellConstraints rchw(int row, int col, int rowSpan, int colSpan) {
      return rchw(row, col, rowSpan, colSpan, DEFAULT, DEFAULT);
  }

  /**
   * Sets the row, column, height, and width; decodes the vertical and
   * horizontal alignments from the given string.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.rchw(3, 1, 1, 2, "bottom, left");
   * cc.rchw(3, 1, 1, 2, "b, l");
   * cc.rchw(3, 1, 3, 7, "fill, center");
   * cc.rchw(3, 1, 3, 7, "f, c");
   * </pre>
   *
   * @param row the new row index
   * @param col the new column index
   * @param rowSpan the row span or grid height
   * @param colSpan the column span or grid width
   * @param encodedAlignments describes the vertical and horizontal alignments
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints rchw(int row, int col, int rowSpan, int colSpan,
                              String encodedAlignments) {
    CellConstraints result = rchw(row, col, rowSpan, colSpan);
    result.setAlignments(encodedAlignments, false);
    return result;
  }

  /**
   * Sets the row, column, height, and width; sets the vertical and horizontal
   * alignment using the specified alignment objects.<p>
   *
   * <strong>Examples:</strong><pre>
   * cc.rchw(3, 1, 1, 2, CellConstraints.BOTTOM, CellConstraints.LEFT);
   * cc.rchw(3, 1, 3, 7, CellConstraints.FILL,   CellConstraints.CENTER);
   * </pre>
   *
   * @param row the new row index
   * @param col the new column index
   * @param rowSpan the row span or grid height
   * @param colSpan the column span or grid width
   * @param rowAlign vertical component alignment
   * @param colAlign horizontal component alignment
   *
   * @return this
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  public CellConstraints rchw(int row, int col, int rowSpan, int colSpan,
                               Alignment rowAlign, Alignment colAlign) {
    return xywh(col, row, colSpan, rowSpan, colAlign, rowAlign);
  }

  /**
   * Returns a short string representation of this constraints object.
   *
   * @return a short string representation of this constraints object
   */
  public String toShortString() {
    return toShortString(null);
  }

  /**
   * Returns a short string representation of this constraints object. This
   * method can use the given <tt>FormLayout</tt> to display extra information
   * how default alignments are mapped to concrete alignments. Therefore it asks
   * the related column and row as specified by this constraints object.
   *
   * @param layout  the layout to be presented as a string
   *
   * @return a short string representation of this constraints object
   */
  public String toShortString(FormLayout layout) {
    StringBuffer buffer = new StringBuffer(80);
    buffer.append('(');
    buffer.append(formatInt(gridX));
    buffer.append(", "); //$NON-NLS-1$
    buffer.append(formatInt(gridY));
    buffer.append(", "); //$NON-NLS-1$
    buffer.append(formatInt(gridWidth));
    buffer.append(", "); //$NON-NLS-1$
    buffer.append(formatInt(gridHeight));
    buffer.append(", \""); //$NON-NLS-1$
    buffer.append(hAlign.abbreviation());
    if ((hAlign == DEFAULT) && (layout != null)) { // NOPMD
      buffer.append('=');
      if (gridWidth == 1) {
        buffer.append(concreteAlignment(hAlign,
                                        layout.getColumnSpec(gridX)).
                                                                abbreviation());
      } else {
        buffer.append(concreteAlignment(hAlign, null).abbreviation());
      }
    }
    buffer.append(", "); //$NON-NLS-1$
    buffer.append(vAlign.abbreviation());
    if (vAlign == DEFAULT && layout != null) { // NOPMD
      buffer.append('=');
      if (gridHeight == 1) {
        buffer.append(concreteAlignment(vAlign,
                                        layout.getRowSpec(gridY)).
                                                               abbreviation());
      } else {
        buffer.append(concreteAlignment(vAlign, null).abbreviation());
      }
    }
    buffer.append('"');
    if (!(Insets.ZERO_INSEST.equals(insets))) {
      buffer.append(", "); //$NON-NLS-1$
      buffer.append(insets);
    }
    buffer.append(')');
    return buffer.toString();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Creates a copy of this cell constraints object.
   *
   * @return a copy of this cell constraints object
   */
  public Object clone() {
    try {
      return super.clone();
    } catch (CloneNotSupportedException e) {
      // This shouldn't happen, since we are Cloneable.
      throw new InternalError(); // NOPMD
    }
  }

  /**
   * Constructs and returns a string representation of this constraints object.
   *
   * @return string representation of this constraints object
   */
  public String toString() {
    StringBuffer buffer = new StringBuffer(80);
    buffer.append("CellConstraints[x="); //$NON-NLS-1$
    buffer.append(gridX);
    buffer.append("; y="); //$NON-NLS-1$
    buffer.append(gridY);
    buffer.append("; w="); //$NON-NLS-1$
    buffer.append(gridWidth);
    buffer.append("; h="); //$NON-NLS-1$
    buffer.append(gridHeight);
    buffer.append("; hAlign="); //$NON-NLS-1$
    buffer.append(hAlign);
    buffer.append("; vAlign="); //$NON-NLS-1$
    buffer.append(vAlign);
    if (!(Insets.ZERO_INSEST.equals(insets))) {
      buffer.append("; insets="); //$NON-NLS-1$
      buffer.append(insets);
    }
    buffer.append(']');
    return buffer.toString();
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Checks and verifies that this constraints object has valid grid index
   * values, i. e. the display area cells are inside the form's grid.
   *
   * @param colCount number of columns in the grid
   * @param rowCount number of rows in the grid
   *
   * @throws IndexOutOfBoundsException if the display area described by this
   *         constraints object is not inside the grid
   */
  void ensureValidGridBounds(int colCount, int rowCount) {
    if (gridX <= 0) {
      throw new IndexOutOfBoundsException(
                                    "The column index " + gridX + //$NON-NLS-1$
                                    " must be positive."); //$NON-NLS-1$
    }
    if (gridX > colCount) {
      throw new IndexOutOfBoundsException(
             "The column index " + gridX + //$NON-NLS-1$
             " must be less than or equal to " + //$NON-NLS-1$ NOPMD
             colCount + '.');
    }
    if (gridX + gridWidth - 1 > colCount) {
      throw new IndexOutOfBoundsException(
                              "The grid width " + gridWidth + //$NON-NLS-1$
                              " must be less than or equal to " + //$NON-NLS-1$
                              (colCount - gridX + 1) + '.');
    }
    if (gridY <= 0) {
      throw new IndexOutOfBoundsException(
                                       "The row index " + gridY + //$NON-NLS-1$
                                       " must be positive."); //$NON-NLS-1$
    }
    if (gridY > rowCount) {
      throw new IndexOutOfBoundsException(
             "The row index " + gridY + //$NON-NLS-1$
             " must be less than or equal to " + rowCount + '.'); //$NON-NLS-1$
    }
    if ((gridY + gridHeight - 1) > rowCount) {
      throw new IndexOutOfBoundsException(
                              "The grid height " + gridHeight + //$NON-NLS-1$
                              " must be less than or equal to " + //$NON-NLS-1$
                              (rowCount - gridY + 1) + '.');
    }
  }

  /**
   * Sets the component's bounds using the given component and cell bounds.
   *
   * @param c the component to set bounds
   * @param layout the FormLayout instance that computes the bounds
   * @param cellBounds the cell's bounds
   * @param minWidthMeasure measures the minimum width
   * @param minHeightMeasure measures the minimum height
   * @param prefWidthMeasure measures the preferred width
   * @param prefHeightMeasure measures the preferred height
   */
  void setBounds(JQComponent c, FormLayout layout, Rectangle cellBounds,
                 FormLayout.Measure minWidthMeasure,
                 FormLayout.Measure minHeightMeasure,
                 FormLayout.Measure prefWidthMeasure,
                 FormLayout.Measure prefHeightMeasure) {
    ColumnSpec colSpec;
    if (gridWidth == 1) {
      colSpec = layout.getColumnSpec(gridX);
    } else {
      colSpec = null;
    }
    RowSpec rowSpec;
    if (gridHeight == 1) {
      rowSpec = layout.getRowSpec(gridY);
    } else {
      rowSpec = null;
    }
    Alignment concreteHAlign = concreteAlignment(this.hAlign, colSpec);
    Alignment concreteVAlign = concreteAlignment(this.vAlign, rowSpec);
    Insets concreteInsets;
    if (this.insets == null) {
      concreteInsets = Insets.ZERO_INSEST;
    } else {
      concreteInsets = this.insets;
    }
    int cellX = cellBounds.getX() + concreteInsets.getLeft();
    int cellY = cellBounds.getY() + concreteInsets.getTop();
    int cellW = cellBounds.getWidth()  - concreteInsets.getLeft() -
                concreteInsets.getRight();
    int cellH = cellBounds.getHeight() - concreteInsets.getTop() -
                concreteInsets.getBottom();
    int compW = componentSize(c, colSpec, cellW, minWidthMeasure,
                              prefWidthMeasure);
    int compH = componentSize(c, rowSpec, cellH, minHeightMeasure,
                              prefHeightMeasure);
    int x = origin(concreteHAlign, cellX, cellW, compW);
    int y = origin(concreteVAlign, cellY, cellH, compH);
    int w = extent(concreteHAlign, cellW, compW);
    int h = extent(concreteVAlign, cellH, compH);
    c.setBounds(x, y, w, h);
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  /**
   * Decodes and returns the grid bounds and alignments for this constraints as
   * an array of six integers. The string representation is a comma separated
   * sequence, one of
   * <pre>
   * "x, y"
   * "x, y, w, h"
   * "x, y, hAlign, vAlign"
   * "x, y, w, h, hAlign, vAlign"
   * </pre>
   *
   * @param encodedConstraints represents horizontal and vertical alignment
   *
   * @throws IllegalArgumentException if the encoded constraints do not follow
   *         the constraint syntax
   */
  private void initFromConstraints(String encodedConstraints) { // NOPMD
    StringTokenizer tokenizer = new StringTokenizer(encodedConstraints, " ,");
    int argCount = tokenizer.countTokens();
    if ((argCount != 2) && (argCount != 4) && (argCount != 6)) {
      throw new IllegalArgumentException(
                        "You must provide 2, 4 or 6 arguments."); //$NON-NLS-1$
    }
    Integer nextInt = decodeInt(tokenizer.nextToken());
    if (nextInt == null) {
      throw new IllegalArgumentException(
              "First cell constraint element must be a number."); //$NON-NLS-1$
    }
    gridX = nextInt.intValue();
    if (gridX <= 0) {
      throw new IndexOutOfBoundsException(
          "The grid x must be a positive number."); //$NON-NLS-1$
    }
    nextInt = decodeInt(tokenizer.nextToken());
    if (nextInt == null) {
          throw new IllegalArgumentException(
             "Second cell constraint element must be a number."); //$NON-NLS-1$
    }
    gridY = nextInt.intValue();
    if (gridY <= 0) {
      throw new IndexOutOfBoundsException(
                        "The grid y must be a positive number."); //$NON-NLS-1$
    }
    if (!tokenizer.hasMoreTokens()) {
      return;
    }
    String token = tokenizer.nextToken();
    nextInt = decodeInt(token);
    if (nextInt != null) {
      // Case: "x, y, w, h" or
      //       "x, y, w, h, hAlign, vAlign"
      gridWidth = nextInt.intValue();
      if (gridWidth <= 0) {
        throw new IndexOutOfBoundsException(
            "The grid width must be a positive number."); //$NON-NLS-1$
      }
      nextInt = decodeInt(tokenizer.nextToken());
      if (nextInt == null) {
        throw new IllegalArgumentException(
           "Fourth cell constraint element must be like third."); //$NON-NLS-1$
      }
      gridHeight = nextInt.intValue();
      if (gridHeight <= 0) {
        throw new IndexOutOfBoundsException(
                   "The grid height must be a positive number."); //$NON-NLS-1$
      }
      if (!tokenizer.hasMoreTokens()) {
        return;
      }
      token = tokenizer.nextToken();
    }
    hAlign = decodeAlignment(token);
    vAlign = decodeAlignment(tokenizer.nextToken());
    ensureValidOrientations(hAlign, vAlign);
  }

  /**
   * Decodes a string description for the horizontal and vertical alignment
   * and sets this CellConstraints' alignment values. If the boolean is
   * <code>true</code> the horizontal alignment is the first token,
   * and the vertical alignment is the second token. if the boolean is
   * <code>false</code> the vertical alignment comes first. <p>
   *
   * Valid horizontal alignments are: left, middle, right, default, and fill.
   * Valid vertical alignments are: top, center, bottom, default, and fill.
   * The anchor's string representation abbreviates the alignment:
   * l, m, r, d, f, t, c, and b.<p>
   *
   * Anchor examples:
   * "m, c" is centered, "l, t" is northwest, "m, t" is north, "r, c" east.
   * "m, d" is horizontally centered and uses the row's default alignment.
   * "d, t" is on top of the cell and uses the column's default alignment.<p>
   *
   * @param encodedAlignments represents horizontal and vertical alignment
   * @param horizontalThenVertical horizontal if first
   *
   * @throws IllegalArgumentException if an alignment orientation is invalid
   */
  private void setAlignments(String encodedAlignments,
                             boolean horizontalThenVertical) {
    StringTokenizer tokenizer = new StringTokenizer(encodedAlignments,
                                                    " ,"); //$NON-NLS-1$
    Alignment first  = decodeAlignment(tokenizer.nextToken());
    Alignment second = decodeAlignment(tokenizer.nextToken());
    if (horizontalThenVertical) {
      hAlign = first;
      vAlign = second;
    } else {
      hAlign = second;
      vAlign = first;
    }
    ensureValidOrientations(hAlign, vAlign);
  }

  /**
   * Decodes an integer string representation and returns the associated Integer
   * or <tt>null</tt> in case of an invalid number format.
   *
   * @param token the encoded integer
   *
   * @return the decoded Integer or <tt>null</tt>
   */
  private Integer decodeInt(String token) {
    try {
      return Integer.decode(token);
    } catch (NumberFormatException e) {
      return null;
    }
  }

  /**
   * Parses an alignment string description and returns the corresponding
   * alignment value.
   *
   * @param encodedAlignment the encoded alignment
   *
   * @return the associated <code>Alignment</code> instance
   */
  private Alignment decodeAlignment(String encodedAlignment) {
    return Alignment.valueOf(encodedAlignment);
  }

  /**
   * Checks and verifies that the horizontal alignment is a horizontal and the
   * vertical alignment is vertical.
   *
   * @param horizontalAlignment the horizontal alignment
   * @param verticalAlignment the vertical alignment
   *
   * @throws IllegalArgumentException if an alignment is invalid
   */
  private void ensureValidOrientations(Alignment horizontalAlignment,
                                       Alignment verticalAlignment) {
    if (!horizontalAlignment.isHorizontal()) {
      throw new IllegalArgumentException(
                    "The horizontal alignment must be one of: " + //$NON-NLS-1$
                    "left, center, right, fill, default."); //$NON-NLS-1$
    }
    if (!verticalAlignment.isVertical()) {
      throw new IllegalArgumentException(
                      "The vertical alignment must be one of: " + //$NON-NLS-1$
                      "top, center, botto, fill, default."); //$NON-NLS-1$
    }
  }

  /**
   * Computes and returns the concrete alignment. Takes into account the cell
   * alignment and <i>the</i> <code>FormSpec</code> if applicable.<p>
   *
   * If this constraints object doesn't belong to a single column or row, the
   * <tt>formSpec</tt> parameter is <tt>null</tt>. In this case the cell
   * alignment is answered, but <tt>DEFAULT</tt> is mapped to <tt>FILL</tt>.<p>
   *
   * If the cell belongs to a single column or row, we use the cell alignment,
   * unless it is <tt>DEFAULT</tt>, where the alignment is inherited from the
   * column or row resp.
   *
   * @param cellAlignment this cell's alignment
   * @param formSpec the associated column or row specification
   * @return the concrete alignment
   */
  private Alignment concreteAlignment(Alignment cellAlignment,
                                      FormSpec formSpec) {
    if (formSpec == null) {
      if (cellAlignment == DEFAULT) { // NOPMD
        return FILL;
      }
      return cellAlignment;
    }
    return usedAlignment(cellAlignment, formSpec);
  }

  /**
   * Returns the alignment used for a given form constraints object. The cell
   * alignment overrides the column or row default, unless it is
   * <tt>DEFAULT</tt>. In the latter case, we use the column or row alignment.
   *
   * @param cellAlignment this cell constraint's alignment
   * @param formSpec the associated column or row specification
   *
   * @return the alignment used
   */
  private Alignment usedAlignment(Alignment cellAlignment, FormSpec formSpec) {
    if (cellAlignment != CellConstraints.DEFAULT) {
      // Cell alignments other than DEFAULT override col/row alignments
      return cellAlignment;
    }
    FormSpec.DefaultAlignment defaultAlignment = formSpec.getDefaultAlignment();
    if (defaultAlignment == FormSpec.FILL_ALIGN) {
      return FILL;
    }
    if (defaultAlignment == ColumnSpec.LEFT) {
      return LEFT;
    }
    if (defaultAlignment == FormSpec.CENTER_ALIGN) {
      return CENTER;
    }
    if (defaultAlignment == ColumnSpec.RIGHT) {
      return RIGHT;
    }
    if (defaultAlignment == RowSpec.TOP) {
      return TOP;
    }
    return BOTTOM;
  }

  /**
   * Computes and returns the pixel size of the given component using the
   * given form specification, measures, and cell size.
   *
   * @param component the component to measure
   * @param formSpec the specification of the component's column/row
   * @param minMeasure the measure for the minimum size
   * @param prefMeasure the measure for the preferred size
   * @param cellSize the cell size
   *
   * @return the component size as measured or a constant
   */
  private int componentSize(JQComponent component, FormSpec formSpec,
                            int cellSize, FormLayout.Measure minMeasure,
                            FormLayout.Measure prefMeasure) {
    if (formSpec == null) {
      return prefMeasure.sizeOf(component);
    }
    if (formSpec.getSize() == Sizes.MINIMUM) {
      return minMeasure.sizeOf(component);
    }
    if (formSpec.getSize() == Sizes.PREFERRED) {
        return prefMeasure.sizeOf(component);
    }
    return Math.min(cellSize, prefMeasure.sizeOf(component));
  }

  /**
   * Computes and returns the component's pixel origin.
   *
   * @param alignment the component's alignment
   * @param cellOrigin the origin of the display area
   * @param cellSize the extent of the display area
   * @param componentSize the component size
   *
   * @return the component's pixel origin
   */
  private int origin(Alignment alignment, int cellOrigin, int cellSize,
                     int componentSize) {
    if ((alignment == RIGHT) || (alignment == BOTTOM)) { // NOPMD
      return cellOrigin + cellSize - componentSize;
    }
    if (alignment == CENTER) { // NOPMD
      return cellOrigin + (cellSize - componentSize) / 2;
    }
    // left, top, fill
     return cellOrigin;
  }

  /**
   * Returns the component's pixel extent.
   *
   * @param alignment the component's alignment
   * @param cellSize the size of the display area
   * @param componentSize the component's size
   *
   * @return the component's pixel extent
   */
  private int extent(Alignment alignment, int cellSize, int componentSize) {
    if (alignment == FILL) { // NOPMD
      return cellSize;
    }
    return componentSize;
  }

  /**
   * Returns an integer that has a minimum of two characters.
   *
   * @param number the number to format
   *
   * @return a string representation for a number with a minimum of two chars
   */
   private String formatInt(int number) {
     String str = Integer.toString(number);
     if (number < 10) {
       return " " + str; //$NON-NLS-1$
     }
     return str;
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * An ordinal-based typesafe enumeration for component
   * alignment types as used by the {@link FormLayout}.
   */
  public static final class Alignment {

    static final int HORIZONTAL = 0;
    static final int VERTICAL = 1;
    static final int BOTH = 2;

    private final String name;
    private final int orientation;

    Alignment(String name, int orientation) {
      this.name = name;
      this.orientation = orientation;
    }

    static Alignment valueOf(String nameOrAbbreviation) { // NOPMD
      String str = nameOrAbbreviation.toLowerCase(Locale.ENGLISH);
      if ("d".equals(str) || "default".equals(str)) { //$NON-NLS-1$ $NON-NLS-2$
        return DEFAULT;
      }
      if ("f".equals(str) || "fill".equals(str)) { //$NON-NLS-1$ $NON-NLS-2$
        return FILL;
      }
      if ("c".equals(str) || "center".equals(str)) { //$NON-NLS-1$ $NON-NLS-2$
        return CENTER;
      }
      if ("l".equals(str) || "left".equals(str)) { //$NON-NLS-1$ $NON-NLS-2$
        return LEFT;
      }
      if ("r".equals(str) || "right".equals(str)) { //$NON-NLS-1$ $NON-NLS-2$
        return RIGHT;
      }
      if ("t".equals(str) || "top".equals(str)) { //$NON-NLS-1$ $NON-NLS-2$
        return TOP;
      }
      if ("b".equals(str) || "bottom".equals(str)) { //$NON-NLS-1$ $NON-NLS-2$
        return BOTTOM;
      }
      throw new IllegalArgumentException(
          "Invalid alignment " + nameOrAbbreviation + //$NON-NLS-1$
          ". Must be one of: left, center, right, top, bottom, " + //$NON-NLS-1$
          "fill, default, l, c, r, t, b, f, d."); //$NON-NLS-1$
    }

    /**
     * Returns the first character of this Alignment's name.
     * Used to identify it in short format strings.
     *
     * @return the name's first character.
     */
    public char abbreviation() {
      return name.charAt(0);
    }

    /**
     * Returns this Alignment's name.
     *
     * @return this alignment's name.
     */
    public String toString() {
      return name;
    }

/*^**************************************************************************^*/
/*- Pacakge-Private Methods.                                                 -*/
/*^**************************************************************************^*/

    boolean isHorizontal() {
      return (orientation != VERTICAL);
    }

    boolean isVertical() {
      return (orientation != HORIZONTAL);
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-11-09 15:38:04 Slobodan
   QSwing port

*****************************************************************************^*/

