/*
 * @(#)ColumnSpec.java  0.1.0 / 2007-11-02
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout;

import java.util.StringTokenizer;

/**
 * Specifies columns in FormLayout by their default orientation, start size and
 * resizing behavior.<p>
 *
 * <strong>Examples:</strong><br>
 * The following examples specify a column with FILL alignment, a size of
 * 10&nbsp;dlu that won't grow.
 * <pre>
 * new ColumnSpec(Sizes.dluX(10));
 * new ColumnSpec(ColumnSpec.FILL, Sizes.dluX(10), 0.0);
 * new ColumnSpec(ColumnSpec.FILL, Sizes.dluX(10), ColumnSpec.NO_GROW);
 * new ColumnSpec("10dlu");
 * new ColumnSpec("10dlu:0");
 * new ColumnSpec("fill:10dlu:0");
 * </pre><p>
 *
 * The
 * {@link yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.factories.FormFactory}
 * provides predefined frequently used ColumnSpec instances.
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.2 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class ColumnSpec extends FormSpec {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * By default put components in the left.
   */
  public static final DefaultAlignment LEFT = FormSpec.LEFT_ALIGN;

  /**
   * By default put the components in the center.
   */
  public static final DefaultAlignment CENTER = FormSpec.CENTER_ALIGN;

  /**
   * By default put components in the middle.
   */
  public static final DefaultAlignment MIDDLE = CENTER;

  /**
   * By default put components in the right.
   */
  public static final DefaultAlignment RIGHT = FormSpec.RIGHT_ALIGN;

  /**
   * By default fill the component into the column.
   */
  public static final DefaultAlignment FILL = FormSpec.FILL_ALIGN;

  /**
   * Unless overridden the default alignment for a column is FILL.
   */
  public static final DefaultAlignment DEFAULT = FILL;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs a ColumnSpec for the given default alignment, size and resize
   * weight.<p>
   *
   * The resize weight must be a non-negative double; you can use
   * <code>NO_GROW</code> as a convenience value for no resize.
   *
   * @param defaultAlignment the column's default alignment
   * @param size constant, component size or bounded size
   * @param resizeWeight the column's non-negative resize weight
   *
   * @throws IllegalArgumentException if the size is invalid or the resize
   *         weight is negative
   */
  public ColumnSpec(DefaultAlignment defaultAlignment, Size size,
                    double resizeWeight) {
    super(defaultAlignment, size, resizeWeight);
  }

  /**
   * Constructs a ColumnSpec for the given size using the default alignment, and
   * no resizing.
   *
   * @param size constant size, component size, or bounded size
   *
   * @throws IllegalArgumentException if the size is invalid
   */
  public ColumnSpec(Size size) {
    super(DEFAULT, size, NO_GROW);
  }

  /**
   * Constructs a ColumnSpec from the specified encoded description. The
   * description will be parsed to set initial values.
   *
   * @param encodedDescription the encoded description
   */
  public ColumnSpec(String encodedDescription) {
    super(DEFAULT, encodedDescription);
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Parses and splits encoded column specifications and returns
   * an array of ColumnSpec objects.
   *
   * @param encodedColumnSpecs  comma separated encoded column specifications
   * @return an array of decoded column specifications
   *
   * @throws IllegalArgumentException if the encoded column specifications
   *         string is <code>null</code>
   */
  public static ColumnSpec[] decodeSpecs(String encodedColumnSpecs) {
    if (encodedColumnSpecs == null) {
      throw new IllegalArgumentException(
                   "The column specification must not be null."); //$NON-NLS-1$
    }
    StringTokenizer tokenizer = new StringTokenizer(encodedColumnSpecs,
                                                    ", "); //$NON-NLS-1$
    int columnCount = tokenizer.countTokens();
    ColumnSpec[] columnSpecs = new ColumnSpec[columnCount];
    for (int i = 0; i < columnCount; i++) {
      columnSpecs[i] = new ColumnSpec(tokenizer.nextToken());
    }
    return columnSpecs;
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns if this is a horizontal specification (vs. vertical). Used to
   * distinct between horizontal and vertical dialog units, which have different
   * conversion factors.
   *
   * @return always true (for horizontal)
   */
  protected boolean isHorizontal() {
    return true;
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-11-02 15:56:10 Slobodan
   QSwing port

*****************************************************************************^*/
