/*
 * @(#)ConstantSize.java  0.1.0 / 2007-11-05
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout;

import java.util.List;
import java.util.Locale;

import yu.ac.bg.etf.javaqx.qswing.JQContainer;
import yu.ac.bg.etf.javaqx.qswing.JQComponent;

/**
 * An implementation of the {@link Size} interface that represents constant
 * sizes described by a value and unit, for example:
 * 10&nbsp;pixel, 15&nbsp;point or 4&nbsp;dialog units.
 * You can get instances of <code>ConstantSize</code> using
 * the factory methods and constants in the {@link Sizes} class.
 * Logical constant sizes that vary with the current layout style
 * are delivered by the
 * {@link yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.util.LayoutStyle}
 * class.<p>
 *
 * This class supports different size units:
 * <table>
 * <tr><td><b>Unit</b>&nbsp;
 * </td><td>&nbsp;<b>Abbreviation</b>&nbsp;</td><td>&nbsp;
 * <b>Size</b></td></tr>
 * <tr><td>Millimeter</td><td>mm</td><td>0.1 cm</td></tr>
 * <tr><td>Centimeter</td><td>cm</td><td>10.0 mm</td></tr>
 * <tr><td>Inch</td><td>in</td><td>25.4 mm</td></tr>
 * <tr><td>DTP Point</td><td>pt</td><td>1/72 in</td></tr>
 * <tr><td>Pixel</td><td>px</td><td>1/(resolution in dpi) in</td></tr>
 * <tr><td>Dialog Unit</td><td>dlu</td><td>honors l&amp;f, resolution, and
 * dialog font size</td></tr>
 * </table><p>
 *
 * <strong>Examples:</strong><pre>
 * Sizes.ZERO;
 * Sizes.DLUX9;
 * Sizes.dluX(42);
 * Sizes.pixel(99);
 * </pre>
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.7 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class ConstantSize implements Size {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /** @todo  comment */

// CS:OFF
  public static final Unit PIXEL = new Unit("Pixel", "px", true);
  public static final Unit POINT = new Unit("Point", "pt", true);
  public static final Unit DIALOG_UNITS_X = new Unit("Dialog units X", "dluX",
                                                     true);
  public static final Unit DLUX = DIALOG_UNITS_X;
  public static final Unit DIALOG_UNITS_Y = new Unit("Dialog units Y", "dluY",
                                                     true);
  public static final Unit DLUY = DIALOG_UNITS_Y;
  public static final Unit MILLIMETER = new Unit("Millimeter", "mm", false);
  public static final Unit MM = MILLIMETER;
  public static final Unit CENTIMETER = new Unit("Centimeter", "cm", false);
  public static final Unit CM = CENTIMETER;
  public static final Unit INCH = new Unit("Inch", "in", false);
  public static final Unit IN = INCH;
// CS:ON

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final double value;
  private final Unit unit;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs a ConstantSize for the given size and unit.
   *
   * @param value the size value interpreted in the given units
   * @param unit the size's unit
   */
  public ConstantSize(int value, Unit unit) {
    this.value = value;
    this.unit = unit;
  }

  /**
   * Constructs a ConstantSize for the given size and unit.
   *
   * @param value the size value interpreted in the given units
   * @param unit the size's unit
   */
  public ConstantSize(double value, Unit unit) {
    this.value = value;
    this.unit = unit;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Converts the size if necessary and returns the value in pixels.
   *
   * @param component  the associated component
   * @return the size in pixels
   */
  public int getPixelSize(JQComponent component) {
    if (unit == PIXEL) { // NOPMD
      return intValue();
    } else if (unit == POINT) { // NOPMD
      return Sizes.pointAsPixel(intValue(), component);
    } else if (unit == INCH) { // NOPMD
      return Sizes.inchAsPixel(value, component);
    } else if (unit == MILLIMETER) { // NOPMD
      return Sizes.millimeterAsPixel(value, component);
    } else if (unit == CENTIMETER) { // NOPMD
      return Sizes.centimeterAsPixel(value, component);
    } else if (unit == DIALOG_UNITS_X) { // NOPMD
      return Sizes.dialogUnitXAsPixel(intValue(), component);
    } else if (unit == DIALOG_UNITS_Y) { // NOPMD
      return Sizes.dialogUnitYAsPixel(intValue(), component);
    }
    throw new IllegalStateException("Invalid unit " + unit);
  }

  /**
   * Returns this size's value.
   *
   * @return the size value
   */
  public double getValue() {
    return value;
  }


  /**
   * Returns this size's unit.
   *
   * @return the size unit
   */
  public Unit getUnit() {
    return unit;
  }

/*^**************************************************************************^*/
/*- Pacakge-Private Static Methods.                                          -*/
/*^**************************************************************************^*/

  /**
   * Creates and returns a ConstantSize from the given encoded size and unit
   * description.
   *
   * @param encodedValueAndUnit the size's value and unit as string
   * @param horizontal true for horizontal, false for vertical
   *
   * @return a constant size for the given encoding and unit description
   *
   * @throws IllegalArgumentException if the unit requires integer but the value
   *                                  is not an integer
   */
  static ConstantSize valueOf(String encodedValueAndUnit, boolean horizontal) {
    String[] split = ConstantSize.splitValueAndUnit(encodedValueAndUnit);
    String encodedValue = split[0];
    String encodedUnit = split[1];
    Unit unit = Unit.valueOf(encodedUnit, horizontal);
    if (unit.requiresIntegers) {
      try {
        return new ConstantSize(Integer.parseInt(encodedValue), unit);
      } catch (NumberFormatException e) {
        throw new IllegalArgumentException(// NOPMD
                          unit.toString() + " value " + //$NON-NLS-1$
                          encodedValue + " must be an integer."); //$NON-NLS-1$
      }
    }
    return new ConstantSize(Double.parseDouble(encodedValue), unit);
  }

  /**
   * Creates and returns a ConstantSize for the specified size value in
   * horizontal dialog units.
   *
   * @param value size value in horizontal dialog units
   *
   * @return the associated Size instance
   */
  static ConstantSize dluX(int value) {
    return new ConstantSize(value, DLUX);
  }

  /**
   * Creates and returns a ConstantSize for the specified size value in vertical
   * dialog units.
   *
   * @param value size value in vertical dialog units
   *
   * @return the associated Size instance
   */
  static ConstantSize dluY(int value) {
    return new ConstantSize(value, DLUY);
  }

/*^**************************************************************************^*/
/*- Size Interface Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * Returns this size as pixel size. Neither requires the component list nor
   * the specified measures.<p>
   *
   * Invoked by {@link yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.FormSpec}
   * to determine the size of a column or row.
   *
   * @param container the layout container
   * @param components the list of components used to compute the size
   * @param minMeasure the measure that determines the minimum sizes
   * @param prefMeasure the measure that determines the preferred sizes
   * @param defaultMeasure the measure that determines the default sizes
   *
   * @return the computed maximum size in pixel
   */
  public int maximumSize(JQContainer container, List components,
                         FormLayout.Measure minMeasure,
                         FormLayout.Measure prefMeasure,
                         FormLayout.Measure defaultMeasure) {
    return getPixelSize(container);
  }

  /**
   * Describes if this Size can be compressed, if container space gets scarce.
   * Used by the FormLayout size computations in <code>#compressedSizes</code>
   * to check whether a column or row can be compressed or not.<p>
   *
   * ConstantSizes are incompressible.
   *
   * @return <code>false</code>
   */
  public boolean compressible() {
    return false;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Indicates whether some other ConstantSize is "equal to" this one.
   *
   * @param o the Object with which to compare
   *
   * @return <tt>true</tt> if this object is the same as the obj argument;
   *         <tt>false</tt> otherwise.
   */
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (!(o instanceof ConstantSize)) {
      return false;
    }
    ConstantSize size = (ConstantSize) o;
    return ((this.value == size.value) && (this.unit  == size.unit));
  }

  /**
   * Returns a hash code value for the object. This method is supported for the
   * benefit of hashtables such as those provided by
   * <tt>java.util.Hashtable</tt>.
   *
   * @return  a hash code value for this object.
   */
  public int hashCode() {
    long bits = Double.doubleToLongBits(value);
    return (int)(bits ^ (bits >>> 32)) + 37 * unit.hashCode();
  }

  /**
   * Returns a string representation of this size object.
   *
   * <strong>Note:</strong> The string representation may change at any time.
   * It is strongly recommended to not use this string for parsing purposes.
   *
   * @return  a string representation of the constant size
   */
  public String toString() {
    if (value == intValue()) {
      return Integer.toString(intValue()) + unit.abbreviation();

    }
    return Double.toString(value) + unit.abbreviation();
  }

/*^**************************************************************************^*/
/*- Pacakge-Private Static Metod.                                            -*/
/*^**************************************************************************^*/

  /**
   * Splits a string that encodes size with unit into the size and unit
   * substrings. Returns an array of two strings.
   *
   * @param encodedValueAndUnit a strings that represents a size with unit
   *
   * @return the first element is size, the second is unit
   */
  static String[] splitValueAndUnit(String encodedValueAndUnit) {
    String[] result = new String[2];
    int len = encodedValueAndUnit.length();
    int firstLetterIndex = len;
    while ((firstLetterIndex > 0) &&
           Character.isLetter(encodedValueAndUnit.
                                               charAt(firstLetterIndex - 1))) {
      firstLetterIndex--;
    }
    result[0] = encodedValueAndUnit.substring(0, firstLetterIndex);
    result[1] = encodedValueAndUnit.substring(firstLetterIndex);
    return result;
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private int intValue() {
    return (int)Math.round(value);
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * An ordinal-based typesafe enumeration for units as used in instances of
   * {@link ConstantSize}.
   */
  public static final class Unit {

    final String name;
    final String abbreviation;
    final boolean requiresIntegers; // NOPMD

    Unit(String name, String abbreviation, boolean requiresIntegers) {
      this.name = name;
      this.abbreviation = abbreviation;
      this.requiresIntegers = requiresIntegers;
    }

    /**
     * Returns a Unit that corresponds to the specified string.
     *
     * @param str the encoded unit
     * @param horizontal true for a horizontal unit, false for vertical
     *
     * @return the corresponding Unit
     *
     * @throws IllegalArgumentException if no Unit exists for the string
     */
    static Unit valueOf(String str, boolean horizontal) {
      String lowerCase = str.toLowerCase(Locale.ENGLISH);
      if ("px".equals(lowerCase) || lowerCase.length() == 0) { //$NON-NLS-1$
        return PIXEL;
      } else if ("dlu".equals(lowerCase)) { //$NON-NLS-1$
        if (horizontal) {
          return DIALOG_UNITS_X;
        }
        return DIALOG_UNITS_Y;
      } else if ("pt".equals(lowerCase)) { //$NON-NLS-1$
        return POINT;
      } else if ("in".equals(lowerCase)) { //$NON-NLS-1$
        return INCH;
      } else if ("mm".equals(lowerCase)) { //$NON-NLS-1$
        return MILLIMETER;
      } else if ("cm".equals(lowerCase)) { //$NON-NLS-1$
        return CENTIMETER;
      }
      throw new IllegalArgumentException(
                    "Invalid unit name '" + str + //$NON-NLS-1$
                    "'. Must be one of: px, dlu, pt, mm, cm, in"); //$NON-NLS-1$
    }

    /**
     * {@inheritDoc}
     */
    public String toString() {
      return name;
    }

    /**
     * Returns the first character of this Unit's name. Used to identify it in
     * short format strings.
     *
     * @return the first character of this Unit's name.
     */
    public String abbreviation() {
      return abbreviation;
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-11-05 11:06:26 Slobodan
   QSwing port

*****************************************************************************^*/
