/*
 * @(#)FormSpec.java  0.1.0 / 2007-11-02
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout;

import java.util.List;
import java.util.Locale;
import java.util.StringTokenizer;

import yu.ac.bg.etf.javaqx.qswing.JQContainer;

/**
 * An abstract class that specifies columns and rows in FormLayout by their
 * default alignment, start size and resizing behavior. API users will use the
 * subclasses {@link ColumnSpec} and  {@link RowSpec}.
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.4 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public abstract class FormSpec {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * Gives a column or row a fixed size.
   */
  public static final double NO_GROW = 0.0d;

  /**
   * The default resize weight.
   */
  public static final double DEFAULT_GROW = 1.0d;

/*^**************************************************************************^*/
/*- Pacakge-Private Static Final Fields.                                     -*/
/*^**************************************************************************^*/

  /**
   * By default put components in the left.
   */
  static final DefaultAlignment LEFT_ALIGN =
                                    new DefaultAlignment("left"); //$NON-NLS-1$

  /**
   * By default put components in the right.
   */
  static final DefaultAlignment RIGHT_ALIGN =
                                   new DefaultAlignment("right"); //$NON-NLS-1$

  /**
   * By default put the components in the top.
   */
  static final DefaultAlignment TOP_ALIGN =
                                     new DefaultAlignment("top"); //$NON-NLS-1$

  /**
   * By default put the components in the bottom.
   */
  static final DefaultAlignment BOTTOM_ALIGN =
                                  new DefaultAlignment("bottom"); //$NON-NLS-1$

  /**
   * By default put the components in the center.
   */
  static final DefaultAlignment CENTER_ALIGN =
                                  new DefaultAlignment("center"); //$NON-NLS-1$

  /**
   * By default fill the column or row.
   */
  static final DefaultAlignment FILL_ALIGN =
                                    new DefaultAlignment("fill"); //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Holds the default alignment that will be used if a cell does not override
   * this default.
   */
  private DefaultAlignment defaultAlignment;

  /**
   * Holds the size that describes how to size this column or row.
   */
  private Size size;

  /**
   * Holds the resize weight; is 0 if not used.
   */
  private double resizeWeight;

/*^**************************************************************************^*/
/*- Protected Constructors.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Constructs a <code>FormSpec</code> for the given default alignment, size,
   * and resize weight. The resize weight must be a non-negative double; you can
   * use <code>NONE</code> as a convenience value for no resize.
   *
   * @param defaultAlignment the spec's default alignment
   * @param size a constant, component or bounded size
   * @param resizeWeight the spec resize weight
   *
   * @throws IllegalArgumentException if the resize weight is negative
   */
  protected FormSpec(DefaultAlignment defaultAlignment, Size size,
                     double resizeWeight) {
    this.defaultAlignment = defaultAlignment;
    this.size = size;
    this.resizeWeight = resizeWeight;
    if (resizeWeight < 0) {
      throw new IllegalArgumentException(
                      "The resize weight must be non-negative."); //$NON-NLS-1$
    }
  }

  /**
   * Constructs a <code>FormSpec</code> from the specified encoded description.
   * The description will be parsed to set initial values.
   *
   * @param defaultAlignment the default alignment
   * @param encodedDescription the encoded description
   */
  protected FormSpec(DefaultAlignment defaultAlignment,
                     String encodedDescription) {
    this(defaultAlignment, Sizes.DEFAULT, NO_GROW);
    parseAndInitValues(encodedDescription.toLowerCase(Locale.ENGLISH));
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the default alignment.
   *
   * @return the default alignment
   */
  public final DefaultAlignment getDefaultAlignment() {
    return defaultAlignment;
  }

  /**
   * Returns the size.
   *
   * @return the size
   */
  public final Size getSize() {
    return size;
  }

  /**
   * Returns the current resize weight.
   *
   * @return the resize weight.
   */
  public final double getResizeWeight() {
    return resizeWeight;
  }

  /**
   * Returns a string representation of this form specification. The string
   * representation consists of three elements separated by a colon
   * (<tt>":"</tt>), first the alignment, second the size, and third the resize
   * spec.<p>
   *
   * This method does <em>not</em> return a decoded version of this object;
   * the contrary is the case. Many instances will return a string that cannot
   * be parsed.<p>
   *
   * <strong>Note:</strong> The string representation may change at any time.
   * It is strongly recommended to not use this string for parsing purposes.
   *
   * @return a string representation of the form specification.
   */
  public final String toShortString() {
    StringBuffer buffer = new StringBuffer();
    buffer.append(defaultAlignment.abbreviation());
    buffer.append(':');
    buffer.append(size.toString());
    buffer.append(':');
    if (resizeWeight == NO_GROW) {
      buffer.append('n');
    } else if (resizeWeight == DEFAULT_GROW) {
      buffer.append('g');
    } else {
      buffer.append("g("); //$NON-NLS-1$
      buffer.append(resizeWeight);
      buffer.append(')');
    }
    return buffer.toString();
  }

/*^**************************************************************************^*/
/*- Pacakge-Private Method.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Checks and answers whether this spec can grow or not. That is the case if
   * and only if the resize weight is != <code>NO_GROW</code>.
   *
   * @return true if it can grow, false if it can't grow
   */
  final boolean canGrow() {
    return (getResizeWeight() != NO_GROW);
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns a string representation of this form specification. The string
   * representation consists of three elements separated by a colon
   * (<tt>":"</tt>), first the alignment, second the size, and third the resize
   * spec.<p>
   *
   * This method does <em>not</em> return a decoded version of this object; the
   * contrary is the case. Many instances will return a string that cannot be
   * parsed.<p>
   *
   * <strong>Note:</strong> The string representation may change at any time.
   * It is strongly recommended to not use this string for parsing purposes.
   *
   * @return a string representation of the form specification.
   */
  public final String toString() {
    StringBuffer buffer = new StringBuffer();
    buffer.append(defaultAlignment);
    buffer.append(':');
    buffer.append(size.toString());
    buffer.append(':');
    if (resizeWeight == NO_GROW) {
      buffer.append("noGrow"); //$NON-NLS-1$
    } else if (resizeWeight == DEFAULT_GROW) {
      buffer.append("grow"); //$NON-NLS-1$
    } else {
      buffer.append("grow("); //$NON-NLS-1$
      buffer.append(resizeWeight);
      buffer.append(')');
    }
    return buffer.toString();
  }

/*^**************************************************************************^*/
/*- Pacakge-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Returns if this is a horizontal specification (vs. vertical). Used to
   * distinct between horizontal and vertical dialog units, which have different
   * conversion factors.
   *
   * @return true for horizontal, false for vertical
   */
  abstract boolean isHorizontal();

  /**
   * Computes the maximum size for the given list of components, using
   * this form spec and the specified measure.<p>
   *
   * Invoked by FormLayout to determine the size of one of my elements
   *
   * @param container the layout container
   * @param components the list of components to measure
   * @param minMeasure the measure used to determine the minimum size
   * @param prefMeasure the measure used to determine the preferred size
   * @param defaultMeasure  the measure used to determine the default size
   *
   * @return the maximum size in pixels
   */
  final int maximumSize(JQContainer container, List components,
                        FormLayout.Measure minMeasure,
                        FormLayout.Measure prefMeasure,
                        FormLayout.Measure defaultMeasure) {
    return size.maximumSize(container, components, minMeasure, prefMeasure,
                            defaultMeasure);
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  /**
   * Parses an encoded form spec and initializes all required fields. The
   * encoded description must be in lower case.
   *
   * @param encodedDescription the FormSpec in an encoded format
   *
   * @throws IllegalArgumentException if the string is empty, has no size, or is
   *                                  otherwise invalid
   */
  private void parseAndInitValues(String encodedDescription) {
    StringTokenizer tokenizer = new StringTokenizer(encodedDescription,
                                                    ":"); //$NON-NLS-1$
    if (!tokenizer.hasMoreTokens()) {
      throw new IllegalArgumentException(
                             "The form spec must not be empty."); //$NON-NLS-1$
    }
    String token = tokenizer.nextToken();
    // Check if the first token is an orientation.
    DefaultAlignment alignment = DefaultAlignment.valueOf(token,
                                                          isHorizontal());
    if (alignment != null) {
      defaultAlignment = alignment;
      if (!tokenizer.hasMoreTokens()) {
        throw new IllegalArgumentException(
                           "The form spec must provide a size."); //$NON-NLS-1$
      }
      token = tokenizer.nextToken();
    }
    parseAndInitSize(token);
    if (tokenizer.hasMoreTokens()) {
       resizeWeight = decodeResize(tokenizer.nextToken());
    }
  }

  /**
   * Parses an encoded size spec and initializes the size fields.
   *
   * @param token a token that represents a size, either bounded or plain
   */
  private void parseAndInitSize(String token) {
    if (token.startsWith("max(") && //$NON-NLS-1$
        token.endsWith(")")) { //$NON-NLS-1$
      size = parseAndInitBoundedSize(token, false);
      return;
    }
    if (token.startsWith("min(") &&  //$NON-NLS-1$
        token.endsWith(")")) { //$NON-NLS-1$
      size = parseAndInitBoundedSize(token, true);
      return;
    }
    size = decodeAtomicSize(token);
  }

  /**
   * Parses an encoded compound size and sets the size fields. The compound size
   * has format:
   * max(&lt;atomic size&gt;;&lt;atomic size2&gt;) |
   * min(&lt;atomic size1&gt;;&lt;atomic size2&gt;)
   * One of the two atomic sizes must be a logical size, the other must  * be a
   * size constant.
   *
   * @param token a token for a bounded size, e.g. "max(50dlu; pref)"
   * @param setMax if true we set a maximum size, otherwise a minimum size
   *
   * @return a Size that represents the parse result
   */
  private Size parseAndInitBoundedSize(String token, boolean setMax) {
    int semicolonIndex = token.indexOf(';');
    String sizeToken1 = token.substring(4, semicolonIndex);
    String sizeToken2 = token.substring(semicolonIndex + 1, token.length() - 1);
    Size size1 = decodeAtomicSize(sizeToken1);
    Size size2 = decodeAtomicSize(sizeToken2);
    // Check valid combinations and set min or max.
    if (size1 instanceof ConstantSize) {
      if (size2 instanceof Sizes.ComponentSize) {
        if (setMax) {
          return new BoundedSize(size2, null, size1);
        }
        return new BoundedSize(size2, size1, null);
      }
      throw new IllegalArgumentException(
                    "Bounded sizes must not be both constants."); //$NON-NLS-1$
    }
    if (size2 instanceof ConstantSize) {
      if (setMax) {
        return new BoundedSize(size1, null, size2);
      }
      return new BoundedSize(size1, size2, null);
    }
    throw new IllegalArgumentException(
                     "Bounded sizes must not be both logical."); //$NON-NLS-1$
  }

  /**
   * Decodes and returns an atomic size that is either a constant size or a
   * component size.
   *
   * @param token the encoded size
   *
   * @return the decoded size either a constant or component size
   */
  private Size decodeAtomicSize(String token) {
    Sizes.ComponentSize componentSize = Sizes.ComponentSize.valueOf(token);
    if (componentSize != null) {
      return componentSize;
    }
    return ConstantSize.valueOf(token, isHorizontal());
  }

  /**
   * Decodes an encoded resize mode and resize weight and answers the resize
   * weight.
   *
   * @param token the encoded resize weight
   * @return the decoded resize weight
   *
   * @throws IllegalArgumentException if the string description is an invalid
   *                                  string representation
   */
  private double decodeResize(String token) {
    if ("g".equals(token) || "grow".equals(token)) { //$NON-NLS-1$ //$NON-NLS-2$
      return DEFAULT_GROW;
    }
    if ("n".equals(token) || //$NON-NLS-1$
        "nogrow".equals(token) || //$NON-NLS-1$
        "none".equals(token)) { //$NON-NLS-1$
      return NO_GROW;
    }
    // Must have format: grow(<double>)
    if ((token.startsWith("grow(") || //$NON-NLS-1$
         token.startsWith("g(")) && //$NON-NLS-1$
        token.endsWith(")")) { //$NON-NLS-1$
      int leftParen  = token.indexOf('(');
      int rightParen = token.indexOf(')');
      return Double.parseDouble(token.substring(leftParen + 1, rightParen));
    }
    throw new IllegalArgumentException(
                      "The resize argument '" + token + //$NON-NLS-1$
                      "' is invalid. Must be one of: grow, " + //$NON-NLS-1$
                      "g, none, n, grow(<double>), g(<double>)"); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * An ordinal-based serializable typesafe enumeration for the
   * column and row default alignment types.
   */
  public static final class DefaultAlignment {

    private final String name;

    DefaultAlignment(String name) {
      this.name = name;
    }

    /**
     * Returns a DefaultAlignment that corresponds to the specified string, null
     * if no such alignment exists.
     *
     * @param str the encoded alignment
     * @param isHorizontal indicates the values orientation
     *
     * @return the corresponding DefaultAlignment or null
     */
    static DefaultAlignment valueOf(String str, boolean isHorizontal) {
      if ("f".equals(str) || "fill".equals(str)) { //$NON-NLS-1$ //$NON-NLS-2$
        return FILL_ALIGN;
      } else if ("c".equals(str) || //$NON-NLS-1$
                 "center".equals(str)) { //$NON-NLS-1$
        return CENTER_ALIGN;
      } else if (isHorizontal) {
        if ("r".equals(str) || //$NON-NLS-1$
            "right".equals(str)) { //$NON-NLS-1$
          return RIGHT_ALIGN;
        } else if ("l".equals(str) || //$NON-NLS-1$
                   "left".equals(str)) { //$NON-NLS-1$
          return LEFT_ALIGN;
        }
        return null;
      } else {
        if ("t".equals(str) || "top".equals(str)) { //$NON-NLS-1$ //$NON-NLS-2$
          return TOP_ALIGN;
        } else if ("b".equals(str) || //$NON-NLS-1$
                   "bottom".equals(str)) { //$NON-NLS-1$
          return BOTTOM_ALIGN;
        }
        return null;
      }
    }

    /**
     * Returns this Alignment's name.
     *
     * @return this alignment's name.
     */
    public String toString()  {
      return name;
    }

    /**
     * Returns the first character of this Alignment's name. Used to identify it
     * in short format strings.
     *
     * @return the name's first character.
     */
    public char abbreviation() {
      return name.charAt(0);
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-11-02 21:45:54 Slobodan
   QSwing port

*****************************************************************************^*/
