/*
 * @(#)RowSpec.java  0.1.0 / 2007-11-02
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout;

import java.util.StringTokenizer;

/**
 * Specifies rows in FormLayout by their default orientation, start size and
 * resizing behavior.<p>
 *
 * <strong>Examples:</strong><br>
 * The following examples specify a centered row with a size of 14&nbsp;dlu
 * that won't grow.
 * <pre>
 * new RowSpec(Sizes.dluX(14));
 * new RowSpec(RowSpec.CENTER, Sizes.dluX(14), 0.0);
 * new RowSpec(rowSpec.CENTER, Sizes.dluX(14), RowSpec.NO_GROW);
 * new RowSpec("14dlu");
 * new RowSpec("14dlu:0");
 * new RowSpec("center:14dlu:0");
 * </pre><p>
 *
 * The
 * {@link yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.factories.FormFactory}
 * provides predefined frequently used RowSpec instances.
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.2 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class RowSpec extends FormSpec {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * By default put the components in the top.
   */
  public static final DefaultAlignment TOP = FormSpec.TOP_ALIGN;

  /**
   * By default put the components in the center.
   */
  public static final DefaultAlignment CENTER = FormSpec.CENTER_ALIGN;

  /**
   * By default put the components in the bottom.
   */
  public static final DefaultAlignment BOTTOM = FormSpec.BOTTOM_ALIGN;

  /**
   * By default fill the component into the row.
   */
  public static final DefaultAlignment FILL = FormSpec.FILL_ALIGN;

  /**
   * Unless overridden the default alignment for a row is CENTER.
   */
  public static final DefaultAlignment DEFAULT = CENTER;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs a RowSpec from the given default orientation, size, and resize
   * weight.<p>
   *
   * The resize weight must be a non-negative double; you can use
   * <code>NO_FILL</code> as a convenience value for no resize.
   *
   * @param defaultAlignment the row's default alignment
   * @param size constant size, component size, or bounded size
   * @param resizeWeight the row's non-negative resize weight
   *
   * @throws IllegalArgumentException if the size is invalid or the resize
   *         weight is negative
   */
  public RowSpec(DefaultAlignment defaultAlignment, Size size,
                 double resizeWeight) {
    super(defaultAlignment, size, resizeWeight);
  }

  /**
   * Constructs a RowSpec for the given size using the default alignment, and no
   * resizing.
   *
   * @param size constant size, component size, or bounded size
   *
   * @throws IllegalArgumentException if the size is invalid
   */
  public RowSpec(Size size) {
    super(DEFAULT, size, NO_GROW);
  }

  /**
   * Constructs a RowSpec from the specified encoded description. The
   * description will be parsed to set initial values.
   *
   * @param encodedDescription the encoded description
   */
   public RowSpec(String encodedDescription) {
     super(DEFAULT, encodedDescription);
   }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
  * Parses and splits encoded row specifications and returns an array of
  * RowSpec objects.
  *
  * @param encodedRowSpecs comma separated encoded row specifications
  *
  * @return an array of decoded row specifications
  *
  * @throws IllegalArgumentException if the encoded row specifications string
  *     is <code>null</code>
  */
  public static RowSpec[] decodeSpecs(String encodedRowSpecs) {
    if (encodedRowSpecs == null) {
      throw new IllegalArgumentException(
                     "The row specification must not be null."); //$NON-NLS-1$
    }
    StringTokenizer tokenizer = new StringTokenizer(encodedRowSpecs,
                                                    ", "); //$NON-NLS-1$
    int rowCount = tokenizer.countTokens();
    RowSpec[] rowSpecs = new RowSpec[rowCount];
    for (int i = 0; i < rowCount; i++) {
      rowSpecs[i] = new RowSpec(tokenizer.nextToken());
    }
    return rowSpecs;
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns if this is a horizontal specification (vs. vertical). Used to
   * distinct between horizontal and vertical dialog units, which have different
   * conversion factors.
   *
   * @return <tt>true</tt> for horizontal, <tt>false</tt> for vertical
   */
  protected boolean isHorizontal() {
    return false;
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-11-02 15:44:50 Slobodan
   QSwing port

*****************************************************************************^*/
