/*
 * @(#)Sizes.java  0.1.0 / 2007-11-02
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout;

import java.util.Iterator;
import java.util.List;

import yu.ac.bg.etf.javaqx.qswing.JQComponent;
import yu.ac.bg.etf.javaqx.qswing.JQContainer;
import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.util.DefaultUnitConverter;
import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.util.UnitConverter;

/**
 * Consists only of static methods that create and convert sizes as required by
 * the FormLayout. The conversion of sizes that are not based on pixel is
 * delegated to an implementation of {@link UnitConverter}.
 * The conversion methods require the layout container as parameter to read its
 * current font and resolution.
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.5 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class Sizes {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /** @todo  comment */

// CS:OFF

  public static final ConstantSize ZERO = pixel(0);
  public static final ConstantSize DLUX1 = dluX(1);
  public static final ConstantSize DLUX2 = dluX(2);
  public static final ConstantSize DLUX3 = dluX(3);
  public static final ConstantSize DLUX4 = dluX(4);
  public static final ConstantSize DLUX5 = dluX(5);
  public static final ConstantSize DLUX6 = dluX(6);
  public static final ConstantSize DLUX7 = dluX(7);
  public static final ConstantSize DLUX8 = dluX(8);
  public static final ConstantSize DLUX9 = dluX(9);
  public static final ConstantSize DLUX11 = dluX(11);
  public static final ConstantSize DLUX14 = dluX(14);
  public static final ConstantSize DLUY1 = dluY(1);
  public static final ConstantSize DLUY2 = dluY(2);
  public static final ConstantSize DLUY3 = dluY(3);
  public static final ConstantSize DLUY4 = dluY(4);
  public static final ConstantSize DLUY5 = dluY(5);
  public static final ConstantSize DLUY6 = dluY(6);
  public static final ConstantSize DLUY7 = dluY(7);
  public static final ConstantSize DLUY8 = dluY(8);
  public static final ConstantSize DLUY9 = dluY(9);
  public static final ConstantSize DLUY11 = dluY(11);
  public static final ConstantSize DLUY14 = dluY(14);

// CS:ON

  /**
   * Use the maximum of all component minimum sizes as column or row size.
   */
  public static final ComponentSize MINIMUM =
                                    new ComponentSize("minimum"); //$NON-NLS-1$

  /**
   * Use the maximum of all component preferred sizes as column or row size.
   */
  public static final ComponentSize PREFERRED =
                                  new ComponentSize("preferred"); //$NON-NLS-1$

  /**
   * Use the maximum of all component sizes as column or row size; measures
   * preferred sizes when asked for the preferred size and minimum sizes when
   * asked for the minimum size.
   */
  public static final ComponentSize DEFAULT =
                                    new ComponentSize("default"); //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Holds the current converter that maps non-pixel sizes to pixels.
   */
  private static UnitConverter unitConverter;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private Sizes() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Creates and returns an instance of <code>ConstantSize</code> from the
   * given encoded size and unit description.
   *
   * @param encodedValueAndUnit value and unit in string representation
   * @param horizontal <tt>true</tt> for horizontal, false for vertical
   *
   * @return a <code>ConstantSize</code> for the given value and unit
   */
  public static ConstantSize constant(String encodedValueAndUnit,
                                      boolean horizontal) {
    return ConstantSize.valueOf(encodedValueAndUnit, horizontal);
  }

  /**
   * Creates and returns a ConstantSize for the specified value in horizontal
   * dialog units.
   *
   * @param value size value in horizontal dialog units
   * @return the associated <code>ConstantSize</code>
   */
  public static ConstantSize dluX(int value) {
    return ConstantSize.dluX(value);
  }

  /**
   * Creates and returns a ConstantSize for the specified value in vertical
   * dialog units.
   *
   * @param value size value in vertical dialog units
   *
   * @return the associated <code>ConstantSize</code>
   */
  public static ConstantSize dluY(int value) {
    return ConstantSize.dluY(value);
  }

  /**
   * Creates and returns a ConstantSize for the specified pixel value.
   *
   * @param value  value in pixel
   *
   * @return the associated <code>ConstantSize</code>
   */
  public static ConstantSize pixel(int value) {
    return new ConstantSize(value, ConstantSize.PIXEL);
  }

  /**
   * Creates and returns a BoundedSize for the given basis using the specified
   * lower and upper bounds.
   *
   * @param basis the base size
   * @param lowerBound the lower bound size
   * @param upperBound the upper bound size
   *
   * @return a <code>BoundedSize</code> for the given basis and bounds
   *
   * @throws NullPointerException if basis is null
   */
  public static Size bounded(Size basis, Size lowerBound, Size upperBound) {
    return new BoundedSize(basis, lowerBound, upperBound);
  }

  /**
   * Converts Inches and returns pixels using the specified resolution.
   *
   * @param in the Inches
   * @param component the component that provides the graphics object
   *
   * @return the given Inches as pixels
   */
  public static int inchAsPixel(double in, JQComponent component) {
    if (in == 0d) {
      return 0;
    }
    return getUnitConverter().inchAsPixel(in, component);
  }

  /**
   * Converts Millimeters and returns pixels using the resolution of the given
   * component's graphics object.
   *
   * @param mm Millimeters
   * @param component the component that provides the graphics object
   *
   * @return the given Millimeters as pixels
   */
  public static int millimeterAsPixel(double mm, JQComponent component) {
    if (mm == 0d) {
      return 0;
    }
    return getUnitConverter().millimeterAsPixel(mm, component);
  }

  /**
   * Converts Centimeters and returns pixels using the resolution of the given
   * component's graphics object.
   *
   * @param cm Centimeters
   * @param component the component that provides the graphics object
   *
   * @return the given Centimeters as pixels
   */
  public static int centimeterAsPixel(double cm, JQComponent component) {
    if (cm == 0d) {
      return 0;
    }
    return getUnitConverter().centimeterAsPixel(cm, component);
  }

  /**
   * Converts DTP Points and returns pixels using the resolution of the given
   * component's graphics object.
   *
   * @param pt DTP Points
   * @param component the component that provides the graphics object
   *
   * @return the given Points as pixels
   */
  public static int pointAsPixel(int pt, JQComponent component) {
    if (pt == 0d) {
      return 0;
    }
    return getUnitConverter().pointAsPixel(pt, component);
  }

  /**
   * Converts horizontal dialog units and returns pixels. Honors the resolution,
   * dialog font size, platform, and l&amp;f.
   *
   * @param dluX the horizontal dialog units
   * @param component the component that provides the graphics object
   *
   * @return the given horizontal dialog units as pixels
   */
  public static int dialogUnitXAsPixel(int dluX, JQComponent component) {
    if (dluX == 0d) {
      return 0;
    }
    return getUnitConverter().dialogUnitXAsPixel(dluX, component);
  }

  /**
   * Converts vertical dialog units and returns pixels.
   * Honors the resolution, dialog font size, platform, and l&amp;f.
   *
   * @param dluY the vertical dialog units
   * @param component the component that provides the graphics object
   *
   * @return the given vertical dialog units as pixels
   */
  public static int dialogUnitYAsPixel(int dluY, JQComponent component) {
    if (dluY == 0d) {
      return 0;
    }
    return getUnitConverter().dialogUnitYAsPixel(dluY, component);
  }

  /**
   * Returns the current {@link UnitConverter}. If it has not been initialized
   * before it will get an instance of {@link DefaultUnitConverter}.
   *
   * @return the current <code>UnitConverter</code>
   */
  public static UnitConverter getUnitConverter() {
    if (unitConverter == null) {
      unitConverter = DefaultUnitConverter.getInstance();
    }
    return unitConverter;
  }

  /**
   * Sets a new UnitConverter that will be used to convert font-dependent sizes
   * to pixel sizes.
   *
   * @param newUnitConverter  the unit converter to be set
   */
  public static void setUnitConverter(UnitConverter newUnitConverter) {
    unitConverter = newUnitConverter;
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * An ordinal-based typesafe enumeration that implements the {@link Size}
   * interface for the component sizes: <em>min, pref, default</em>.
   */
  static final class ComponentSize implements Size {

    private final String name;

    ComponentSize(String name) {
      this.name = name;
    }

    /**
     * Returns an instance of <code>ComponentSize</code> that corresponds to the
     * specified string.
     *
     * @param str the encoded component size
     *
     * @return the corresponding ComponentSize or null if none matches
     */
    static ComponentSize valueOf(String str) {
      if ("m".equals(str) || "min".equals(str)) { //$NON-NLS-1$ //$NON-NLS-2$
        return MINIMUM;
      }
      if ("p".equals(str) || "pref".equals(str)) { //$NON-NLS-1$ //$NON-NLS-2$
        return PREFERRED;
      }
      if ("d".equals(str) || //$NON-NLS-1$
          "default".equals(str)) { //$NON-NLS-1$
        return DEFAULT;
      }
      return null;
    }

    /**
     * Computes the maximum size for the given list of components, using this
     * form spec and the specified measure.<p>
     *
     * Invoked by FormLayout to determine the size of one of my elements
     *
     * @param container the layout container
     * @param components the list of components to measure
     * @param minMeasure the measure used to determine the minimum size
     * @param prefMeasure the measure used to determine the preferred size
     * @param defaultMeasure the measure used to determine the default size
     *
     * @return the maximum size in pixels for the given list of components
     */
    public int maximumSize(JQContainer container, List components,
                           FormLayout.Measure minMeasure,
                           FormLayout.Measure prefMeasure,
                           FormLayout.Measure defaultMeasure) {
      FormLayout.Measure measure;
      if (MINIMUM == this) {
        measure = minMeasure;
      } else if (PREFERRED == this) {
        measure = defaultMeasure;
      } else {
        measure = minMeasure;
      }
      int maximum = 0;
      for (Iterator i = components.iterator(); i.hasNext();) {
        JQComponent c = (JQComponent)i.next();
        maximum = Math.max(maximum, measure.sizeOf(c));
      }
      return maximum;
    }

    /**
     * Describes if this Size can be compressed, if container space gets scarce.
     * Used by the FormLayout size computations in <code>#compressedSizes</code>
     * to check whether a column or row can be compressed or not.<p>
     *
     * The DEFAULT ComponentSize is compressible, MINIMUM and PREFERRED are
     * incompressible.
     *
     * @return <tt>true</tt> for the DEFAULT size, <tt>false</tt> otherwise
     */
    public boolean compressible() {
      return (DEFAULT == this);
    }

    public String toString() {
      return name.substring(0, 1);
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-11-02 00:57:13 Slobodan
   QSwing port

*****************************************************************************^*/
