/*
 * @(#)BoundedSize.java  0.2.0 / 2007-11-27
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.factories;

import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.ColumnSpec;
import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.ConstantSize;
import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.RowSpec;
import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.Sizes;
import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.util.LayoutStyle;

/**
 * A factory that creates instances of FormLayout for frequently used form
 * layouts. It makes form creation easier and more consistent.<p>
 *
 * The forms are described by major and minor columns. Major columnsconsist of a
 * leading label and a set of related components, for example: a label plus
 * textfield, or label plus textfield plus button. The component part of a major
 * column is divided into minor columns as shown in this layout:
 * <pre>
 * <-    major column 1        ->  <-     major column 2       ->
 * label1 textfield1a textfield1b  label2 textfield2a textfield2b
 * label3 textfield3a textfield3b  label4 textfield4
 * label5 textfield5               label6 textfield6
 * </pre><p>
 *
 * Many forms use 1, 2, 3 or 4 major columns, which in turn are often split into
 * 1, 2, 3 or 4 minor columns.
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.3 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class FormFactory {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * An unmodifyable <tt>ColumnSpec</tt> that determines its width by computing
   * the maximum of all column component minimum widths.
   */
  public static final ColumnSpec MIN_COLSPEC = new ColumnSpec(Sizes.MINIMUM);

  /**
   * An unmodifyable <tt>ColumnSpec</tt> that determines its width by computing
   * the maximum of all column component preferred widths.
   */
  public static final ColumnSpec PREF_COLSPEC = new ColumnSpec(Sizes.PREFERRED);

  /**
   * An unmodifyable <tt>ColumnSpec</tt> that determines its preferred width by
   * computing the maximum of all column component preferred widths and its
   * minimum width by computing all column component minimum widths.<p>
   *
   * Useful to let a column shrink from preferred width to minimum width if the
   * container space gets scarce.
   */
  public static final ColumnSpec DEFAULT_COLSPEC =
                                                 new ColumnSpec(Sizes.DEFAULT);

  /**
   * An unmodifyable <tt>ColumnSpec</tt> that has an initial width of 0 pixels
   * and that grows. Useful to describe <em>glue</em> columns that fill the
   * space between other columns.
   */
  public static final ColumnSpec GLUE_COLSPEC =
        new ColumnSpec(ColumnSpec.DEFAULT, Sizes.ZERO, ColumnSpec.DEFAULT_GROW);

  /**
   * Describes a logical horizontal gap between a label and an associated
   * component. Useful for builders that automatically fill a grid with labels
   * and components.<p>
   *
   * <strong>Note:</strong> In a future version this constant will likely be
   * moved to a class <code>LogicalSize</code> or <code>StyledSize</code>.
   */
  public static final ColumnSpec LABEL_COMPONENT_GAP_COLSPEC =
          createGapColumnSpec(LayoutStyle.getCurrent().getLabelComponentPadX());

  /**
   * Describes a logical horizontal gap between two related components. For
   * example the <em>OK</em> and <em>Cancel</em> buttons are considered related.
   * <p>
   *
   * <strong>Note:</strong> In a future version this constant will likely
   * be moved to a class <code>LogicalSize</code> or <code>StyledSize</code>.
   */
  public static final ColumnSpec RELATED_GAP_COLSPEC =
       createGapColumnSpec(LayoutStyle.getCurrent().getRelatedComponentsPadX());

  /**
   * Describes a logical horizontal gap between two unrelated components.<p>
   *
   * <strong>Note:</strong> In a future version this constant will likely
   * be moved to a class <code>LogicalSize</code> or <code>StyledSize</code>.
   */
  public static final ColumnSpec UNRELATED_GAP_COLSPEC =
    createGapColumnSpec(LayoutStyle.getCurrent().getUnrelatedComponentsPadX());

  /**
   * Describes a logical horizontal column for a fixed size button. This spec
   * honors the current layout style's default button minimum width.<p>
   *
   * <strong>Note:</strong> In a future version this constant will likely be
   * moved to a class <code>LogicalSize</code> or <code>StyledSize</code>.
   */
  public static final ColumnSpec BUTTON_COLSPEC =
      new ColumnSpec(Sizes.bounded(Sizes.PREFERRED, LayoutStyle.getCurrent().
                                   getDefaultButtonWidth(), null));

  /**
   * Describes a logical horizontal column for a growing button. This spec
   * does <em>not</em> use the layout style's default button minimum width.<p>
   *
   * <strong>Note:</strong> In a future version this constant will likely
   * be moved to a class <code>LogicalSize</code> or <code>StyledSize</code>.
   */
  public static final ColumnSpec GROWING_BUTTON_COLSPEC =
      new ColumnSpec(ColumnSpec.DEFAULT, BUTTON_COLSPEC.getSize(),
                     ColumnSpec.DEFAULT_GROW);

  /**
   * An unmodifyable <code>RowSpec</code> that determines its height by
   * computing the maximum of all column component minimum heights.
   */
  public static final RowSpec MIN_ROWSPEC = new RowSpec(Sizes.MINIMUM);

  /**
   * An unmodifyable <code>RowSpec</code> that determines its height by
   * computing the maximum of all column component preferred heights.
   */
  public static final RowSpec PREF_ROWSPEC = new RowSpec(Sizes.PREFERRED);

  /**
   * An unmodifyable <code>RowSpec</code> that determines its preferred
   * height by computing the maximum of all column component preferred heights
   * and its minimum height by computing all column component minimum heights.
   * <p>
   *
   * Useful to let a column shrink from preferred height to minimum height
   * if the container space gets scarce.
   */
  public static final RowSpec DEFAULT_ROWSPEC = new RowSpec(Sizes.DEFAULT);

  /**
   * An unmodifyable <code>RowSpec</code> that has an initial height of 0 pixels
   * and that grows. Useful to describe <em>glue</em> rows that fill the space
   * between other rows.
   */
  public static final RowSpec GLUE_ROWSPEC =
      new RowSpec(RowSpec.DEFAULT, Sizes.ZERO, RowSpec.DEFAULT_GROW);

  /**
   * Describes a logical vertzical gap between two related components. For
   * example the <em>OK</em> and <em>Cancel</em> buttons are considered
   * related.<p>
   *
   * <strong>Note:</strong> In a future version this constant will likely
   * be moved to a class <code>LogicalSize</code> or <code>StyledSize</code>.
   */
  public static final RowSpec RELATED_GAP_ROWSPEC =
      createGapRowSpec(LayoutStyle.getCurrent().getRelatedComponentsPadY());

  /**
   * Describes a logical vertical gap between two unrelated components.<p>
   *
   * <strong>Note:</strong> In a future version this constant will likely
   * be moved to a class <code>LogicalSize</code> or <code>StyledSize</code>.
   *
   * @see #RELATED_GAP_ROWSPEC
   */
  public static final RowSpec UNRELATED_GAP_ROWSPEC =
      createGapRowSpec(LayoutStyle.getCurrent().getUnrelatedComponentsPadY());

  /**
   * Describes a logical vertical narrow gap between two rows in the grid.
   * Useful if the vertical space is scarce or if an individual vertical gap
   * shall be small than the default line gap.<p>
   *
   * <strong>Note:</strong> In a future version this constant will likely
   * be moved to a class <code>LogicalSize</code> or <code>StyledSize</code>.
   */
  public static final RowSpec NARROW_LINE_GAP_ROWSPEC =
      createGapRowSpec(LayoutStyle.getCurrent().getNarrowLinePad());

  /**
   * Describes the logical vertical default gap between two rows in the grid.
   * A little bit larger than the narrow line gap.<p>
   *
   * <strong>Note:</strong> In a future version this constant will likely
   * be moved to a class <code>LogicalSize</code> or <code>StyledSize</code>.
   */
  public static final RowSpec LINE_GAP_ROWSPEC =
      createGapRowSpec(LayoutStyle.getCurrent().getLinePad());

  /**
   * Describes the logical vertical default gap between two paragraphs in
   * the layout grid. This gap is larger than the default line gap.<p>
   *
   * <strong>Note:</strong> In a future version this constant will likely
   * be moved to a class <code>LogicalSize</code> or <code>StyledSize</code>.
   */
  public static final RowSpec PARAGRAPH_GAP_ROWSPEC =
      createGapRowSpec(LayoutStyle.getCurrent().getParagraphPad());

  /**
   * Creates and returns a {@link ColumnSpec} that represents a gap with the
   * specified {@link ConstantSize}.
   *
   * @param gapSize a <code>ConstantSize</code> that specifies the gap
   *
   * @return a <code>ColumnSpec</code> that describes a horizontal gap
   */
  public static ColumnSpec createGapColumnSpec(ConstantSize gapSize) {
    return new ColumnSpec(ColumnSpec.LEFT, gapSize, ColumnSpec.NO_GROW);
  }

  /**
   * Creates and returns a {@link RowSpec} that represents a gap with the
   * specified {@link ConstantSize}.
   *
   * @param gapSize a <tt>ConstantSize</tt> that specifies the gap
   *
   * @return a <tt>RowSpec</tt> that describes a vertical gap
   */
  public static RowSpec createGapRowSpec(ConstantSize gapSize) {
    return new RowSpec(RowSpec.TOP, gapSize, RowSpec.NO_GROW);
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private FormFactory() {
    // empty constructor body
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-11-27 12:29:19 Slobodan
   QSwing port

*****************************************************************************^*/
