/*
 * @(#)AbstractUnitConverter.java  0.2.0 / 2007-10-30
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.util;

import yu.ac.bg.etf.javaqx.qswing.JQComponent;
import yu.ac.bg.etf.javaqx.qswing.JQDesktop;
import yu.ac.bg.etf.javaqx.qswing.graphics.FontMetrics;

/**
 * An abstract implementation of the {@link UnitConverter} interface that
 * minimizes the effort required to convert font-dependent sizes to pixels.
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.2 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public abstract class AbstractUnitConverter implements UnitConverter {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final int DTP_RESOLUTION = 72;
  private static int screenResolution = -1;

/*^**************************************************************************^*/
/*- Unit Converter Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * Converts Inches and returns pixels using the specified resolution.
   *
   * @param in the inches
   * @param component the component that provides the graphics object
   *
   * @return the given Inches as pixels
   */
  public int inchAsPixel(double in, JQComponent component) {
    return inchAsPixel(in, getScreenResolution());
  }

  /**
   * Converts Millimeters and returns pixels using the resolution of the given
   * component's graphics object.
   *
   * @param mm the millimeters
   * @param component the component that provides the graphics object
   *
   * @return the given millimeters as pixels
   */
  public int millimeterAsPixel(double mm, JQComponent component) {
    return millimeterAsPixel(mm, getScreenResolution());
  }

  /**
   * Converts Centimeters and returns pixels using the resolution of the
   * given component's graphics object.
   *
   * @param cm the centimeters
   * @param component the component that provides the graphics object
   *
   * @return the given Centimeters as pixels
   */
  public int centimeterAsPixel(double cm, JQComponent component) {
    return centimeterAsPixel(cm, getScreenResolution());
  }

  /**
   * Converts DTP Points and returns pixels using the resolution of the
   * given component's graphics object.
   *
   * @param pt the DTP points
   * @param component the component that provides the graphics object
   *
   * @return the given Points as pixels
   */
  public int pointAsPixel(int pt, JQComponent component) {
    return pointAsPixel(pt, getScreenResolution());
  }

  /**
   * Converts horizontal dialog units and returns pixels. Honors the resolution,
   * dialog font size, platform, and Style.
   *
   * @param dluX the horizontal dialog units
   * @param c a Component that provides the font and graphics
   *
   * @return the given horizontal dialog units as pixels
   */
  public int dialogUnitXAsPixel(int dluX, JQComponent c) {
    return dialogUnitXAsPixel(dluX, getDialogBaseUnitsX(c));
  }

  /**
   * Converts vertical dialog units and returns pixels. Honors the resolution,
   * dialog font size, platform, and Style.
   *
   * @param dluY the vertical dialog units
   * @param c a Component that provides the font and graphics
   *
   * @return the given vertical dialog units as pixels
   */
  public int dialogUnitYAsPixel(int dluY, JQComponent c) {
    return dialogUnitYAsPixel(dluY, getDialogBaseUnitsY(c));
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Gets and returns the horizontal dialog base units. Implementations are
   * encouraged to cache previously computed dialog base units.
   *
   * @param component a component that provides the font and graphics
   *
   * @return the horizontal dialog base units
   */
  protected abstract double getDialogBaseUnitsX(JQComponent component);

  /**
   * Gets and returns the vertical dialog base units.
   * Implementations are encouraged to cache previously computed
   * dialog base units.
   *
   * @param component a Component that provides the font and graphics
   *
   * @return the vertical dialog base units
   */
  protected abstract double getDialogBaseUnitsY(JQComponent component);

  /**
   * Converts Inches and returns pixels using the specified resolution.
   *
   * @param in the inches
   * @param dpi the resolution
   *
   * @return the given Inches as pixels
   */
  protected final int inchAsPixel(double in, int dpi) {
    return (int)Math.round(dpi * in);
  }

  /**
   * Converts Millimeters and returns pixels using the specified resolution.
   *
   * @param mm the millimeters
   * @param dpi the resolution
   *
   * @return the given Millimeters as pixels
   */
  protected final int millimeterAsPixel(double mm, int dpi) {
    return (int)Math.round(dpi * mm * 10 / 254);
  }

  /**
   * Converts Centimeters and returns pixels using the specified resolution.
   *
   * @param cm Centimeters
   * @param dpi the resolution
   *
   * @return the given Centimeters as pixels
   */
  protected final int centimeterAsPixel(double cm, int dpi) {
    return (int)Math.round(dpi * cm * 100 / 254);
  }

  /**
   * Converts DTP Points and returns pixels using the specified resolution.
   *
   * @param pt DTP Points
   * @param dpi the resolution in dpi
   *
   * @return the given Points as pixels
   */
  protected final int pointAsPixel(int pt, int dpi) {
    return Math.round((float)(dpi * pt) / DTP_RESOLUTION);
  }

  /**
   * Converts horizontal dialog units and returns pixels.
   *
   * @param dluX the horizontal dialog units
   * @param dialogBaseUnitsX the horizontal dialog base units
   *
   * @return the given dialog base units as pixels
   */
  protected int dialogUnitXAsPixel(int dluX, double dialogBaseUnitsX) {
    return (int)Math.round(dluX * dialogBaseUnitsX / 4);
  }

  /**
   * Converts vertical dialog units and returns pixels.
   *
   * @param dluY the vertical dialog units
   * @param dialogBaseUnitsY the vertical dialog base units
   *
   * @return the given dialog base units as pixels
   */
  protected int dialogUnitYAsPixel(int dluY, double dialogBaseUnitsY) {
    return (int)Math.round(dluY * dialogBaseUnitsY / 8);
  }

  /**
   * Computes and returns the average character width
   * of the specified test string using the given FontMetrics.
   * The test string shall represent an "average" text.
   *
   * @param metrics used to compute the test string's width
   * @param testString the string that shall represent an "average" text
   *
   * @return the test string's average character width.
   */
  protected double computeAverageCharWidth(FontMetrics metrics,
                                           String testString) {
     int width = metrics.stringWidth(testString);
     double average = (double) width / testString.length();
     //System.out.println("Average width of '" + testString + "'=" + average);
     return average;
   }

  /**
   * Returns the components screen resolution or the default screen
   * resolution if the component is null or has no toolkit assigned yet.
   *
   * @return the component's screen resolution
   */
  protected int getScreenResolution() {
    if (screenResolution == -1) {
      screenResolution = JQDesktop.getScreenResolution();
    }
    return screenResolution;
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-10-30 16:32:44 Slobodan
   QSwing port

*****************************************************************************^*/
