/*
 * @(#)DefaultUnitConverter.java  0.1.0 / 2007-10-30
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.util;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Map;

import yu.ac.bg.etf.javaqx.qswing.JQComponent;
import yu.ac.bg.etf.javaqx.qswing.QSwing;
import yu.ac.bg.etf.javaqx.qswing.graphics.Font;
import yu.ac.bg.etf.javaqx.qswing.graphics.FontMetrics;

/**
 * This is the default implementation of the {@link UnitConverter} interface.
 * It converts horizontal and vertical dialog base units to pixels.<p>
 *
 * The horizontal base unit is equal to the average width, in pixels, of the
 * characters in the system font; the vertical base unit is equal to the height,
 * in pixels, of the font.
 * Each horizontal base unit is equal to 4 horizontal dialog units;
 * each vertical base unit is equal to 8 vertical dialog units.<p>
 *
 * The DefaultUnitConverter computes dialog base units using a default font
 * and a test string for the average character width. You can configure
 * the font and the test string via the bound Bean properties
 * <em>defaultDialogFont</em> and <em>averageCharacterWidthTestString</em>.
 * See also Microsoft's suggestion for a custom computation
 * <a href="http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnwue/html/ch14e.asp">here</a>.<p>
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.2 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class DefaultUnitConverter extends AbstractUnitConverter {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static DefaultUnitConverter instance;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private String averageCharWidthTestString;
  private Font defaultDialogFont;
  private FontMetrics defaultDialogFontMetrics;
  private DialogBaseUnits cachedGlobalDialogBaseUnits;
  private final Map cachedDialogBaseUnits;
  private Font cachedDefaultDialogFont;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs a DefaultUnitConverter and registers
   * a listener that handles changes in the Style.
   */
  private DefaultUnitConverter() {
    averageCharWidthTestString = "X"; //$NON-NLS-1$
    cachedGlobalDialogBaseUnits = computeGlobalDialogBaseUnits();
    cachedDialogBaseUnits = new HashMap();
    StyleChangeListener scl = new StyleChangeListener();
    QSwing.addPropertyChangeListener(QSwing.PROPERTY_STYLE, scl);
    QSwing.addPropertyChangeListener(QSwing.PROPERTY_THEME, scl);
    QSwing.addPropertyChangeListener(QSwing.PROPERTY_STYLE_SHEET, scl);
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Lazily instantiates and returns the sole instance.
   *
   * @return the lazily instantiated sole instance
   */
  public static DefaultUnitConverter getInstance() {
    if (instance == null) { // NOPMD
      instance = new DefaultUnitConverter();
    }
    return instance;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the string used to compute the average character width.
   * By default it is initialized to &quot;X&quot;.
   *
   * @return the test string used to compute the average character width
   */
  public String getAverageCharacterWidthTestString() {
    return averageCharWidthTestString;
  }

  /**
   * Sets a string that will be used to compute the average character width.
   * By default it is initialized to &quot;X&quot;. You can provide other test
   * strings, for example:
   * <ul>
   *  <li>&quot;Xximeee&quot;</li>
   *  <li>&quot;ABCEDEFHIJKLMNOPQRSTUVWXYZ&quot;</li>
   *  <li>&quot;abcdefghijklmnopqrstuvwxyz&quot;</li>
   * </ul>
   *
   * @param testString the test string to be used
   *
   * @throws IllegalArgumentException if the test string is empty
   * @throws NullPointerException     if the test string is <code>null</code>
   */
  public void setAverageCharacterWidthTestString(String testString) {
    if (testString == null) {
      throw new IllegalArgumentException("Null test string"); //$NON-NLS-1$
    }
    if (testString.length() == 0) {
      throw new IllegalArgumentException("Empty test string"); //$NON-NLS-1$
    }
    averageCharWidthTestString = testString;
  }

  /**
   * Returns the dialog font that is used to compute the dialog base units.
   * If a default dialog font has been set using
   * {@link #setDefaultDialogFont(Font)}, this font will be returned.
   * Otherwise a cached fallback will be lazily created.
   *
   * @return the font used to compute the dialog base units
   */
  public Font getDefaultDialogFont() {
    if (defaultDialogFont == null) {
      return getCachedDefaultDialogFont();
    }
    return defaultDialogFont;
  }

  /**
   * Sets a dialog font that will be used to compute the dialog base units.
   *
   * @param defaultDialogFont the default dialog font to be set
   */
  public void setDefaultDialogFont(Font defaultDialogFont) {
    this.defaultDialogFont = defaultDialogFont;
    defaultDialogFontMetrics = null;
  }

  /**
   * Returns default dialog font metrics.
   *
   * @return default dialog font metrics
   */
  public FontMetrics getDefaultDialogFontMetrics() {
    if (defaultDialogFontMetrics == null) {
      defaultDialogFontMetrics = new FontMetrics(getDefaultDialogFont());
    }
    return defaultDialogFontMetrics;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Returns the cached or computed horizontal dialog base units.
   *
   * @param component a Component that provides the font and graphics
   *
   * @return the horizontal dialog base units
   */
  protected double getDialogBaseUnitsX(JQComponent component) {
    return getDialogBaseUnits(component).x;
  }

  /**
   * Returns the cached or computed vertical dialog base units
   * for the given component.
   *
   * @param component a Component that provides the font and graphics
   *
   * @return the vertical dialog base units
   */
  protected double getDialogBaseUnitsY(JQComponent component) {
    return getDialogBaseUnits(component).y;
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Invalidates the caches. Resets the global dialog base units, clears the Map
   * from <code>FontMetrics</code> to dialog base units, and resets the fallback
   * for the default dialog font. This is invoked after a change of the Style.
   */
  void invalidateCaches() {
    cachedGlobalDialogBaseUnits = null;
    cachedDialogBaseUnits.clear();
    cachedDefaultDialogFont = null;
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  /**
   * Lazily computes and answer the global dialog base units.
   * Should be re-computed if the Style, platform, or screen changes.
   *
   * @return a cached DialogBaseUnits object used globally if no container is
   *         available
   */
  private DialogBaseUnits getGlobalDialogBaseUnits() {
    if (cachedGlobalDialogBaseUnits == null) {
      cachedGlobalDialogBaseUnits = computeGlobalDialogBaseUnits();
    }
    return cachedGlobalDialogBaseUnits;
  }

  /**
   * Looks up and returns the dialog base units for the given component.
   * In case the component is <code>null</code> the global dialog base units
   * are answered.<p>
   *
   * Before we compute the dialog base units we check whether they have been
   * computed and cached before - for the same component <tt>FontMetrics</tt>.
   *
   * @param c  the component that provides the graphics object
   *
   * @return the DialogBaseUnits object for the given component
   */
  private DialogBaseUnits getDialogBaseUnits(JQComponent c) {
    if (c == null) {
      return getGlobalDialogBaseUnits();
    }
    FontMetrics fm = getDefaultDialogFontMetrics();
    DialogBaseUnits dialogBaseUnits = (DialogBaseUnits)
                                                 cachedDialogBaseUnits.get(fm);
    if (dialogBaseUnits == null) {
      dialogBaseUnits = computeDialogBaseUnits(fm);
      cachedDialogBaseUnits.put(fm, dialogBaseUnits);
    }
    return dialogBaseUnits;
  }

  /**
   * Computes and returns the horizontal dialog base units. Honors the font,
   * font size and resolution.<p>
   *
   * Implementation Note: 14dluY map to 22 pixel for 8pt Tahoma on 96 dpi.
   * I could not yet manage to compute the Microsoft complaint font height.
   * Therefore this method adds a correction value that seems to work well with
   * the vast majority of desktops.<p>
   *
   * @todo Revise the computation of vertical base units as soon as there are
   * more information about the original computation in Microsoft environments.
   *
   * @param metrics  the FontMetrics used to measure the dialog font
   *
   * @return the horizontal and vertical dialog base units
   */
  private DialogBaseUnits computeDialogBaseUnits(FontMetrics metrics) {
    double averageCharWidth =  computeAverageCharWidth(metrics,
                                                   averageCharWidthTestString);
    int ascent = metrics.getAscent();
    if (ascent > 14) {
      return new DialogBaseUnits(averageCharWidth, ascent);
    }
    return new DialogBaseUnits(averageCharWidth, ascent + (15 - ascent) / 3);
  }

  /**
   * Computes the global dialog base units. The current implementation assumes a
   * fixed 8pt font and on 96 or 120 dpi. A better implementation should ask for
   * the main dialog font and should honor the current screen resolution.<p>
   *
   * Should be re-computed if the Style, platform, or screen changes.
   *
   * @return a DialogBaseUnits object used globally if no container is available
   */
  private DialogBaseUnits computeGlobalDialogBaseUnits() {
    return computeDialogBaseUnits(getDefaultDialogFontMetrics());
  }

  /**
   * Lazily creates and returns a fallback for the dialog font that is used to
   * compute the dialog base units. This fallback font is cached and will be
   * reset if the Style changes.
   *
   * @return the cached fallback font used to compute the dialog base units
   */
  private Font getCachedDefaultDialogFont() {
    if (cachedDefaultDialogFont == null) {
      cachedDefaultDialogFont = QSwing.getTheme().getFontSet().getButtonFont();
    }
    return cachedDefaultDialogFont;
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Describes horizontal and vertical dialog base units.
   */
  static final class DialogBaseUnits {
    final double x;
    final double y;
    public DialogBaseUnits(double dialogBaseUnitsX, double dialogBaseUnitsY) {
      x = dialogBaseUnitsX;
      y = dialogBaseUnitsY;
    }
    public String toString() {
      return "DBU(x=" + x + "; y=" + y + //$NON-NLS-1$ //$NON-NLS-2$
              ")"; //$NON-NLS-1$
    }
  }

  /**
   * Listens to changes of the Style and invalidates the cache.
   */
  final class StyleChangeListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent evt) {
      invalidateCaches();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-10-30 17:56:26 Slobodan
   Initial version

*****************************************************************************^*/
