/*
 * @(#)LayoutStyle.java  0.1.0 / 2007-11-20
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.util;

import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.ConstantSize;
import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.Size;

/**
 * An abstract class that describes a layout and design style guide. It provides
 * constants used to lay out panels consistently.<p>
 *
 * This class is work in progress and the API may change without notice.
 * Therefore it is recommended to not write custom subclasses for production
 * code. A future version of this class will likely collaborate with a class
 * <code>LogicalSize</code> or <code>StyledSize</code>.
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.2 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public abstract class LayoutStyle {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Holds the current layout style.
   */
  private static LayoutStyle current = initialLayoutStyle();

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns the current <code>LayoutStyle</code>.
   *
   * @return the current <code>LayoutStyle</code>
   */
  public static LayoutStyle getCurrent() {
    return current;
  }

  /**
   * Set a new <code>LayoutStyle</code>.
   *
   * @param newLayoutStyle the style to be set
   */
  public static void setCurrent(LayoutStyle newLayoutStyle) {
    current = newLayoutStyle;
  }

/*^**************************************************************************^*/
/*- Public Absract Methods.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Returns this style's default button width.
   *
   * @return the default button width
   */
  public abstract Size getDefaultButtonWidth();

  /**
   * Returns this style's default button height.
   *
   * @return the default button height
   */
  public abstract Size getDefaultButtonHeight();

  /**
   * Returns this style's horizontal margin for general dialogs.
   *
   * @return the horizontal margin for general dialogs
   */
  public abstract ConstantSize getDialogMarginX();

  /**
   * Returns this style's vertical margin for general dialogs.
   *
   * @return the vertical margin for general dialogs
   */
  public abstract ConstantSize getDialogMarginY();

  /**
   * Returns this style's horizontal margin for dialogs that consist of a tabbed
   * pane.
   *
   * @return the horizontal margin for dialogs that consist of a tabbed pane
   */
  public abstract ConstantSize getTabbedDialogMarginX();

  /**
   * Returns this style's vertical margin for dialogs that consist of a tabbed
   * pane.
   *
   * @return the vertical margin for dialogs that consist of a tabbed pane
   */
  public abstract ConstantSize getTabbedDialogMarginY();

  /**
   * Returns a gap used to separate a label and associated control.
   *
   * @return a gap between label and associated control
   */
  public abstract ConstantSize getLabelComponentPadX();

  /**
   * Returns a horizontal gap used to separate related controls.
   *
   * @return a horizontal gap between related controls
   */
  public abstract ConstantSize getRelatedComponentsPadX();

  /**
   * Returns a vertical gap used to separate related controls.
   *
   * @return a vertical gap between related controls
   */
  public abstract ConstantSize getRelatedComponentsPadY();

  /**
   * Returns a horizontal gap used to separate unrelated controls.
   *
   * @return a horizontal gap between unrelated controls
   */
  public abstract ConstantSize getUnrelatedComponentsPadX();

  /**
   * Returns a vertical gap used to separate unrelated controls.
   *
   * @return a vertical gap between unrelated controls
   */
  public abstract ConstantSize getUnrelatedComponentsPadY();

  /**
   * Returns a narrow vertical pad used to separate lines.
   *
   * @return a narrow vertical pad used to separate lines
   */
  public abstract ConstantSize getNarrowLinePad();

  /**
   * Returns a narrow vertical pad used to separate lines.
   *
   * @return a vertical pad used to separate lines
   */
  public abstract ConstantSize getLinePad();

  /**
   * Returns a pad used to separate paragraphs.
   *
   * @return a vertical pad used to separate paragraphs
   */
  public abstract ConstantSize getParagraphPad();

  /**
   * Returns a pad used to separate a button bar from a component.
   *
   * @return a vertical pad used to separate paragraphs
   */
  public abstract ConstantSize getButtonBarPad();

  /**
   * Checks and answers whether buttons are typically ordered from left to right
   * or from right to left. Useful for building button bars that shall comply
   * with the platform's layout style guide.<p>
   *
   * For example the Windows style guide recommends to layout out
   * <em>OK, Cancel, Apply</em> from left to right, where the Mac Aqua style
   * guide recommends to layout out these buttons from right to left.<p>
   *
   * Although most button sequences shall honor this order
   * some buttons require a left to right order. For example
   * <em>Back, Next</em> or <em>Move Left, Move Right</em>.<p>
   *
   * @return <tt>true</tt> if buttons are typically ordered from left to right
   */
  public abstract boolean isLeftToRightButtonOrder();

/*^**************************************************************************^*/
/*- Private Static Method.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Computes and returns the initial <code>LayoutStyle</code>. Checks the OS
   * name and returns <code>MacLayoutStyle</code> on Mac OS X and
   * <code>WindowLayoutStyle</code> on all other platforms.
   *
   * @return MacLayoutStyle on Mac, WindowsLayoutStyle on all other platforms
   */
  private static LayoutStyle initialLayoutStyle() {
    /** @todo  finish */
    //if (isOSMac()) return MacLayoutStyle.INSTANCE;
    return WindowsLayoutStyle.INSTANCE;
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-11-20 18:23:00 Slobodan
   QSwing port

*****************************************************************************^*/
