/*
 * @(#)WindowsLayoutStyle.java  0.1.0 / 2007-11-20
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.util;

import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.ConstantSize;
import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.Size;
import yu.ac.bg.etf.javaqx.qswing.layouts.formlayout.Sizes;

/**
 * A {@link LayoutStyle} that aims to provide layout constants as defined by
 * Microsoft's <i>Design Specifications and Guidelines - Visual Design</i>.
 *
 * <p>
 *   <tt>NOTE: This code is part of the
 *       <a href="http://www.jgoodies.org">JGoodies Forms</a> port.</tt>
 * </p>
 *
 * @author Karsten Lentzsch
 * @version $Revision: 1.2 $
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
final class WindowsLayoutStyle extends LayoutStyle {

/*^**************************************************************************^*/
/*- Pacakge-Private Static Final Field.                                      -*/
/*^**************************************************************************^*/

    static final WindowsLayoutStyle INSTANCE = new WindowsLayoutStyle();

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final Size BUTTON_WIDTH = Sizes.dluX(50);
  private static final Size BUTTON_HEIGHT = Sizes.dluY(14);
  private static final ConstantSize DIALOG_MARGIN_X = Sizes.DLUX7;
  private static final ConstantSize DIALOG_MARGIN_Y = Sizes.DLUY7;
  private static final ConstantSize TABBED_DIALOG_MARGIN_X = Sizes.DLUX4;
  private static final ConstantSize TABBED_DIALOG_MARGIN_Y = Sizes.DLUY4;
  private static final ConstantSize LABEL_COMPONENT_PADX = Sizes.DLUX3;
  private static final ConstantSize RELATED_COMPONENTS_PADX = Sizes.DLUX4;
  private static final ConstantSize UNRELATED_COMPONENTS_PADX = Sizes.DLUX7;
  private static final ConstantSize RELATED_COMPONENTS_PADY = Sizes.DLUY4;
  private static final ConstantSize UNRELATED_COMPONENTS_PADY = Sizes.DLUY7;
  private static final ConstantSize NARROW_LINE_PAD = Sizes.DLUY2;
  private static final ConstantSize LINE_PAD = Sizes.DLUY3;
  private static final ConstantSize PARAGRAPH_PAD = Sizes.DLUY9;
  private static final ConstantSize BUTTON_BAR_PAD = Sizes.DLUY5;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private WindowsLayoutStyle() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Returns this style's default button width.
   *
   * @return the default button width
   */
  public Size getDefaultButtonWidth() {
    return BUTTON_WIDTH;
  }

  /**
   * Returns this style's default button height.
   *
   * @return the default button height
   */
  public Size getDefaultButtonHeight() {
    return BUTTON_HEIGHT;
  }

  /**
   * Returns this style's horizontal margin for general dialogs.
   *
   * @return the horizontal margin for general dialogs
   */
  public ConstantSize getDialogMarginX() {
    return DIALOG_MARGIN_X;
  }

  /**
   * Returns this style's vertical margin for general dialogs.
   *
   * @return the vertical margin for general dialogs
   */
  public ConstantSize getDialogMarginY() {
    return DIALOG_MARGIN_Y;
  }

  /**
   * Returns this style's horizontal margin for dialogs that consist of a tabbed
   * pane.
   *
   * @return the horizontal margin for dialogs that consist of a tabbed pane
   */
  public ConstantSize getTabbedDialogMarginX() {
    return TABBED_DIALOG_MARGIN_X;
  }

  /**
   * Returns this style's vertical margin for dialogs that consist of
   * a tabbed pane.
   *
   * @return the vertical margin for dialogs that consist of a tabbed pane
   */
  public ConstantSize getTabbedDialogMarginY() {
    return TABBED_DIALOG_MARGIN_Y;
  }

  /**
   * Returns a gap used to separate a label and associated control.
   *
   * @return a gap between label and associated control
   */
  public ConstantSize getLabelComponentPadX() {
    return LABEL_COMPONENT_PADX;
  }

  /**
   * Returns a horizontal gap used to separate related controls.
   *
   * @return a horizontal gap between related controls
   */
  public ConstantSize getRelatedComponentsPadX() {
    return RELATED_COMPONENTS_PADX;
  }

  /**
   * Returns a vertical gap used to separate related controls.
   *
   * @return a vertical gap between related controls
   */
  public ConstantSize getRelatedComponentsPadY() {
    return RELATED_COMPONENTS_PADY;
  }

  /**
   * Returns a horizontal gap used to separate unrelated controls.
   *
   * @return a horizontal gap between unrelated controls
   */
  public ConstantSize getUnrelatedComponentsPadX() {
    return UNRELATED_COMPONENTS_PADX;
  }

  /**
   * Returns a vertical gap used to separate unrelated controls.
   *
   * @return a vertical gap between unrelated controls
   */
  public ConstantSize getUnrelatedComponentsPadY() {
    return UNRELATED_COMPONENTS_PADY;
  }

  /**
   * Returns a narrow vertical pad used to separate lines.
   *
   * @return a narrow vertical pad used to separate lines
   */
  public ConstantSize getNarrowLinePad() {
    return NARROW_LINE_PAD;
  }

  /**
   * Returns a narrow vertical pad used to separate lines.
   *
   * @return a vertical pad used to separate lines
   */
  public ConstantSize getLinePad() {
    return LINE_PAD;
  }

  /**
   * Returns a pad used to separate paragraphs.
   *
   * @return a vertical pad used to separate paragraphs
   */
  public ConstantSize getParagraphPad() {
    return PARAGRAPH_PAD;
  }

  /**
   * Returns a pad used to separate a button bar from a component.
   *
   * @return a vertical pad used to separate paragraphs
   */
  public ConstantSize getButtonBarPad() {
    return BUTTON_BAR_PAD;
  }

  /**
   * Checks and answers whether buttons are typically ordered from
   * left to right or from right to left. Useful for building button bars
   * that shall comply with the platform's layout style guide.<p>
   *
   * For example the Windows style guide recommends to layout out
   * <em>OK, Cancel, Apply</em> from left to right, where the
   * Mac Aqua style guide recommends to layout out these buttons
   * from right to left.<p>
   *
   * Although most button sequences shall honor this order
   * some buttons require a left to right order. For example
   * <em>Back, Next</em> or <em>Move Left, Move Right</em>.<p>
   *
   * @return true if buttons are typically ordered from left to right
   */
  public boolean isLeftToRightButtonOrder() {
    return true;
  }

}

/*^*****************************************************************************
  File History:

 - 2007-09-07 Karsten
   Original version 1.1.0

 - 2007-11-20 18:23:00 Slobodan
   Initial version

*****************************************************************************^*/
