/*
 * @(#)QObjectBridge.cpp  0.2.0 / 2007-09-07
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "QObjectBridge.h"

/*^**************************************************************************^*/
/*- QObjectBridge :: Protected Constructors.                                 -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new QObjectBridge object.
 *
 * \param env the JNIEnv pointer
 * \param jniObj the JavaQx object
 * \param qObj the Qt object
 *
 * \exception OutOfMemoryError
 */
QObjectBridge::QObjectBridge(JNIEnv *env, jobject jniObj, QObject *qObj)
  : QObject(), m_jniObject(new JNIObject(env, jniObj)), m_qObject(0),
    m_notifyDelete(true)
{
  if (qObj) {
    setQObject(qObj);
  }
}

/*!
 * \brief Constructs new QObjectBridge object.
 *
 * \param env the JNIEnv pointer
 * \param jniObj the JavaQx object
 *
 * \exception OutOfMemoryError
 */
QObjectBridge::QObjectBridge(JNIEnv *env, jobject jniObj)
  : QObject(), m_jniObject(new JNIObject(env, jniObj)), m_qObject(0),
    m_notifyDelete(true)
{
  // empty constructor body
}

/*^**************************************************************************^*/
/*- QObjectBridge :: Protected Destructor.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Destructs QObjectBridge object.
 */
QObjectBridge::~QObjectBridge()
{
  if (m_qObject) {
    delete m_qObject;
  }
}

/*^**************************************************************************^*/
/*- QObjectBridge :: Private Constructor.                                    -*/
/*^**************************************************************************^*/

QObjectBridge::QObjectBridge(const QObjectBridge &)
  : QObject()
{
  // copy constructor is disabled
}

/*^**************************************************************************^*/
/*- QObjectBridge :: Public Member Functions.                                -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets the object name.
 *
 * \param env the JNIEnv pointer 
 * \param name the object name 
 */
void QObjectBridge::setName(JNIEnv *env, jstring name)
{
  Q_ASSERT(env);
  Q_ASSERT(m_qObject);
  m_qObject->setObjectName(QStringAdapter(env, name));
}

/*!
 * \brief Posts the Qt event to the Qt object.
 *
 * \param e the event to be posted
 */
void QObjectBridge::postEvent(QEvent *e)
{
  Q_ASSERT(m_qObject);
  Q_ASSERT(e);
  QCoreApplication::postEvent(m_qObject, e);
}

/*!
 * \brief Sends the Qt event to the Qt object.
 *
 * \param e the event
 */
void QObjectBridge::sendEvent(QEvent *e)
{
  Q_ASSERT(m_qObject);
  Q_ASSERT(e);
  QCoreApplication::sendEvent(m_qObject, e);
}

/*!
 * \brief Destroys the QObjectBridge object.
 *
 * \param env the JNIEnv pointer
 * \param notify if \c true the Java object will be notified about destroy
 */
void QObjectBridge::destroy(JNIEnv *env, bool notify)
{
  Q_ASSERT(env);
  m_notifyDelete = notify;
  if (m_jniObject->isSameThread(env)) {
    delete this; // OK - referenced only from java
  } else {
    deleteLater();
  }
}

/*^**************************************************************************^*/
/*- QObjectBridge :: Public Static Member Function.                          -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes QObjectBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JavaQx AbstractDisposeable class
 *
 * \exception FatalError if failed
 */
void QObjectBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_disposeMID);
  if (!INIT_MID(dispose, cls, "()V")) {
    FATAL_LOGGED_ERROR("QObjectBridge initialization failed");
  }
}

/*^**************************************************************************^*/
/*- QObjectBridge :: Protected Member Functions.                             -*/
/*^**************************************************************************^*/

/*!
 * \brief Returns the JNI bridge of the javaqx object.
 *
 * \return the bridge of the javaqx object
 */
jlong QObjectBridge::bridgePointer()
{
  return m_jniObject->getLongField(bridgePointerFID());
}

/*!
 * \brief Object destroyed slot.
 *
 * \return qObj the destroyed object
 */
void QObjectBridge::objectDestroyed(QObject *qObj)
{
  if (qObj == m_qObject) {
    m_qObject = 0;
    deleteLater();
  }
}

/*!
 * \brief Sets the JNI bridge of the javaqx object.
 *
 * \param bridgePointer the bridge of the javaqx object
 */
void QObjectBridge::setBridgePointer(jlong bridgePointer)
{
  m_jniObject->setLongField(bridgePointerFID(), bridgePointer);
}

/*!
 * \brief Sets Qt object.
 *
 * \param qObj the Qt object
 */
void QObjectBridge::setQObject(QObject *qObj)
{
  if (m_qObject) {
    m_qObject->removeEventFilter(this);
    m_qObject->setProperty(BRIDGE_PROPERTY_NAME, INVALID_QVARIANT);
    disconnect(m_qObject, SIGNAL(destroyed(QObject *)),
               SLOT(objectDestroyed(QObject *)));
  }
  if (qObj) {
    m_qObject = qObj;
    connect(qObj, SIGNAL(destroyed(QObject *)), this,
            SLOT(objectDestroyed(QObject *)));
    qObj->setProperty(BRIDGE_PROPERTY_NAME, qVariantFromValue<QObject *>(this));
    qObj->installEventFilter(this);
  } else {
    m_qObject = 0;
  }
}

/*^**************************************************************************^*/
/*- QObjectBridge :: Private Static Data Member.                             -*/
/*^**************************************************************************^*/

jmethodID QObjectBridge::sm_disposeMID = 0;

/*^**************************************************************************^*/
/*- QObjectBridge :: Public Static Data Members.                             -*/
/*^**************************************************************************^*/

const char *const QObjectBridge::BRIDGE_PROPERTY_NAME = "QObjectBridge";
const QVariant QObjectBridge::INVALID_QVARIANT(QVariant::Invalid);

/*^*****************************************************************************
  File History:

 - 2007-05-20 20:47:52 Slobodan
   Initial version

*****************************************************************************^*/
