/*
 * @(#)QObjectBridge.h  0.2.0 / 2007-09-07
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#ifndef QOBJECTBRIDGE_H
#define QOBJECTBRIDGE_H

#include "JavaQxCore.h"

/*^**************************************************************************^*/
/*- QObjectBridge Class.                                                     -*/
/*^**************************************************************************^*/

/*!
 * \brief Abstract QObject-JavaQx Object Bridge.
 *
 * \author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * \version 0.2.0
 */
class JAVAQX_EXPORT QObjectBridge : public QObject
{
  Q_OBJECT
protected:
  QObjectBridge(JNIEnv *env, jobject jniObj, QObject *qObj);
  QObjectBridge(JNIEnv *env, jobject jniObj);
  virtual ~QObjectBridge();
private:
  QObjectBridge(const QObjectBridge &);
public:
  void setName(JNIEnv *env, jstring name);
  void postEvent(QEvent *e);
  void sendEvent(QEvent *e);
public:
  virtual void destroy(JNIEnv *env, bool notify = false);
public:
  static void initialize(JNIEnv *env, jclass cls);
protected:
  jlong bridgePointer();
  void setBridgePointer(jlong bridgePointer);
  void setQObject(QObject *qObj);
protected:
  virtual jfieldID bridgePointerFID() const = 0;
protected slots:
  virtual void objectDestroyed(QObject *qObj);
protected:
  inline JNIObject *jniObject() const;
  inline QObject *qObject() const;
  inline bool notifyDelete() const;
  inline void setNotifyDelete(bool notifyDelete);
  inline bool connectAuto(const QObject *sender, const char *signal,
                          const char *member) const;
  inline bool connectDirect(const QObject *sender, const char *signal,
                            const char *member) const;
  inline bool connectQueued(const char *signal, const char *member) const;
  inline bool disconnect(const char *signal, const char *member);
  inline bool disconnect(const QObject *sender, const char *signal,
                         const char *member);
protected:
  inline static jmethodID disposeMID();
private:
  JNIObject *m_jniObject;
  QObject *m_qObject;
  bool m_notifyDelete;
private:
  static jmethodID sm_disposeMID;
public:
  static const char *const BRIDGE_PROPERTY_NAME;
  static const QVariant INVALID_QVARIANT;
};

/*^**************************************************************************^*/
/*- QObjectBridge :: Protected Inline Member Functions.                      -*/
/*^**************************************************************************^*/

/*!
 * \brief Returns JNIObject pointer.
 *
 * \return JNIObject pointer
 */
inline JNIObject *QObjectBridge::jniObject() const
{
  return m_jniObject;
}

/*!
 * \brief Returns QObject pointer.
 *
 * \return QObject pointer
 */
inline QObject *QObjectBridge::qObject() const
{
  return m_qObject;
}

/*!
 * \brief Returns \c true if notifies about delete.
 *
 * \return \c true if notifies about delete
 */
inline bool QObjectBridge::notifyDelete() const
{
  return m_notifyDelete;
}

/*!
 * \brief Sets notification on object delete.
 *
 * \return \c true if notifies
 */
inline void QObjectBridge::setNotifyDelete(bool notifyDelete)
{
  m_notifyDelete = notifyDelete;
}

/*!
 * \brief Connects a signal and a slot of this object using AutoConnection mode.
 *
 * \param sender the sender
 * \param signal the signal
 * \param member the member
 *
 * \return \c true if succeed
 */
inline bool QObjectBridge::connectAuto(const QObject *sender,
                                         const char *signal,
                                         const char *member) const
{
  return connect(sender, signal, this, member, Qt::AutoConnection);
}

/*!
 * \brief Connects a signal and a slot of this object using DirectConnection
 *        mode.
 *
 * \param sender the sender
 * \param signal the signal
 * \param member the member
 *
 * \return \c true if succeed
 */
inline bool QObjectBridge::connectDirect(const QObject *sender,
                                         const char *signal,
                                         const char *member) const
{
  return connect(sender, signal, this, member, Qt::DirectConnection);
}

/*!
 * \brief Connects a signal and a slot of this object using QueuedConnection
 *        mode.
 *
 * \param signal the signal
 * \param member the member
 *
 * \return \c true if succeed
 */
inline bool QObjectBridge::connectQueued(const char *signal,
                                         const char *member) const
{
  return connect(this, signal, this, member, Qt::QueuedConnection);
}

/*!
 * \brief Disconnects a signal and a slot of this object.
 *
 * \param signal the signal
 * \param member the member
 *
 * \return \c true if succeed
 */
inline bool QObjectBridge::disconnect(const char *signal, const char *member)
{
  return QObject::disconnect(this, signal, this, member);
}

/*!
 * \brief Disconnects a signal and a slot of this object.
 *
 * \param sender the sender
 * \param signal the signal
 * \param member the member
 *
 * \return \c true if succeed
 */
inline bool QObjectBridge::disconnect(const QObject *sender, const char *signal,
                                      const char *member)
{
  return QObject::disconnect(sender, signal, this, member);
}

/*^**************************************************************************^*/
/*- QObjectBridge :: Protected Static Inline Member Function.                -*/
/*^**************************************************************************^*/

/*!
 * \brief Returns \c dispose method ID.
 *
 * \return \c dispose method ID
 */
inline jmethodID QObjectBridge::disposeMID()
{
  return sm_disposeMID;
}

/*^**************************************************************************^*/
/*- Related Non-Member Function.                                             -*/
/*^**************************************************************************^*/

/*!
 * \brief Returns JavaQx Bridge object for the specified QObject.
 *
 * \param qObj the QObject;
 *
 * \return JavaQx Bridge object for the specified QObject
 */
template <typename T>
inline T qObjectBridgePointer(QObject *qObj)
{
  return static_cast<T>(qVariantValue<QObject *>(
                          qObj->property(QObjectBridge::BRIDGE_PROPERTY_NAME)));
}

/*^**************************************************************************^*/
/*- Macro Definition.                                                        -*/
/*^**************************************************************************^*/

#define qObjectBridge (reinterpret_cast<QObjectBridge *>(bridge))

#endif // QOBJECTBRIDGE_H

/*^*****************************************************************************
  File History:

 - 2007-05-20 20:44:27 Slobodan
   Initial version

*****************************************************************************^*/
