/*
 * @(#)QApplicationAdapter.cpp  0.2.0 / 2007-05-01
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "QApplicationAdapter.h"

#include "JQComponentBridge.h"
#include "QCursorAdapter.h"

/*^**************************************************************************^*/
/*- QApplicationAdapter :: Public Constructor.                               -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new QApplicationAdapter object.
 *
 * \param argc the number of command line arguments
 * \param argv the command line arguments
 */
QApplicationAdapter::QApplicationAdapter(int &argc, char **argv)
  : QApplication(argc, argv), m_fontSet(0)
{
  setWindowIcon(QIcon(":/GreenTeaCup.png"));
  connect(this, SIGNAL(aboutToQuit()), this, SLOT(aboutToQuitSlot()));
  connect(this, SIGNAL(aboutQSwingSignal()), this, SLOT(aboutQSwingSlot()),
          Qt::BlockingQueuedConnection);
  connect(this, SIGNAL(setTranslationSignal(bool *, const QString &)), this,
          SLOT(setTranslationSlot(bool *, const QString &)),
          Qt::BlockingQueuedConnection);
  connect(this, SIGNAL(removeTranslationSignal()), this,
          SLOT(removeTranslationSlot()), Qt::BlockingQueuedConnection);
  connect(this, SIGNAL(setStyleSheetSignal(const QString &)), this,
          SLOT(setStyleSheetSlot(const QString &)),
          Qt::BlockingQueuedConnection);
  connect(this, SIGNAL(setOverrideCursorSignal(jint)), this,
          SLOT(setOverrideCursorSlot(jint)), Qt::BlockingQueuedConnection);
  connect(this, SIGNAL(changeOverrideCursorSignal(jint)), this,
          SLOT(changeOverrideCursorSlot(jint)), Qt::BlockingQueuedConnection);
  connect(this, SIGNAL(restoreOverrideCursorSignal()), this,
          SLOT(restoreOverrideCursorSlot()), Qt::BlockingQueuedConnection);
  connect(this, SIGNAL(aboutQtSignal()), this, SLOT(aboutQt()),
          Qt::BlockingQueuedConnection);
}

/*^**************************************************************************^*/
/*- QApplicationAdapter :: Public Member Function.                           -*/
/*^**************************************************************************^*/

/*!
 * \brief Displays about Qt message box.
 *
 * \param env the JNIEnv pointer
 */
void QApplicationAdapter::aboutQt(JNIEnv *env)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    QApplication::aboutQt();
  } else {
    emit aboutQtSignal();
  }
}

/*!
 * \brief Displays about QSwing message box.
 *
 * \param env the JNIEnv pointer
 */
void QApplicationAdapter::aboutQSwing(JNIEnv *env)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    aboutQSwingSlot();
  } else {
    emit aboutQSwingSignal();
  }
}

/*!
 * \brief Sets translation.
 *
 * \param env the JNIEnv pointer
 * \param fileName the translation file name
 *
 * \return \c true if succeed
 */
bool QApplicationAdapter::setTranslation(JNIEnv *env, const QString &fileName)
{
  bool succeed;
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    setTranslationSlot(&succeed, fileName);
  } else {
    emit setTranslationSignal(&succeed, fileName);
  }
  return succeed;
}

/*!
 * \brief Removes translation.
 *
 * \param env the JNIEnv pointer
 */
void QApplicationAdapter::removeTranslation(JNIEnv *env)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    removeTranslationSlot();
  } else {
    emit removeTranslationSignal();
  }
}

/*!
 * \brief Sets style sheet.
 *
 * \param env the JNIEnv pointer
 * \param styleSheetText the style sheet text
 */
void QApplicationAdapter::setStyleSheet(JNIEnv *env, jstring styleSheetText)
{
  QStringAdapter styleSheet(env, styleSheetText);
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    setStyleSheetSlot(styleSheet);
  } else {
    emit setStyleSheetSignal(styleSheet);
  }
}

/*!
 * \brief Sets application override cursor.
 *
 * \param env the JNIEnv pointer 
 * \param type the style cursor type
 */
void QApplicationAdapter::setOverrideCursor(JNIEnv *env, jint type)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    setOverrideCursorSlot(type);
  } else {
    emit setOverrideCursorSignal(type);
  }
}

/*!
 * \brief Changes application override cursor.
 *
 * \param env the JNIEnv pointer
 * \param type the style cursor type
 */
void QApplicationAdapter::changeOverrideCursor(JNIEnv *env, jint type)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    changeOverrideCursorSlot(type);
  } else {
    emit changeOverrideCursorSignal(type);
  }
}

/*!
 * \brief Undos last setOverrideCursor().
 *
 * \param env the JNIEnv pointer
 */
void QApplicationAdapter::restoreOverrideCursor(JNIEnv *env)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    restoreOverrideCursorSlot();
  } else {
    emit restoreOverrideCursorSignal();
  }
}

/*!
 * \brief Sets font set.
 *
 * \param fontSet the font set
 */
void QApplicationAdapter::setFontSet(QFontSet *fontSet)
{
  if (fontSet != m_fontSet) {
    setFont(fontSet->defaultFont());
    setFont(fontSet->fixedFont(), "QTextArea");
    setFont(fontSet->componentFont(), "QWidget");
    QFont buttonFont = fontSet->buttonFont();
    setFont(buttonFont, "QAbstractButton");
    setFont(buttonFont, "QCheckBox");
    setFont(buttonFont, "QPushButton");
    setFont(buttonFont, "QRadioButton");
    setFont(buttonFont, "QToolButton");
    setFont(fontSet->menuFont(), "QMenu");
    setFont(fontSet->menuBarFont(), "QMenuBar");
    setFont(fontSet->textFont(), "QLabel");
    setFont(fontSet->toolTipFont(), "QTipLabel");
    m_fontSet = fontSet;
  }
}

/*^**************************************************************************^*/
/*- QApplicationAdapter :: Public Static Member Function.                    -*/
/*^**************************************************************************^*/

/*!
 * \brief Quits main loop.
 *
 * \param quitOnLastWindowClosed if \c true exits main loop when last window is
 *                               closed
 *
 * \return loop exit code
 */
int QApplicationAdapter::exec(bool quitOnLastWindowClosed)
{
  QApplication::setQuitOnLastWindowClosed(quitOnLastWindowClosed);
  return QApplication::exec();
}

/*^**************************************************************************^*/
/*- QApplicationAdapter :: Private Member Functions.                         -*/
/*^**************************************************************************^*/

void QApplicationAdapter::aboutQSwingSlot()
{
  QMessageBox mb(QApplication::activeWindow());
  mb.setWindowTitle(translate("JavaQx", "About QSwing"));
  mb.setText(
    "<h3>About QSwing</h3>"
    "<p>QSwing is a Swing-like GUI for Java developed on "
    "Qt&nbsp;toolkit.</p>"
    "<p>It is published under the LGPL, version 2.1, license.</p>"
    "<p>Copyright (c) 2007, "
    "<a href=\"http://www.etf.bg.ac.yu/\">ETF</a> and contributors. "
    "All rights reserved.</p>");
  mb.setIconPixmap(QPixmap(":/GreenTeaCupLarge.png"));
  mb.addButton(QMessageBox::Ok);
  mb.exec();
}

void QApplicationAdapter::setTranslationSlot(bool *succeed,
                                             const QString &fileName)
{
  if (sm_translator) {
    removeTranslator(sm_translator);
    delete sm_translator;
  }
  sm_translator = new QTranslator();
  if (sm_translator->load(fileName)) {
    installTranslator(sm_translator);
    *succeed = true;
  } else {
    delete sm_translator;
    sm_translator = 0;
    *succeed = false;
  }
}

void QApplicationAdapter::removeTranslationSlot()
{
  if (sm_translator) {
    removeTranslator(sm_translator);
    delete sm_translator;
    sm_translator = 0;
  }
}

void QApplicationAdapter::setStyleSheetSlot(const QString &styleSheetText)
{
  bool removed = styleSheetText.isEmpty();
  QApplication::setStyleSheet(styleSheetText);
  QWidgetList tlws = topLevelWidgets();
  for (int i = tlws.size(); --i >= 0;) {
    QWidget *qSwingWidget = tlws[i];
    JQComponentBridge *bridgeComponent =
                        qObjectBridgePointer<JQComponentBridge *>(qSwingWidget);
    if (bridgeComponent) {
      bridgeComponent->styleSheetChanged(removed);
    }
  }
}

void QApplicationAdapter::setOverrideCursorSlot(jint type)
{
  QApplication::setOverrideCursor(QCursorAdapter(type));
}

void QApplicationAdapter::changeOverrideCursorSlot(jint type)
{
  QApplication::changeOverrideCursor(QCursorAdapter(type));
}

void QApplicationAdapter::restoreOverrideCursorSlot()
{
  QApplication::restoreOverrideCursor();
}

void QApplicationAdapter::aboutToQuitSlot()
{
  QWidgetList aws = allWidgets();
  QWidgetList tlws;
  int awsSize = aws.size();
  for (int i = 0; i < awsSize; i++) {
    QWidget *widget = aws.at(i);
    if (widget->isWindow() && !widget->parent() &&
        widget->property(QObjectBridge::BRIDGE_PROPERTY_NAME).isValid()) {
      tlws.append(widget);
    }
  }
  for (int i = tlws.size(); --i >= 0;) {
    delete tlws.at(i);;
  }
}

/*^**************************************************************************^*/
/*- QApplicationAdapter :: Private Static Data Member.                       -*/
/*^**************************************************************************^*/

QTranslator *QApplicationAdapter::sm_translator = 0;

/*^*****************************************************************************
  File History:

 - 2007-05-01 18:51:59 Slobodan
   Initial version

*****************************************************************************^*/
