/*
 * @(#)QSwingGraphics.cpp  0.2.0 / 2007-09-11
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "QSwingGraphics.h"

/*^**************************************************************************^*/
/*- QSwingGraphics :: Public Constructors.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new QSwingGraphics object.
 *
 * \param device the QPaintDevice
 */
QSwingGraphics::QSwingGraphics(QPaintDevice *device)
  : QPainter(device)
{
  Q_ASSERT(device);
}

/*!
 * \brief Constructs new QSwingGraphics object.
 *
 * \param device the QPaintDevice
 * \param x the x coordinate of the clipping region
 * \param y the y coordinate of the clipping region
 * \param width the width of the clipping region
 * \param height the height of the clipping region
 */
QSwingGraphics::QSwingGraphics(QPaintDevice *device, jint x, jint y,
                               jint width, jint height)
  : QPainter(device)
{
  Q_ASSERT(device);
  setClipRect(x, y, width, height);
}

/*^**************************************************************************^*/
/*- QSwingGraphics :: Public Member Functions.                               -*/
/*^**************************************************************************^*/

/*!
 * \brief Returns current color of the graphics.
 *
 * \return current color of the graphics
 */
jobject QSwingGraphics::getColor()
{
  return QColorAdapter::toJColor(pen().color());
}

/*!
 * \brief Sets current color of the graphics.
 *
 * \param argb the argb value of the color
 */
void QSwingGraphics::setColor(jint argb)
{
  QPen p = pen();
  p.setColor(QColorAdapter(argb));
  setPen(p);
}

/*!
 * \brief Returns current font of the graphics.
 *
 * \return current font of the graphics
 */
jobject QSwingGraphics::getFont()
{
  return QFontAdapter::toJFont(font());
}

/*!
 * \brief Sets current font of the graphics.
 *
 * \param font the QSwing font object
 */
void QSwingGraphics::setFont(jobject font)
{
  if (font) {
    QPainter::setFont(*QFontAdapter::toQFont(font));
  } else {
    QPainter::setFont(QFont());
  }
}

/*!
 * \brief Sets linear gradient.
 *
 * \param x1 the x coordinate of the gradient's start point
 * \param y1 the y coordinate of the gradient's start point
 * \param argb1 the argb value of the first color
 * \param x2 the x coordinate of the gradient's end point
 * \param y2 the y coordinate of the gradient's end point
 * \param argb2 the argb value of the second color
 * \param cyclic \c true if the gradient is cyclic
 */
void QSwingGraphics::setLinearGradient(jint x1, jint y1, jint argb1, jint x2,
                                       jint y2, jint argb2, jboolean cyclic)
{
  QLinearGradient linearGradient(x1, y1, x2, y2);
  if (cyclic) {
    linearGradient.setSpread(QGradient::ReflectSpread);
  }
  linearGradient.setColorAt(0, QColorAdapter(argb1));
  linearGradient.setColorAt(1, QColorAdapter(argb2));
  QBrush b(linearGradient);
  QPen p = pen();
  p.setBrush(b);
  setPen(p);
  setBrush(b);
}

/*!
 * \brief Sets texture gradient.
 *
 * \param image the patern image
 */
void QSwingGraphics::setTextureGradient(QPixmap *image)
{
  if (image) {
    QBrush b(*image);
    QPen p = pen();
    p.setBrush(b);
    setPen(p);
    setBrush(b);
  }
}

/*!
 * \brief Sets basic stroke.
 *
 * \param width the stroke width
 * \param cap the stroke cap
 * \param join the stroke join
 * \param miterLimit the stroke miter limit
 * \param dash the stroke dash
 * \param dashPhase the stroke dash phase
 */
void QSwingGraphics::setBasicStroke(jfloat width, jint cap, jint join,
                                    jfloat miterLimit, jfloatArray dash,
                                    jfloat dashPhase)
{
  Q_UNUSED(dashPhase); // dash pahse is not currently supported
  QPen p = pen();
  p.setWidthF(width);
  p.setCapStyle(PEN_CAP_STYLE[cap]);
  p.setJoinStyle(PEN_JOIN_STYLE[join]);
  p.setMiterLimit(miterLimit);
  if (dash) {
    jsize dashLenght = QSWING_ENV->GetArrayLength(dash);
    float *dashArray = (float *)QSWING_ENV->GetPrimitiveArrayCritical(dash, 0);
    QVector<qreal> dashes(dashLenght);
    for (int i = 0; i < dashLenght; i++) {
      dashes.append(dashArray[i]);
    }
    QSWING_ENV->ReleasePrimitiveArrayCritical(dash, dashArray, JNI_ABORT);
    p.setDashPattern(dashes);
  } else {
    p.setStyle(Qt::SolidLine);
  }
  setPen(p);
}

/*!
 * \brief Pushes the state of the graphics.
 */
void QSwingGraphics::push()
{
  // TODO: add stack state (like QPainter::QPainterState)
  end();
}

/*!
 * \brief Pops the state of the graphics.
 */
void QSwingGraphics::pop()
{
  // TODO: use state stack
  begin(device());
}

/*!
 * \brief Pops the state of the graphics.
 *
 * \param imageData the image data
 * \param length the length of the image data
 * \param x the x coordinate of the image
 * \param y the y coordinate of the image
 */
void QSwingGraphics::drawImage(jbyteArray imageData, jint length, jint x,
                               jint y)
{
  if (imageData) {
    QPixmap pixmap;
    uchar *bytes = (uchar *)QSWING_ENV->GetPrimitiveArrayCritical(imageData, 0);
    bool loadSucceed = pixmap.loadFromData(bytes, length);
    QSWING_ENV->ReleasePrimitiveArrayCritical(imageData, bytes, JNI_ABORT);
    if (loadSucceed) {
      drawPixmap(x, y, pixmap);
    }
  }
}

/*!
 * \brief Fills rectangle.
 *
 * \param x the x coordinate of the rectangle
 * \param y the y coordinate of the rectangle
 * \param w the width of the rectangle
 * \param h the height of the rectangle
 */
void QSwingGraphics::fillRect(jint x, jint y, jint w, jint h)
{
  QBrush oldBrush = brush();
  QPen oldPen = pen();
  QBrush b = oldBrush;
  b.setStyle(Qt::SolidPattern);
  b.setColor(oldPen.color());
  setBrush(b);
  setPen(Qt::NoPen);
  drawRect(x, y, w, h);
  setPen(oldPen);
  setBrush(oldBrush);
}

/*!
 * \brief Fills oval.
 *
 * \param x the x coordinate of the oval
 * \param y the y coordinate of the oval
 * \param w the width of the oval
 * \param h the height of the oval
 */
void QSwingGraphics::fillOval(jint x, jint y, jint w, jint h)
{
  QBrush oldBrush = brush();
  QPen oldPen = pen();
  QBrush b = oldBrush;
  b.setStyle(Qt::SolidPattern);
  b.setColor(oldPen.color());
  setBrush(b);
  setPen(Qt::NoPen);
  drawEllipse(x, y, w, h);
  setPen(oldPen);
  setBrush(oldBrush);
}

/*^**************************************************************************^*/
/*- QSwingGraphics ::Private Static Constant Data Members.                   -*/
/*^**************************************************************************^*/

const Qt::PenCapStyle QSwingGraphics::PEN_CAP_STYLE[] = {
  Qt::FlatCap,
  Qt::RoundCap,
  Qt::SquareCap
};
const Qt::PenJoinStyle QSwingGraphics::PEN_JOIN_STYLE[] = {
  Qt::MiterJoin,
  Qt::RoundJoin,
  Qt::BevelJoin
};

/*^*****************************************************************************
  File History:

 - 2007-09-11 13:03:23 Slobodan
   Initial version

*****************************************************************************^*/
