/*
 * @(#)QTextPane.cpp  0.2.0 / 2007-12-27
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "QTextPane.h"

#include "JavaQxQSwing.h"
#include "JQTextPaneBridge.h"
#include "QSwingGraphics.h"

/*^**************************************************************************^*/
/*- QTextPane :: Protected Constructor.                                      -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new QTextPane object.
 *
 * \param bridge the JQTextPane bridge
 */
QTextPane::QTextPane(JQTextPaneBridge *bridge)
  : QTextEdit(), m_document(0), m_paintView(false), m_bridge(bridge),
    m_paintViewEvent(0)
{
  if (sm_defaultDocumentCount == 0) {
    sm_defaultDocument = document();
    sm_defaultDocument->setParent(0);
  }
  ++sm_defaultDocumentCount;
}

/*^**************************************************************************^*/
/*- QTextPane :: Protected Destructor.                                       -*/
/*^**************************************************************************^*/

/*!
 * \brief Destructs QTextPane object.
 */
QTextPane::~QTextPane()
{
  if (--sm_defaultDocumentCount == 0) {
    delete sm_defaultDocument;
  }
}

/*^**************************************************************************^*/
/*- QTextPane :: Protected Member Functions.                                 -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets text document.
 *
 * \param document the document
 */
void QTextPane::setDocument(QTextDocumentAdapter *document)
{
  if (m_document != document) {
    if (m_document) {
      disconnectSignals();
    }
    if (document) {
      m_document = document;
      connectSignals();
      QTextEdit::setDocument(document);
    } else {
      QTextEdit::setDocument(sm_defaultDocument);
    }
  }
}

/*!
 * \brief Sets whether or not paints component view.
 *
 * \param paintView \c true if paint
 */
void QTextPane::setPaintView(bool paintView)
{
  if (m_paintView != paintView) {
    if (paintView) {
      m_paintView = true;
      connect(horizontalScrollBar(), SIGNAL(valueChanged(int)), this,
              SLOT(valueChangedSlot(int)));
      connect(verticalScrollBar(), SIGNAL(valueChanged(int)), this,
              SLOT(valueChangedSlot(int)));
    } else {
      m_paintView = false;
      disconnect(horizontalScrollBar(), SIGNAL(valueChanged(int)), this,
                 SLOT(valueChangedSlot(int)));
      disconnect(verticalScrollBar(), SIGNAL(valueChanged(int)), this,
                 SLOT(valueChangedSlot(int)));
    }
  }
}

/*!
 * \brief Default view painting.
 *
 * \param graphicsPointer the QSwingGraphics pointer
 */
void QTextPane::paintView(jlong graphicsPointer)
{
  if (m_paintView && m_paintViewEvent) {
    qSwingGraphics->push();
    QTextEdit::paintEvent(m_paintViewEvent);
    qSwingGraphics->pop();
    // TODO: Move into QSwingGraphics the following code
    const int xOffset = horizontalScrollBar()->value();
    const int yOffset = verticalScrollBar()->value();
    qSwingGraphics->translate(-xOffset, -yOffset);
    QRect clipRect(xOffset, yOffset, viewport()->width(), viewport()->height());
    qSwingGraphics->setClipRect(clipRect);
  }
}

/*!
 * \brief Connects signals.
 */
void QTextPane::connectSignals()
{
  connect(m_document, SIGNAL(destroying()), SLOT(destroyingSlot()));
  connect(m_document, SIGNAL(textCleared()), SLOT(textClearedSlot()));
  connect(m_document, SIGNAL(textSet()), SLOT(textSetSlot()));
  connect(m_document, SIGNAL(textAppended()), SLOT(textAppendedSlot()));
  connect(this, SIGNAL(undoAvailable(bool)), m_bridge,
          SLOT(undoAvailableSlot(bool)));
  connect(this, SIGNAL(redoAvailable(bool)), m_bridge,
          SLOT(redoAvailableSlot(bool)));
  connect(this, SIGNAL(copyAvailable(bool)), SLOT(copyAvailableSlot(bool)));
  connect(m_document, SIGNAL(selectAllAvailable(bool)),
          SLOT(selectAllAvailableSlot(bool)));
}

/*!
 * \brief Disconnects signals.
 */
void QTextPane::disconnectSignals()
{
  disconnect(m_document, SIGNAL(destroying()), this,
             SLOT(destroyingSlot()));
  disconnect(m_document, SIGNAL(textCleared()), this,
             SLOT(textClearedSlot()));
  disconnect(m_document, SIGNAL(textSet()), this, SLOT(textSetSlot()));
  disconnect(m_document, SIGNAL(textAppended()), this,
             SLOT(textAppendedSlot()));
  disconnect(this, SIGNAL(undoAvailable(bool)), m_bridge,
             SLOT(undoAvailableSlot(bool)));
  disconnect(this, SIGNAL(redoAvailable(bool)), m_bridge,
             SLOT(redoAvailableSlot(bool)));
  disconnect(this, SIGNAL(copyAvailable(bool)), this,
             SLOT(copyAvailableSlot(bool)));
  disconnect(m_document, SIGNAL(selectAllAvailable(bool)), this,
             SLOT(selectAllAvailableSlot(bool)));
}

/*^**************************************************************************^*/
/*- QTextPane :: Protected Member Functions.                                 -*/
/*^**************************************************************************^*/

/*!
 * \brief \c destroying slot.
 */
void QTextPane::destroyingSlot()
{
  disconnectSignals();
  m_document = 0;
  QTextEdit::setDocument(sm_defaultDocument);
  setReadOnly(true);
}

/*!
 * \brief \c valueChanged slot.
 *
 * \param value the chnaged value
 */
void QTextPane::valueChangedSlot(int value)
{
  Q_UNUSED(value);
  viewport()->update(); // TODO: optimize
}

/*!
 * \brief \c textSet slot.
 */
void QTextPane::textClearedSlot()
{
  m_bridge->cutAvailable(false);
  m_bridge->copyAvailable(false);
  m_bridge->clearAvailable(false);
  m_bridge->selectAllAvailable(false);
  if (!isReadOnly()) {
    setFocus();
  }
}

/*!
 * \brief \c textSet slot.
 */
void QTextPane::textSetSlot()
{
  moveCursor(QTextCursor::Start);
  if (!isReadOnly()) {
    setFocus();
  }
}

/*!
 * \brief \c textAppended slot.
 */
void QTextPane::textAppendedSlot()
{
  moveCursor(QTextCursor::End);
}

/*!
 * \brief \c selectionChanged slot.
 *
 * \param available \c true if available
 */
void QTextPane::copyAvailableSlot(bool available)
{
  if (!isReadOnly()) {
    m_bridge->cutAvailable(available);
    m_bridge->clearAvailable(available);
  }
  m_bridge->copyAvailable(available);
  m_bridge->selectAllAvailable(available);
}

/*!
 * \brief \c selectAllAvailable slot.
 *
 * \param available \c true if available
 */
void QTextPane::selectAllAvailableSlot(bool available)
{
  m_bridge->selectAllAvailable(available);
}

/*^**************************************************************************^*/
/*- QTextPane :: Overridden Member Function.                                 -*/
/*^**************************************************************************^*/

/*!
 * \inheritDoc
 */
void QTextPane::paintEvent(QPaintEvent *e)
{
  if (m_paintView) {
    const int xOffset = horizontalScrollBar()->value();
    const int yOffset = verticalScrollBar()->value();
    QSwingGraphics qsg(viewport());
    qsg.translate(-xOffset, -yOffset);
    QRect clipRect(xOffset, yOffset, viewport()->width(), viewport()->height());
    qsg.setClipRect(clipRect);
    m_paintViewEvent = e;
    m_bridge->paintView(reinterpret_cast<jlong>(&qsg), clipRect.x(),
                        clipRect.y(), clipRect.width(), clipRect.height());
    m_paintViewEvent->accept();
    m_paintViewEvent = 0;
  } else {
    QTextEdit::paintEvent(e);
  }
}

/*^**************************************************************************^*/
/*- QTextPane :: Private Static Data Members.                                -*/
/*^**************************************************************************^*/

QTextDocument *QTextPane::sm_defaultDocument = 0;
int QTextPane::sm_defaultDocumentCount = 0;

/*^*****************************************************************************
  File History:

 - 2007-12-27 19:34:19 Slobodan
   Initial version

*****************************************************************************^*/
