/*
 * @(#)JQAbstractButtonBridge.cpp  0.2.0 / 2007-08-16
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQAbstractButtonBridge.h"

#include "QAbstractButtonAdapter.h"
#include "QApplicationBridge.h"
#include "QSwingBridge.h"

/*^**************************************************************************^*/
/*- JQAbstractButtonBridge :: Protected Constructors.                        -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQAbstractButtonBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQAbstractButton object
 * \param button the QAbstractButton object
 */
JQAbstractButtonBridge::JQAbstractButtonBridge(JNIEnv *env, jobject obj,
                                               QAbstractButton *button)
  : JQComponentBridge(env, obj, button), m_modifiers(Qt::NoModifier),
    m_iconType(QIconAdapter::DefaultIcon)
{
  connectDirect(qAbstractButton(), SIGNAL(pressed()), SLOT(pressedSlot()));
  connectDirect(qAbstractButton(), SIGNAL(released()), SLOT(releasedSlot()));
  connectDirect(qAbstractButton(), SIGNAL(clicked()), SLOT(clickedSlot()));
}

/*!
 * \brief Constructs new JQAbstractButtonBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQAbstractButton object
 */
JQAbstractButtonBridge::JQAbstractButtonBridge(JNIEnv *env, jobject obj)
  : JQComponentBridge(env, obj, new QAbstractButtonAdapter()),
    m_modifiers(Qt::NoModifier), m_iconType(QIconAdapter::DefaultIcon)
{
  connectDirect(qAbstractButton(), SIGNAL(pressed()), SLOT(pressedSlot()));
  connectDirect(qAbstractButton(), SIGNAL(released()), SLOT(releasedSlot()));
  connectDirect(qAbstractButton(), SIGNAL(clicked()), SLOT(clickedSlot()));
}

/*^**************************************************************************^*/
/*- JQAbstractButtonBridge :: Native Calls.                                  -*/
/*^**************************************************************************^*/

/*!
 * \brief Pressed/unpressed the button.
 *
 * \param env the JNIEnv pointer 
 * \param pressed \c true if pressed
 */
void JQAbstractButtonBridge::setPressed(JNIEnv *env, jboolean pressed)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setPressed, QSWING_ENV_ARG, JBOOLEAN_ARG(pressed))
  qAbstractButton()->setDown(pressed);
}

/*!
 * \brief Sets button text.
 *
 * \param env the JNIEnv pointer 
 * \param text the text
 */
void JQAbstractButtonBridge::setText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setText, QSWING_ENV_ARG, JSTRING_ARG(text))
  qAbstractButton()->setText(QStringAdapter(env, text));
}

/*!
 * \brief Sets the icon set.
 *
 * \param env the JNIEnv pointer 
 * \param iconSet the icon set
 */
void JQAbstractButtonBridge::setIcon(JNIEnv *env, jobject iconSet)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setIcon, QSWING_ENV_ARG, JOBJECT_ARG(iconSet))
  if (iconSet) {
    qAbstractButton()->setIcon(QIconAdapter(env, iconSet, &m_iconType));
    qAbstractButton()->setMouseTracking(true);
  } else {
    qAbstractButton()->setIcon(QIcon());
  }
}

/*!
 * \brief Sets icon size.
 *
 * \param env the JNIEnv pointer 
 * \param width the icon width 
 * \param height the icon height 
 */
void JQAbstractButtonBridge::setIconSize(JNIEnv *env, jint width, jint height)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setIconSize, QSWING_ENV_ARG, JINT_ARG(width),
                   JINT_ARG(height))
  qAbstractButton()->setIconSize(QSize(width, height));
}

/*^**************************************************************************^*/
/*- JQAbstractButtonBridge :: Static Native Calls.                           -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQAbstractButtonBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQAbstractButton class
 *
 * \exception FatalError if failed
 */
void JQAbstractButtonBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT (!sm_buttonPressedMID);
  if (!INIT_UMID(buttonPressed, cls, "()Z") ||
      !INIT_UMID(buttonReleased, cls, "()Z") ||
      !INIT_UMID(buttonClicked, cls, "(I)V")) {
    FATAL_LOGGED_ERROR("JQAbstractButtonBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object.
 *
 * \param env the JNIEnv pointer 
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQAbstractButtonBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQAbstractButtonBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQAbstractButtonBridge :: Native Callbacks.                              -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls buttonPressed method of the JQAbstractButton object.
 *
 * \return \c true if the button is pressed
 */
jboolean JQAbstractButtonBridge::buttonPressed()
{
  jboolean result;
  jniObject()->callBooleanMethod(result, sm_buttonPressedMID);
  return result;
}

/*!
 * \brief Calls buttonReleased method of the JQAbstractButton object.
 *
 * \return \c true if the button is released
 */
jboolean JQAbstractButtonBridge::buttonReleased()
{
  jboolean result;
  jniObject()->callBooleanMethod(result, sm_buttonReleasedMID);
  return result;
}

/*!
 * \brief Calls buttonClicked method of the JQAbstractButton object.
 */
void JQAbstractButtonBridge::buttonClicked(jint modifiers)
{
  jniObject()->callVoidMethod(sm_buttonClickedMID, modifiers);
}

/*^**************************************************************************^*/
/*- JQAbstractButtonBridge :: Overridden Member Functions.                   -*/
/*^**************************************************************************^*/

/*!
 * \inheritDoc
 */
jobject JQAbstractButtonBridge::getBackground(JNIEnv *env, jclass colorClass)
{
  return QColorAdapter::toJColor(env, colorClass,
                          qAbstractButton()->palette().color(QPalette::Button));
}

/*!
 * \inheritDoc
 */
void JQAbstractButtonBridge::setBackground(JNIEnv *env, jint argb)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setBackground, QSWING_ENV_ARG, JINT_ARG(argb))
  QColorAdapter color(argb);
  QPalette palette(qAbstractButton()->palette());
  if (color.isValid()) {
    palette.setColor(QPalette::Button, color);
    palette.setColor(QPalette::Window, color);
  } else {
    palette.setColor(QPalette::Button,
                     QApplication::palette().color(QPalette::Button));
    palette.setColor(QPalette::Window,
                     QApplication::palette().color(QPalette::Window));
  }
  qAbstractButton()->setPalette(palette);
}

/*!
 * \inheritDoc
 */
void JQAbstractButtonBridge::setNullBackground(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setNullBackground, QSWING_ENV_ARG)
  QPalette palette(qWidget()->palette());
  palette.setColor(QPalette::Button,
                   QApplication::palette().color(QPalette::Button));
  palette.setColor(QPalette::Window,
                   QApplication::palette().color(QPalette::Window));
  qAbstractButton()->setPalette(palette);
}

/*!
 * \inheritDoc
 */
jobject JQAbstractButtonBridge::getForeground(JNIEnv *env, jclass colorClass)
{
  return QColorAdapter::toJColor(env, colorClass,
                      qAbstractButton()->palette().color(QPalette::ButtonText));
}

/*!
 * \inheritDoc
 */
void JQAbstractButtonBridge::setForeground(JNIEnv *env, jint argb)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setForeground, QSWING_ENV_ARG, JINT_ARG(argb))
  QColorAdapter color(argb);
  QPalette palette(qAbstractButton()->palette());
  if (color.isValid()) {
    palette.setColor(QPalette::ButtonText, color);
    palette.setColor(QPalette::WindowText, color);
  } else {
    palette.setColor(QPalette::ButtonText,
                     QApplication::palette().color(QPalette::ButtonText));
    palette.setColor(QPalette::WindowText,
                     QApplication::palette().color(QPalette::WindowText));
  }
  qAbstractButton()->setPalette(palette);
}

/*!
 * \inheritDoc
 */
void JQAbstractButtonBridge::setNullForeground(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setNullForeground, QSWING_ENV_ARG)
  QPalette palette(qWidget()->palette());
  palette.setColor(QPalette::ButtonText,
                   QApplication::palette().color(QPalette::ButtonText));
  palette.setColor(QPalette::WindowText,
                   QApplication::palette().color(QPalette::WindowText));
  qAbstractButton()->setPalette(palette);
}

/*^**************************************************************************^*/
/*- JQAbstractButtonBridge :: Overridden Member Functions.                   -*/
/*^**************************************************************************^*/

/*!
 * \inheritDoc
 */
bool JQAbstractButtonBridge::filterKeyPressedEvent(QObject *obj, QKeyEvent *e)
{
  m_modifiers |= e->modifiers();
  return JQComponentBridge::filterKeyPressedEvent(obj, e);
}

/*!
 * \inheritDoc
 */
bool JQAbstractButtonBridge::filterKeyReleasedEvent(QObject *obj, QKeyEvent *e)
{
  m_modifiers &= ~QSwingBridge::toKeyboardModifiers(e->key());
  return JQComponentBridge::filterKeyPressedEvent(obj, e);
}

/*!
 * \inheritDoc
 */
bool JQAbstractButtonBridge::filterMouseEnterEvent(QObject *obj, QMouseEvent *e)
{
  if (qAbstractButton()->isChecked()) {
    setIconType(QIconAdapter::RolloverSelectedIcon);
  } else {
    setIconType(QIconAdapter::RolloverIcon);
  }
  qAbstractButton()->update();
  return JQComponentBridge::filterMouseEnterEvent(obj, e);
}

/*!
 * \inheritDoc
 */
bool JQAbstractButtonBridge::filterMouseLeaveEvent(QObject *obj, QMouseEvent *e)
{
  if (qAbstractButton()->isChecked()) {
    setIconType(QIconAdapter::SelectedIcon);
  } else {
    setIconType(QIconAdapter::DefaultIcon);
  }
  qAbstractButton()->update();
  return JQComponentBridge::filterMouseLeaveEvent(obj, e);
}

/*^**************************************************************************^*/
/*- JQAbstractButtonBridge :: Protected Member Functions.                    -*/
/*^**************************************************************************^*/

/*!
 * \brief Pressed slot.
 */
void JQAbstractButtonBridge::pressedSlot()
{
  if (!buttonPressed()) {
    setEventFilterEnabled(false);
    if (qAbstractButton()->isCheckable()) {
      qAbstractButton()->setChecked(!qAbstractButton()->isChecked());
    } else {
      qAbstractButton()->setDown(false);
    }
    setEventFilterEnabled(true);
  } else {
    setIconType(QIconAdapter::PressedIcon);
    qAbstractButton()->update();
  }
}

/*!
 * \brief Released slot.
 */
void JQAbstractButtonBridge::releasedSlot()
{
  if (!buttonReleased()) {
    setEventFilterEnabled(false);
    qAbstractButton()->setDown(true);
    setEventFilterEnabled(true);
  } else {
    if (qAbstractButton()->isChecked()) {
      if (qAbstractButton()->underMouse()) {
        setIconType(QIconAdapter::RolloverSelectedIcon);
      } else {
        setIconType(QIconAdapter::SelectedIcon);
      }
    } else {
      if (qAbstractButton()->underMouse()) {
        setIconType(QIconAdapter::RolloverIcon);
      } else {
        setIconType(QIconAdapter::DefaultIcon);
      }
    }
    qAbstractButton()->update();
  }
}

/*!
 * \brief \c clicked slot.
 */
void JQAbstractButtonBridge::clickedSlot()
{
  buttonClicked(QSwingBridge::toQSwingModifiers(m_modifiers));
}

/*^**************************************************************************^*/
/*- JQAbstractButtonBridge :: Private Static Data Members.                   -*/
/*^**************************************************************************^*/

jmethodID JQAbstractButtonBridge::sm_buttonPressedMID = 0;
jmethodID JQAbstractButtonBridge::sm_buttonReleasedMID = 0;
jmethodID JQAbstractButtonBridge::sm_buttonClickedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-08-16 07:56:37 Slobodan
   Initial version

*****************************************************************************^*/
