/*
 * @(#)JQCheckBoxMenuItemBridge.cpp  0.2.0 / 2007-11-14
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQCheckBoxMenuItemBridge.h"

#include "QApplicationBridge.h"
#include "QIconAdapter.h"

/*^**************************************************************************^*/
/*- JQCheckBoxMenuItemBridge :: Protected Constructor.                       -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQCheckBoxMenuItemBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQCheckBoxMenuItem object
 */
JQCheckBoxMenuItemBridge::JQCheckBoxMenuItemBridge(JNIEnv *env, jobject obj)
  : JQActionComponentBridge(env, obj), m_shortcut(0)
{
  // empty constructor body
}

/*^**************************************************************************^*/
/*- JQCheckBoxMenuItemBridge :: Native Calls.                                -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets text.
 *
 * \param env the JNIEnv pointer
 * \param text the text
 */
void JQCheckBoxMenuItemBridge::setText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setText, QSWING_ENV_ARG, JSTRING_ARG(text))
  if (isQWidget()) {
    checkBox()->setText(QStringAdapter(env, text));
  } else {
    action()->setText(QStringAdapter(env, text));
  }
}

/*!
 * \brief Sets short cut.
 *
 * \param env the JNIEnv pointer
 * \param shortcut the shortcut
 */
void JQCheckBoxMenuItemBridge::setShortcut(JNIEnv *env, jstring shortcut)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setShortcut, QSWING_ENV_ARG, JSTRING_ARG(shortcut))
  QStringAdapter shortcutString(env, shortcut);
  if (isQWidget()) {
    if (!m_shortcut) {
      m_shortcut = new QShortcut(widget());
    }
    if (shortcutString.isEmpty()) {
      m_shortcut->setKey(QKeySequence::UnknownKey);
    } else {
      m_shortcut->setKey(QKeySequence(shortcutString));
    }
  } else {
    if (shortcutString.isEmpty()) {
      action()->setShortcut(QKeySequence::UnknownKey);
    } else {
      action()->setShortcut(QKeySequence(shortcutString));
    }
  }
}

/*!
 * \brief Sets icon.
 *
 * \param env the JNIEnv pointer
 * \param iconSet the icon
 */
void JQCheckBoxMenuItemBridge::setIcon(JNIEnv *env, jobject iconSet)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setIcon, QSWING_ENV_ARG, JOBJECT_ARG(iconSet))
  if (isQWidget()) {
    if (iconSet) {
      checkBox()->setIcon(QIconAdapter(env, iconSet));
    } else {
      checkBox()->setIcon(QIcon());
    }
  } else {
    if (iconSet) {
      action()->setIcon(QIconAdapter(env, iconSet));
    } else {
      action()->setIcon(QIcon());
    }
  }
}

/*!
 * \brief Sets whether or not the item is checked.
 *
 * \param env the JNIEnv pointer
 * \param checked \c true if checked
 */
void JQCheckBoxMenuItemBridge::setChecked(JNIEnv *env, jboolean checked)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setChecked, QSWING_ENV_ARG, JBOOLEAN_ARG(checked))
  if (isQWidget()) {
    checkBox()->setChecked(checked);
  } else {
    action()->setChecked(checked);
  }
}

/*^**************************************************************************^*/
/*- JQCheckBoxMenuItemBridge :: Static Native Calls.                         -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQCheckBoxMenuItemBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQCheckBoxMenuItem class
 *
 * \exception FatalError if failed
 */
void JQCheckBoxMenuItemBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT (!sm_checkedChangedMID);
  if (!INIT_UMID(checkedChanged, cls, "(IZ)V")) {
    FATAL_LOGGED_ERROR("JQCheckBoxMenuItemBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQCheckBoxMenuItemBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQCheckBoxMenuItemBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQCheckBoxMenuItemBridge :: Native Callbacks.                            -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls \c checkedChanged method of the JQMenuItemBridge object.
 *
 * \param modifiers the keyboard modifiers
 * \param checked \c true if the item is checked
 */
void JQCheckBoxMenuItemBridge::checkedChanged(jint modifiers, jboolean checked)
{
  jniObject()->callVoidMethod(sm_checkedChangedMID, modifiers, checked);
}

/*^**************************************************************************^*/
/*- JQCheckBoxMenuItemBridge :: Protected Member Functions.                  -*/
/*^**************************************************************************^*/

/*!
 * \brief Toggled slot.
 *
 * \param checked \c true if the item is checked
 */
void JQCheckBoxMenuItemBridge::toggledSlot(bool checked)
{
  checkedChanged(QApplicationBridge::keyboardModifiers(), checked);
}

/*!
 * \brief State changed slot.
 *
 * \param state the state
 */
void JQCheckBoxMenuItemBridge::stateChangedSlot(int state)
{
  checkedChanged(QApplicationBridge::keyboardModifiers(), state);
}

/*^**************************************************************************^*/
/*- JQCheckBoxMenuItemBridge :: Overridden Member Functions.                 -*/
/*^**************************************************************************^*/

/*!
 * \brief Creates QWidget adaptee.
 *
 * \return QWidget adaptee
 */
QWidget *JQCheckBoxMenuItemBridge::createQWidget()
{
  QCheckBox *checkBox = new QCheckBox();
  connectDirect(checkBox, SIGNAL(stateChanged(int)),
                SLOT(stateChangedSlot(int)));
  return checkBox;
}

/*!
 * \brief Creates QAction adaptee.
 *
 * \return QAction adaptee
 */
QAction *JQCheckBoxMenuItemBridge::createQAction()
{
  QAction *action = new QAction(this);
  action->setCheckable(true);
  connectDirect(action, SIGNAL(toggled(bool)), SLOT(toggledSlot(bool)));
  return action;
}

/*!
 * \inheritDoc
 */
void JQCheckBoxMenuItemBridge::deleteQWidget(QWidget *widget)
{
  if (m_shortcut) {
    delete m_shortcut;
    m_shortcut = 0;
  }
  connectDirect(widget, SIGNAL(stateChanged(int)), SLOT(stateChangedSlot(int)));
  JQActionComponentBridge::deleteQWidget(widget);
}

/*!
 * \inheritDoc
 */
void JQCheckBoxMenuItemBridge::deleteQAction(QAction *action)
{
  disconnect(action, SIGNAL(toggled(bool)), SLOT(toggledSlot(bool)));
  JQActionComponentBridge::deleteQAction(action);
}

/*!
 * \inheritDoc
 */
QWidget *JQCheckBoxMenuItemBridge::convertToQWidget(QAction *action)
{
  QCheckBox *checkBox = static_cast<QCheckBox *>(
                             JQActionComponentBridge::convertToQWidget(action));
  checkBox->setText(action->text());
  if (m_shortcut) {
    delete m_shortcut;
  }
  m_shortcut = new QShortcut(checkBox);
  m_shortcut->setKey(action->shortcut());
  checkBox->setChecked(action->isChecked());
  checkBox->setIcon(action->icon());
  return checkBox;
}

/*!
 * \inheritDoc
 */
QAction *JQCheckBoxMenuItemBridge::convertToQAction(QWidget *widget)
{
  QCheckBox *checkBox = static_cast<QCheckBox *>(widget);
  QAction *action = JQActionComponentBridge::convertToQAction(checkBox);
  action->setText(checkBox->text());
  if (m_shortcut) {
    action->setShortcut(m_shortcut->key());
  }
  action->setChecked(checkBox->isChecked());
  action->setIcon(checkBox->icon());
  return action;
}

/*^**************************************************************************^*/
/*- JQCheckBoxMenuItemBridge :: Private Static Data Member.                  -*/
/*^**************************************************************************^*/

jmethodID JQCheckBoxMenuItemBridge::sm_checkedChangedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-14 09:03:27 Slobodan
   Initial version

*****************************************************************************^*/
