/*
 * @(#)JQDesktopPaneBridge.cpp  0.2.0 / 2007-11-19
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQDesktopPaneBridge.h"

#include "JQInternalFrameBridge.h"

/*^**************************************************************************^*/
/*- JQDesktopPaneBridge :: Protected Constructor.                            -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQDesktopPaneBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQDesktopPane object
 */
JQDesktopPaneBridge::JQDesktopPaneBridge(JNIEnv *env, jobject obj)
  : JQContainerBridge(env, obj, new QMdiArea())
{
  qMdiArea()->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
  qMdiArea()->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
  qMdiArea()->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
  connectDirect(qMdiArea(), SIGNAL(subWindowActivated(QMdiSubWindow *)),
                SLOT(subWindowActivatedSlot(QMdiSubWindow *)));
}

/*^**************************************************************************^*/
/*- JQDesktopPaneBridge :: Native Calls.                                     -*/
/*^**************************************************************************^*/

/*!
 * \brief Inserts child.
 *
 * \param env the JNIEnv pointer
 * \param childBridge the child pointer
 * \param index the child index
 */
void JQDesktopPaneBridge::insertChild(JNIEnv *env, jlong childBridge,
                                      jint index)
{
  Q_UNUSED(index);
  Q_ASSERT(env);
  QSWING_REENTRANT(insertChild, QSWING_ENV_ARG, JLONG_ARG(childBridge),
                   JINT_ARG(index))
  JQComponentBridge *childComponentBridge = jqComponentCast(childBridge);
  QWidget *childWidget = childComponentBridge->qWidget();
  QString tempTitle = childWidget->windowTitle();
  QPoint oldPosition = childWidget->pos();
  QSize oldSize = childWidget->size();
  setEventFilterEnabled(false);
  childComponentBridge->setEventFilterEnabled(false);
  qMdiArea()->addSubWindow(childWidget);
  childComponentBridge->setParentBridge(this);
  if (oldPosition != childWidget->pos()) {
    childWidget->move(oldPosition);
  }
  if (oldSize != childWidget->size()) {
    childWidget->resize(oldSize);
  }
  childComponentBridge->setEventFilterEnabled(true);
  setEventFilterEnabled(true);
  if (qMdiArea()->isVisible()) {
    childWidget->setVisible(true);
  }
}

/*!
 * \brief Removes the specified child.
 *
 * \param env the JNIEnv pointer
 * \param childBridge the child bridge pointer
 */
void JQDesktopPaneBridge::removeChild(JNIEnv *env, jlong childBridge)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(removeChild, QSWING_ENV_ARG, JLONG_ARG(childBridge))
  qMdiArea()->removeSubWindow(jqComponentCast(childBridge)->qWidget());
  jqComponentCast(childBridge)->setParentBridge(0);
}

/*!
 * \brief Sets current component.
 *
 * \param env the JNIEnv pointer
 * \param componentBridge the component bridge pointer
 */
void JQDesktopPaneBridge::setCurrentComponent(JNIEnv *env,
                                              jlong componentBridge)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setCurrentComponent, QSWING_ENV_ARG,
                   JLONG_ARG(componentBridge))
  if (componentBridge) {
    qMdiArea()->setActiveSubWindow(
                        jqInternalFrameCast(componentBridge)->qMdiSubWindow());
  } else {
    qMdiArea()->setActiveSubWindow(0);
  }
}

/*!
 * \brief Selects next comopnent.
 *
 * \param env the JNIEnv pointer
 */
void JQDesktopPaneBridge::selectNextComponent(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(selectNextComponent, QSWING_ENV_ARG)
  qMdiArea()->activateNextSubWindow();
}

/*!
 * \brief Selects previous component.
 *
 * \param env the JNIEnv pointer
 */
void JQDesktopPaneBridge::selectPreviousComponent(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(selectPreviousComponent, QSWING_ENV_ARG)
  qMdiArea()->activatePreviousSubWindow();
}

/*!
 * \brief Closes the current component.
 *
 * \param env the JNIEnv pointer
 */
void JQDesktopPaneBridge::closeCurrentComponent(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(closeCurrentComponent, QSWING_ENV_ARG)
  qMdiArea()->closeActiveSubWindow();
}

/*!
 * \brief Closes all components.
 *
 * \param env the JNIEnv pointer
 */
void JQDesktopPaneBridge::closeAllComponents(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(closeAllComponents, QSWING_ENV_ARG)
  qMdiArea()->closeAllSubWindows();
}

/*!
 * \brief Cascades the components.
 *
 * \param env the JNIEnv pointer
 */
void JQDesktopPaneBridge::cascadeComponents(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(cascadeComponents, QSWING_ENV_ARG)
  qMdiArea()->cascadeSubWindows();
}

/*!
 * \brief Tiles the components.
 *
 * \param env the JNIEnv pointer
 */
void JQDesktopPaneBridge::tileComponents(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(tileComponents, QSWING_ENV_ARG)
  qMdiArea()->tileSubWindows();
}

/*^**************************************************************************^*/
/*- JQDesktopPaneBridge :: Static Native Calls.                              -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQDesktopPaneBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQDesktopPane class
 *
 * \exception FatalError if failed
 */
void JQDesktopPaneBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_activeComponentChangedMID);
  if (!INIT_UMID(activeComponentChanged, cls,
      "(Lyu/ac/bg/etf/javaqx/qswing/JQComponent;)V")) {
    FATAL_LOGGED_ERROR("JQDesktopPaneBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQDesktopPaneBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQDesktopPaneBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQDesktopPaneBridge :: Native Callbacks.                                 -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls \c activeComponentChanged method of the desktop pane object.
 *
 * \param component the component
 */
void JQDesktopPaneBridge::activeComponentChanged(jobject component)
{
  jniObject()->callVoidMethod(sm_activeComponentChangedMID, component);
}

/*^**************************************************************************^*/
/*- JQDesktopPaneBridge :: Protected Member Function.                        -*/
/*^**************************************************************************^*/

/*!
 * \brief Sub window activated slot.
 *
 * \param window the activated window
 */
void JQDesktopPaneBridge::subWindowActivatedSlot(QMdiSubWindow *window)
{ 
  if (window) {
    JQInternalFrameBridge *internalFrameBridge =
                          qObjectBridgePointer<JQInternalFrameBridge *>(window);
    activeComponentChanged(internalFrameBridge->jniObject()->ref());
  } else {
    activeComponentChanged(0);
  }
}

/*^**************************************************************************^*/
/*- JQDesktopPaneBridge :: Overridden Member Functions.                      -*/
/*^**************************************************************************^*/

/*!
 * \inheritDoc
 */
jobject JQDesktopPaneBridge::getBackground(JNIEnv *env, jclass colorClass)
{
  return QColorAdapter::toJColor(env, colorClass,
                                 qMdiArea()->background().color());
}

/*!
 * \inheritDoc
 */
void JQDesktopPaneBridge::setBackground(JNIEnv *env, jint argb)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setBackground, QSWING_ENV_ARG, JINT_ARG(argb))
  qMdiArea()->setBackground(QColorAdapter(argb));
}

/*!
 * \inheritDoc
 */
void JQDesktopPaneBridge::setNullBackground(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setNullBackground, QSWING_ENV_ARG)
  qMdiArea()->setBackground(QApplication::palette().brush(QPalette::Dark));
}

/*^**************************************************************************^*/
/*- JQDesktopPaneBridge :: Private Static Data Member.                       -*/
/*^**************************************************************************^*/

jmethodID JQDesktopPaneBridge::sm_activeComponentChangedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-19 09:55:17 Slobodan
   Initial version

*****************************************************************************^*/
