/*
 * @(#)JQInternalFrameBridge.cpp  0.2.0 / 2007-11-19
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQInternalFrameBridge.h"

#include "InternalFrameWidget.h"
#include "QCursorAdapter.h"

/*^**************************************************************************^*/
/*- JQInternalFrameBridge :: Protected Constructor.                          -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQInternalFrameBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj JQInternalFrame object
 */
JQInternalFrameBridge::JQInternalFrameBridge(JNIEnv *env, jobject obj)
  : JQWindowBridge(env, obj, new QMdiSubWindow())
{
  m_internalWidget = new InternalFrameWidget(this);
  qMdiSubWindow()->setWidget(m_internalWidget);
  m_internalWidget->installEventFilter(this);
}

/*^**************************************************************************^*/
/*- JQInternalFrameBridge :: Native Calls.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Inserts child.
 *
 * \param env the JNIEnv pointer
 * \param childBridge the child bridge pointer
 * \param index the child index
 */
void JQInternalFrameBridge::insertChild(JNIEnv *env, jlong childBridge,
                                        jint index)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(insertChild, QSWING_ENV_ARG, JLONG_ARG(childBridge),
                   JINT_ARG(index))
  JQComponentBridge *childComponentBridge = jqComponentCast(childBridge);
  QWidget *childWidget = childComponentBridge->qWidget();
  QRect geom = childWidget->geometry();
  childWidget->setParent(qMdiSubWidget());
  childComponentBridge->setParentBridge(this);
  if (!geom.isNull()) {
    childWidget->setGeometry(geom);
  }
  if (childWidget->isVisible()) {
    childWidget->setVisible(true);
  }
}

/*^**************************************************************************^*/
/*- JQInternalFrameBridge :: Static Native Calls.                            -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQInternalFrameBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQInternalFrame class
 *
 * \exception FatalError if failed
 */
void JQInternalFrameBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_getLayoutPreferredSizeMID);
  if (!INIT_UMID(getLayoutPreferredSize, cls,
                "()Lyu/ac/bg/etf/javaqx/qswing/graphics/Dimension;")) {
    FATAL_LOGGED_ERROR("JQInternalFrameBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQInternalFrameBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQInternalFrameBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQInternalFrameBridge :: Native Callbacks.                               -*/
/*^**************************************************************************^*/

/*!
 * \brief Returns layout's preferred size.
 */
QSize JQInternalFrameBridge::getLayoutPreferredSize()
{
  jobject preferredSize;
  if (jniObject()->callObjectMethod(preferredSize,
                                    sm_getLayoutPreferredSizeMID)) {
    jint preferredWidth;
    jint preferredHeight;
    DimensionObject preferredSizeDimension(jniObject()->env(), preferredSize);
    if (preferredSizeDimension.getWidth(preferredWidth) &&
        preferredSizeDimension.getHeight(preferredHeight)) {
      return QSize(preferredWidth, preferredHeight);
    }
  }
  return QSize();
}

/*^**************************************************************************^*/
/*- JQInternalFrameBridge :: Overridden Member Function.                     -*/
/*^**************************************************************************^*/

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::setFocusable(JNIEnv *env, jboolean focusable)
{
  Q_UNUSED(env);
  Q_UNUSED(focusable);
  // empty method
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::requestFocus(JNIEnv *env)
{
  Q_UNUSED(env);
  // empty method
}

/*!
 * \inheritDoc
 */
jobject JQInternalFrameBridge::getBackground(JNIEnv *env, jclass colorClass)
{
  return QColorAdapter::toJColor(env, colorClass,
                          m_internalWidget->palette().color(QPalette::Window));
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::setBackground(JNIEnv *env, jint argb)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setBackground, QSWING_ENV_ARG, JINT_ARG(argb))
  QColorAdapter color(argb);
  QPalette palette(m_internalWidget->palette());
  if (color.isValid()) {
    m_internalWidget->setAutoFillBackground(true);
    palette.setColor(QPalette::Window, color);
  } else {
    m_internalWidget->setAutoFillBackground(false);
    palette.setColor(QPalette::Window,
                     QApplication::palette().color(QPalette::Window));
  }
  m_internalWidget->setPalette(palette);
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::setNullBackground(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setNullBackground, QSWING_ENV_ARG)
  QPalette palette(m_internalWidget->palette());
  palette.setColor(QPalette::Window,
                   QApplication::palette().color(QPalette::Window));
  m_internalWidget->setPalette(palette);
}

/*!
 * \inheritDoc
 */
jobject JQInternalFrameBridge::getForeground(JNIEnv *env, jclass colorClass)
{
  return QColorAdapter::toJColor(env, colorClass,
                       m_internalWidget->palette().color(QPalette::WindowText));
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::setForeground(JNIEnv *env, jint argb)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setForeground, QSWING_ENV_ARG, JINT_ARG(argb))
  QColorAdapter color(argb);
  QPalette palette(m_internalWidget->palette());
  if (color.isValid()) {
    palette.setColor(QPalette::WindowText, color);
  } else {
    palette.setColor(QPalette::WindowText,
                     QApplication::palette().color(QPalette::WindowText));
  }
  m_internalWidget->setPalette(palette);
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::setNullForeground(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setNullForeground, QSWING_ENV_ARG)
  QPalette palette(m_internalWidget->palette());
  palette.setColor(QPalette::WindowText,
                   QApplication::palette().color(QPalette::WindowText));
  m_internalWidget->setPalette(palette);
}

/*!
 * \inheritDoc
 */
jobject JQInternalFrameBridge::getFont(JNIEnv *env, jclass fontClass)
{
  return QFontAdapter::toJFont(env, fontClass, m_internalWidget->font());
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::setFont(JNIEnv *env, jobject font)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setFont, QSWING_ENV_ARG, JOBJECT_ARG(font))
  if (font) {
    m_internalWidget->setFont(*QFontAdapter::toQFont(env, font));
  } else {
    m_internalWidget->setFont(QFont());
  }
}

/*!
 * \inheritDoc
 */
jint JQInternalFrameBridge::getCursor(JNIEnv *env)
{
  Q_UNUSED(env);
  return QCursorAdapter::cursorType(m_internalWidget->cursor());
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::setCursor(JNIEnv *env, jint type)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setCursor, QSWING_ENV_ARG, JINT_ARG(type))
  m_internalWidget->setCursor(QCursorAdapter(type));
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::setComponentOrientation(JNIEnv *env,
                                                    jint orientation)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setComponentOrientation, QSWING_ENV_ARG,
                   JINT_ARG(orientation))
  switch(orientation) {
    case 1:
      m_internalWidget->unsetLayoutDirection();
      qMdiSubWindow()->unsetLayoutDirection();
      break;
    case 4:
      m_internalWidget->setLayoutDirection(Qt::LeftToRight);
      qMdiSubWindow()->setLayoutDirection(Qt::LeftToRight);
      break;
    default:
      m_internalWidget->setLayoutDirection(Qt::RightToLeft);
      qMdiSubWindow()->setLayoutDirection(Qt::RightToLeft);
      break;
  }
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::setToolTipText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setToolTipText, QSWING_ENV_ARG, JSTRING_ARG(text))
  m_internalWidget->setToolTip(QStringAdapter(env, text));
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::setStatusTipText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setStatusTipText, QSWING_ENV_ARG, JSTRING_ARG(text))
  m_internalWidget->setStatusTip(QStringAdapter(env, text));
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::setWhatsThisText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setWhatsThisText, QSWING_ENV_ARG, JSTRING_ARG(text))
  m_internalWidget->setWhatsThis(QStringAdapter(env, text));
}

/*!
 * \inheritDoc
 */
bool JQInternalFrameBridge::eventFilter(QObject *obj, QEvent *e)
{
  bool filtered = false;
  if (eventFilterEnabled()) {
    if (obj == qObject()) {
      switch ((int)e->type()) { // cast for QEvent::User types
        case QEvent::Enter:
          // falls through
        case QEvent::Leave:
          // falls through
        case QEvent::Resize:
          // falls through
        case QEvent::Paint:
          // falls through
        case QEvent::KeyPress:
          // falls through
        case QEvent::KeyRelease:
          // falls through
        case QSwingKeyTypedEvent::TYPE:
          // falls through
        case QEvent::MouseButtonPress:
          // falls through
        case QEvent::MouseButtonRelease:
          // falls through
        case QSwingMouseClickedEvent::TYPE:
          // falls through
        case QEvent::MouseButtonDblClick:
          // falls through
        case QEvent::ContextMenu:
          // falls through
        case QEvent::MouseMove:
          // falls through
        case QEvent::Wheel:
          // falls through
        case QEvent::FocusIn:
          // falls through
        case QEvent::FocusOut:
          break;
        default:
          filtered = JQWindowBridge::eventFilter(obj, e);
          break;
      }
    } else {
      switch ((int)e->type()) { // cast for QEvent::User types
        case QEvent::Enter:
          JQComponentBridge::eventFilter(obj, e);
          break;
          // falls through
        case QEvent::Leave:
          // falls through
        case QEvent::Resize:
          // falls through
        case QEvent::Paint:
          // falls through
        case QEvent::KeyPress:
          // falls through
        case QEvent::KeyRelease:
          // falls through
        case QSwingKeyTypedEvent::TYPE:
          // falls through
        case QEvent::MouseButtonPress:
          // falls through
        case QEvent::MouseButtonRelease:
          // falls through
        case QSwingMouseClickedEvent::TYPE:
          // falls through
        case QEvent::MouseButtonDblClick:
          // falls through
        case QEvent::ContextMenu:
          // falls through
        case QEvent::MouseMove:
          // falls through
        case QEvent::Wheel:
          // falls through
        case QEvent::FocusIn:
          // falls through
        case QEvent::FocusOut:
          filtered = JQComponentBridge::eventFilter(obj, e);
          break;
        default:
          break;
      }
    }
  }
  return filtered;
}

/*!
 * \inheritDoc
 */
void JQInternalFrameBridge::filterEnabledEvents()
{
  if (eventEnabled(QSWING_EVENT(MOUSE_MOTION_EVENT_MASK))) {
    m_internalWidget->setMouseTracking(true); // not m_widget!
  } else {
    m_internalWidget->setMouseTracking(false); // not m_widget!
  }
}

/*!
 * \inheritDoc
 */
QWidget *JQInternalFrameBridge::paintingWidget()
{
  return m_internalWidget;
}

/*^**************************************************************************^*/
/*- JQInternalFrameBridge :: Private Static Data Members.                    -*/
/*^**************************************************************************^*/

jmethodID JQInternalFrameBridge::sm_getLayoutPreferredSizeMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-19 16:04:41 Slobodan
   Initial version

*****************************************************************************^*/
