/*
 * @(#)JQLabelBridge.cpp  0.2.0 / 2007-07-28
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQLabelBridge.h"

#include "QIconAdapter.h"
#include "QSwingBridge.h"

/*^**************************************************************************^*/
/*- JQLabelBridge :: Protected Constructor.                                  -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQLabelBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQLabel object
 */
JQLabelBridge::JQLabelBridge(JNIEnv *env, jobject obj)
  : JQComponentBridge(env, obj, new QLabelAdapter())
{
  // empty constructor body
}

/*^**************************************************************************^*/
/*- JQLabelBridge :: Native Calls.                                           -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets label text.
 *
 * \param env the JNIEnv pointer 
 * \param text the text 
 */
void JQLabelBridge::setText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setText, QSWING_ENV_ARG, JSTRING_ARG(text))
  qLabelAdapter()->setText(QStringAdapter(env, text));
}

/*!
 * \brief Sets text format.
 *
 * \param env the JNIEnv pointer 
 * \param textFormat the text format
 */
void JQLabelBridge::setTextFormat(JNIEnv *env, jint textFormat)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setTextFormat, QSWING_ENV_ARG, JINT_ARG(textFormat))
  qLabelAdapter()->setTextFormat((Qt::TextFormat)textFormat);
}

/*!
 * \brief Sets text wrap status.
 *
 * \param env the JNIEnv pointer 
 * \param wordWrap if \c true the label will wrap the text
 */
void JQLabelBridge::setWordWrap(JNIEnv *env, jboolean wordWrap)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setWordWrap, QSWING_ENV_ARG, JBOOLEAN_ARG(wordWrap))
  qLabelAdapter()->setWordWrap(wordWrap);
}

/*!
 * \brief Sets text alignment.
 *
 * \param env the JNIEnv pointer 
 * \param ha the horizontal alignment
 * \param va the vertical alignment
 */
void JQLabelBridge::setAlignment(JNIEnv *env, jint ha, jint va)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setAlignment, QSWING_ENV_ARG, JINT_ARG(ha), JINT_ARG(va))
  qLabelAdapter()->setAlignment(QSwingBridge::toHorizontalAlignment(ha) |
                                QSwingBridge::toVerticalAlignment(va));
}

/*!
 * \brief Sets the icon set.
 *
 * \param env the JNIEnv pointer 
 * \param iconSet the icon set
 */
void JQLabelBridge::setIcon(JNIEnv *env, jobject iconSet)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setIcon, QSWING_ENV_ARG, JOBJECT_ARG(iconSet))
  if (iconSet) {
    qLabelAdapter()->setIcon(QIconAdapter(env, iconSet));
  } else {
    qLabelAdapter()->setIcon(QIcon());
  }
}

/*!
 * \brief Sets icon size.
 *
 * \param env the JNIEnv pointer 
 * \param width the icon width 
 * \param height the icon height 
 */
void JQLabelBridge::setIconSize(JNIEnv *env, jint width, jint height)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setIconSize, QSWING_ENV_ARG, JINT_ARG(width),
                   JINT_ARG(height))
  qLabelAdapter()->setIconSize(QSize(width, height));
}

/*!
 * \brief Sets border type.
 *
 * \param env the JNIEnv pointer
 * \param borderType the border type
 */
void JQLabelBridge::setBorderType(JNIEnv *env, jint borderType)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setBorderType, QSWING_ENV_ARG, JINT_ARG(borderType))
  qLabelAdapter()->setFrameShape(QFrame::Shape(borderType));
}

/*!
 * \brief Sets border style.
 *
 * \param env the JNIEnv pointer
 * \param borderStyle the border style
 */
void JQLabelBridge::setBorderStyle(JNIEnv *env, jint borderStyle)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setBorderStyle, QSWING_ENV_ARG, JINT_ARG(borderStyle))
  qLabelAdapter()->setFrameShadow(QFrame::Shadow(borderStyle));
}

/*!
 * \brief Sets border width.
 *
 * \param env the JNIEnv pointer
 * \param borderWidth the border width
 */
void JQLabelBridge::setBorderWidth(JNIEnv *env, jint borderWidth)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setBorderWidth, QSWING_ENV_ARG, JINT_ARG(borderWidth))
  qLabelAdapter()->setMidLineWidth(borderWidth);
}

/*^**************************************************************************^*/
/*- JQLabelBridge :: Static Native Calls.                                    -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQLabelBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQLabel class
 *
 * \exception FatalError if failed
 */
void JQLabelBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_UNUSED(env);
  Q_UNUSED(cls);
  // todo: Implement action links
}

/*!
 * \brief Returns new instance of the bridge object.
 *
 * \param env the JNIEnv pointer 
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQLabelBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQLabelBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^*****************************************************************************
  File History:

 - 2007-07-28 12:57:43 Slobodan
   Initial version

*****************************************************************************^*/
