/*
 * @(#)JQMenuItemBridge.cpp  0.2.0 / 2007-11-09
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQMenuItemBridge.h"

#include "QApplicationBridge.h"
#include "QIconAdapter.h"

/*^**************************************************************************^*/
/*- JQMenuItemBridge :: Protected Constructor.                               -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQMenuItemBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQMenuItem object
 */
JQMenuItemBridge::JQMenuItemBridge(JNIEnv *env, jobject obj)
  : JQActionComponentBridge(env, obj), m_shortcut(0)
{
  // empty constructor body
}

/*^**************************************************************************^*/
/*- JQMenuItemBridge :: Native Calls.                                        -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets text
 *
 * \param env the JNIEnv pointer
 * \param text the text
 */
void JQMenuItemBridge::setText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setText, QSWING_ENV_ARG, JSTRING_ARG(text))
  if (isQWidget()) {
    label()->setText(QStringAdapter(env, text));
  } else {
    action()->setText(QStringAdapter(env, text));
  }
}

/*!
 * \brief Sets short cut
 *
 * \param env the JNIEnv pointer
 * \param shortcut the shortcut
 */
void JQMenuItemBridge::setShortcut(JNIEnv *env, jstring shortcut)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setShortcut, QSWING_ENV_ARG, JSTRING_ARG(shortcut))
  QStringAdapter shortcutString(env, shortcut);
  if (isQWidget()) {
    if (!m_shortcut) {
      m_shortcut = new QShortcut(widget());
    }
    if (shortcutString.isEmpty()) {
      m_shortcut->setKey(QKeySequence::UnknownKey);
    } else {
      m_shortcut->setKey(QKeySequence(shortcutString));
    }
  } else {
    if (shortcutString.isEmpty()) {
      action()->setShortcut(QKeySequence::UnknownKey);
    } else {
      action()->setShortcut(QKeySequence(shortcutString));
    }
  }
}

/*!
 * \brief Sets icon
 *
 * \param env the JNIEnv pointer
 * \param iconSet the icon
 */
void JQMenuItemBridge::setIcon(JNIEnv *env, jobject iconSet)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setIcon, QSWING_ENV_ARG, JOBJECT_ARG(iconSet))
  if (isQWidget()) {
    if (iconSet) {
      label()->setIcon(QIconAdapter(env, iconSet));
    } else {
      label()->setIcon(QIcon());
    }
  } else {
    if (iconSet) {
      action()->setIcon(QIconAdapter(env, iconSet));
    } else {
      action()->setIcon(QIcon());
    }
  }
}

/*^**************************************************************************^*/
/*- JQMenuItemBridge :: Static Native Calls.                                 -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQMenuItemBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQMenuItem class
 *
 * \exception FatalError if failed
 */
void JQMenuItemBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT (!sm_actionTriggeredMID);
  if (!INIT_UMID(actionTriggered, cls, "(I)V")) {
    FATAL_LOGGED_ERROR("JQMenuItemBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQMenuItemBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQMenuItemBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQMenuItemBridge :: Native Callbacks.                                    -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls \c actionTriggered method of the JQMenuItemBridge object.
 *
 * \param modifiers the modifiers
 */
void JQMenuItemBridge::actionTriggered(jint modifiers)
{
  jniObject()->callVoidMethod(sm_actionTriggeredMID, modifiers);
}

/*^**************************************************************************^*/
/*- JQMenuItemBridge :: Protected Member Function.                           -*/
/*^**************************************************************************^*/

/*!
 * \brief Triggered lot.
 */
void JQMenuItemBridge::triggeredSlot()
{
  actionTriggered(QApplicationBridge::keyboardModifiers());
}

/*^**************************************************************************^*/
/*- JQMenuItemBridge :: Overridden Member Functions.                         -*/
/*^**************************************************************************^*/

/*!
 * \brief Creates QWidget adaptee.
 *
 * \return QWidget adaptee
 */
QWidget *JQMenuItemBridge::createQWidget()
{
  return new QLabelAdapter();
}

/*!
 * \brief Creates QAction adaptee.
 *
 * \return QAction adaptee
 */
QAction *JQMenuItemBridge::createQAction()
{
  QAction *action = new QAction(this);
  connectDirect(action, SIGNAL(triggered()), SLOT(triggeredSlot()));
  return action;
}

/*!
 * \inheritDoc
 */
void JQMenuItemBridge::deleteQWidget(QWidget *widget)
{
  if (m_shortcut) {
    delete m_shortcut;
    m_shortcut = 0;
  }
  JQActionComponentBridge::deleteQWidget(widget);
}

/*!
 * \inheritDoc
 */
void JQMenuItemBridge::deleteQAction(QAction *action)
{
  disconnect(action, SIGNAL(triggered()), SLOT(triggeredSlot()));
  JQActionComponentBridge::deleteQAction(action);
}

/*!
 * \inheritDoc
 */
QWidget *JQMenuItemBridge::convertToQWidget(QAction *action)
{
  QLabelAdapter *label = static_cast<QLabelAdapter *>(
                             JQActionComponentBridge::convertToQWidget(action));
  label->setText(action->text());
  if (m_shortcut) {
    delete m_shortcut;
  }
  m_shortcut = new QShortcut(label);
  m_shortcut->setKey(action->shortcut());
  label->setIcon(action->icon());
  return label;
}

/*!
 * \inheritDoc
 */
QAction *JQMenuItemBridge::convertToQAction(QWidget *widget)
{
  QLabelAdapter *label = static_cast<QLabelAdapter *>(widget);
  QAction *action = JQActionComponentBridge::convertToQAction(widget);
  action->setText(label->text());
  if (m_shortcut) {
    action->setShortcut(m_shortcut->key());
  }
  action->setIcon(label->icon());
  return action;
}

/*^**************************************************************************^*/
/*- JQMenuItemBridge :: Private Static Data Member.                          -*/
/*^**************************************************************************^*/

jmethodID JQMenuItemBridge::sm_actionTriggeredMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-09 19:21:03 Slobodan
   Initial version

*****************************************************************************^*/
