/*
 * @(#)JQRadioButtonMenuItemBridge.cpp  0.2.0 / 2007-11-14
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQRadioButtonMenuItemBridge.h"

#include "QApplicationBridge.h"
#include "QIconAdapter.h"

/*^**************************************************************************^*/
/*- JQRadioButtonMenuItemBridge :: Protected Constructor.                    -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQRadioButtonMenuItemBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQRadioButtonMenuItem object
 */
JQRadioButtonMenuItemBridge::JQRadioButtonMenuItemBridge(JNIEnv *env,
                                                         jobject obj)
  : JQActionComponentBridge(env, obj), m_shortcut(0)
{
  // empty constructor body
}

/*^**************************************************************************^*/
/*- JQRadioButtonMenuItemBridge :: Native Calls.                             -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets text
 *
 * \param env the JNIEnv pointer
 * \param text the text
 */
void JQRadioButtonMenuItemBridge::setText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  Q_ASSERT(text);
  QSWING_REENTRANT(setText, QSWING_ENV_ARG, JSTRING_ARG(text))
  if (isQWidget()) {
    radioButton()->setText(QStringAdapter(env, text));
  } else {
    action()->setText(QStringAdapter(env, text));
  }
}

/*!
 * \brief Sets short cut.
 *
 * \param env the JNIEnv pointer
 * \param shortcut the shortcut
 */
void JQRadioButtonMenuItemBridge::setShortcut(JNIEnv *env, jstring shortcut)
{
  Q_ASSERT(env);
  Q_ASSERT(shortcut);
  QSWING_REENTRANT(setShortcut, QSWING_ENV_ARG, JSTRING_ARG(shortcut))
  QStringAdapter shortcutString(env, shortcut);
  if (isQWidget()) {
    if (!m_shortcut) {
      m_shortcut = new QShortcut(widget());
    }
    if (shortcutString.isEmpty()) {
      m_shortcut->setKey(QKeySequence::UnknownKey);
    } else {
      m_shortcut->setKey(QKeySequence(shortcutString));
    }
  } else {
    if (shortcutString.isEmpty()) {
      action()->setShortcut(QKeySequence::UnknownKey);
    } else {
      action()->setShortcut(QKeySequence(shortcutString));
    }
  }
}

/*!
 * \brief Sets icon.
 *
 * \param env the JNIEnv pointer
 * \param iconSet the icon
 */
void JQRadioButtonMenuItemBridge::setIcon(JNIEnv *env, jobject iconSet)
{
  Q_ASSERT(env);
  Q_ASSERT(iconSet);
  QSWING_REENTRANT(setIcon, QSWING_ENV_ARG, JOBJECT_ARG(iconSet))
  if (isQWidget()) {
    if (iconSet) {
      radioButton()->setIcon(QIconAdapter(env, iconSet));
    } else {
      radioButton()->setIcon(QIcon());
    }
  } else {
    if (iconSet) {
      action()->setIcon(QIconAdapter(env, iconSet));
    } else {
      action()->setIcon(QIcon());
    }
  }
}

/*!
 * \brief Sets whether or not the item is selected.
 *
 * \param env the JNIEnv pointer
 * \param selected \c true if selected
 */
void JQRadioButtonMenuItemBridge::setSelected(JNIEnv *env, jboolean selected)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setSelected, QSWING_ENV_ARG, JBOOLEAN_ARG(selected))
  if (isQWidget()) {
    radioButton()->setChecked(selected);
  } else {
    action()->setChecked(selected);
  }
}

/*^**************************************************************************^*/
/*- JQRadioButtonMenuItemBridge :: Static Native Calls.                      -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQRadioButtonMenuItemBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQRadioButtonMenuItem class
 *
 * \exception FatalError if failed
 */
void JQRadioButtonMenuItemBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT (!sm_selectedChangedMID);
  if (!INIT_UMID(selectedChanged, cls, "(IZ)V")) {
    FATAL_LOGGED_ERROR("JQRadioButtonMenuItemBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQRadioButtonMenuItemBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQRadioButtonMenuItemBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQRadioButtonMenuItemBridge :: Native Callbacks.                         -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls \c selectedChanged method of the JQMenuItemBridge object.
 *
 * \param modifiers the keyboard modifiers
 * \param selected \c true if the item is selected
 */
void JQRadioButtonMenuItemBridge::selectedChanged(jint modifiers,
                                                  jboolean selected)
{
  jniObject()->callVoidMethod(sm_selectedChangedMID, modifiers, selected);
}

/*^**************************************************************************^*/
/*- JQRadioButtonMenuItemBridge :: Protected Member Function.                -*/
/*^**************************************************************************^*/

/*!
 * \brief Toggled slot.
 *
 * \param checked \c true if the item is checked
 */
void JQRadioButtonMenuItemBridge::toggledSlot(bool checked)
{
  selectedChanged(QApplicationBridge::keyboardModifiers(), checked);
}

/*^**************************************************************************^*/
/*- JQRadioButtonMenuItemBridge :: Overridden Member Functions.              -*/
/*^**************************************************************************^*/

/*!
 * \brief Creates QWidget adaptee.
 *
 * \return QWidget adaptee
 */
QWidget *JQRadioButtonMenuItemBridge::createQWidget()
{
  QRadioButton *radioButton = new QRadioButton();
  radioButton->setAutoExclusive(false);
  connectDirect(radioButton, SIGNAL(toggled(bool)), SLOT(toggledSlot(bool)));
  return radioButton;
}

/*!
 * \brief Creates QAction adaptee.
 *
 * \return QAction adaptee
 */
QAction *JQRadioButtonMenuItemBridge::createQAction()
{
  QAction *action = new QAction(this);
  action->setCheckable(true);
  m_actionGroup = new QActionGroup(this);
  action->setActionGroup(m_actionGroup);
  connectDirect(action, SIGNAL(toggled(bool)), SLOT(toggledSlot(bool)));
  return action;
}

/*!
 * \inheritDoc
 */
void JQRadioButtonMenuItemBridge::deleteQWidget(QWidget *widget)
{
  if (m_shortcut) {
    delete m_shortcut;
    m_shortcut = 0;
  }
  connectDirect(widget, SIGNAL(toggled(bool)), SLOT(toggledSlot(bool)));
  JQActionComponentBridge::deleteQWidget(widget);
}

/*!
 * \inheritDoc
 */
void JQRadioButtonMenuItemBridge::deleteQAction(QAction *action)
{
  disconnect(action, SIGNAL(toggled(bool)), SLOT(toggledSlot(bool)));
  JQActionComponentBridge::deleteQAction(action);
  delete m_actionGroup;
}

/*!
 * \inheritDoc
 */
QWidget *JQRadioButtonMenuItemBridge::convertToQWidget(QAction *action)
{
  QRadioButton *radioButton = static_cast<QRadioButton *>(
                             JQActionComponentBridge::convertToQWidget(action));
  radioButton->setText(action->text());
  if (m_shortcut) {
    delete m_shortcut;
  }
  m_shortcut = new QShortcut(radioButton);
  m_shortcut->setKey(action->shortcut());
  radioButton->setChecked(action->isChecked());
  radioButton->setIcon(action->icon());
  return radioButton;
}

/*!
 * \inheritDoc
 */
QAction *JQRadioButtonMenuItemBridge::convertToQAction(QWidget *widget)
{
  QRadioButton *radioButton = static_cast<QRadioButton *>(widget);
  QAction *action = JQActionComponentBridge::convertToQAction(radioButton);
  action->setText(radioButton->text());
  if (m_shortcut) {
    action->setShortcut(m_shortcut->key());
  }
  action->setChecked(radioButton->isChecked());
  action->setIcon(radioButton->icon());
  return action;
}

/*^**************************************************************************^*/
/*- JQRadioButtonMenuItemBridge :: Private Static Data Member.               -*/
/*^**************************************************************************^*/

jmethodID JQRadioButtonMenuItemBridge::sm_selectedChangedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-14 12:51:06 Slobodan
   Initial version

*****************************************************************************^*/
