/*
 * @(#)JQScrollBarBridge.cpp  0.2.0 / 2007-11-06
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQScrollBarBridge.h"

/*^**************************************************************************^*/
/*- JQScrollBarBridge :: Protected Constructor.                              -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQScrollBarBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQScrollBar object
 */
JQScrollBarBridge::JQScrollBarBridge(JNIEnv *env, jobject obj)
  : JQComponentBridge(env, obj, new QScrollBar(Qt::Vertical))
{
  qScrollBar()->setMaximum(100);
  qScrollBar()->setFocusPolicy(Qt::StrongFocus);
  connectDirect(qScrollBar(), SIGNAL(valueChanged(int)),
                SLOT(valueChangedSlot(int)));
  connectDirect(qScrollBar(), SIGNAL(sliderPressed()),
                SLOT(sliderPressedSlot()));
  connectDirect(qScrollBar(), SIGNAL(sliderReleased()),
                SLOT(sliderReleasedSlot()));
}

/*^**************************************************************************^*/
/*- JQScrollBarBridge :: Native Calls.                                       -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets scroll bar model.
 *
 * \param env the JNIEnv pointer
 * \param minimum the minimum scroll value
 * \param maximum the maximum scroll value
 * \param value the scroll value
 */
void JQScrollBarBridge::setModel(JNIEnv *env, jint minimum, jint maximum,
                                 jint value)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setModel, QSWING_ENV_ARG, JINT_ARG(minimum),
                   JINT_ARG(maximum), JINT_ARG(value))
  qScrollBar()->setRange(minimum, maximum);
  qScrollBar()->setValue(value);
}

/*!
 * \brief Sets scroll value.
 *
 * \param env the JNIEnv pointer
 * \param value the scroll value
 */
void JQScrollBarBridge::setValue(JNIEnv *env, jint value)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setValue, QSWING_ENV_ARG, JINT_ARG(value))
  qScrollBar()->setValue(value);
}

/*!
 * \brief Sets minimum scroll value.
 *
 * \param env the JNIEnv pointer
 * \param minimum the minimum scroll value
 */
void JQScrollBarBridge::setMinimum(JNIEnv *env, jint minimum)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setMinimum, QSWING_ENV_ARG, JINT_ARG(minimum))
  qScrollBar()->setMinimum(minimum);
}

/*!
 * \brief Sets maximum scroll value.
 *
 * \param env the JNIEnv pointer
 * \param maximum the maximum scroll value
 */
void JQScrollBarBridge::setMaximum(JNIEnv *env, jint maximum)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setMaximum, QSWING_ENV_ARG, JINT_ARG(maximum))
  qScrollBar()->setMaximum(maximum);
}

/*!
 * \brief Sets scroll bar orientation.
 *
 * \param env the JNIEnv pointer
 * \param orientation the orientation
 */
void JQScrollBarBridge::setOrientation(JNIEnv *env, jint orientation)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setOrientation, QSWING_ENV_ARG, JINT_ARG(orientation))
  switch(orientation) {
    case 0:
      qScrollBar()->setOrientation(Qt::Horizontal);
      break;
    case 1:
      qScrollBar()->setOrientation(Qt::Vertical);
      break;
  }
}

/*!
 * \brief Sets scroll unit increment.
 *
 * \param env the JNIEnv pointer
 * \param unitIncrement the unit increment
 */
void JQScrollBarBridge::setUnitIncrement(JNIEnv *env, jint unitIncrement)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setUnitIncrement, QSWING_ENV_ARG, JINT_ARG(unitIncrement))
  qScrollBar()->setSingleStep(unitIncrement);
}

/*!
 * \brief Sets scroll block increment.
 *
 * \param env the JNIEnv pointer
 * \param blockIncrement the block increment
 */
void JQScrollBarBridge::setBlockIncrement(JNIEnv *env, jint blockIncrement)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setBlockIncrement, QSWING_ENV_ARG, JINT_ARG(blockIncrement))
  qScrollBar()->setPageStep(blockIncrement);
}

/*^**************************************************************************^*/
/*- JQScrollBarBridge :: Static Native Calls.                                -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes the JQScrollBarBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQScrollBar class
 *
 * \exception FatalError if failed
 */
void JQScrollBarBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_valueChangedMID);
  if (!INIT_UMID(valueChanged, cls, "(I)V") ||
      !INIT_UMID(valueIsAdjusting, cls, "(Z)V")) {
    FATAL_LOGGED_ERROR("JQScrollBarBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQScrollBarBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQScrollBarBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQScrollBarBridge :: Native Callbacks.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls \c valueChanged method of the JQScrollBar object.
 *
 * \param value the changed value
 */
void JQScrollBarBridge::valueChanged(jint value)
{
  jniObject()->callVoidMethod(sm_valueChangedMID, value);
}

/*!
 * \brief Calls \c valueIsAdjusting method of the JQScrollBar object.
 *
 * \param adjusting \c true if adjusting
 */
void JQScrollBarBridge::valueIsAdjusting(jboolean adjusting)
{
  jniObject()->callVoidMethod(sm_valueIsAdjustingMID, adjusting);
}

/*^**************************************************************************^*/
/*- JQScrollBarBridge :: Protected Member Functions.                         -*/
/*^**************************************************************************^*/

/*!
 * \brief \c valueChanged slot.
 *
 * \param value the value
 */
void JQScrollBarBridge::valueChangedSlot(int value)
{
  valueChanged(value);
}

/*!
 * \brief \c sliderPressed slot.
 */
void JQScrollBarBridge::sliderPressedSlot()
{
  valueIsAdjusting(true);
}

/*!
 * \brief \c sliderReleased slot.
 */
void JQScrollBarBridge::sliderReleasedSlot()
{
  valueIsAdjusting(false);
}

/*^**************************************************************************^*/
/*- JQScrollBarBridge :: Static Data Members.                                -*/
/*^**************************************************************************^*/

jmethodID JQScrollBarBridge::sm_valueChangedMID = 0;
jmethodID JQScrollBarBridge::sm_valueIsAdjustingMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-06 06:51:23 JavaQx Native Generator
   Initial version

*****************************************************************************^*/
