/*
 * @(#)JQSeparatorBridge.cpp  0.2.0 / 2007-11-09
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQSeparatorBridge.h"

#include "JQToolBarBridge.h"

/*^**************************************************************************^*/
/*- JQSeparatorData :: Protected Constructor.                                -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQSeparatorData object.
 */
JQSeparatorData::JQSeparatorData()
  : JQActionComponentData(), orientation(Qt::Horizontal)
{
  // empty constructor body
}

/*^**************************************************************************^*/
/*- JQSeparatorBridge :: Protected Constructor.                              -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQSeparatorBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQSeparator object
 */
JQSeparatorBridge::JQSeparatorBridge(JNIEnv *env, jobject obj)
  : JQActionComponentBridge(env, obj)
{
  // empty constructor body
}

/*^**************************************************************************^*/
/*- JQSeparatorBridge :: Native Calls.                                       -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets separator orientation.
 *
 * \param env the JNIEnv pointer
 * \param orientation the orientation
 */
void JQSeparatorBridge::setOrientation(JNIEnv *env, jint orientation)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setOrientation, QSWING_ENV_ARG, JINT_ARG(orientation))
  switch(orientation) {
    case 0:
      if (isQWidget()) {
        separator()->setOrientation(Qt::Horizontal);
      } else {
        separatorActionData()->orientation = Qt::Horizontal;
      }
      break;
    case 1:
      if (isQWidget()) {
        separator()->setOrientation(Qt::Vertical);
      } else {
        separatorActionData()->orientation = Qt::Vertical;
      }
      break;
  }
  jniObject()->callVoidMethod(sm_orientationChangedMID, orientation);
}

/*!
 * \brief Sets separator text.
 *
 * \param env the JNIEnv pointer
 * \param text the text
 */
void JQSeparatorBridge::setText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  Q_ASSERT(text);
  QSWING_REENTRANT(setText, QSWING_ENV_ARG, JSTRING_ARG(text))
  if (isQWidget()) {
    separator()->setText(QStringAdapter(env, text));
  } else {
    action()->setText(QStringAdapter(env, text));
  }
}

/*^**************************************************************************^*/
/*- JQSeparatorBridge :: Static Native Calls.                                -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQSeparatorBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQSeparator class
 *
 * \exception FatalError if failed
 */
void JQSeparatorBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_orientationChangedMID);
  if (!INIT_UMID(orientationChanged, cls, "(I)V")) {
    FATAL_LOGGED_ERROR("JQSeparatorBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQSeparatorBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQSeparatorBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQSeparatorBridge :: Native Callback.                                    -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls \c orientationChanged method of the separator object.
 *
 * \param orientation the orientation
 */
void JQSeparatorBridge::orientationChanged(jint orientation)
{
  jniObject()->callVoidMethod(sm_orientationChangedMID, orientation);
}

/*^**************************************************************************^*/
/*- JQSeparatorBridge :: Protected Method.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief orientationChanged slot.
 *
 * \param orientation the orientation
 */
void JQSeparatorBridge::orientationChangedSlot(Qt::Orientation orientation)
{
  switch(orientation) {
    case Qt::Horizontal:
      orientationChanged(0);
      break;
    case Qt::Vertical:
      orientationChanged(1);
      break;
  }
}

/*^**************************************************************************^*/
/*- JQSeparatorBridge :: Overridden Member Functions.                        -*/
/*^**************************************************************************^*/

/*!
 * \inheritDoc
 */
void JQSeparatorBridge::setParentBridge(JQContainerBridge *parentBridge)
{
  JQContainerBridge *oldParentBridge = this->parentBridge();
  JQToolBarBridge *toolBarBridge;
  if (oldParentBridge) {
    toolBarBridge = qobject_cast<JQToolBarBridge *>(oldParentBridge);
    if (toolBarBridge) {
      disconnect(toolBarBridge->qToolBar(),
                 SIGNAL(orientationChanged(Qt::Orientation)),
                 SLOT(orientationChangedSlot(Qt::Orientation)));
    }
  }
  if (parentBridge) {
    toolBarBridge = qobject_cast<JQToolBarBridge *>(parentBridge);
    if (toolBarBridge) {
      connectDirect(toolBarBridge->qToolBar(),
                    SIGNAL(orientationChanged(Qt::Orientation)),
                    SLOT(orientationChangedSlot(Qt::Orientation)));
      orientationChangedSlot(toolBarBridge->qToolBar()->orientation());
    }
  }
  JQActionComponentBridge::setParentBridge(parentBridge);
}

/*!
 * \inheritDoc
 */
void JQSeparatorBridge::setForeground(JNIEnv *env, jint argb)
{
  if (isQWidget()) {
    separator()->setUseStylePalette(false);
  }
  JQActionComponentBridge::setForeground(env, argb);
}

/*!
 * \inheritDoc
 */
void JQSeparatorBridge::setNullForeground(JNIEnv *env)
{
  if (isQWidget()) {
    separator()->setUseStylePalette(true);
  }
  JQActionComponentBridge::setNullForeground(env);
}

/*!
 * \brief Creates QWidget adaptee.
 *
 * \return QWidget adaptee
 */
QWidget *JQSeparatorBridge::createQWidget()
{
  return new QSeparatorAdapter();
}

/*!
 * \brief Creates QAction adaptee.
 *
 * \return QAction adaptee
 */
QAction *JQSeparatorBridge::createQAction()
{
  QAction *action = new QAction(this);
  action->setSeparator(true);
  return action;
}

/*!
 * \inheritDoc
 */
QWidget *JQSeparatorBridge::convertToQWidget(QAction *action)
{
  QSeparatorAdapter *separator = static_cast<QSeparatorAdapter *>(
                             JQActionComponentBridge::convertToQWidget(action));
  separator->setText(action->text());
  JQSeparatorData *data = static_cast<JQSeparatorData *>(getActionData(action));
  if (data) {
    separator->setOrientation(data->orientation);
    separator->setUseStylePalette(!data->foreground.isValid());
  }
  return separator;
}

/*!
 * \inheritDoc
 */
QAction *JQSeparatorBridge::convertToQAction(QWidget *widget)
{
  QSeparatorAdapter *separator = static_cast<QSeparatorAdapter *>(widget);
  QAction *action = JQActionComponentBridge::convertToQAction(widget);
  action->setText(separator->text());
  JQSeparatorData *data = static_cast<JQSeparatorData *>(getActionData(action));
  data->orientation = separator->orientation();
  return action;
}

/*!
 * \inheritDoc
 */
JQActionComponentData *JQSeparatorBridge::createActionData()
{
  return new JQSeparatorData();
}

/*^**************************************************************************^*/
/*- JQSeparatorBridge :: Private Static Data Member.                         -*/
/*^**************************************************************************^*/

jmethodID JQSeparatorBridge::sm_orientationChangedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-09 17:31:07 JavaQx Native Generator
   Initial version

*****************************************************************************^*/
