/*
 * @(#)JQSliderBridge.cpp  0.2.0 / 2007-11-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQSliderBridge.h"

/*^**************************************************************************^*/
/*- JQSliderBridge :: Protected Constructor.                                 -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQSliderBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj JQSlider object
 */
JQSliderBridge::JQSliderBridge(JNIEnv *env, jobject obj)
  : JQComponentBridge(env, obj, new QSlider(Qt::Horizontal))
{
  qSlider()->setMaximum(100);
  qSlider()->setValue(50);
  qSlider()->setTickInterval(10);
  qSlider()->setTickPosition(QSlider::NoTicks);
  connectDirect(qSlider(), SIGNAL(valueChanged(int)),
                SLOT(valueChangedSlot(int)));
  connectDirect(qSlider(), SIGNAL(sliderPressed()),
                SLOT(sliderPressedSlot()));
  connectDirect(qSlider(), SIGNAL(sliderReleased()),
                SLOT(sliderReleasedSlot()));
}

/*^**************************************************************************^*/
/*- JQSliderBridge :: Native Calls.                                          -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets slider model.
 *
 * \param env the JNIEnv pointer
 * \param minimum the minimum slider value
 * \param maximum the maximum slider value
 * \param value the slider value
 */
void JQSliderBridge::setModel(JNIEnv *env, jint minimum, jint maximum,
                              jint value)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setModel, QSWING_ENV_ARG, JINT_ARG(minimum),
                   JINT_ARG(maximum), JINT_ARG(value))
  qSlider()->setRange(minimum, maximum);
  qSlider()->setValue(value);
}

/*!
 * \brief Sets slider value.
 *
 * \param env the JNIEnv pointer
 * \param value the slider value
 */
void JQSliderBridge::setValue(JNIEnv *env, jint value)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setValue, QSWING_ENV_ARG, JINT_ARG(value))
  qSlider()->setValue(value);
}

/*!
 * \brief Sets minimum slider value.
 *
 * \param env the JNIEnv pointer
 * \param minimum the minimum slider value
 */
void JQSliderBridge::setMinimum(JNIEnv *env, jint minimum)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setMinimum, QSWING_ENV_ARG, JINT_ARG(minimum))
  qSlider()->setMinimum(minimum);
}

/*!
 * \brief Sets maximum slider value.
 *
 * \param env the JNIEnv pointer
 * \param maximum the maximum slider value
 */
void JQSliderBridge::setMaximum(JNIEnv *env, jint maximum)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setMaximum, QSWING_ENV_ARG, JINT_ARG(maximum))
  qSlider()->setMaximum(maximum);
}

/*!
 * \brief Sets slider orientation.
 *
 * \param env the JNIEnv pointer
 * \param orientation the orientation
 */
void JQSliderBridge::setOrientation(JNIEnv *env, jint orientation)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setOrientation, QSWING_ENV_ARG, JINT_ARG(orientation))
  switch(orientation) {
    case 0:
      qSlider()->setOrientation(Qt::Horizontal);
      break;
    case 1:
      qSlider()->setOrientation(Qt::Vertical);
      break;
  }
}

/*!
 * \brief Sets tick position.
 *
 * \param env the JNIEnv pointer
 * \param tickPosition the tick position
 */
void JQSliderBridge::setTickPosition(JNIEnv *env, jint tickPosition)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setTickPosition, QSWING_ENV_ARG, JINT_ARG(tickPosition))
  qSlider()->setTickPosition(QSlider::TickPosition(tickPosition));
}

/*!
 * \brief Sets tick spacing
 *
 * \param env the JNIEnv pointer
 * \param tickSpacing the tick spacing
 */
void JQSliderBridge::setTickSpacing(JNIEnv *env, jint tickSpacing)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setTickSpacing, QSWING_ENV_ARG, JINT_ARG(tickSpacing))
  qSlider()->setTickInterval(tickSpacing);
}

/*^**************************************************************************^*/
/*- JQSliderBridge :: Static Native Calls.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes the JQSliderBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQSlider class
 *
 * \exception FatalError if failed
 */
void JQSliderBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_valueChangedMID);
  if (!INIT_UMID(valueChanged, cls, "(I)V") ||
      !INIT_UMID(valueIsAdjusting, cls, "(Z)V")) {
    FATAL_LOGGED_ERROR("JQSliderBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQSliderBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQSliderBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQSliderBridge :: Native Callbacks.                                      -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls \c valueChanged method of the JQScrollBar object.
 *
 * \param value the changed value
 */
void JQSliderBridge::valueChanged(jint value)
{
  jniObject()->callVoidMethod(sm_valueChangedMID, value);
}

/*!
 * \brief Calls \c valueIsAdjusting method of the JQScrollBar object.
 *
 * \param adjusting \c true if adjusting
 */
void JQSliderBridge::valueIsAdjusting(jboolean adjusting)
{
  jniObject()->callVoidMethod(sm_valueIsAdjustingMID, adjusting);
}

/*^**************************************************************************^*/
/*- JQSliderBridge :: Protected Member Functions.                            -*/
/*^**************************************************************************^*/

/*!
 * \brief \c valueChanged slot.
 *
 * \param value the value
 */
void JQSliderBridge::valueChangedSlot(int value)
{
  valueChanged(value);
}

/*!
 * \brief \c sliderPressed slot.
 */
void JQSliderBridge::sliderPressedSlot()
{
  valueIsAdjusting(true);
}

/*!
 * \brief \c sliderReleased slot.
 */
void JQSliderBridge::sliderReleasedSlot()
{
  valueIsAdjusting(false);
}

/*^**************************************************************************^*/
/*- JQSliderBridge :: Private Static Data Members.                           -*/
/*^**************************************************************************^*/

jmethodID JQSliderBridge::sm_valueChangedMID = 0;
jmethodID JQSliderBridge::sm_valueIsAdjustingMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-04 23:07:33 JavaQx Native Generator
   Initial version

*****************************************************************************^*/
