/*
 * @(#)JQSplitPaneBridge.cpp  0.2.0 / 2007-12-04
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQSplitPaneBridge.h"

/*^**************************************************************************^*/
/*- JQSplitPaneBridge :: Protected Constructor.                              -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQSplitPaneBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQSplitPane object
 */
JQSplitPaneBridge::JQSplitPaneBridge(JNIEnv *env, jobject obj)
  : JQComponentBridge(env, obj, new QSplitterAdapter())
{
  connectDirect(qSplitterAdapter(), SIGNAL(splitterMoved(int, int)),
                SLOT(splitterMovedSlot(int, int)));
}

/*^**************************************************************************^*/
/*- JQSplitPaneBridge :: Native Calls.                                       -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets left component of the split pane.
 *
 * \param env the JNIEnv pointer
 * \param componentBridge the left component bridge pointer
 */
void JQSplitPaneBridge::setLeftComponent(JNIEnv *env, jlong componentBridge)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setLeftComponent, QSWING_ENV_ARG, JLONG_ARG(componentBridge))
  if (componentBridge) {
    qSplitterAdapter()->insertWidget(0, jqComponentCast(componentBridge)->
                                                                     qWidget());
  } else {
    qSplitterAdapter()->widget(0)->setParent(0);
  }
}

/*!
 * \brief Sets right component of the split pane.
 *
 * \param env the JNIEnv pointer
 * \param componentBridge the right component bridge pointer
 */
void JQSplitPaneBridge::setRightComponent(JNIEnv *env, jlong componentBridge)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setRightComponent, QSWING_ENV_ARG,
                   JLONG_ARG(componentBridge))
  if (componentBridge) {
    qSplitterAdapter()->insertWidget(1, jqComponentCast(componentBridge)->
                                                                     qWidget());
  } else {
    QWidget *rightWidget = qSplitterAdapter()->widget(1);
    if (!rightWidget) {
      rightWidget = qSplitterAdapter()->widget(0);
    }
    rightWidget->setParent(0);
  }
}

/*!
 * \brief Sets orientation of the split pane.
 *
 * \param env the JNIEnv pointer
 * \param orientation the orientation
 */
void JQSplitPaneBridge::setOrientation(JNIEnv *env, jint orientation)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setOrientation, QSWING_ENV_ARG, JINT_ARG(orientation))
  switch(orientation) { // do not cast
    case 0:
      qSplitterAdapter()->setOrientation(Qt::Horizontal);
      break;
    case 1:
      qSplitterAdapter()->setOrientation(Qt::Vertical);
      break;
    default:
      LOG_WARNING("Invalid orientation");
      break;
  }
}

/*!
 * \brief Sets whether or not the resizing of components is opaque
 *
 * \param env the JNIEnv pointer
 * \param continuousLayout \c true if opaque
 */
void JQSplitPaneBridge::setContinuousLayout(JNIEnv *env,
                                            jboolean continuousLayout)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setContinuousLayout, QSWING_ENV_ARG,
                   JBOOLEAN_ARG(continuousLayout))
  qSplitterAdapter()->setOpaqueResize(continuousLayout);
}

/*!
 * \brief Sets resize weight.
 *
 * \param env the JNIEnv pointer
 * \param resizeWeight the resize weight
 */
void JQSplitPaneBridge::setResizeWeight(JNIEnv *env, jdouble resizeWeight)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setResizeWeight, QSWING_ENV_ARG, JDOUBLE_ARG(resizeWeight))
  qSplitterAdapter()->setResizeWeight(resizeWeight);
}

/*!
 * \brief Returns divider size.
 *
 * \param env the JNIEnv pointer
 * 
 * \return divider size
 */
jint JQSplitPaneBridge::getDividerSize(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT_RETURN(getDividerSize, jint, QSWING_ENV_ARG)
  return qSplitterAdapter()->handleWidth();
}

/*!
 * \brief Sets divider size.
 *
 * \param env the JNIEnv pointer
 * \param dividerSize the divider size
 */
void JQSplitPaneBridge::setDividerSize(JNIEnv *env, jint dividerSize)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setDividerSize, QSWING_ENV_ARG, JINT_ARG(dividerSize))
  qSplitterAdapter()->setHandleWidth(dividerSize);
}

/*!
 * \brief Sets divider location.
 *
 * \param env the JNIEnv pointer
 * \param dividerLocation the divider location
 */
void JQSplitPaneBridge::setDividerLocation(JNIEnv *env, jint dividerLocation)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setDividerLocation, QSWING_ENV_ARG,
                   JINT_ARG(dividerLocation))
  qSplitterAdapter()->setHandlePosition(dividerLocation);
}

/*!
 * \brief Sets whether or not the comopnents of the split pane can be quickly
 *        expanded/collapsed.
 *
 * \param env the JNIEnv pointer
 * \param oneTouchExpandable \c true if expandable
 */
void JQSplitPaneBridge::setOneTouchExpandable(JNIEnv *env,
                                              jboolean oneTouchExpandable)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setOneTouchExpandable, QSWING_ENV_ARG,
                   JBOOLEAN_ARG(oneTouchExpandable))
  qSplitterAdapter()->setOneTouchExpandable(oneTouchExpandable);
}

/*^**************************************************************************^*/
/*- JQSplitPaneBridge :: Static Native Calls.                                -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQSplitPaneBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQSplitPane class
 */
void JQSplitPaneBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_dividerLocationChangedMID);
  if (!INIT_UMID(dividerLocationChanged, cls, "(I)V")) {
    FATAL_LOGGED_ERROR("JQSplitPaneBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQSplitPaneBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQSplitPaneBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQSplitPaneBridge :: Native Callbacks.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls \c dividerLocationChanged method of the split pane object.
 *
 * \param dividerLocation the divider location
 */
void JQSplitPaneBridge::dividerLocationChanged(jint dividerLocation)
{
  jniObject()->callVoidMethod(sm_dividerLocationChangedMID, dividerLocation);
}

/*^**************************************************************************^*/
/*- JQSplitPaneBridge :: Protected Method.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief splitterMoved slot.
 *
 * \param position the position
 * \param index the index
 */
void JQSplitPaneBridge::splitterMovedSlot(int position, int index)
{
  Q_ASSERT(index == 1);
  Q_UNUSED(index);
  dividerLocationChanged(position);
}

/*^**************************************************************************^*/
/*- JQSplitPaneBridge :: Private Static Data Member.                         -*/
/*^**************************************************************************^*/

jmethodID JQSplitPaneBridge::sm_dividerLocationChangedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-12-04 20:30:14 Slobodan
   Initial version

*****************************************************************************^*/
