/*
 * @(#)JQStatusBarBridge.cpp  0.2.0 / 2007-11-07
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQStatusBarBridge.h"

/*^**************************************************************************^*/
/*- JQStatusBarBridge :: Protected Constructor.                              -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQStatusBarBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing JQStatusBar object
 */

JQStatusBarBridge::JQStatusBarBridge(JNIEnv *env, jobject obj)
  : JQContainerBridge(env, obj, new QStatusBar())
{
  qStatusBar()->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
  connectDirect(qStatusBar(), SIGNAL(messageChanged(const QString &)),
                SLOT(messageChangedSlot(const QString &)));
}

/*^**************************************************************************^*/
/*- JQStatusBarBridge :: Native Calls.                                       -*/
/*^**************************************************************************^*/

/*!
 * \brief Displays the specified message
 *
 * \param env the JNIEnv pointer
 * \param message the message
 * \param timeout the message time out
 */
void JQStatusBarBridge::showMessage(JNIEnv *env, jstring message,
                                    jint timeout)
{
  Q_ASSERT(env);
  Q_ASSERT(message);
  QSWING_REENTRANT(showMessage, QSWING_ENV_ARG, JSTRING_ARG(message),
                   JINT_ARG(timeout))
  qStatusBar()->showMessage(QStringAdapter(env, message), timeout);
}

/*!
 * \brief Clears status message
 *
 * \param env the JNIEnv pointer
 */
void JQStatusBarBridge::clearMessage(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(clearMessage, QSWING_ENV_ARG)
  qStatusBar()->clearMessage();
}

/*!
 * \brief Inserts temporary child.
 *
 * \param env the JNIEnv pointer
 * \param childBridge the child bridge
 * \param index the child index
 */
void JQStatusBarBridge::insertChild(JNIEnv *env, jlong childBridge, jint index)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(insertChild, QSWING_ENV_ARG, JLONG_ARG(childBridge),
                   JINT_ARG(index))
  qStatusBar()->insertWidget(index, jqComponentCast(childBridge)->qWidget());
  jqComponentCast(childBridge)->setParentBridge(this);
}

/*!
 * \brief Inserts permanent child.
 *
 * \param env the JNIEnv pointer
 * \param childBridge the child bridge pointer
 * \param index the child index
 */
void JQStatusBarBridge::insertPermanentChild(JNIEnv *env, jlong childBridge,
                                             jint index)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(insertPermanentChild, QSWING_ENV_ARG, JLONG_ARG(childBridge),
                   JINT_ARG(index))
  qStatusBar()->insertPermanentWidget(index,
                                      jqComponentCast(childBridge)->qWidget());
  jqComponentCast(childBridge)->setParentBridge(this);
}

/*!
 * \brief Removes the specified child.
 *
 * \param env the JNIEnv pointer
 * \param childBridge the child bridge pointer
 */
void JQStatusBarBridge::removeChild(JNIEnv *env, jlong childBridge)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(removeChild, QSWING_ENV_ARG, JLONG_ARG(childBridge))
  qStatusBar()->removeWidget(jqComponentCast(childBridge)->qWidget());
  jqComponentCast(childBridge)->setParentBridge(0);
}

/*!
 * \brief Sets whether or not the size grip is visible.
 *
 * \param env the JNIEnv pointer
 * \param enabled \c true if enabled
 */
void JQStatusBarBridge::setSizeGripEnabled(JNIEnv *env, jboolean enabled)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setSizeGripEnabled, QSWING_ENV_ARG, JBOOLEAN_ARG(enabled))
  qStatusBar()->setSizeGripEnabled(enabled);
}

/*^**************************************************************************^*/
/*- JQStatusBarBridge :: Static Native Calls.                                -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQStatusBarBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQStatusBar class
 *
 * \exception FatalError if failed
 */
void JQStatusBarBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_statusMessgeChangedMID);
  if (!INIT_UMID(statusMessgeChanged, cls, "(Ljava/lang/String;)V")) {
    FATAL_LOGGED_ERROR("JQStatusBarBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQStatusBarBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQStatusBarBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQStatusBarBridge :: Native Callbacks.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls statusMessgeChanged method of the JQStatusBar object.
 *
 * \param message the new status message
 */
void JQStatusBarBridge::statusMessgeChanged(jstring message)
{
  jniObject()->callVoidMethod(sm_statusMessgeChangedMID, message);
}

/*^**************************************************************************^*/
/*- JQStatusBarBridge :: Protected Member Function.                          -*/
/*^**************************************************************************^*/

/*!
 * \brief Message changend slot.
 *
 * \param message the status message
 */
void JQStatusBarBridge::messageChangedSlot(const QString &message)
{ 
  if (message.isNull()) {
    statusMessgeChanged(0);
  } else {
    statusMessgeChanged(QSwingString::toJString(message));
  }
}

/*^**************************************************************************^*/
/*- JQStatusBarBridge :: Overridden Member Function.                         -*/
/*^**************************************************************************^*/

/*!
 * \inheritDoc
 */
void JQStatusBarBridge::setParentBridge(JQContainerBridge *parentBridge)
{
  JQContainerBridge::setParentBridge(parentBridge);
  if (parentBridge) {
     // QSizeGrip bug workaround
    bool sizeGripEnabled = qStatusBar()->isSizeGripEnabled();
    qStatusBar()->setSizeGripEnabled(!sizeGripEnabled);
    qStatusBar()->setSizeGripEnabled(sizeGripEnabled);
  }
}

/*^**************************************************************************^*/
/*- JQStatusBarBridge :: Private Static Data Member.                         -*/
/*^**************************************************************************^*/

jmethodID JQStatusBarBridge::sm_statusMessgeChangedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-07 08:45:32 Slobodan
   Initial version

*****************************************************************************^*/
