/*
 * @(#)JQTabbedPaneBridge.cpp  0.2.0 / 2007-11-18
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQTabbedPaneBridge.h"

#include "QIconAdapter.h"

/*^**************************************************************************^*/
/*- JQTabbedPaneBridge :: Protected Constructor.                             -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQTabbedPaneBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj JQTabbedPane object
 */
JQTabbedPaneBridge::JQTabbedPaneBridge(JNIEnv *env, jobject obj)
  : JQContainerBridge(env, obj, new QTabWidget())
{
  connectDirect(qTabWidget(), SIGNAL(currentChanged(int)),
                SLOT(currentChangedSlot(int)));
}

/*^**************************************************************************^*/
/*- JQTabbedPaneBridge :: Native Calls.                                      -*/
/*^**************************************************************************^*/

/*!
 * \brief Inserts tab at specified position.
 *
 * \param env the JNIEnv pointer
 * \param tabBridge the tab brige pointer
 * \param index the index
 * \param title the tab title
 */
void JQTabbedPaneBridge::insertTab(JNIEnv *env, jlong tabBridge, jint index,
                                   jstring title)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(insertTab, QSWING_ENV_ARG, JLONG_ARG(tabBridge),
                   JINT_ARG(index), JSTRING_ARG(title))
  qTabWidget()->insertTab(index, jqComponentCast(tabBridge)->qWidget(),
                          QStringAdapter(env, title));
  jqComponentCast(tabBridge)->setParentBridge(this);
}

/*!
 * \brief Removes the specified child.
 *
 * \param env the JNIEnv pointer
 * \param childBridge the child bridge pointer
 */
void JQTabbedPaneBridge::removeChild(JNIEnv *env, jlong childBridge)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(removeChild, QSWING_ENV_ARG, JLONG_ARG(childBridge))
  qTabWidget()->removeTab(
               qTabWidget()->indexOf(jqComponentCast(childBridge)->qWidget()));
  jqComponentCast(childBridge)->setParentBridge(0);
}

/*!
 * \brief Sets tab placement
 *
 * \param env the JNIEnv pointer
 * \param tabPlacement the tab placement
 */
void JQTabbedPaneBridge::setTabPlacement(JNIEnv *env, jint tabPlacement)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setTabPlacement, QSWING_ENV_ARG, JINT_ARG(tabPlacement))
  switch(tabPlacement) {
    case 1:
      qTabWidget()->setTabPosition(QTabWidget::North);
      break;
    case 2:
      qTabWidget()->setTabPosition(QTabWidget::West);
      break;
    case 3:
      qTabWidget()->setTabPosition(QTabWidget::South);
      break;
    case 4:
      qTabWidget()->setTabPosition(QTabWidget::East);
      break;
  }
}

/*!
 * \brief Enables/disables the specified tab.
 *
 * \param env the JNIEnv pointer
 * \param index the tab index
 * \param enabled \c true if enabled
 */
void JQTabbedPaneBridge::setTabEnabled(JNIEnv *env, jint index,
                                       jboolean enabled)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setTabEnabled, QSWING_ENV_ARG, JINT_ARG(index),
                   JBOOLEAN_ARG(enabled))
  qTabWidget()->setTabEnabled(index, enabled);
}

/*!
 * \brief Sets tab title.
 *
 * \param env the JNIEnv pointer
 * \param index the tab index
 * \param titleText the title text
 */
void JQTabbedPaneBridge::setTabTitleText(JNIEnv *env, jint index,
                                         jstring titleText)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setTabTitleText, QSWING_ENV_ARG, JINT_ARG(index),
                   JSTRING_ARG(titleText))
  qTabWidget()->setTabText(index, QStringAdapter(env, titleText));
}

/*!
 * \brief Sets tooltip of the specified tab.
 *
 * \param env the JNIEnv pointer
 * \param index the tab index
 * \param toolTipText the tool tip text
 */
void JQTabbedPaneBridge::setTabToolTipText(JNIEnv *env, jint index,
                                           jstring toolTipText)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setTabToolTipText, QSWING_ENV_ARG, JINT_ARG(index),
                   JSTRING_ARG(toolTipText))
  qTabWidget()->setTabToolTip(index, QStringAdapter(env, toolTipText));
}

/*!
 * \brief Sets what's this text of the specified tab.
 *
 * \param env the JNIEnv pointer
 * \param index the tab index
 * \param whatsThisText the what's this text
 */
void JQTabbedPaneBridge::setTabWhatsThisText(JNIEnv *env, jint index,
                                             jstring whatsThisText)
{
  Q_ASSERT(env);
  Q_ASSERT(whatsThisText);
  QSWING_REENTRANT(setTabWhatsThisText, QSWING_ENV_ARG, JINT_ARG(index),
                   JSTRING_ARG(whatsThisText))
  qTabWidget()->setTabWhatsThis(index, QStringAdapter(env, whatsThisText));
}

/*!
 * \brief Sets tab icon
 *
 * \param env the JNIEnv pointer
 * \param index the tab index
 * \param iconSet the tab icon
 */
void JQTabbedPaneBridge::setTabIcon(JNIEnv *env, jint index, jobject iconSet)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setTabIcon, QSWING_ENV_ARG, JINT_ARG(index),
                   JOBJECT_ARG(iconSet))
  if (iconSet) {
    qTabWidget()->setTabIcon(index, QIconAdapter(env, iconSet));
  } else {
    qTabWidget()->setTabIcon(index, QIcon());
  }
}

/*!
 * \brief Sets currently selected tab index.
 *
 * \param env the JNIEnv pointer
 * \param index the index
 */
void JQTabbedPaneBridge::setCurrentIndex(JNIEnv *env, jint index)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setCurrentIndex, QSWING_ENV_ARG, JINT_ARG(index))
  qTabWidget()->setCurrentIndex(index);
}

/*^**************************************************************************^*/
/*- JQTabbedPaneBridge :: Static Native Calls.                               -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQTabbedPaneBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQTabbedPane class
 *
 * \exception FatalError if failed
 */
void JQTabbedPaneBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_currentChangedMID);
  if (!INIT_UMID(currentChanged, cls, "(I)V")) {
    FATAL_LOGGED_ERROR("JQTabbedPaneBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQTabbedPaneBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQTabbedPaneBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQTabbedPaneBridge :: Native Callbacks.                                  -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls \c currentChanged method of the tabbed pan object.
 *
 * \param selectedIndex the selected index
 */
void JQTabbedPaneBridge::currentChanged(jint selectedIndex)
{
  jniObject()->callVoidMethod(sm_currentChangedMID, selectedIndex);
}

/*^**************************************************************************^*/
/*- JQToolBarBridge :: Protected Method.                                     -*/
/*^**************************************************************************^*/

/*!
 * \brief currentChanged slot.
 *
 * \param index the index
 */
void JQTabbedPaneBridge::currentChangedSlot(int index)
{
  currentChanged(index);
}

/*^**************************************************************************^*/
/*- JQTabbedPaneBridge :: Private Static Data Member.                        -*/
/*^**************************************************************************^*/

jmethodID JQTabbedPaneBridge::sm_currentChangedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-18 18:39:57 Slobodan
   Initial version

*****************************************************************************^*/
