/*
 * @(#)JQTextAreaBridge.cpp  0.2.0 / 2007-12-09
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQTextAreaBridge.h"

#include "QCursorAdapter.h"
#include "TextDocumentBridge.h"

/*^**************************************************************************^*/
/*- JQTextAreaBridge :: Protected Constructor.                               -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQTextAreaBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQTextArea object.
 */
JQTextAreaBridge::JQTextAreaBridge(JNIEnv *env, jobject obj)
  : JQTextComponentBridge(env, obj, new QTextArea(this))
{
  connect(QApplication::clipboard(), SIGNAL(dataChanged()),
          SLOT(clipboardDataChanged()));
  clipboardDataChanged();
}

/*^**************************************************************************^*/
/*- JQTextAreaBridge :: Native Calls.                                        -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets text wrapping mode.
 *
 * \param env the JNIEnv pointer
 * \param textWrap \c true if wrapping text
 */
void JQTextAreaBridge::setWrapText(JNIEnv *env, jboolean textWrap)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setWrapText, QSWING_ENV_ARG, JBOOLEAN_ARG(textWrap))
  if (textWrap) {
    qTextArea()->setLineWrapMode(QTextEdit::WidgetWidth);
    qTextArea()->setWordWrapMode(QTextOption::WordWrap);
  } else {
    qTextArea()->setLineWrapMode(QTextEdit::NoWrap);
    qTextArea()->setWordWrapMode(QTextOption::NoWrap);
  }
}

/*!
 * \brief Paints default component view.
 *
 * \param env the JNIEnv pointer
 * \param graphicsPointer the QSwingGraphics pointer
 */
void JQTextAreaBridge::paintComponentView(JNIEnv *env, jlong graphicsPointer)
{
  Q_UNUSED(env);
  Q_ASSERT(graphicsPointer);
  qTextArea()->paintView(graphicsPointer);
}

/*^**************************************************************************^*/
/*- JQTextAreaBridge :: Static Native Calls.                                 -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQTextAreaBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQTextArea class
 *
 * \exception FatalError if failed
 */
void JQTextAreaBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_paintViewMID);
  if (!INIT_UMID(paintView, cls, "(JIIII)V")) {
    FATAL_LOGGED_ERROR("JQTextAreaBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQTextAreaBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQTextAreaBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQTextAreaBridge :: Native Callbacks.                                    -*/
/*^**************************************************************************^*/

/*!
 * \brief Paints component view.
 *
 * \param graphicsPointer the graphics pointer
 * \param x the x-coordinate of the view
 * \param y the y-coordinate of the view
 * \param width width the width of the view
 * \param height height the height of the view
 */
void JQTextAreaBridge::paintView(jlong graphicsPointer, jint x, jint y,
                                 jint width, jint height)
{
  jniObject()->callVoidMethod(sm_paintViewMID, graphicsPointer, x, y, width,
                              height);
}

/*^**************************************************************************^*/
/*- JQTextAreaBridge :: Overridden Member Functions.                         -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets document.
 *
 * \param env the JNIEnv pointer
 * \param document the new document 
 */
void JQTextAreaBridge::setDocument(JNIEnv *env, jobject document)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setDocument, QSWING_ENV_ARG, JOBJECT_ARG(document))
  qTextArea()->setDocument(TextDocumentBridge::toQDocument(env, document));
}

/*!
 * \brief Sets whether or not text area is editable.
 *
 * \param env the JNIEnv pointer
 * \param editable \c true if editable
 */
void JQTextAreaBridge::setEditable(JNIEnv *env, jboolean editable)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setEditable, QSWING_ENV_ARG, JBOOLEAN_ARG(editable))
  if (editable) {
    cutAvailable(false);
    clearAvailable(false);
    qTextArea()->setReadOnly(false);
    connect(QApplication::clipboard(), SIGNAL(dataChanged()),
            SLOT(clipboardDataChanged()));
  } else {
    qTextArea()->setReadOnly(true);
    disconnect(QApplication::clipboard(), SIGNAL(dataChanged()),
               SLOT(clipboardDataChanged()));
  }
}

/*!
 * \brief Inserts text.
 *
 * \param env the JNIEnv pointer
 * \param text text the text to be inserted
 */
void JQTextAreaBridge::insertText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  Q_ASSERT(text);
  QSWING_REENTRANT(insertText, QSWING_ENV_ARG, JSTRING_ARG(text))
  qTextArea()->textCursor().insertText(QStringAdapter(env, text));
}

/*!
 * \brief Cuts selected text.
 *
 * \param env the JNIEnv pointer
 */
void JQTextAreaBridge::cut(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(cut, QSWING_ENV_ARG)
  qTextArea()->cut();
}

/*!
 * \brief 
 *
 * \param env the JNIEnv pointer
 */
void JQTextAreaBridge::copy(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(copy, QSWING_ENV_ARG)
  qTextArea()->copy();
}

/*!
 * \brief Paste text form the system clipboard.
 *
 * \param env the JNIEnv pointer
 */
void JQTextAreaBridge::paste(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(paste, QSWING_ENV_ARG)
  qTextArea()->paste();
}

/*!
 * \brief Clears selected text.
 *
 * \param env the JNIEnv pointer
 */
void JQTextAreaBridge::clear(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(clear, QSWING_ENV_ARG)
  qTextArea()->textCursor().removeSelectedText();
}

/*!
 * \brief Selects all text.
 *
 * \param env the JNIEnv pointer
 */
void JQTextAreaBridge::selectAll(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(selectAll, QSWING_ENV_ARG)
  qTextArea()->selectAll();
}

/*!
 * \inheritDoc
 */
jobject JQTextAreaBridge::getBackground(JNIEnv *env, jclass colorClass)
{
  return QColorAdapter::toJColor(env, colorClass,
                        qTextArea()->viewport()->palette().color(
                                    qTextArea()->viewport()->backgroundRole()));
}

/*!
 * \inheritDoc
 */
void JQTextAreaBridge::setBackground(JNIEnv *env, jint argb)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setBackground, QSWING_ENV_ARG, JINT_ARG(argb))
  QColorAdapter color(argb);
  QPalette palette(qTextArea()->viewport()->palette());
  if (color.isValid()) {
    palette.setColor(qTextArea()->viewport()->backgroundRole(), color);
  } else {
    palette.setColor(qTextArea()->viewport()->backgroundRole(),
                     QApplication::palette().color(
                                   qTextArea()->viewport()->backgroundRole()));
  }
  qTextArea()->viewport()->setPalette(palette);
}

/*!
 * \inheritDoc
 */
void JQTextAreaBridge::setNullBackground(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setNullBackground, QSWING_ENV_ARG)
  QPalette palette(qTextArea()->viewport()->palette());
  palette.setColor(qTextArea()->viewport()->backgroundRole(),
                   QApplication::palette().color(
                                 qTextArea()->viewport()->backgroundRole()));
  qTextArea()->viewport()->setPalette(palette);
}

/*!
 * \inheritDoc
 */
jint JQTextAreaBridge::getCursor(JNIEnv *env)
{
  Q_UNUSED(env);
  return QCursorAdapter::cursorType(qTextArea()->viewport()->cursor());
}

/*!
 * \inheritDoc
 */
void JQTextAreaBridge::setCursor(JNIEnv *env, jint type)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setCursor, QSWING_ENV_ARG, JINT_ARG(type))
  if (type < 0) {
    qTextArea()->viewport()->unsetCursor();
  } else {
    qTextArea()->viewport()->setCursor(QCursorAdapter(type));
  }
}

/*!
 * \inheritDoc.
 */
void JQTextAreaBridge::filterEnabledEvents()
{
  JQComponentBridge::filterEnabledEvents();
  qTextArea()->setPaintView(eventEnabled(QSWING_EVENT(PAINT_VIEW_EVENT_MASK)));
}

/*^**************************************************************************^*/
/*- JQTextAreaBridge :: Private Static Data Members.                        -*/
/*^**************************************************************************^*/

jmethodID JQTextAreaBridge::sm_paintViewMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-12-09 08:59:52 Slobodan
   Initial version

*****************************************************************************^*/
