/*
 * @(#)JQTextFieldBridge.cpp  0.2.0 / 2007-11-21
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQTextFieldBridge.h"

#include "QApplicationBridge.h"
#include "QSwingBridge.h"

/*^**************************************************************************^*/
/*- JQTextFieldBridge :: Protected Constructor.                              -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQTextFieldBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQTextField object
 */
JQTextFieldBridge::JQTextFieldBridge(JNIEnv *env, jobject obj)
  : JQComponentBridge(env, obj, new QLineEdit())
{
  connectDirect(qLineEdit(), SIGNAL(returnPressed()),
                SLOT(returnPressedSlot()));
}

/*^**************************************************************************^*/
/*- JQTextFieldBridge :: Native Calls.                                       -*/
/*^**************************************************************************^*/

/*!
 * \brief Returns text.
 *
 * \param env the JNIEnv pointer
 * 
 * \return text
 */
jstring JQTextFieldBridge::getText(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT_RETURN(getText, jstring, QSWING_ENV_ARG)
  return QStringAdapter::toJString(env, qLineEdit()->text());
}

/*!
 * \brief Sets text.
 *
 * \param env the JNIEnv pointer
 * \param text the text to be set
 */
void JQTextFieldBridge::setText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setText, QSWING_ENV_ARG, JSTRING_ARG(text))
  qLineEdit()->setText(QStringAdapter(env, text));
}

/*!
 * \brief Sets horizontal alignment.
 *
 * \param env the JNIEnv pointer
 * \param alignment the horizontal alignment
 */
void JQTextFieldBridge::setHorizontalAlignment(JNIEnv *env, jint alignment)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setHorizontalAlignment, QSWING_ENV_ARG, JINT_ARG(alignment))
  qLineEdit()->setAlignment(QSwingBridge::toHorizontalAlignment(alignment));
}

/*^**************************************************************************^*/
/*- JQTextFieldBridge :: Static Native Calls.                                -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQTextFieldBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQTextField class
 *
 * \exception FatalError if failed
 */
void JQTextFieldBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT (!sm_returnPressedMID);
  if (!INIT_UMID(returnPressed, cls, "(I)V")) {
    FATAL_LOGGED_ERROR("JQTextFieldBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQTextFieldBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQTextFieldBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQTextFieldBridge :: Native Callbacks.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls returnPressed method of the JQTextField object.
 */
void JQTextFieldBridge::returnPressed(jint modifiers)
{
  jniObject()->callVoidMethod(sm_returnPressedMID, modifiers);
}

/*^**************************************************************************^*/
/*- JQTextFieldBridge :: Protected Member Functions.                         -*/
/*^**************************************************************************^*/

/*!
 * \brief \c returnPressed slot.
 */
void JQTextFieldBridge::returnPressedSlot()
{
  returnPressed(QApplicationBridge::keyboardModifiers());
}

/*^**************************************************************************^*/
/*- JQTextFieldBridge :: Private Static Data Members.                        -*/
/*^**************************************************************************^*/

jmethodID JQTextFieldBridge::sm_returnPressedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-21 19:22:48 Slobodan
   Initial version

*****************************************************************************^*/
