/*
 * @(#)JQTextPaneBridge.cpp  0.2.0 / 2007-12-08
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQTextPaneBridge.h"

#include "QCursorAdapter.h"
#include "TextDocumentBridge.h"

/*^**************************************************************************^*/
/*- JQTextPaneBridge :: Protected Constructor.                               -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQTextPaneBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQTextPane object
 */
JQTextPaneBridge::JQTextPaneBridge(JNIEnv *env, jobject obj)
  : JQTextComponentBridge(env, obj, new QTextPane(this))
{
  // empty constructor body
}

/*^**************************************************************************^*/
/*- JQTextPaneBridge :: Native Calls.                                        -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets text wrapping mode.
 *
 * \param env the JNIEnv pointer
 * \param textWrap \c true if wrapping text
 */
void JQTextPaneBridge::setWrapText(JNIEnv *env, jboolean textWrap)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setWrapText, QSWING_ENV_ARG, JBOOLEAN_ARG(textWrap))
  if (textWrap) {
    qTextPane()->setLineWrapMode(QTextEdit::WidgetWidth);
  } else {
    qTextPane()->setLineWrapMode(QTextEdit::NoWrap);
  }
}

/*!
 * \brief Paints default component view.
 *
 * \param env the JNIEnv pointer
 * \param graphicsPointer the QSwingGraphics pointer
 */
void JQTextPaneBridge::paintComponentView(JNIEnv *env, jlong graphicsPointer)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(paintComponentView, QSWING_ENV_ARG,
                   JLONG_ARG(graphicsPointer))
  qTextPane()->paintView(graphicsPointer);
}

/*^**************************************************************************^*/
/*- JQTextPaneBridge :: Static Native Calls.                                 -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQTextPaneBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQTextArea class
 *
 * \exception FatalError if failed
 */
void JQTextPaneBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_paintViewMID);
  if (!INIT_UMID(paintView, cls, "(JIIII)V")) {
    FATAL_LOGGED_ERROR("JQTextPaneBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQTextPaneBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQTextPaneBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQTextPaneBridge :: Native Callbacks.                                    -*/
/*^**************************************************************************^*/

/*!
 * \brief Paint component view
 *
 * \param graphicsPointer the graphics pointer
 * \param x the x-coordinate of the view
 * \param y the y-coordinate of the view
 * \param width width the width of the view
 * \param height height the height of the view
 */
void JQTextPaneBridge::paintView(jlong graphicsPointer, jint x, jint y,
                                 jint width, jint height)
{
  jniObject()->callVoidMethod(sm_paintViewMID, graphicsPointer, x, y, width,
                              height);
}


/*^**************************************************************************^*/
/*- JQTextPaneBridge :: Overridden Member Functions.                         -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets document.
 *
 * \param env the JNIEnv pointer
 * \param document the new document 
 */
void JQTextPaneBridge::setDocument(JNIEnv *env, jobject document)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setDocument, QSWING_ENV_ARG, JOBJECT_ARG(document))
  qTextPane()->setDocument(TextDocumentBridge::toQDocument(env, document));
}

/*!
 * \brief Sets whether or not text pane is editable.
 *
 * \param env the JNIEnv pointer
 * \param editable \c true if editable
 */
void JQTextPaneBridge::setEditable(JNIEnv *env, jboolean editable)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setEditable, QSWING_ENV_ARG, JBOOLEAN_ARG(editable))
  qTextPane()->setReadOnly(!editable);
}

/*!
 * \brief Inserts text.
 *
 * \param env the JNIEnv pointer
 * \param text text the text to be inserted
 */
void JQTextPaneBridge::insertText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  Q_ASSERT(text);
  QSWING_REENTRANT(insertText, QSWING_ENV_ARG, JSTRING_ARG(text))
  qTextPane()->textCursor().insertText(QStringAdapter(env, text));
}

/*!
 * \brief Cuts selected text.
 *
 * \param env the JNIEnv pointer
 */
void JQTextPaneBridge::cut(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(cut, QSWING_ENV_ARG)
  qTextPane()->cut();
}

/*!
 * \brief 
 *
 * \param env the JNIEnv pointer
 */
void JQTextPaneBridge::copy(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(copy, QSWING_ENV_ARG)
  qTextPane()->copy();
}

/*!
 * \brief Paste text form the system clipboard.
 *
 * \param env the JNIEnv pointer
 */
void JQTextPaneBridge::paste(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(paste, QSWING_ENV_ARG)
  qTextPane()->paste();
}

/*!
 * \brief Clears selected text.
 *
 * \param env the JNIEnv pointer
 */
void JQTextPaneBridge::clear(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(clear, QSWING_ENV_ARG)
  qTextPane()->textCursor().removeSelectedText();
}

/*!
 * \brief Selects all text.
 *
 * \param env the JNIEnv pointer
 */
void JQTextPaneBridge::selectAll(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(selectAll, QSWING_ENV_ARG)
  qTextPane()->selectAll();
}

/*!
 * \inheritDoc
 */
jobject JQTextPaneBridge::getBackground(JNIEnv *env, jclass colorClass)
{
  return QColorAdapter::toJColor(env, colorClass,
                        qTextPane()->viewport()->palette().color(
                                    qTextPane()->viewport()->backgroundRole()));
}

/*!
 * \inheritDoc
 */
void JQTextPaneBridge::setBackground(JNIEnv *env, jint argb)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setBackground, QSWING_ENV_ARG, JINT_ARG(argb))
  QColorAdapter color(argb);
  QPalette palette(qTextPane()->viewport()->palette());
  if (color.isValid()) {
    palette.setColor(qTextPane()->viewport()->backgroundRole(), color);
  } else {
    palette.setColor(qTextPane()->viewport()->backgroundRole(),
                     QApplication::palette().color(
                                   qTextPane()->viewport()->backgroundRole()));
  }
  qTextPane()->viewport()->setPalette(palette);
}

/*!
 * \inheritDoc
 */
void JQTextPaneBridge::setNullBackground(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setNullBackground, QSWING_ENV_ARG)
  QPalette palette(qTextPane()->viewport()->palette());
  palette.setColor(qTextPane()->viewport()->backgroundRole(),
                   QApplication::palette().color(
                                 qTextPane()->viewport()->backgroundRole()));
  qTextPane()->viewport()->setPalette(palette);
}

/*!
 * \inheritDoc
 */
jint JQTextPaneBridge::getCursor(JNIEnv *env)
{
  Q_UNUSED(env);
  return QCursorAdapter::cursorType(qTextPane()->viewport()->cursor());
}

/*!
 * \inheritDoc
 */
void JQTextPaneBridge::setCursor(JNIEnv *env, jint type)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setCursor, QSWING_ENV_ARG, JINT_ARG(type))
  if (type < 0) {
    qTextPane()->viewport()->unsetCursor();
  } else {
    qTextPane()->viewport()->setCursor(QCursorAdapter(type));
  }
}

/*!
 * \inheritDoc.
 */
void JQTextPaneBridge::filterEnabledEvents()
{
  JQComponentBridge::filterEnabledEvents();
  qTextPane()->setPaintView(eventEnabled(QSWING_EVENT(PAINT_VIEW_EVENT_MASK)));
}

/*^**************************************************************************^*/
/*- JQTextPaneBridge :: Private Static Data Members.                        -*/
/*^**************************************************************************^*/

jmethodID JQTextPaneBridge::sm_paintViewMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-12-08 20:06:34 Slobodan
   Initial version

*****************************************************************************^*/
