/*
 * @(#)JQTimerBridge.cpp  0.2.0 / 2007-09-05
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQTimerBridge.h"

#include "QSwingBridge.h"
#include "QSwingUtilitiesBridge.h"

/*^**************************************************************************^*/
/*- JQTimerBridge :: Protected Constructor.                                  -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQTimerBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing timer object
 * \param delay the number of milliseconds between timer events
 * \param initialDelay the initial delay
 * \param singleShot \c true if the timer is single-shot timer
 */
JQTimerBridge::JQTimerBridge(JNIEnv *env, jobject obj, jint delay,
                             jint initialDelay, jboolean singleShot)
  : m_objref(JNIWrapper::newWeakGlobalRef(env, obj)), m_timer(0)
{
  m_timer.setInterval(delay);
  m_timer.setSingleShot(singleShot);
  if (m_objref) { // check for OutOfMemoryError
    connect(&m_timer, SIGNAL(timeout()), this, SLOT(timeout()));
    m_timer.start(initialDelay);
  }
}

/*^**************************************************************************^*/
/*- JQTimerBridge :: Protected Destructor.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Destructs JQTimerBridge object.
 */
JQTimerBridge::~JQTimerBridge()
{
  if (m_objref) {
    JNIQSwingWrapper::deleteWeakGlobalRef(m_objref);
  }
}

/*^**************************************************************************^*/
/*- JQTimerBridge :: Native Calls.                                           -*/
/*^**************************************************************************^*/

/*!
 * \brief Destroys the bridge object.
 *
 * \param env the JNIEnv pointer
 */
void JQTimerBridge::destroy(JNIEnv *env)
{
  Q_UNUSED(env);
  if (QSwingBridge::isDisposed()) {
    delete this; // OK - referenced only from java
  } else {
    deleteLater();
  }
}

/*!
 * \brief Sets timer interval.
 *
 * \param env the JNIEnv pointer
 * \param interval the timer interval
 */
void JQTimerBridge::setInterval(JNIEnv *env, jint interval)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setInterval, QSWING_ENV_ARG, JINT_ARG(interval))
  m_timer.setInterval(interval);
}

/*!
 * \brief Sets whether or not the timer is single shot-timer.
 *
 * \param env the JNIEnv pointer
 * \param singleShot if \c true the timer is single shot-timer
 */
void JQTimerBridge::setSingleShot(JNIEnv *env, jboolean singleShot)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setSingleShot, QSWING_ENV_ARG, JBOOLEAN_ARG(singleShot))
  m_timer.setSingleShot(singleShot);
}

/*^**************************************************************************^*/
/*- JQTimerBridge :: Static Native Calls.                                    -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQTimerBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JavaQx JQTimer class
 *
 * \exception FatalError if failed
 */
void JQTimerBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_createMID);
  if (!INIT_UMID(create, cls, "(IIZ)J") || !INIT_UMID(timeout, cls, "()V")) {
    FATAL_LOGGED_ERROR("JQTimerBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JavaQx JQTimer object
 * \param delay the number of milliseconds between timer events
 * \param initialDelay the initial delay
 * \param singleShot \c true if the timer is single-shot timer
 *
 * \return new instance of the bridge object.
 */
jlong JQTimerBridge::newInstance(JNIEnv *env, jobject obj, jint delay,
                                 jint initialDelay, jboolean singleShot)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQTimerBridge(env, obj, delay,
                                                     initialDelay, singleShot));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, sm_createMID, delay,
                                                  initialDelay, singleShot);
}

/*^**************************************************************************^*/
/*- JQTimerBridge :: Native Callback.                                        -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls timeout method of the JQTimer object.
 */
void JQTimerBridge::timeout()
{
  if (QSwingBridge::isDisposed()) {
    m_timer.stop();
    delete this;
  } else {
    JNIQSwingWrapper::callVoidMethod(m_objref, sm_timeoutMID);
  }
}

/*^**************************************************************************^*/
/*- JQTimerBridge :: Private Static Data Members.                            -*/
/*^**************************************************************************^*/

jmethodID JQTimerBridge::sm_createMID = 0;
jmethodID JQTimerBridge::sm_timeoutMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-09-05 17:42:41 JavaQx Native Generator
   Initial version

*****************************************************************************^*/
