/*
 * @(#)JQToolBarBridge.cpp  0.2.0 / 2007-11-07
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQToolBarBridge.h"

/*^**************************************************************************^*/
/*- JQToolBarBridge :: Protected Constructor.                                -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQToolBarBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing JQToolBar object
 */
JQToolBarBridge::JQToolBarBridge(JNIEnv *env, jobject obj)
  : JQContainerBridge(env, obj, new QToolBar())
{
  connectDirect(qToolBar(), SIGNAL(orientationChanged(Qt::Orientation)),
                SLOT(orientationChangedSlot(Qt::Orientation)));
}

/*^**************************************************************************^*/
/*- JQToolBarBridge :: Native Calls.                                         -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets object name.
 *
 * \param env the JNIEnv pointer
 * \param name the tool bar title name
 */
void JQToolBarBridge::setName(JNIEnv *env, jstring name)
{
  Q_ASSERT(env);
  Q_ASSERT(name);
  QSWING_REENTRANT(setName, QSWING_ENV_ARG, JSTRING_ARG(name))
  QStringAdapter title(env, name);
  qToolBar()->setObjectName(title);
  qToolBar()->setWindowTitle(title);
}

/*!
 * \brief Inserts child.
 *
 * \param env the JNIEnv pointer
 * \param childBridge the child pointer
 * \param index the child index
 */
void JQToolBarBridge::insertChild(JNIEnv *env, jlong childBridge, jint index)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(insertChild, QSWING_ENV_ARG, JLONG_ARG(childBridge),
                   JINT_ARG(index))
  QAction *before = 0;
  if ((index >= 0) && (index < qToolBar()->actions().size())) {
    before = qToolBar()->actions().at(index);
  }
  JQComponentBridge *childComponentBridge = jqComponentCast(childBridge);
  qToolBar()->insertAction(before, childComponentBridge->qAction());
  childComponentBridge->setParentBridge(this);
  childComponentBridge->setParent(this);
}

/*!
 * \brief Removes child.
 *
 * \param env the JNIEnv pointer
 * \param childBridge the child bridge pointer
 */
void JQToolBarBridge::removeChild(JNIEnv *env, jlong childBridge)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(removeChild, QSWING_ENV_ARG, JLONG_ARG(childBridge))
  JQComponentBridge *childComponentBridge = jqComponentCast(childBridge);
  qToolBar()->removeAction(childComponentBridge->qAction());
  childComponentBridge->setParentBridge(0);
  childComponentBridge->setParent(0);
}

/*!
 * \brief Sets whether or not toolbar can be moved.
 *
 * \param env the JNIEnv pointer
 * \param floatable \c true if the toolbar can be moved
 */
void JQToolBarBridge::setFloatable(JNIEnv *env, jboolean floatable)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setFloatable, QSWING_ENV_ARG, JBOOLEAN_ARG(floatable))
  qToolBar()->setMovable(floatable);
}

/*!
 * \brief Sets toolbar orientation
 *
 * \param env the JNIEnv pointer
 * \param orientation the orientation
 */
void JQToolBarBridge::setOrientation(JNIEnv *env, jint orientation)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setOrientation, QSWING_ENV_ARG, JINT_ARG(orientation))
  switch(orientation) { // do not cast
    case 0:
      qToolBar()->setOrientation(Qt::Horizontal);
      break;
    case 1:
      qToolBar()->setOrientation(Qt::Vertical);
      break;
  }
}

/*!
 * \brief Sets icon size.
 *
 * \param env the JNIEnv pointer
 * \param width the width
 * \param height the height
 */
void JQToolBarBridge::setIconSize(JNIEnv *env, jint width, jint height)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setOrientation, QSWING_ENV_ARG, JINT_ARG(width),
                   JINT_ARG(height))
  qToolBar()->setIconSize(QSize(width, height));
}

/*^**************************************************************************^*/
/*- JQToolBarBridge :: Static Native Calls.                                  -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQToolBarBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQToolBar class
 *
 * \exception FatalError if failed
 */
void JQToolBarBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_orientationChangedMID);
  if (!INIT_UMID(orientationChanged, cls, "(I)V")) {
    FATAL_LOGGED_ERROR("JQToolBarBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQToolBarBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQToolBarBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQToolBarBridge :: Native Callback.                                      -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls \c orientationChanged method of the toolbar object.
 *
 * \param orientation the orientation
 */
void JQToolBarBridge::orientationChanged(jint orientation)
{
  jniObject()->callVoidMethod(sm_orientationChangedMID, orientation);
}

/*^**************************************************************************^*/
/*- JQToolBarBridge :: Protected Method.                                     -*/
/*^**************************************************************************^*/

/*!
 * \brief orientationChanged slot.
 *
 * \param orientation the orientation
 */
void JQToolBarBridge::orientationChangedSlot(Qt::Orientation orientation)
{
  switch(orientation) {
    case Qt::Horizontal:
      orientationChanged(0);
      break;
    case Qt::Vertical:
      orientationChanged(1);
      break;
  }
}

/*^**************************************************************************^*/
/*- JQToolBarBridge :: Private Static Data Member.                           -*/
/*^**************************************************************************^*/

jmethodID JQToolBarBridge::sm_orientationChangedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-07 16:06:14 Slobodan
   Initial version

*****************************************************************************^*/
