/*
 * @(#)JQToolTipBridge.cpp  0.2.0 / 2007-12-01
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQToolTipBridge.h"

#include "JQComponentBridge.h"

/*^**************************************************************************^*/
/*- JQToolTipBridge :: Static Native Calls.                                  -*/
/*^**************************************************************************^*/

/*!
 * \brief Enables/disables the tooltips.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQToolTip class
 * \param enabled \c true if enabled
 */
void JQToolTipBridge::setEnabled(JNIEnv *env, jclass cls, jboolean enabled)
{
  Q_UNUSED(env);
  Q_UNUSED(cls);
  sm_enabled = enabled;
}

/*!
 * \brief Sets font for tooltips.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQToolTip class
 * \param font the font
 */
void JQToolTipBridge::setFont(JNIEnv *env, jclass cls, jobject font)
{
  Q_ASSERT(env);
  QSWING_STATIC_REENTRANT(JQToolTipBridge, setFont, QSWING_ENV_ARG,
                          JCLASS_ARG(cls), JOBJECT_ARG(font))
  if (font) {
    QToolTip::setFont(*QFontAdapter::toQFont(font));
  } else {
    QToolTip::setFont(QFont());
  }
}

/*!
 * \brief Sets background color of the tooltips.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQToolTip class
 * \param argb the argb value of the color
 */
void JQToolTipBridge::setBackground(JNIEnv *env, jclass cls, jint argb)
{
  Q_ASSERT(env);
  QSWING_STATIC_REENTRANT(JQToolTipBridge, setBackground, QSWING_ENV_ARG,
                          JCLASS_ARG(cls), JINT_ARG(argb))
  QColorAdapter color(argb);
  QPalette palette(QToolTip::palette());
  if (color.isValid()) {
    palette.setColor(QPalette::Window, color);
  } else {
    palette.setColor(QPalette::Window,
                     QApplication::palette().color(QPalette::Window));
  }
  QToolTip::setPalette(palette);
}

/*!
 * \brief Sets default background color for tooltips.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQToolTip class
 */
void JQToolTipBridge::setNullBackground(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  QSWING_STATIC_REENTRANT(JQToolTipBridge, setNullBackground, QSWING_ENV_ARG,
                          JCLASS_ARG(cls))
  QPalette palette(QToolTip::palette());
  palette.setColor(QPalette::Window,
                   QApplication::palette().color(QPalette::Light));
  QToolTip::setPalette(palette);
}

/*!
 * \brief Sets foreground color of the tooltips.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQToolTip class
 * \param argb the argb value of the color
 */
void JQToolTipBridge::setForeground(JNIEnv *env, jclass cls, jint argb)
{
  Q_ASSERT(env);
  QSWING_STATIC_REENTRANT(JQToolTipBridge, setForeground, QSWING_ENV_ARG,
                          JCLASS_ARG(cls), JINT_ARG(argb))
  QColorAdapter color(argb);
  QPalette palette(QToolTip::palette());
  if (color.isValid()) {
    palette.setColor(QPalette::WindowText, color);
  } else {
    palette.setColor(QPalette::WindowText,
                     QApplication::palette().color(QPalette::WindowText));
  }
  QToolTip::setPalette(palette);
}

/*!
 * \brief Sets default foreground color for tooltips.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQToolTip class
 */
void JQToolTipBridge::setNullForeground(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  QSWING_STATIC_REENTRANT(JQToolTipBridge, setNullForeground, QSWING_ENV_ARG,
                          JCLASS_ARG(cls))
  QPalette palette(QToolTip::palette());
  palette.setColor(QPalette::WindowText,
                   QApplication::palette().color(QPalette::WindowText));
  QToolTip::setPalette(palette);
}

/*!
 * \brief Displays tooltip for the specified component.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQToolTip class
 * \param text the tooltip text
 * \param x the x-coordinate of the tooltip
 * \param y the y-coordinate of the tooltip
 * \param componentBridge the component bridge pointer
 */
void JQToolTipBridge::showText(JNIEnv *env, jclass cls, jstring text, jint x,
                               jint y, jlong componentBridge)
{
  Q_ASSERT(env);
  QSWING_STATIC_REENTRANT(JQToolTipBridge, showText, QSWING_ENV_ARG,
                          JCLASS_ARG(cls), JSTRING_ARG(text), JINT_ARG(x),
                          JINT_ARG(y), JLONG_ARG(componentBridge))
  if (sm_enabled) {
    QWidget *widget = 0;
    if (componentBridge) {
      widget = jqComponentCast(componentBridge)->qWidget();
      QToolTip::showText(widget->mapToGlobal(QPoint(x, y)), QSwingString(text),
                         widget);
    } else {
      QToolTip::showText(QPoint(x, y), QSwingString(text));
    }
  }
}

/*!
 * \brief Hides tooltip.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQToolTip class
 */
void JQToolTipBridge::hideText(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  QSWING_STATIC_REENTRANT(JQToolTipBridge, hideText, QSWING_ENV_ARG,
                          JCLASS_ARG(cls))
  if (sm_enabled) {
    QToolTip::hideText();
  }
}

/*^**************************************************************************^*/
/*- JQToolTipBridge :: Private Static Data Member.                           -*/
/*^**************************************************************************^*/

bool JQToolTipBridge::sm_enabled = true;

/*^*****************************************************************************
  File History:

 - 2007-12-01 14:37:05 Slobodan
   Initial version

*****************************************************************************^*/
