/*
 * @(#)JQTreeBridge.cpp  0.2.0 / 2007-12-17
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQTreeBridge.h"

#include "QCursorAdapter.h"

/*^**************************************************************************^*/
/*- JQTreeBridge :: Protected Constructor.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQTreeBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQTree object
 */
JQTreeBridge::JQTreeBridge(JNIEnv *env, jobject obj)
  : JQComponentBridge(env, obj, new QTreeViewAdapter(this)), m_nodeIcon(0),
    m_treeModel(0), m_treeSelectionModel(0)
{
  // empty constructor body
}

/*^**************************************************************************^*/
/*- JQTreeBridge :: Protected Destructor.                                    -*/
/*^**************************************************************************^*/

/*!
 * \brief Destructs JQTreeBridge object.
 */
JQTreeBridge::~JQTreeBridge()
{
  if (m_nodeIcon) {
    delete m_nodeIcon;
  }
  if (m_treeSelectionModel) {
    delete m_treeSelectionModel;
  }
  if (m_treeModel) {
    delete m_treeModel;
  }
}

/*^**************************************************************************^*/
/*- JQTreeBridge :: Native Calls.                                            -*/
/*^**************************************************************************^*/

/*!
 * \brief Creates icon.
 *
 * \param env the JNIEnv pointer
 * \param iconSet the icon set
 */
void JQTreeBridge::createIcon(JNIEnv *env, jobject iconSet)
{
  Q_ASSERT(env);
  Q_ASSERT(iconSet);
  QSWING_REENTRANT(createIcon, QSWING_ENV_ARG, JOBJECT_ARG(iconSet))
  m_nodeIcon = new QIconAdapter(env, iconSet);
}

/*!
 * \brief Sets whether or not the tree is using tree model.
 *
 * \param env the JNIEnv pointer
 * \param useModel \c true if using
 */
void JQTreeBridge::setUseModel(JNIEnv *env, jboolean useModel)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setUseModel, QSWING_ENV_ARG, JBOOLEAN_ARG(useModel))
  TreeModel *oldTreeModel = m_treeModel;
  QItemSelectionModel *oldTreeSelectionModel = m_treeSelectionModel;
  if (useModel) {
    m_treeModel = new TreeModel(this);
    m_treeSelectionModel = new QItemSelectionModel(m_treeModel);
    qTreeView()->setModel(m_treeModel);
    qTreeView()->setSelectionModel(m_treeSelectionModel);
    connect(m_treeSelectionModel,
            SIGNAL(selectionChanged(const QItemSelection &,
                                    const QItemSelection &)),
            SLOT(selectionChangedSlot(const QItemSelection &,
                                      const QItemSelection &)));
    qTreeView()->expandToDepth(0); // TODO: temp
    qTreeView()->resizeColumnToContents(0);
  } else {
    qTreeView()->setSelectionModel(0);
    qTreeView()->setModel(0);
  }
  if (oldTreeSelectionModel) {
    delete oldTreeSelectionModel;
  }
  if (oldTreeModel) {
    disconnect(oldTreeModel,
               SIGNAL(selectionChanged(const QItemSelection &,
                                       const QItemSelection &)),
               SLOT(selectionChangedSlot(const QItemSelection &,
                                         const QItemSelection &)));
    delete oldTreeModel;
  }
}

/*^**************************************************************************^*/
/*- JQTreeBridge :: Static Native Calls.                                     -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes the JQTreeBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQTree class
 * \param treeNodeClass the ThreeNode class
 *
 * \exception FatalError if failed
 */
void JQTreeBridge::initialize(JNIEnv *env, jclass cls, jclass treeNodeClass)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_getRootNodeMID);
  TreeNode::initialize(env, treeNodeClass);
  if (!INIT_UMID(getRootNode, cls,
                 "()Lyu/ac/bg/etf/javaqx/qswing/tree/TreeNode;") ||
      !INIT_UMID(setNodeIcon, cls,
                 "(Lyu/ac/bg/etf/javaqx/qswing/tree/TreeNode;)Z") ||
      !INIT_UMID(nodeSelected, cls,
                 "(Lyu/ac/bg/etf/javaqx/qswing/tree/TreeNode;)V")) {
    FATAL_LOGGED_ERROR("JQTreeBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQTreeBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQTreeBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQTreeBridge :: Native Callbacks.                                        -*/
/*^**************************************************************************^*/

/*!
 * \brief Returns root node.
 * 
 * \return root node
 */
jobject JQTreeBridge::getRootNode()
{
  jobject result;
  if (jniObject()->callObjectMethod(result, sm_getRootNodeMID)) {
    return result;
  }
  return 0;
}

/*!
 * \brief Calls \c nodeSelected method of the tree object.
 *
 * \param node the node
 */
void JQTreeBridge::nodeSelected(jobject node)
{
  jniObject()->callVoidMethod(sm_nodeSelectedMID, node);
}

/*!
 * \brief Calls \c setNodeIcon method of the tree object.
 *
 * \param node the node
 */
jboolean JQTreeBridge::setNodeIcon(jobject node)
{
  jboolean result;
  if (jniObject()->callBooleanMethod(result, sm_setNodeIconMID, node)) {
    return result;
  }
  return false;
}

/*^**************************************************************************^*/
/*- JQTreeBridge :: Public Member Functions.                                 -*/
/*^**************************************************************************^*/

/*!
 * \brief Returns the icon for the specified node.
 *
 * \param node the node
 *
 * \return the icon for the specified node.
 */
QIcon *JQTreeBridge::getNodeIcon(jobject node)
{
  Q_ASSERT(node);
  if (setNodeIcon(node)) {
    return m_nodeIcon;
  }
  return 0;
}

/*^**************************************************************************^*/
/*- JQTreeBridge :: Overridden Member Functions.                             -*/
/*^**************************************************************************^*/

/*!
 * \inheritDoc
 */
jobject JQTreeBridge::getBackground(JNIEnv *env, jclass colorClass)
{
  return QColorAdapter::toJColor(env, colorClass,
                        qTreeView()->viewport()->palette().color(
                                    qTreeView()->viewport()->backgroundRole()));
}

/*!
 * \inheritDoc
 */
void JQTreeBridge::setBackground(JNIEnv *env, jint argb)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setBackground, QSWING_ENV_ARG, JINT_ARG(argb))
  QColorAdapter color(argb);
  QPalette palette(qTreeView()->viewport()->palette());
  if (color.isValid()) {
    palette.setColor(qTreeView()->viewport()->backgroundRole(), color);
  } else {
    palette.setColor(qTreeView()->viewport()->backgroundRole(),
                     QApplication::palette().color(
                                   qTreeView()->viewport()->backgroundRole()));
  }
  qTreeView()->viewport()->setPalette(palette);
}

/*!
 * \inheritDoc
 */
void JQTreeBridge::setNullBackground(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setNullBackground, QSWING_ENV_ARG)
  QPalette palette(qTreeView()->viewport()->palette());
  palette.setColor(qTreeView()->viewport()->backgroundRole(),
                   QApplication::palette().color(
                                 qTreeView()->viewport()->backgroundRole()));
  qTreeView()->viewport()->setPalette(palette);
}

/*!
 * \inheritDoc
 */
jint JQTreeBridge::getCursor(JNIEnv *env)
{
  Q_UNUSED(env);
  return QCursorAdapter::cursorType(qTreeView()->viewport()->cursor());
}

/*!
 * \inheritDoc
 */
void JQTreeBridge::setCursor(JNIEnv *env, jint type)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setCursor, QSWING_ENV_ARG, JINT_ARG(type))
  if (type < 0) {
    qTreeView()->viewport()->unsetCursor();
  } else {
    qTreeView()->viewport()->setCursor(QCursorAdapter(type));
  }
}

/*^**************************************************************************^*/
/*- JQTreeBridge :: Protected Member Function.                               -*/
/*^**************************************************************************^*/

/*!
 * \brief \c selectionChanged slot.
 *
 * \param selected the selected item
 * \param deselected the deselected item
 */
void JQTreeBridge::selectionChangedSlot(const QItemSelection &selected,
                                        const QItemSelection &deselected)
{
  Q_UNUSED(deselected);
  if (!selected.empty()) {
    // Qt Bug ???
    //QModelIndexList items = qTreeView()->selectionModel()->selectedIndexes();
    QModelIndex index = selected.at(0).topLeft();
    if (index.isValid()) {
      TreeNode *node = static_cast<TreeNode *>(index.internalPointer());
      if (node) {
        nodeSelected(node->node());
      }
    }
  }
}

/*^**************************************************************************^*/
/*- JQTreeBridge :: Private Static Data Members.                             -*/
/*^**************************************************************************^*/

jmethodID JQTreeBridge::sm_getRootNodeMID = 0;
jmethodID JQTreeBridge::sm_setNodeIconMID = 0;
jmethodID JQTreeBridge::sm_nodeSelectedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-12-17 14:20:21 Slobodan
   Initial version

*****************************************************************************^*/
