/*
 * @(#)JQWindowBridge.cpp  0.2.0 / 2007-10-31
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "JQWindowBridge.h"

#include "QSwingImageBridge.h"
#include "QSwingUtilitiesBridge.h"
#include "yu_ac_bg_etf_javaqx_qswing_JQWindow.h"

/*^**************************************************************************^*/
/*- JQWindowBridge :: Protected Constructors.                                -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new JQWindowBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQWindow object
 * \param widget the window widget
 */
JQWindowBridge::JQWindowBridge(JNIEnv *env, jobject obj, QWidget *widget)
  : JQContainerBridge(env, obj, widget), m_acceptClose(true)
{
  qWidget()->setAttribute(Qt::WA_DeleteOnClose, false);
  qWidget()->setFocusPolicy(Qt::WheelFocus);
  sm_windows.append(this);
}

/*!
 * \brief Constructs new JQWindowBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the JQWindow object
 */
JQWindowBridge::JQWindowBridge(JNIEnv *env, jobject obj)
  : JQContainerBridge(env, obj, new QWidget(0, Qt::Window)), m_acceptClose(true)
{
  qWidget()->setAttribute(Qt::WA_DeleteOnClose, false);
  qWidget()->setFocusPolicy(Qt::WheelFocus);
  sm_windows.append(this);
}

/*^**************************************************************************^*/
/*- JQWindowBridge :: Protected Destructor.                                  -*/
/*^**************************************************************************^*/

/*!
 * \brief Destructs JQWindowBridge object.
 */
JQWindowBridge::~JQWindowBridge()
{
  if (notifyDelete() && eventEnabled(QSWING_EVENT(WINDOW_EVENT_MASK))) {
    windowClosed();
  }
  sm_windows.removeAt(sm_windows.lastIndexOf(this));
}

/*^**************************************************************************^*/
/*- JQWindowBridge :: Native Calls.                                          -*/
/*^**************************************************************************^*/

/*!
 * \brief Sets window type.
 *
 * \param env the JNIEnv pointer
 * \param windowType the window type
 */
void JQWindowBridge::setWindowType(JNIEnv *env, jint windowType)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setWindowType, QSWING_ENV_ARG, JINT_ARG(windowType))
  QIcon windowIcon = qWidget()->windowIcon();
  Qt::WindowFlags flags = qWidget()->windowFlags();
  flags = flags & (Qt::WindowStaysOnTopHint | Qt::X11BypassWindowManagerHint);
  switch(windowType) { // avoids Qt::WindowFlags cast
    case yu_ac_bg_etf_javaqx_qswing_JQWindow_TYPE_WINDOW:
      flags = flags | Qt::Window;
      break;
    case yu_ac_bg_etf_javaqx_qswing_JQWindow_TYPE_DIALOG:
      flags = flags | Qt::Dialog;
      break;
    case yu_ac_bg_etf_javaqx_qswing_JQWindow_TYPE_POPUP:
      flags = flags | Qt::Popup;
      break;
    case yu_ac_bg_etf_javaqx_qswing_JQWindow_TYPE_TOOL:
      flags = flags | Qt::Tool;
      break;
    case yu_ac_bg_etf_javaqx_qswing_JQWindow_TYPE_SPLASH_SCREEN:
      flags = flags | Qt::SplashScreen;
      break;
  }
  qWidget()->setWindowFlags(flags);
  qWidget()->setWindowIcon(windowIcon);
}

/*!
 * \brief Sets window title.
 *
 * \param env the JNIEnv pointer
 * \param title the title
 */
void JQWindowBridge::setTitle(JNIEnv *env, jstring title)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setTitle, QSWING_ENV_ARG, JSTRING_ARG(title))
  if (title) {
    qWidget()->setWindowTitle(QStringAdapter(env, title));
  } else {
    qWidget()->setWindowTitle(" "); // avoids "javaw" title
  }
}

/*!
 * \brief Sets window icon.
 *
 * \param env the JNIEnv pointer
 * \param imagePointer the image pointer
 */
void JQWindowBridge::setIconImage(JNIEnv *env, jlong imagePointer)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setIconImage, QSWING_ENV_ARG, JLONG_ARG(imagePointer))
  if (imagePointer) {
    QPixmap *pixmap = qSwingImage;
    if (pixmap->isNull()) {
      qWidget()->setWindowIcon(QApplication::windowIcon());
    } else {
      qWidget()->setWindowIcon(QIcon(*pixmap));
    }
  } else {
    qWidget()->setWindowIcon(QApplication::windowIcon());
  }
}

/*!
 * \brief Sets window icon.
 *
 * \param env the JNIEnv pointer
 * \param imageData the image data
 * \param length the image data length
 */
void JQWindowBridge::setIconImage(JNIEnv *env, jbyteArray imageData,
                                  jint length)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setIconImage, QSWING_ENV_ARG, JBYTEARRAY_ARG(imageData),
                   JINT_ARG(length))
  if (imageData) {
    QPixmap pixmap;
    if (QSwingImageBridge::loadImage(env, imageData, length, &pixmap)) {
      qWidget()->setWindowIcon(QIcon(pixmap));
      return;
    }
  }
  qWidget()->setWindowIcon(QApplication::windowIcon());
}

/*!
 * \brief Sets window opacity.
 *
 * \param env the JNIEnv pointer
 * \param opacity the opacity
 */
void JQWindowBridge::setOpacity(JNIEnv *env, jdouble opacity)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setOpacity, QSWING_ENV_ARG, JDOUBLE_ARG(opacity))
  qWidget()->setWindowOpacity(opacity);
}

/*!
 * \brief Sets whether or not the window can be closed.
 *
 * \param env the JNIEnv pointer
 * \param accept if \c true the window can be closed
 */
void JQWindowBridge::setAcceptClose(JNIEnv *env, jboolean accept)
{
  Q_UNUSED(env);
  m_acceptClose = accept;
}

/*!
 * \brief Sets window modality.
 *
 * \param env the JNIEnv pointer
 * \param modality the modality
 */
void JQWindowBridge::setModality(JNIEnv *env, jint modality)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setModality, QSWING_ENV_ARG, JINT_ARG(modality))
  if ((modality >= Qt::NonModal) && (modality <= Qt::ApplicationModal)) {
    qWidget()->setWindowModality((Qt::WindowModality)modality);
  }
}

/*!
 * \brief Sets window state
 *
 * \param env the JNIEnv pointer
 * \param state the window state
 */
void JQWindowBridge::setState(JNIEnv *env, jint state)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setState, QSWING_ENV_ARG, JINT_ARG(state))
  switch (state) {
    case 0:
      qWidget()->setWindowState(Qt::WindowNoState);
      break;
    case 1:
      qWidget()->setWindowState(Qt::WindowMinimized);
      break;
    case 8:
      qWidget()->setWindowState(Qt::WindowMaximized);
      break;
    case 16:
      qWidget()->setWindowState(Qt::WindowFullScreen);
      break;
  }
}

/*!
 * \brief Sets whether or not the window is always on top.
 *
 * \param env the JNIEnv pointer
 * \param alwaysOnTop if \c true the window is always on top
 */
void JQWindowBridge::setAlwaysOnTop(JNIEnv *env, jboolean alwaysOnTop)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setAlwaysOnTop, QSWING_ENV_ARG, JBOOLEAN_ARG(alwaysOnTop))
  QWidget *widget = qWidget();
  bool visible = widget->isVisible();
  QIcon windowIcon = widget->windowIcon();
  Qt::WindowFlags flags = widget->windowType();
  if (alwaysOnTop) {
    flags |= Qt::WindowStaysOnTopHint;
    #ifdef JAVAQX_BYPASS_WINDOW_MANAGER_HINT
      flags |= Qt::X11BypassWindowManagerHint
    #endif // JAVAQX_BYPASS_WINDOW_MANAGER_HINT
  } else {
    flags &= ~Qt::WindowStaysOnTopHint;
    #ifdef JAVAQX_BYPASS_WINDOW_MANAGER_HINT
      flags &= Qt::X11BypassWindowManagerHint
    #endif // JAVAQX_BYPASS_WINDOW_MANAGER_HINT
  }
  widget->setWindowFlags(flags);
  widget->setWindowIcon(windowIcon);
  if (visible) {
    widget->setVisible(true);
  }
}

/*!
 * \brief Sets window location relative to the specified window.
 *
 * \param env the JNIEnv pointer
 * \param windowBridge the referent window
 */
void JQWindowBridge::setLocationRelativeTo(JNIEnv *env, jlong windowBridge)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setLocationRelativeTo, QSWING_ENV_ARG,
                   JLONG_ARG(windowBridge))
  QWidget *widget = qWidget();
  QRect widgetGeometry = widget->frameGeometry();
  QRect centralGeometry;
  if (windowBridge) {
    centralGeometry = jqWindowCast(windowBridge)->qWidget()->frameGeometry();
  } else {
    int screenNumber = 0;
    QWidget *topWidget = widget->topLevelWidget();
    QDesktopWidget *desktopWidget = QApplication::desktop();
    if (topWidget) {
      screenNumber = desktopWidget->screenNumber(topWidget);
    } else if(desktopWidget->isVirtualDesktop()) {
      screenNumber = desktopWidget->screenNumber(QCursor::pos());
    } else {
      screenNumber = desktopWidget->screenNumber(widget);
    }
    centralGeometry = desktopWidget->availableGeometry(screenNumber);
  }
  widget->move(QPoint(centralGeometry.x() +
                      (centralGeometry.width() - widgetGeometry.width()) / 2,
                      centralGeometry.y() +
                      (centralGeometry.height() -
                                                widgetGeometry.height()) / 2));
}

/*!
 * \brief Brings the window to front.
 *
 * \param env the JNIEnv pointer
 */
void JQWindowBridge::toFront(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(toFront, QSWING_ENV_ARG)
  qWidget()->raise();
  qWidget()->activateWindow();
}

/*!
 * \brief Moves the window to back.
 *
 * \param env the JNIEnv pointer
 */
void JQWindowBridge::toBack(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(toBack, QSWING_ENV_ARG)
  qWidget()->clearFocus();
  qWidget()->lower();
}

/*!
 * \brief Sets border decoration.
 *
 * \param env the JNIEnv pointer
 * \param borderDecoration the border decoration
 */
void JQWindowBridge::setBorderDecoration(JNIEnv *env, jint borderDecoration)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setBorderDecoration, QSWING_ENV_ARG,
                   JINT_ARG(borderDecoration))
  QIcon windowIcon = qWidget()->windowIcon();
  Qt::WindowFlags flags = qWidget()->windowFlags();
  flags = flags & (Qt::WindowStaysOnTopHint | Qt::X11BypassWindowManagerHint |
                   Qt::WindowType_Mask);
  qWidget()->setWindowFlags(flags | Qt::WindowFlags(borderDecoration));
  qWidget()->setWindowIcon(windowIcon);
}

/*^**************************************************************************^*/
/*- JQWindowBridge :: Static Native Calls.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes JQWindowBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQWindow class
 *
 * \exception FatalError if failed
 */
void JQWindowBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_windowClass);
  if (!INIT_CLASS(window, cls) ||
      !INIT_UMID(windowOpened, cls, "()V") ||
      !INIT_UMID(windowClosing, cls, "()V") ||
      !INIT_UMID(windowClosed, cls, "()V") ||
      !INIT_UMID(windowActivated, cls, "()V") ||
      !INIT_UMID(windowDeactivated, cls, "()V") ||
      !INIT_UMID(windowGainedFocus, cls, "()V") ||
      !INIT_UMID(windowLostFocus, cls, "()V") ||
      !INIT_UMID(windowStateChanged, cls, "(II)V")) {
    FATAL_LOGGED_ERROR("JQWindowBridge initialization failed");
  }
}

/*!
 * \brief Returns \c true if the system supports window opacity.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQWindow class
 * 
 * \return \c true if the sitem supports window opacity
 */
jboolean JQWindowBridge::opacitySupported(JNIEnv *env, jclass cls)
{
  Q_UNUSED(env);
  Q_UNUSED(cls);
  #ifdef Q_OS_WIN32
    return (QSysInfo::WindowsVersion >= QSysInfo::WV_2000);
  #else
    return true;
  #endif // Q_OS_WIN32
}

/*!
 * \brief Returns \c true if the system supports always on top windows.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQWindow class
 * 
 * \return \c true if the system supports always on top windows
 */
jboolean JQWindowBridge::alwaysOnTopSupported(JNIEnv *env, jclass cls)
{
  Q_UNUSED(env);
  Q_UNUSED(cls);
  return true;
}

/*!
 * \brief Returns \c true if the system supports full screen windows.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQWindow class
 * 
 * \return \c true if the system supports full screen windows
 */
jboolean JQWindowBridge::fullScreenSupported(JNIEnv *env, jclass cls)
{
  Q_UNUSED(env);
  Q_UNUSED(cls);
  return true;
}

/*!
 * \brief \c true if the system supports non-rectangular windows.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQWindow class
 * 
 * \return \c true if the system supports non-rectangular windows
 */
jboolean JQWindowBridge::windowShapeSupported(JNIEnv *env, jclass cls)
{
  Q_UNUSED(env);
  Q_UNUSED(cls);
  return true;
}

/*!
 * \brief Returns all non deleted windows.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQWindow class
 * 
 * \return all non deleted windows
 */
jobjectArray JQWindowBridge::getWindows(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  QSWING_STATIC_REENTRANT_RETURN(JQWindowBridge, getWindows, jobjectArray,
                                 QSWING_ENV_ARG, JCLASS_ARG(cls))
  int windowCount = sm_windows.size();
  jobjectArray jwindows = JNIWrapper::newObjectArray(env, sm_windowClass,
                                                     windowCount);
  if (jwindows) {
    for (int i = 0; i < windowCount; i++) {
      env->SetObjectArrayElement(jwindows, i,
                                 sm_windows.at(i)->jniObject()->ref());
    }
  }
  return jwindows;
}

/*!
 * \brief Returns active window.
 *
 * \return active window
 */
jobject JQWindowBridge::getActiveWindow(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  QSWING_STATIC_REENTRANT_RETURN(JQWindowBridge, getActiveWindow, jobject,
                                 QSWING_ENV_ARG, JCLASS_ARG(cls))
  QWidget *activeWindow = QApplication::activeWindow();
  if (activeWindow) {
    JQWindowBridge *windowBridge =
                          qObjectBridgePointer<JQWindowBridge *>(activeWindow);
    if (windowBridge) {
      return windowBridge->jniObject()->ref();
    }
  }
  return 0;
}

/*!
 * \brief Sets the owner of the specified window.
 *
 * \param env the JNIEnv pointer
 * \param cls the JQWindow class
 * \param windowBridge the window bridge pointer
 * \param ownerBridge the owner bridge pointer
 */
void JQWindowBridge::setWindowOwner(JNIEnv *env, jclass cls, jlong windowBridge,
                                    jlong ownerBridge)
{
  Q_ASSERT(env);
  QSWING_STATIC_REENTRANT(JQWindowBridge, setWindowOwner, QSWING_ENV_ARG,
                          JCLASS_ARG(cls), JLONG_ARG(windowBridge),
                          JLONG_ARG(ownerBridge))
  JQWindowBridge *windowBridgePointer = jqWindowCast(windowBridge);
  QWidget *window = windowBridgePointer->qWidget();
  QIcon windowIcon = window->windowIcon();
  bool visible = window->isVisible();
  if (ownerBridge) {
    JQWindowBridge *ownerBridgePointer = jqWindowCast(ownerBridge);
    window->setParent(ownerBridgePointer->qWidget(), window->windowFlags());
    windowBridgePointer->setParentBridge(ownerBridgePointer);
  } else {
    window->setParent(0, window->windowFlags());
    windowBridgePointer->setParentBridge(0);
  }
  window->setWindowIcon(windowIcon);
  if (visible) {
    window->setVisible(true);
  }
}

/*!
 * \brief Returns new instance of the bridge object
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing object
 * 
 * \return new instance of the bridge object
 */
jlong JQWindowBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new JQWindowBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, createMID());
}

/*^**************************************************************************^*/
/*- JQWindowBridge :: Native Callbacks.                                      -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls windowOpened method of the JQComponent object.
 */
void JQWindowBridge::windowOpened()
{
  jniObject()->callVoidMethod(sm_windowOpenedMID);
}

/*!
 * \brief Calls windowClosing method of the JQComponent object.
 */
void JQWindowBridge::windowClosing()
{
  jniObject()->callVoidMethod(sm_windowClosingMID);
}

/*!
 * \brief Calls windowClosed method of the JQComponent object.
 */
void JQWindowBridge::windowClosed()
{
  jniObject()->callVoidMethod(sm_windowClosedMID);
}

/*!
 * \brief Calls windowActivated method of the JQComponent object.
 */
void JQWindowBridge::windowActivated()
{
  jniObject()->callVoidMethod(sm_windowActivatedMID);
}

/*!
 * \brief Calls windowDeactivated method of the JQComponent object.
 */
void JQWindowBridge::windowDeactivated()
{
  jniObject()->callVoidMethod(sm_windowDeactivatedMID);
}

/*!
 * \brief Calls windowGainedFocus method of the JQComponent object.
 */
void JQWindowBridge::windowGainedFocus()
{
  jniObject()->callVoidMethod(sm_windowGainedFocusMID);
}

/*!
 * \brief Calls windowLostFocus method of the JQComponent object.
 */
void JQWindowBridge::windowLostFocus()
{
  jniObject()->callVoidMethod(sm_windowLostFocusMID);
}

/*!
 * \brief Calls windowStateChanged method of the JQComponent object.
 *
 * \param oldState the old window state
 * \param newState the new window state
 */
void JQWindowBridge::windowStateChanged(jint oldState, jint newState)
{
  jniObject()->callVoidMethod(sm_windowStateChangedMID, oldState, newState);
}

/*^**************************************************************************^*/
/*- JQWindowBridge :: Overridden Member Functions.                           -*/
/*^**************************************************************************^*/

/*!
 * \inheritDoc
 */
bool JQWindowBridge::filterEvent(QObject *obj, QEvent *e)
{
  bool filtered = true;
  switch (e->type()) {
    case QEvent::Close:
      windowClosing();
      filtered = obj->event(e);
      if (m_acceptClose) {
        e->accept();
      } else {
        e->ignore();
      }
      break;
    case QEvent::WindowActivate:
      filtered = obj->event(e);
      if (eventEnabled(QSWING_EVENT(WINDOW_EVENT_MASK))) {
        windowActivated();
      }
      break;
    case QEvent::WindowDeactivate:
      filtered = obj->event(e);
      if (eventEnabled(QSWING_EVENT(WINDOW_EVENT_MASK))) {
        windowDeactivated();
      }
      break;
    case QEvent::FocusIn:
      filtered = obj->event(e);
      if (eventEnabled(QSWING_EVENT(WINDOW_EVENT_MASK))) {
        windowGainedFocus();
      }
      focusGained(); // little opt.
      break;
    case QEvent::FocusOut:
      filtered = obj->event(e);
      if (eventEnabled(QSWING_EVENT(WINDOW_EVENT_MASK))) {
        windowLostFocus();
      }
      focusLost(); // little opt.
      break;
    case QEvent::WindowStateChange:
      filtered = obj->event(e);
      if (eventEnabled(QSWING_EVENT(WINDOW_EVENT_MASK))) {
        QWindowStateChangeEvent *wsce =
                                     static_cast<QWindowStateChangeEvent *>(e);
        Qt::WindowStates oldState = wsce->oldState();
        Qt::WindowStates newState = qWidget()->windowState();
        Qt::WindowStates oldActiveState = (oldState & Qt::WindowActive);
        Qt::WindowStates newActiveState = (newState & Qt::WindowActive);
        oldState = (oldState & ~Qt::WindowActive);
        newState = (newState & ~Qt::WindowActive);
        if (oldState != newState) {
          windowStateChanged(converWindowState(oldState),
                             converWindowState(newState));
        }
        if (oldActiveState != newActiveState) {
          if (newActiveState) {
            windowActivated();
          } else {
            windowDeactivated();
          }
        }
      }
      break;
    default:
      filtered = JQContainerBridge::filterEvent(obj, e);
  }
  return filtered;
}

/*!
 * \inheritDoc
 */
void JQWindowBridge::showWindow(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(showWindow, QSWING_ENV_ARG)
  if (!qWidget()->testAttribute(Qt::WA_Resized)) {
    qWidget()->adjustSize();
  }
  qWidget()->setVisible(true);
  if (eventEnabled(QSWING_EVENT(WINDOW_EVENT_MASK))) {
    windowOpened();
  }
  qWidget()->setFocus(Qt::ActiveWindowFocusReason);
}

/*^**************************************************************************^*/
/*- JQWindowBridge :: Private Member Function.                               -*/
/*^**************************************************************************^*/

jint JQWindowBridge::converWindowState(Qt::WindowStates states)
{
  switch(states) {
    case Qt::WindowMinimized:
      return 1;
    case Qt::WindowMaximized:
      return 8;
    case Qt::WindowFullScreen:
      return 16;
    default:
      return 0;
  }
}

/*^**************************************************************************^*/
/*- JQWindowBridge :: Private Static Data Members.                           -*/
/*^**************************************************************************^*/

QList<JQWindowBridge *> JQWindowBridge::sm_windows;
jclass JQWindowBridge::sm_windowClass = 0;
jmethodID JQWindowBridge::sm_windowOpenedMID = 0;
jmethodID JQWindowBridge::sm_windowClosingMID = 0;
jmethodID JQWindowBridge::sm_windowClosedMID = 0;
jmethodID JQWindowBridge::sm_windowActivatedMID = 0;
jmethodID JQWindowBridge::sm_windowDeactivatedMID = 0;
jmethodID JQWindowBridge::sm_windowGainedFocusMID = 0;
jmethodID JQWindowBridge::sm_windowLostFocusMID = 0;
jmethodID JQWindowBridge::sm_windowStateChangedMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-10-31 17:36:22 Slobodan
   Initial version

*****************************************************************************^*/
