/*
 * @(#)PrinterBridge.cpp  0.2.0 / 2007-11-15
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "PrinterBridge.h"

#include "QSwingBridge.h"

/*^**************************************************************************^*/
/*- PrinterBridge :: Protected Constructor.                                  -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new PrinterBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing Printer object
 */
PrinterBridge::PrinterBridge(JNIEnv *env, jobject obj)
  : QObject()
{
  m_wobjref = JNIWrapper::newWeakGlobalRef(env, obj);
  if (m_wobjref) {
    m_printer = new QPrinter();
  }
}

/*^**************************************************************************^*/
/*- QObjectBridge :: Protected Destructor.                                   -*/
/*^**************************************************************************^*/

/*!
 * \brief Destructs PrinterBridge object.
 */
PrinterBridge::~PrinterBridge()
{
  if (m_wobjref) {
    JNIQSwingWrapper::deleteWeakGlobalRef(m_wobjref);
  }
  if (m_printer) {
    delete m_printer;
  }
}

/*^**************************************************************************^*/
/*- PrinterBridge :: Native Calls.                                           -*/
/*^**************************************************************************^*/

/*!
 * \brief Destroys the PrinterBridge object.
 *
 * \param env the JNIEnv pointer
 */
void PrinterBridge::destroy(JNIEnv *env)
{
  Q_UNUSED(env);
  if (QSwingBridge::isDisposed()) {
    delete this; // OK - referenced only from java
  } else {
    deleteLater();
  }
}

/*!
 * \brief Displays page dialog.
 *
 * \param env the JNIEnv pointer
 */
void PrinterBridge::pageDialog(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(pageDialog, QSWING_ENV_ARG)
  QPageSetupDialog pageSetup(m_printer, QApplication::activeWindow());
  pageSetup.exec();
}

/*!
 * \brief Displays printer dialog.
 *
 * \param env the JNIEnv pointer
 * 
 * \return \c it the dialog is accepted
 */
jboolean PrinterBridge::printerDialog(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT_RETURN(printerDialog, jboolean, QSWING_ENV_ARG)
  QPrintDialog printDialog(m_printer, QApplication::activeWindow());
  return (printDialog.exec() == QDialog::Accepted);
}

/*^**************************************************************************^*/
/*- PrinterBridge :: Static Native Calls.                                    -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes PrinterBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the Printer class
 *
 * \exception FatalError if failed
 */
void PrinterBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT(!sm_bridgeFID);
  if (!INIT_UFID(bridge, cls, "J") || !INIT_UMID(create, cls, "()J")) {
    FATAL_LOGGED_ERROR("PrinterBridge initialization failed");
  }
}

/*!
 * \brief Returns new instance of the bridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the Printer object
 * 
 * \return new instance of the bridge object
 */
jlong PrinterBridge::newInstance(JNIEnv *env, jobject obj)
{
  if (JNIQSwingWrapper::isQSwingThread(env)) {
    return reinterpret_cast<jlong>(new PrinterBridge(env, obj));
  }
  return QSwingUtilitiesBridge::invokeLongAndWait(env, obj, sm_createMID);
}

/*^**************************************************************************^*/
/*- PrinterBridge :: Public Static Member Function.                          -*/
/*^**************************************************************************^*/

/*!
 * \brief Returns QPrinter object of the specified QSwing Printer object.
 *
 * \param env the JNIEnv pointer
 * \param printer the QSwing printer object
 * 
 * \return QPrinter object of the specified QSwing Printer object
 */
QPrinter *PrinterBridge::toQPrinter(JNIEnv *env, jobject printer)
{
  return printerCast(JNIWrapper::getLongField(env, printer,
                                              sm_bridgeFID))->qPrinter();
}

/*^**************************************************************************^*/
/*- PrinterBridge :: Private Static Data Members.                            -*/
/*^**************************************************************************^*/

jfieldID PrinterBridge::sm_bridgeFID = 0;
jmethodID PrinterBridge::sm_createMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-11-15 08:27:12 Slobodan
   Initial version

*****************************************************************************^*/
