/*
 * @(#)QSwingUtilitiesBridge.cpp  0.2.0 / 2007-07-28
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "QSwingUtilitiesBridge.h"

#include "JavaQxQSwing.h"

/*^**************************************************************************^*/
/*- QSwingUtilitiesBridge :: Private Constructors.                           -*/
/*^**************************************************************************^*/

QSwingUtilitiesBridge::QSwingUtilitiesBridge()
  : QObject(0)
{
  connect(this, SIGNAL(invokeLaterSignal(jobject)), this,
          SLOT(invokeLaterSlot(jobject)), Qt::QueuedConnection);
  connect(this, SIGNAL(invokeAndWaitSignal(TargetFrame *)), this,
          SLOT(invokeAndWaitSlot(TargetFrame *)), Qt::BlockingQueuedConnection);
  connect(this, SIGNAL(invokeSafeAndWaitSignal(jobject)), this,
          SLOT(invokeLaterSlot(jobject)), Qt::BlockingQueuedConnection);
  connect(this, SIGNAL(invokeIntAndWaitSignal(IntMethodFrame *)), this,
          SLOT(invokeIntAndWaitSlot(IntMethodFrame *)),
          Qt::BlockingQueuedConnection);
  connect(this, SIGNAL(invokeLongAndWaitSignal(LongMethodFrame *)), this,
          SLOT(invokeLongAndWaitSlot(LongMethodFrame *)),
          Qt::BlockingQueuedConnection);
}

QSwingUtilitiesBridge::QSwingUtilitiesBridge(const QSwingUtilitiesBridge &)
  : QObject(0)
{
  // disabled
}

/*^**************************************************************************^*/
/*- QSwingUtilitiesBridge :: Private Destructor.                             -*/
/*^**************************************************************************^*/

QSwingUtilitiesBridge::~QSwingUtilitiesBridge()
{
  // disabled
}

/*^**************************************************************************^*/
/*- QSwingUtilitiesBridge :: Static Native Calls.                            -*/
/*^**************************************************************************^*/

/*!
 * \brief Invokes runnable target from the GUI loop.
 *
 * \param env the JNIEnv pointer 
 * \param cls the JavaQx QSwingUtilities class
 * \param target the runnable target object
 */
void QSwingUtilitiesBridge::invokeLater(JNIEnv *env, jclass cls,
                                        jobject target)
{
  Q_UNUSED(cls);
  qSwingUtilitiesBridge->invokeLater(JNIWrapper::newGlobalRef(env, target));
}

/*!
 * \brief Invokes runnable target from the  GUI loop and waits until finish.
 *
 * \param env the JNIEnv pointer 
 * \param cls the JavaQx QSwingUtilities class
 * \param target the runnable target object
 */
void QSwingUtilitiesBridge::invokeAndWait(JNIEnv *env, jclass cls,
                                          jobject target)
{
  Q_UNUSED(cls);
  TargetFrame targetFrame;
  targetFrame.target = JNIWrapper::newGlobalRef(env, target);
  qSwingUtilitiesBridge->invokeAndWait(&targetFrame);
  if (targetFrame.exception) {
    jclass cls = env->FindClass("java/lang/reflect/InvocationTargetException");
    if (cls != 0) {
      jthrowable ite = (jthrowable)env->NewObject(cls, sm_iteConstructorMID,
                                                  targetFrame.exception);
      if (ite) {
        env->Throw(ite);
        env->DeleteLocalRef(ite);
      } else {
        JNIWrapper::checkException(env);
      }
      env->DeleteLocalRef(cls);
    } else {
      JNIWrapper::checkException(env);
    }
    env->DeleteGlobalRef(targetFrame.exception);
  }
  env->DeleteGlobalRef(targetFrame.target);
}

/*!
 * \brief Invokes "safe" runnable target from the GUI loop and waits until
 *        finish.
 *
 * \param env the JNIEnv pointer
 * \param cls the JavaQx QSwingUtilities class
 * \param safeTarget the safe runnable target object
 */
void QSwingUtilitiesBridge::invokeSafeAndWait(JNIEnv *env, jclass cls,
                                              jobject safeTarget)
{
  Q_UNUSED(cls);
  qSwingUtilitiesBridge->invokeSafeAndWait(
                                    JNIWrapper::newGlobalRef(env, safeTarget));
}

/*^**************************************************************************^*/
/*- QSwingUtilitiesBridge :: Static Member Functions.                        -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes QSwingUtilitiesBridge class.
 *
 * \param env the JNIEnv pointer 
 * \param runnableClass the Java Runnable class
 * \param iteClass the Java InvocationTargetException class
 */
void QSwingUtilitiesBridge::initialize(JNIEnv *env, jclass runnableClass,
                                       jclass iteClass)
{
  Q_ASSERT(env);
  Q_ASSERT(runnableClass);
  Q_ASSERT(iteClass);
  Q_ASSERT(!sm_runMID);
  if (!INIT_MID(run, runnableClass, "()V") ||
      !INIT_CONSTRUCTOR(iteConstructor, iteClass, "(Ljava/lang/Throwable;)V")) {
    FATAL_LOGGED_ERROR("QSwingUtilitiesBridge initialization failed");
  }
  qSwingUtilitiesBridge = new QSwingUtilitiesBridge();
}

/*!
 * \brief Invokes int method of the specifed object from the QSwing thread and
 *        waits until finish.
 *
 * \param env the JNIEnv pointer
 * \param obj the JavaQx object
 * \param mid the method ID
 * \param ... the method arguments
 *
 * \return method result
 */
jint QSwingUtilitiesBridge::invokeIntAndWait(JNIEnv *env, jobject obj,
                                             jmethodID mid, ...)
{
  Q_ASSERT(mid);
  IntMethodFrame methodFrame;
  methodFrame.obj = JNIWrapper::newGlobalRef(env, obj);
  methodFrame.mid = mid;
  va_start(methodFrame.args, mid);
  qSwingUtilitiesBridge->invokeIntAndWait(&methodFrame);
  va_end(methodFrame.args);
  env->DeleteGlobalRef(methodFrame.obj);
  return methodFrame.result;
}

/*!
 * \brief Invokes long method of the specifed object from the QSwing thread and
 *        waits until finish.
 *
 * \param env the JNIEnv pointer
 * \param obj the JavaQx object
 * \param mid the method ID
 * \param ... the method arguments
 *
 * \return method result
 */
jlong QSwingUtilitiesBridge::invokeLongAndWait(JNIEnv *env, jobject obj,
                                               jmethodID mid, ...)
{
  Q_ASSERT(mid);
  LongMethodFrame methodFrame;
  methodFrame.obj = JNIWrapper::newGlobalRef(env, obj);
  methodFrame.mid = mid;
  va_start(methodFrame.args, mid);
  qSwingUtilitiesBridge->invokeLongAndWait(&methodFrame);
  va_end(methodFrame.args);
  env->DeleteGlobalRef(methodFrame.obj);
  return methodFrame.result;
}

/*^**************************************************************************^*/
/*- QSwingUtilitiesBridge :: Private Member Functions.                       -*/
/*^**************************************************************************^*/

void QSwingUtilitiesBridge::invokeLater(jobject target)
{
  emit invokeLaterSignal(target);
}

void QSwingUtilitiesBridge::invokeAndWait(TargetFrame *targetFrame)
{
  emit invokeAndWaitSignal(targetFrame);
}

void QSwingUtilitiesBridge::invokeSafeAndWait(jobject safeTarget)
{
  emit invokeSafeAndWaitSignal(safeTarget);
}

void QSwingUtilitiesBridge::invokeIntAndWait(IntMethodFrame *methodFrame)
{
  emit invokeIntAndWaitSignal(methodFrame);
}

void QSwingUtilitiesBridge::invokeLongAndWait(LongMethodFrame *methodFrame)
{
  emit invokeLongAndWaitSignal(methodFrame);
}

void QSwingUtilitiesBridge::invokeLaterSlot(jobject target)
{
  JNIQSwingWrapper::callVoidMethod(target, sm_runMID);
  JNIQSwingWrapper::deleteGlobalRef(target);
}

void QSwingUtilitiesBridge::invokeAndWaitSlot(TargetFrame *targetFrame)
{
  QSWING_ENV->CallVoidMethod(targetFrame->target, sm_runMID);
  jthrowable exception = QSWING_ENV->ExceptionOccurred();
  if (exception) {
    targetFrame->exception = (jthrowable)QSWING_ENV->NewGlobalRef(exception);
    QSWING_ENV->ExceptionClear();
    QSWING_ENV->DeleteLocalRef(exception);
  } else {
    targetFrame->exception = 0;
  }
}

void QSwingUtilitiesBridge::invokeIntAndWaitSlot(IntMethodFrame *methodFrame)
{
  methodFrame->result = QSWING_ENV->CallIntMethodV(methodFrame->obj,
                                                   methodFrame->mid,
                                                   methodFrame->args);
  if (JNIQSwingWrapper::testException()) {
    methodFrame->result = 0;
  }
}

void QSwingUtilitiesBridge::invokeLongAndWaitSlot(LongMethodFrame *methodFrame)
{
  methodFrame->result = QSWING_ENV->CallLongMethodV(methodFrame->obj,
                                                    methodFrame->mid,
                                                    methodFrame->args);
  if (JNIQSwingWrapper::testException()) {
    methodFrame->result = 0;
  }
}

/*^**************************************************************************^*/
/*- QSwingUtilitiesBridge :: Private Static Data Members.                    -*/
/*^**************************************************************************^*/

jmethodID QSwingUtilitiesBridge::sm_runMID = 0;
jmethodID QSwingUtilitiesBridge::sm_iteConstructorMID = 0;
QSwingUtilitiesBridge *QSwingUtilitiesBridge::qSwingUtilitiesBridge = 0;

/*^*****************************************************************************
  File History:

 - 2007-07-28 11:43:10 Slobodan
   Initial version

*****************************************************************************^*/
