/*
 * @(#)TextDocumentBridge.cpp  0.2.0 / 2007-12-11
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
#include "TextDocumentBridge.h"

#include "PrinterBridge.h"
#include "QSwingBridge.h"
#include "QTextCharFormatAdapter.h"

/*^**************************************************************************^*/
/*- TextDocumentBridge :: Protected Constructor.                             -*/
/*^**************************************************************************^*/

/*!
 * \brief Constructs new TextDocumentBridge object.
 *
 * \param env the JNIEnv pointer
 * \param obj the QSwing TextDocument object
 * \param document the Qt QTextDocument
 */
TextDocumentBridge::TextDocumentBridge(JNIEnv *env, jobject obj,
                                       QTextDocumentAdapter *document)
  : QObjectBridge(env, obj, document)
{
  // empty constructor body
}

/*^**************************************************************************^*/
/*- TextDocumentBridge :: Native Calls.                                      -*/
/*^**************************************************************************^*/

/*!
 * \brief Clears the text.
 *
 * \param env the JNIEnv pointer
 */
void TextDocumentBridge::clearText(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(clearText, QSWING_ENV_ARG)
  qTextDocument()->clear();
}

/*!
 * \brief Returns text of the document.
 *
 * \param env the JNIEnv pointer
 * 
 * \return text of the document
 */
jstring TextDocumentBridge::getText(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT_RETURN(getText, jstring, QSWING_ENV_ARG)
  return QStringAdapter::toJString(env, qTextDocument()->getText());
}

/*!
 * \brief Sets text of the document.
 *
 * \param env the JNIEnv pointer
 * \param text the text of the document
 */
void TextDocumentBridge::setText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setText, QSWING_ENV_ARG, JSTRING_ARG(text))
  qTextDocument()->setText(QStringAdapter(env, text));
}

/*!
 * \brief Appends text to the end of the document.
 *
 * \param env the JNIEnv pointer
 * \param text text the text to be appended
 */
void TextDocumentBridge::appendText(JNIEnv *env, jstring text)
{
  Q_ASSERT(env);
  Q_ASSERT(text);
  QSWING_REENTRANT(appendText, QSWING_ENV_ARG, JSTRING_ARG(text))
  qTextDocument()->appendText(QStringAdapter(env, text));
}

/*!
 * \brief Returns \c true if undo operation is available.
 *
 * \param env the JNIEnv pointer
 * 
 * \return \c true if undo operation is available
 */
jboolean TextDocumentBridge::isUndoAvailable(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT_RETURN(isUndoAvailable, jboolean, QSWING_ENV_ARG)
  return qTextDocument()->isUndoAvailable();
}

/*!
 * \brief Returns \c true if redo operation is available.
 *
 * \param env the JNIEnv pointer
 * 
 * \return \c true if undo operation is available
 */
jboolean TextDocumentBridge::isRedoAvailable(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT_RETURN(isRedoAvailable, jboolean, QSWING_ENV_ARG)
  return qTextDocument()->isRedoAvailable();
}

/*!
 * \brief Undos last action.
 *
 * \param env the JNIEnv pointer
 */
void TextDocumentBridge::undo(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(undo, QSWING_ENV_ARG)
  qTextDocument()->undo();
}

/*!
 * \brief Redos last action.
 *
 * \param env the JNIEnv pointer
 */
void TextDocumentBridge::redo(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(redo, QSWING_ENV_ARG)
  qTextDocument()->redo();
}

/*!
 * \brief Sets whether or not the undo stack is enabled.
 *
 * \param env the JNIEnv pointer
 * \param enabled \c true if enabled
 */
void TextDocumentBridge::setUndoRedoEnabled(JNIEnv *env, jboolean enabled)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setUndoRedoEnabled, QSWING_ENV_ARG, JBOOLEAN_ARG(enabled))
  qTextDocument()->setUndoRedoEnabled(enabled);
}

/*!
 * \brief Sets undo/redo listening.
 *
 * \param env the JNIEnv pointer
 * \param listening \c true if listening
 */
void TextDocumentBridge::setUndoRedoListening(JNIEnv *env, jboolean listening)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setUndoRedoListening, QSWING_ENV_ARG,
                   JBOOLEAN_ARG(listening))
  if (listening) {
    connectAuto(qTextDocument(), SIGNAL(undoAvailable(bool)),
                SLOT(undoAvailableSlot(bool)));
    connectAuto(qTextDocument(), SIGNAL(redoAvailable(bool)),
                SLOT(redoAvailableSlot(bool)));
  } else {
    disconnect(qTextDocument(), SIGNAL(undoAvailable(bool)),
               SLOT(undoAvailableSlot(bool)));
    disconnect(qTextDocument(), SIGNAL(redoAvailable(bool)),
               SLOT(redoAvailableSlot(bool)));
  }
}

/*!
 * \brief Returns \c true if the document is modified.
 *
 * \param env the JNIEnv pointer
 * 
 * \return \c true if the document is modified
 */
jboolean TextDocumentBridge::isModified(JNIEnv *env)
{
  Q_ASSERT(env);
  QSWING_REENTRANT_RETURN(isModified, jboolean, QSWING_ENV_ARG)
  return qTextDocument()->isModified();
}

/*!
 * \brief Sets whether or not the document is modified.
 *
 * \param env the JNIEnv pointer
 * \param modified \c true if modified
 */
void TextDocumentBridge::setModified(JNIEnv *env, jboolean modified)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setModified, QSWING_ENV_ARG, JBOOLEAN_ARG(modified))
  qTextDocument()->setModified(modified);
}

/*!
 * \brief Prints the document using the specified printer object.
 *
 * \param env the JNIEnv pointer
 * \param printer the QSwing printer object
 */
void TextDocumentBridge::print(JNIEnv *env, jobject printer)
{
  Q_ASSERT(env);
  Q_ASSERT(printer);
  QSWING_REENTRANT(print, QSWING_ENV_ARG, JOBJECT_ARG(printer))
  qTextDocument()->print(PrinterBridge::toQPrinter(env, printer));
}

/*!
 * \brief Sets whether or not the document is using the highlighter.
 *
 * \param env the JNIEnv pointer
 * \param useHighlighter \c true if using
 */
void TextDocumentBridge::setUseHighlighter(JNIEnv *env,
                                           jboolean useHighlighter)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setUseHighlighter, QSWING_ENV_ARG,
                   JBOOLEAN_ARG(useHighlighter))
  qTextDocument()->setUseHighlighter(useHighlighter);
}

/*!
 * \brief Sets the character format for the specified text segment.
 *
 * \param env the JNIEnv pointer
 * \param start the start of the segment
 * \param length the length of the segment
 * \param format the format
 */
void TextDocumentBridge::setFormat(JNIEnv *env, jint start, jint length,
                                   jobject format)
{
  Q_ASSERT(env);
  QSWING_REENTRANT(setFormat, QSWING_ENV_ARG, JINT_ARG(start), JINT_ARG(length),
                   JOBJECT_ARG(format))
  qTextDocument()->setFormat(start, length, QTextCharFormatAdapter(format));
}

/*^**************************************************************************^*/
/*- TextDocumentBridge :: Static Native Calls.                               -*/
/*^**************************************************************************^*/

/*!
 * \brief Initializes TextDocumentBridge class.
 *
 * \param env the JNIEnv pointer
 * \param cls the TextDocument class
 *
 * \exception FatalError if failed
 */
void TextDocumentBridge::initialize(JNIEnv *env, jclass cls)
{
  Q_ASSERT(env);
  Q_ASSERT(cls);
  Q_ASSERT (!sm_bridgeFID);
  if (!INIT_UFID(bridge, cls, "J") || !INIT_UMID(create, cls, "()J") ||
      !INIT_UMID(undoAvailable, cls, "(Z)V") || 
      !INIT_UMID(redoAvailable, cls, "(Z)V") || 
      !INIT_UMID(highlightBlock, cls, "(ILjava/lang/String;)V")) {
    FATAL_LOGGED_ERROR("TextDocumentBridge initialization failed");
  }
}

/*^**************************************************************************^*/
/*- TextDocumentBridge :: Native Callbacks.                                  -*/
/*^**************************************************************************^*/

/*!
 * \brief Calls \c undoAvailable method of the TextDocument object.
 *
 * \param available \c true if available
 */
void TextDocumentBridge::undoAvailable(jboolean available)
{
  jniObject()->callVoidMethod(sm_undoAvailableMID, available);
}

/*!
 * \brief Calls \c redoAvailable method of the TextDocument object.
 *
 * \param available \c true if available
 */
void TextDocumentBridge::redoAvailable(jboolean available)
{
  jniObject()->callVoidMethod(sm_redoAvailableMID, available);
}

/*!
 * \brief Calls \c highlightDocument method of the TextDocument object.
 *
 * \param blockStart the block start
 * \param blockText the block text
 */
void TextDocumentBridge::highlightBlock(jint blockStart, jstring blockText)
{
  jniObject()->callVoidMethod(sm_highlightBlockMID, blockStart, blockText);
}

/*^**************************************************************************^*/
/*- TextDocumentBridge :: Overridden Member Functions.                       -*/
/*^**************************************************************************^*/

/*!
 * \inheritDoc
 */
void TextDocumentBridge::destroy(JNIEnv *env)
{
  Q_UNUSED(env);
  qTextDocument()->setDestroying();
  if (QSwingBridge::isDisposed()) {
    delete this; // OK - referenced only from java
  } else {
    deleteLater();
  }
}

/*!
 * \brief Returns bridge pointer FID.
 *
 * \return bridge pointer FID 
 */
jfieldID TextDocumentBridge::bridgePointerFID() const
{
  return sm_bridgeFID;
}

/*^**************************************************************************^*/
/*- TextDocumentBridge :: Public Static Member Function.                     -*/
/*^**************************************************************************^*/

/*!
 * \brief Returns QTextDocumentAdapter object of the specified QSwing
 *        TextDocument object.
 *
 * \param env the JNIEnv pointer
 * \param document the QSwing TextDocumen object
 * 
 * \return QTextDocumentAdapter object of the specified TextDocument object
 */
QTextDocumentAdapter *TextDocumentBridge::toQDocument(JNIEnv *env,
                                                      jobject document)
{
  if (document) {
    return textDocumentCast(
        JNIWrapper::getLongField(env, document, sm_bridgeFID))->qTextDocument();
  }
  return 0;
}

/*^**************************************************************************^*/
/*- TextDocumentBridge :: Protected Member Functions.                        -*/
/*^**************************************************************************^*/

/*!
 * \brief \c undoAvailable slot.
 *
 * \param available \c true if available
 */
void TextDocumentBridge::undoAvailableSlot(bool available)
{
  undoAvailable(available);
}

/*!
 * \brief \c redAvailable slot.
 *
 * \param available \c true if available
 */
void TextDocumentBridge::redoAvailableSlot(bool available)
{
  redoAvailable(available);
}

/*^**************************************************************************^*/
/*- TextDocumentBridge :: Private Static Data Members.                       -*/
/*^**************************************************************************^*/

jfieldID TextDocumentBridge::sm_bridgeFID = 0;
jmethodID TextDocumentBridge::sm_createMID = 0;
jmethodID TextDocumentBridge::sm_undoAvailableMID = 0;
jmethodID TextDocumentBridge::sm_redoAvailableMID = 0;
jmethodID TextDocumentBridge::sm_highlightBlockMID = 0;

/*^*****************************************************************************
  File History:

 - 2007-12-11 21:49:59 Slobodan
   Initial version

*****************************************************************************^*/
