/*
 * @(#)ActiveTestSuite.java  0.1.0 / 31.01.2007
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the Common Public License Version 1.0.
 * A copy of this license is included in the file LICENSE-CPL_V10.txt.
 */
package junit.extensions;

import junit.framework.Test;
import junit.framework.TestResult;
import junit.framework.TestSuite;

/**
 * A TestSuite for active Tests. It runs each test in a separate thread and
 * waits until all threads have terminated.
 *
 * -- Aarhus Radisson Scandinavian Center 11th floor
 *
 * @author Erich Gamma
 * @author Kent Beck
 * @version 3.8.1 (original)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.1.0 (lite)
 */
public class ActiveTestSuite extends TestSuite {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private volatile int activeTestDeathCount;
  private final Object lock;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ActiveTestSuite object.
   */
  public ActiveTestSuite() {
    lock = new Object();
  }

  /**
   * Constructs new ActiveTestSuite object.
   *
   * @param cls the class
   */
  public ActiveTestSuite(Class cls) {
    super(cls);
    lock = new Object();
  }

  /**
   * Constructs new ActiveTestSuite object.
   *
   * @param name the name
   */
  public ActiveTestSuite(String name) {
    super(name);
    lock = new Object();
  }

  /**
   * Constructs new ActiveTestSuite object.
   *
   * @param cls the class
   * @param name the name
   */
  public ActiveTestSuite(Class cls, String name) {
    super(cls, name);
    lock = new Object();
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void run(TestResult result) {
    activeTestDeathCount = 0;
    super.run(result);
    waitUntilFinished();
  }

  /**
   * {@inheritDoc}
   */
  public void runTest(Test test, TestResult result) {
    new TestThread(this, test, result).start();
  }

/*^**************************************************************************^*/
/*- Package-Private Method.                                                  -*/
/*^**************************************************************************^*/

  void waitUntilFinished() {
    synchronized (lock) {
      try {
        while (activeTestDeathCount < testCount()) {
          lock.wait();
        }
      } catch (InterruptedException e) { // NOPMD
        // empty catch
      }
    }
  }

  void runFinished(Test test) {
    synchronized (lock) {
      activeTestDeathCount++;
      lock.notifyAll();
    }
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Test Thread.
   */
  private static class TestThread extends Thread {
    private final ActiveTestSuite ats;
    private final Test test;
    private final TestResult result;
    public TestThread(ActiveTestSuite ats, Test test, TestResult result) {
      super("TestThread"); //$NON-NLS-1$
      this.ats = ats;
      this.test = test;
      this.result = result;
    }
    public void run() {
      try {
        test.run(result);
      } finally {
        ats.runFinished(test);
      }
    }
  }
}

/*^*****************************************************************************
  File History:

 - 31.08.2002 Erich Gamma and Kent Beck
   Original 3.8.1 version

 - 31.01.2007 02:05:18 Slobodan
   Initial lite version

*****************************************************************************^*/
