/*
 * @(#)Assert.java  0.1.0 / 30.01.2007
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the Common Public License Version 1.0.
 * A copy of this license is included in the file LICENSE-CPL_V10.txt.
 */
package junit.framework; // NOPMD

/**
 * A set of assert methods. Messages are only displayed when an assert fails.
 *
 * @author Erich Gamma
 * @author Kent Beck
 * @version 3.8.1 (original)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.1.0 (lite)
 */
public class Assert {

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Protect constructor since it is a static only class.
   */
  protected Assert() {
    // empty implementation
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Asserts that a condition is true. If it isn't it throws an
   * AssertionFailedError with the given message.
   *
   * @param message the fail message
   * @param condition the condition
   */
  public static void assertTrue(String message, boolean condition) {
    if (!condition) {
      fail(message);
    }
  }

  /**
   * Asserts that a condition is true. If it isn't it throws an
   * AssertionFailedError.
   *
   * @param condition the condition
   */
  public static void assertTrue(boolean condition) {
    assertTrue(null, condition);
  }

  /**
   * Asserts that a condition is false. If it isn't it throws an
   * AssertionFailedError with the given message.
   *
   * @param message the fail message
   * @param condition the condition
   */
  public static void assertFalse(String message, boolean condition) {
    assertTrue(message, !condition); // NOPMD
  }

  /**
   * Asserts that a condition is false. If it isn't it throws an
   * AssertionFailedError with.
   *
   * @param condition the condition
   */
  public static void assertFalse(boolean condition) {
    assertFalse(null, condition);
  }

  /**
   * Fails a test with the given message.
   *
   * @param message the fail message
   */
  public static void fail(String message) {
    throw new AssertionFailedError(message);
  }

  /**
   * Fails a test with no message.
   */
  public static void fail() {
    fail(null);
  }

  /**
   * Asserts that two objects are equal. If they are not an
   * AssertionFailedError is thrown with the given message.
   *
   * @param message the fail message
   * @param expected the expected object
   * @param actual the actual object
   */
  public static void assertEquals(String message, Object expected,
                                  Object actual) {
    if ((expected == null) && (actual == null)) {
      return;
    }
    if ((expected != null) && expected.equals(actual)) {
      return;
    }
    failNotEquals(message, expected, actual);
  }

  /**
   * Asserts that two objects are equal. If they are not an
   * AssertionFailedError is thrown.
   *
   * @param expected the expected object
   * @param actual the actual object
   */
  public static void assertEquals(Object expected, Object actual) {
    assertEquals(null, expected, actual);
  }

  /**
   * Asserts that two Strings are equal.
   *
   * @param message the fail message
   * @param expected the expected string
   * @param actual the actual string
   */
  public static void assertEquals(String message, String expected,
                                  String actual) {
    if ((expected == null) && (actual == null)) {
      return;
    }
    if ((expected != null) && (expected.equals(actual))) {
      return;
    }
    throw new ComparisonFailure(message, expected, actual);
  }

  /**
   * Asserts that two Strings are equal.
   *
   * @param expected the expected string
   * @param actual the actual string
   */
  public static void assertEquals(String expected, String actual) {
    assertEquals(null, expected, actual);
  }

  /**
   * Asserts that two doubles are equal concerning a delta.  If they are not
   * an AssertionFailedError is thrown with the given message. If the expected
   * value is infinity then the delta value is ignored.
   *
   * @param message the fail message
   * @param expected the expected double
   * @param actual the actual double
   * @param delta the delta
   */
  public static void assertEquals(String message, double expected,
                                  double actual, double delta) {
    // handle infinity specially since subtracting to infinite values gives NaN
    // and the the following test fails
    if (Double.isInfinite(expected)) {
      if (!(expected == actual)) {
        failNotEquals(message, new Double(expected), new Double(actual));
      }
    } else if (!(Math.abs(expected - actual) <= delta)) {
      // Because comparison with NaN always returns false
      failNotEquals(message, new Double(expected), new Double(actual));
    }
  }

  /**
   * Asserts that two doubles are equal concerning a delta. If the expected
   * value is infinity then the delta value is ignored.
   *
   * @param expected the expected double
   * @param actual the actual double
   * @param delta the delta
   */
  public static void assertEquals(double expected, double actual,
                                  double delta) {
    assertEquals(null, expected, actual, delta);
  }

  /**
   * Asserts that two floats are equal concerning a delta. If they are not
   * an AssertionFailedError is thrown with the given message.  If the expected
   * value is infinity then the delta value is ignored.
   *
   * @param message the fail message
   * @param expected the expected float
   * @param actual the actual float
   * @param delta the delta
   */
  public static void assertEquals(String message, float expected, float actual,
                                  float delta) {
    // handle infinity specially since subtracting to infinite values gives NaN
    // and the the following test fails
    if (Float.isInfinite(expected)) {
      if (!(expected == actual)) {
        failNotEquals(message, new Float(expected), new Float(actual));
      }
    } else if (!(Math.abs(expected - actual) <= delta)) {
      failNotEquals(message, new Float(expected), new Float(actual));
    }
  }

  /**
   * Asserts that two floats are equal concerning a delta. If the expected
   * value is infinity then the delta value is ignored.
   *
   * @param expected the expected float
   * @param actual the actual float
   * @param delta the delta
   */
  public static void assertEquals(float expected, float actual, float delta) {
    assertEquals(null, expected, actual, delta);
  }

  /**
   * Asserts that two longs are equal. If they are not an AssertionFailedError
   * is thrown with the given message.
   *
   * @param message the fail message
   * @param expected the expected long
   * @param actual the actual long
   */
  public static void assertEquals(String message, long expected, long actual) {
    assertEquals(message, new Long(expected), new Long(actual));
  }

  /**
   * Asserts that two longs are equal.
   *
   * @param expected the expected long
   * @param actual the actual long
   */
  public static void assertEquals(long expected, long actual) {
    assertEquals(null, expected, actual);
  }

  /**
   * Asserts that two booleans are equal. If they are not
   * an AssertionFailedError is thrown with the given message.
   *
   * @param message the fail message
   * @param expected the expected boolean
   * @param actual the actual boolean
   */
  public static void assertEquals(String message, boolean expected,
                                  boolean actual) {
    assertEquals(message, new Boolean(expected), new Boolean(actual)); // NOPMD
  }

  /**
   * Asserts that two booleans are equal.
   *
   * @param expected the expected boolean
   * @param actual the actual boolean
   */
  public static void assertEquals(boolean expected, boolean actual) {
    assertEquals(null, expected, actual);
  }

  /**
   * Asserts that two bytes are equal. If they are not an AssertionFailedError
   * is thrown with the given message.
   *
   * @param message the fail message
   * @param expected the expected byte
   * @param actual the actual byte
   */
  public static void assertEquals(String message, byte expected, byte actual) {
    assertEquals(message, new Byte(expected), new Byte(actual));
  }

  /**
   * Asserts that two bytes are equal.
   *
   * @param expected the expected byte
   * @param actual the actual byte
   */
  public static void assertEquals(byte expected, byte actual) {
    assertEquals(null, expected, actual);
  }

  /**
   * Asserts that two chars are equal. If they are not an AssertionFailedError
   * is thrown with the given message.
   *
   * @param message the fail message
   * @param expected the expected char
   * @param actual the actual char
   */
  public static void assertEquals(String message, char expected, char actual) {
    assertEquals(message, new Character(expected), new Character(actual));
  }

  /**
   * Asserts that two chars are equal.
   *
   * @param expected the expected char
   * @param actual the actual char
   */
  public static void assertEquals(char expected, char actual) {
    assertEquals(null, expected, actual);
  }

  /**
   * Asserts that two shorts are equal. If they are not an AssertionFailedError
   * is thrown with the given message.
   *
   * @param message the fail message
   * @param expected the expected short
   * @param actual the actual short
   */
  public static void assertEquals(String message, short expected,
                                  short actual) {
    assertEquals(message, new Short(expected), new Short(actual));
  }

  /**
   * Asserts that two shorts are equal.
   *
   * @param expected the expected short
   * @param actual the actual short
   */
  public static void assertEquals(short expected, short actual) {
    assertEquals(null, expected, actual);
  }

  /**
   * Asserts that two ints are equal. If they are not an AssertionFailedError is
   * thrown with the given message.
   *
   * @param message the fail message
   * @param expected the expected int
   * @param actual the actual int
   */
  public static void assertEquals(String message, int expected, int actual) {
    assertEquals(message, new Integer(expected), new Integer(actual));
  }

  /**
   * Asserts that two ints are equal.
   *
   * @param expected the expected char
   * @param actual the actual char
   */
  public static void assertEquals(int expected, int actual) {
    assertEquals(null, expected, actual);
  }

  /**
   * Asserts that an object isn't null. If it is an AssertionFailedError is
   * thrown with the given message.
   *
   * @param message the fail message
   * @param object the object
   */
  public static void assertNotNull(String message, Object object) {
    assertTrue(message, object != null); // NOPMD
  }

  /**
   * Asserts that an object isn't null.
   *
   * @param object the object
   */
  public static void assertNotNull(Object object) {
    assertNotNull(null, object);
  }

  /**
   * Asserts that an object is null.  If it is not an AssertionFailedError is
   * thrown with the given message.
   *
   * @param message the fail message
   * @param object the object
   */
  public static void assertNull(String message, Object object) {
    assertTrue(message, object == null); // NOPMD
  }

  /**
   * Asserts that an object is null.
   *
   * @param object the object
   */
  public static void assertNull(Object object) {
    assertNull(null, object);
  }

  /**
   * Asserts that two objects refer to the same object. If they are not an
   * AssertionFailedError is thrown with the given message.
   *
   * @param message the fail message
   * @param expected the expected object
   * @param actual the actual object
   */
  public static void assertSame(String message, Object expected,
                                Object actual) {
    if (expected != actual) { // NOPMD
      failNotSame(message, expected, actual);
    }
  }

  /**
   * Asserts that two objects refer to the same object. If they are not the same
   * an AssertionFailedError is thrown.
   *
   * @param expected the expected object
   * @param actual the actual object
   */
  public static void assertSame(Object expected, Object actual) {
    assertSame(null, expected, actual);
  }

  /**
   * Asserts that two objects refer to the same object. If they are not an
   * AssertionFailedError is thrown with the given message.
   *
   * @param message the fail message
   * @param expected the expected object
   * @param actual the actual object
   */
  public static void assertNotSame(String message, Object expected,
                                   Object actual) {
    if (expected == actual) { // NOPMD
      failSame(message);
    }
  }

  /**
   * Asserts that two objects refer to the same object. If they are not the same
   * an AssertionFailedError is thrown.
   *
   * @param expected the expected object
   * @param actual the actual object
   */
  public static void assertNotSame(Object expected, Object actual) {
    assertNotSame(null, expected, actual);
  }

/*^**************************************************************************^*/
/*- Pacakge-Private Static Method.                                           -*/
/*^**************************************************************************^*/

  static String format(String message, Object expected, Object actual) {
    if (message != null) {
      return message + " expected:<" + expected + //$NON-NLS-1$
             "> but was:<" + actual + ">"; //$NON-NLS-1$ $NON-NLS-2$
    }
    return "expected:<" + expected + "> but was:<" + //$NON-NLS-1$ $NON-NLS-2$
            actual + ">"; //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Private Static Methods.                                                -  */
/*^**************************************************************************^*/

  private static void failSame(String message) {
    if (message == null) {
      fail("expected not same"); //$NON-NLS-1$
    } else {
      fail(message + " expected not same"); //$NON-NLS-1$
    }
  }

  private static void failNotSame(String message, Object expected,
                                  Object actual) {
    if (message == null) {
      fail("expected same:<" + expected + //$NON-NLS-1$
           "> was not:<" + actual + ">"); //$NON-NLS-1$ $NON-NLS-2$
    } else {
      fail(message + " expected same:<" + expected + //$NON-NLS-1$
           "> was not:<" + actual + ">"); //$NON-NLS-1$ $NON-NLS-2$
    }
  }

  private static void failNotEquals(String message, Object expected,
                                    Object actual) {
    fail(format(message, expected, actual));
  }

}

/*^*****************************************************************************
  File History:

 - 31.08.2002 Erich Gamma and Kent Beck
   Original 3.8.1 version

 - 30.01.2007 18:22:33 Slobodan
   Initial lite version

*****************************************************************************^*/
