/*
 * @(#)TestCase.java  0.1.0 / 30.01.2007
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the Common Public License Version 1.0.
 * A copy of this license is included in the file LICENSE-CPL_V10.txt.
 */
package junit.framework;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

/**
 * A test case defines the fixture to run multiple tests. To define a test case
 * <ol>
 *   <li>implement a subclass of TestCase</li>
 *   <li>define instance variables that store the state of the fixture</li>
 *   <li>initialize the fixture state by overriding <code>setUp</code></li>
 *   <li>clean-up after a test by overriding <code>tearDown</code>.</li>
 * </ol>
 *
 * Each test runs in its own fixture so there can be no side effects among test
 * runs. Here is an example:
 * <pre>
 * public class MathTest extends TestCase {
 *
 *   protected double value1;
 *   protected double value2;
 *
 *   protected void setUp() {
 *      value1 = 2.0;
 *      value2 = 3.0;
 *   }
 * }
 * </pre>
 *
 * For each test implement a method which interacts with the fixture. Verify the
 * expected results with assertions specified by calling <code>assertTrue</code>
 * with a boolean.
 * <pre>
 *    public void testAdd() {
 *      double result = value1 + value2;
 *      assertTrue(result == 5.0);
 *    }
 * </pre>
 *
 * Once the methods are defined you can run them. The framework supports both a
 * static type safe and more dynamic way to run a test. In the static way you
 * override the runTest method and define the method to be invoked. A convenient
 * way to do so is with an anonymous inner class.
 * <pre>
 * TestCase test = new MathTest("add") {
 *     public void runTest() {
 *       testAdd();
 *     }
 * };
 * test.run();
 * </pre>
 *
 * The dynamic way uses reflection to implement <code>runTest</code>. It
 * dynamically finds and invokes a method. In this case the name of the test
 * case has to correspond to the test method to be run.
 * <pre>
 * TestCase = new MathTest("testAdd");
 * test.run();
 * </pre>
 *
 * The tests to be run can be collected into a TestSuite. JUnit provides
 * different <i>test runners</i> which can run a test suite and collect the
 * results. A test runner either expects a static method <code>suite</code> as
 * the entry point to get a test to run or it will extract the suite
 * automatically.
 * <pre>
 * public static Test suite() {
 *   suite.addTest(new MathTest("testAdd"));
 *   suite.addTest(new MathTest("testDivideByZero"));
 *   return suite;
 * }
 * </pre>
 *
 * @author Erich Gamma
 * @author Kent Beck
 * @version 3.8.1 (original)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.1.0 (lite)
 */
public abstract class TestCase extends Assert implements Test { // NOPMD

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private String name;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new TestCase object.
   */
  public TestCase() {
    // empty implementation
  }

  /**
   * Constructs new TestCase object with the given name.
   *
   * @param name the test name
   */
  public TestCase(String name) {
    this.name = name;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the name of the test case.
   *
   * @return the name of the test case
   */
  public String getName() {
    return name;
  }

  /**
   * Sets the name the test case.
   *
   * @param name the name to set
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * Counts the number of test cases executed by run(TestResult result).
   *
   * @return the number of test cases executed by run(TestResult result)
   */
  public int countTestCases() {
    return 1;
  }

  /**
   * A convenience method to run this test, collecting the results with a
   * default TestResult object.
   *
   * @return test result
   */
  public TestResult run() {
    TestResult result = createResult();
    run(result);
    return result;
  }

  /**
   * Runs the bare test sequence.
   *
   * @exception Throwable if any exception is thrown
   */
  public void runBare() throws Throwable {
    setUp();
    try {
      runTest();
    } finally {
      tearDown();
    }
  }

  /**
   * Override to run the test and assert its state.
   *
   * @exception Throwable if any exception is thrown
   */
  protected void runTest() throws Throwable {
    assertNotNull(name);
    Method runMethod = null;
    try {
      // use getMethod to get all public inherited methods. getDeclaredMethods
      // returns all methods of this class but excludes the inherited ones.
      runMethod = getClass().getMethod(name, null);
    } catch (NoSuchMethodException e) {
      fail("Method \"" + name + "\" not found"); //$NON-NLS-1$ $NON-NLS-1$
    }
    if (!Modifier.isPublic(runMethod.getModifiers())) {
      fail("Method \"" + name + //$NON-NLS-1$
           "\" should be public"); //$NON-NLS-1$
    }
    try {
      runMethod.invoke(this, new Class[0]);
    } catch (InvocationTargetException e) {
      e.fillInStackTrace();
      throw e.getTargetException();
    } catch (IllegalAccessException e) {
      e.fillInStackTrace();
      throw e;
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Creates a default TestResult object.
   *
   * @return a default TestResult object
   */
  protected TestResult createResult() {
    return new TestResult();
  }

  /**
   * Sets up the fixture, for example, open a network connection. This method is
   * called before a test is executed.
   *
   * @throws Exception if any exception is thrown
   */
  protected void setUp() throws Exception { // NOPMD
  }

  /**
   * Tears down the fixture, for example, close a network connection. This
   * method is called after a test is executed.
   *
   * @throws Exception if any exception is thrown
   */
  protected void tearDown() throws Exception { // NOPMD
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Returns a string representation of the test case.
   *
   * @return a string representation of the test case
   */
  public String toString() {
    return getName() + "(" + getClass().getName() + //$NON-NLS-1$
           ")"; //$NON-NLS-1$
  }

  /**
   * Runs the test case and collects the results in TestResult.
   *
   * @param result the test result collector
   */
  public void run(TestResult result) {
    result.run(this);
  }

}

/*^*****************************************************************************
  File History:

 - 31.08.2002 Erich Gamma and Kent Beck
   Original 3.8.1 version

 - 30.01.2007 22:12:00 Slobodan
   Initial lite version

*****************************************************************************^*/
