/*
 * @(#)TestFailure.java  0.1.0 / 30.01.2007
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the Common Public License Version 1.0.
 * A copy of this license is included in the file LICENSE-CPL_V10.txt.
 */
package junit.framework;



/**
 * A <code>TestFailure</code> collects a failed test together with the caught
 * exception.
 *
 * @author Erich Gamma
 * @author Kent Beck
 * @version 3.8.1 (original)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.1.0 (lite)
 */
public class TestFailure {

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  protected Test fFailedTest;
  protected Throwable fThrownException;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs a TestFailure with the given test and exception.
   *
   * @param failedTest the failed test
   * @param thrownException the thrown exception
   *
   * @throws IllegalArgumentException if failedTest or thrownException is
   *         <tt>null</tt>
   */
  public TestFailure(Test failedTest, Throwable thrownException) {
    if (failedTest == null) {
      throw new IllegalArgumentException("Null failed test"); //$NON-NLS-1$
    }
    if (thrownException == null) {
      throw new IllegalArgumentException("Null thrown exception"); //$NON-NLS-1$
    }
    fFailedTest = failedTest;
    fThrownException = thrownException;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the failed test.
   *
   * @return the failed test
   */
  public Test failedTest() {
    return fFailedTest;
  }

  /**
   * Returns the thrown exception.
   *
   * @return the thrown exception
   */
  public Throwable thrownException() {
    return fThrownException;
  }

  /**
   * Returns <tt>true</tt> if the thrown exception if instance of the
   * AssertionFailedError class.
   *
   * @return <tt>true</tt> if the thrown exception is failure
   */
  public boolean isFailure() {
    return (thrownException() instanceof AssertionFailedError);
  }

  /**
   * Returns the detail message of the thrown exception.
   *
   * @return the detail message of the thrown exception
   */
  public String exceptionMessage() {
    return thrownException().getMessage();
  }

  /**
   * Returns exception's stack trace.
   *
   * @return exception's stack trace
   */
  public String trace() {
    return TestSuite.exceptionToString(thrownException());
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns a string representation of the test failure.
   *
   * @return a string representation of the test failure
   */
  public String toString() {
    return fFailedTest + ": " + thrownException().getMessage(); //$NON-NLS-1$
  }

}

/*^*****************************************************************************
  File History:

 - 31.08.2002 Erich Gamma and Kent Beck
   Original 3.8.1 version

 - 30.01.2007 21:42:34 Slobodan
   Initial lite version

*****************************************************************************^*/
