/*
 * @(#)TestResult.java  0.1.0 / 04.02.2007
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the Common Public License Version 1.0.
 * A copy of this license is included in the file LICENSE-CPL_V10.txt.
 */
package junit.framework;

import java.util.Enumeration;
import java.util.Vector;

/**
 * A <code>TestResult</code> collects the results of executing a test case. It
 * is an instance of the Collecting Parameter pattern. The test framework
 * distinguishes between <i>failures</i> and <i>errors</i>. A failure is
 * anticipated and checked for with assertions. Errors are unanticipated
 * problems like an <code>ArrayIndexOutOfBoundsException</code>.
 *
 * @author Erich Gamma
 * @author Kent Beck
 * @version 3.8.1 (original)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.1.0 (lite)
 */
public class TestResult {

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  protected Vector fFailures; // NOPMD
  protected Vector fErrors; // NOPMD
  protected Vector fListeners; // NOPMD
  protected int fRunTests; // NOPMD

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean stop;
  private final Object lock;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new TestResult object.
   */
  public TestResult() {
    lock = new Object();
    fFailures = new Vector(); // NOPMD
    fErrors = new Vector(); // NOPMD
    fListeners = new Vector(); // NOPMD
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Adds an error to the list of errors. The passed in exception caused the
   * error.
   *
   * @param test the test
   * @param t the throwable
   */
  public void addError(Test test, Throwable t) {
    synchronized (lock) {
      fErrors.addElement(new TestFailure(test, t));
      for (Enumeration e = cloneListeners().elements(); e.hasMoreElements();) {
        ((TestListener)e.nextElement()).addError(test, t);
      }
    }
  }

  /**
   * Adds a failure to the list of failures. The passed in exception caused the
   * failure.
   *
   * @param test the test
   * @param t the assertion failed error
   */
  public void addFailure(Test test, AssertionFailedError t) {
    synchronized (lock) {
      fFailures.addElement(new TestFailure(test, t));
      for (Enumeration e = cloneListeners().elements(); e.hasMoreElements();) {
        ((TestListener)e.nextElement()).addFailure(test, t);
      }
    }
  }

  /**
   * Registers a TestListener.
   *
   * @param listener the TestListener
   */
  public void addListener(TestListener listener) {
    synchronized (lock) {
      fListeners.addElement(listener);
    }
  }

  /**
   * Unregisters a TestListener.
   *
   * @param listener the TestListener
   */
  public void removeListener(TestListener listener) {
    synchronized (lock) {
      fListeners.removeElement(listener);
    }
  }

  /**
   * Informs the result that a test was completed.
   *
   * @param test the test
   */
  public void endTest(Test test) {
    for (Enumeration e = cloneListeners().elements(); e.hasMoreElements();) {
      ((TestListener)e.nextElement()).endTest(test);
    }
  }

  /**
   * Returns the number of detected errors.
   *
   * @return the number of detected errors
   */
  public int errorCount() {
    synchronized (lock) {
      return fErrors.size();
    }
  }

  /**
   * Returns an Enumeration for the errors.
   *
   * @return an Enumeration for the errors
   */
  public Enumeration errors() {
    synchronized (lock) {
      return fErrors.elements();
    }
  }

  /**
   * Returns the number of detected failures.
   *
   * @return the number of detected failures
   */
  public int failureCount() {
    synchronized (lock) {
      return fFailures.size();
    }
  }

  /**
   * Returns an Enumeration for the failures.
   *
   * @return an Enumeration for the failures
   */
  public Enumeration failures() {
    synchronized (lock) {
      return fFailures.elements();
    }
  }

  /**
   * Returns the number of run tests.
   *
   * @return the number of run tests
   */
  public int runCount() {
    synchronized (lock) {
      return fRunTests;
    }
  }

  /**
   * Runs a TestCase.
   *
   * @param test the test
   * @param p the protectable
   */
  public void runProtected(Test test, Protectable p) {
    try {
      p.protect();
    } catch (AssertionFailedError e) {
      addFailure(test, e);
    } catch (ThreadDeath e) { // don't catch ThreadDeath by accident
      throw e;
    } catch (Throwable e) {
      addError(test, e);
    }
  }

  /**
   * Checks whether the test run should stop.
   *
   * @return whether the test run should stop
   */
  public boolean shouldStop() {
    synchronized (lock) {
      return stop;
    }
  }

  /**
   * Informs the result that a test will be started.
   *
   * @param test the test
   */
  public void startTest(Test test) {
    final int count = test.countTestCases();
    synchronized (lock) {
      fRunTests += count;
    }
    for (Enumeration e = cloneListeners().elements(); e.hasMoreElements();) {
      ((TestListener)e.nextElement()).startTest(test);
    }
  }

  /**
   * Marks that the test run should stop.
   */
  public void stop() {
    synchronized (lock) {
      stop = true;
    }
  }

  /**
   * Returns whether the entire test was successful or not.
   *
   * @return whether the entire test was successful or not
   */
  public boolean wasSuccessful() {
    synchronized (lock) {
      return ((failureCount() == 0) && (errorCount() == 0));
    }
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Runs a TestCase.
   *
   * @param test the test case
   */
  protected void run(TestCase test) {
    startTest(test);
    runProtected(test, new TestResultProtectable(test));
    endTest(test);
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns a copy of the listeners.
   *
   * @return a copy of the listeners
   */
  private Vector cloneListeners() { // NOPMD
    synchronized (lock) {
      return (Vector)fListeners.clone(); // NOPMD
    }
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Protectable TestSetup.
   */
  private static class TestResultProtectable implements Protectable {
    private final TestCase testCase;
    public TestResultProtectable(TestCase testCase) {
      this.testCase = testCase;
    }
    public void protect() throws Throwable  {
      testCase.runBare();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 31.08.2002 Erich Gamma and Kent Beck
   Original 3.8.1 version

 - 04.02.2007 17:40:08 Slobodan
   Initial lite version

*****************************************************************************^*/
