/*
 * @(#)TestSuite.java  0.1.0 / 31.01.2007
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the Common Public License Version 1.0.
 * A copy of this license is included in the file LICENSE-CPL_V10.txt.
 */
package junit.framework;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Vector;

/**
 * A <code>TestSuite</code> is a <code>Composite</code> of Tests. It runs a
 * collection of test cases. Here is an example using the dynamic test
 * definition.
 * <pre>
 * TestSuite suite= new TestSuite();
 * suite.addTest(new MathTest("testAdd"));
 * suite.addTest(new MathTest("testDivideByZero"));
 * </pre>
 *
 * Alternatively, a TestSuite can extract the tests to be run automatically.
 * To do so you pass the class of your TestCase class to the TestSuite
 * constructor.
 * <pre>
 * TestSuite suite= new TestSuite(MathTest.class);
 * </pre>
 *
 * This constructor creates a suite with all the methods starting with "test"
 * that take no arguments.
 *
 * @author Erich Gamma
 * @author Kent Beck
 * @version 3.8.1 (original)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.1.0 (lite)
 */
public class TestSuite implements Test {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final Class[] ONE_STRING_CLASS_ARRAY = {String.class};
  private static final Class[] ZERO_CLASS_ARRAY = {};
  private static final Object[] ZERO_OBJECT_ARRAY = {};

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Vector tests; // NOPMD
  private String name;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs an empty TestSuite.
   */
  public TestSuite() {
    tests = new Vector(); // NOPMD
  }

  /**
   * Constructs a TestSuite from the given class. Adds all the methods
   * starting with "test" as test cases to the suite.
   *
   * @param cls the class
   */
  public TestSuite(Class cls) {
    this();
    name = cls.getName();
    try {
      getTestConstructor(cls);
      if (Modifier.isPublic(cls.getModifiers())) {
        Class superClass = cls;
        ArrayList names = new ArrayList();
        while (Test.class.isAssignableFrom(superClass)) {
          Method[] methods = superClass.getDeclaredMethods();
          for (int i = 0; i < methods.length; i++) {
            addTestMethod(methods[i], names, cls); // NOPMD
          }
          superClass = superClass.getSuperclass();
        }
        if (tests.size() == 0) {
          addWarningTest("No tests found in " + //$NON-NLS-1$ NOPMD
                         cls.getName());
        }
      } else {
        addWarningTest("Class " + cls.getName() + //$NON-NLS-1$ NOPMD
                       " is not public"); //$NON-NLS-1$
      }
    } catch (NoSuchMethodException e) {
      addWarningTest("Class " + cls.getName() + //$NON-NLS-1$ NOPMD
                     " has no public constructor " +  //$NON-NLS-1$
                     "TestCase(String name) or TestCase()"); //$NON-NLS-1$
    }
  }

  /**
   * Constructs a TestSuite from the given class with the given name.
   *
   * @param cls the class
   * @param name the name
   */
  public TestSuite(Class cls, String name) {
    this(cls);
    setName(name); // NOPMD
  }

  /**
   * Constructs an empty TestSuite.
   *
   * @param name the name
   */
  public TestSuite(String name) {
    this();
    setName(name); // NOPMD
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Creates test for the specified class.
   *
   * @param c the class
   * @param name the test method name
   *
   * @return the test
   */
  public static Test createTest(Class cls, String name) {
    try {
      Constructor constructor = getTestConstructor(cls);
      Object test;
      if (constructor.getParameterTypes().length == 0) {
        test = constructor.newInstance(ZERO_OBJECT_ARRAY);
        if (test instanceof TestCase) {
          ((TestCase)test).setName(name);
        }
      } else {
        test = constructor.newInstance(new Object[] {name});
      }
      return (Test)test;
    } catch (NoSuchMethodException e) {
      return warning("Class " + cls.getName() + //$NON-NLS-1$
                     " has no public constructor " + //$NON-NLS-1$
                     "TestCase(String name) or TestCase()"); //$NON-NLS-1$
    } catch (InstantiationException e) {
      return warning("Cannot instantiate test case: " + name + //$NON-NLS-1$
                     " (" + exceptionToString(e) + ")"); //$NON-NLS-1$
    } catch (InvocationTargetException e) {
      return warning("Exception in constructor: " + name + //$NON-NLS-1$
                     " (" + //$NON-NLS-1$
                     exceptionToString(e.getTargetException()) +
                     ")"); //$NON-NLS-1$
    } catch (IllegalAccessException e) {
      return warning("Cannot access test case: " + name +  //$NON-NLS-1$
                     " (" + exceptionToString(e) + //$NON-NLS-1$
                     ")"); //$NON-NLS-1$
    }
  }

  /**
   * Returns the constructor which takes a single String as its argument or a
   * no arg constructor.
   *
   * @param cls the class
   *
   * @return the constructor
   *
   * @throws NoSuchMethodException if there is no such constructor
   */
  public static Constructor getTestConstructor(Class cls) throws
                                                        NoSuchMethodException {
    try {
      return cls.getConstructor(ONE_STRING_CLASS_ARRAY);
    } catch (NoSuchMethodException e) {
      return cls.getConstructor(ZERO_CLASS_ARRAY);
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the name of the suite. Not all test suites have a name and this
   * method can return null.
   *
   * @return the name of the suite
   */
  public String getName() {
    return name;
  }

  /**
   * Sets the name of the suite.
   *
   * @param name the name of the suite
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * Returns the number of tests in this suite.
   *
   * @return the number of tests in this suite
   */
  public int testCount() {
    return tests.size();
  }

  /**
   * Returns the test at the given index.
   *
   * @param index the test index
   *
   * @return the test at the given index
   */
  public Test testAt(int index) {
    return (Test)tests.get(index);
  }

  /**
   * Adds a test to the suite.
   *
   * @param test the test to be added
   */
  public void addTest(Test test) {
    tests.add(test);
  }

  /**
   * Adds the tests from the given class to the suite.
   *
   * @param cls the class of the test suite to be added
   */
  public void addTestSuite(Class cls) {
    addTest(new TestSuite(cls));
  }

  /**
   * Returns the tests as an enumeration.
   *
   * @return the tests as an enumeration
   */
  public Enumeration tests() {
    return tests.elements();
  }

  /**
   * Runs the specified test.
   *
   * @param test the test
   * @param result the test result
   */
  public void runTest(Test test, TestResult result) {
    test.run(result);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String toString() {
    String name = getName();
    if (name != null) {
      return name;
    }
    return super.toString();
  }

  /**
   * Counts the number of test cases that will be run by this test.
   *
   * @return the number of test cases that will be run by this test
   */
  public int countTestCases() {
    int count = 0;
    for (Enumeration e = tests(); e.hasMoreElements();) {
      Test test = (Test)e.nextElement();
      count = count + test.countTestCases();
    }
    return count;
  }

  /**
   * Runs the tests and collects their result in a TestResult.
   *
   * @param result the test result
   */
  public void run(TestResult result) {
    for (Enumeration e = tests(); e.hasMoreElements();) {
      if (result.shouldStop()) {
        break;
      }
      Test test = (Test)e.nextElement();
      runTest(test, result);
    }
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void addTestMethod(Method m, ArrayList names, Class cls) {
    String methodName = m.getName();
    if (!names.contains(methodName) && isTestMethod(m)) {
      if (Modifier.isPublic(m.getModifiers())) { // is public method
        names.add(methodName);
        addTest(createTest(cls, methodName));
      } else {
        addWarningTest("Test method isn't public: " + //$NON-NLS-1$
                        methodName);
      }
    }
  }

  private void addWarningTest(String message) {
    addTest(new WarningTestCase(message));
  }

/*^**************************************************************************^*/
/*- Package-Private Static Method.                                           -*/
/*^**************************************************************************^*/

  /**
   * Converts the stack trace into a string.
   *
   * @param t the throwable
   *
   * @return string representation of the stack trace
   */
  static String exceptionToString(Throwable t) {
    StringWriter stringWriter = new StringWriter();
    PrintWriter printWriter = new PrintWriter(stringWriter);
    t.printStackTrace(printWriter);
    return stringWriter.getBuffer().toString();
  }

/*^**************************************************************************^*/
/*- Private Staic Methods.                                                   -*/
/*^**************************************************************************^*/

  private static boolean isTestMethod(Method method) {
    String methodName = method.getName();
    Class[] parameters = method.getParameterTypes();
    Class returnType = method.getReturnType();
    return ((parameters.length == 0) &&
            methodName.startsWith("test") && //$NON-NLS-1$
            returnType.equals(Void.TYPE));
  }

  private static Test warning(String message) {
    return new WarningTestCase(message);
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Warning TestCase.
   */
  private static class WarningTestCase extends TestCase {
    private final String message;
    public WarningTestCase(String message) {
      super("warning"); //$NON-NLS-1$
      this.message = message;
    }
    protected void runTest() {
      fail(message);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 31.08.2002 Erich Gamma and Kent Beck
   Original 3.8.1 version

 - 31.01.2007 00:47:35 Slobodan
   Initial lite version

*****************************************************************************^*/
