/*
 * @(#)ResultPrinter.java  0.1.0 / 14.02.2007
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the Common Public License Version 1.0.
 * A copy of this license is included in the file LICENSE-CPL_V10.txt.
 */
package junit.textui;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintStream;
import java.io.StringReader;
import java.text.NumberFormat;
import java.util.Enumeration;

import junit.framework.AssertionFailedError;
import junit.framework.Test;
import junit.framework.TestFailure;
import junit.framework.TestListener;
import junit.framework.TestResult;
import junit.framework.TestSuite;

/**
 * Test result printer.
 *
 * @author Erich Gamma
 * @author Kent Beck
 * @version 3.8.1 (original)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.1.0 (lite)
 */
final class ResultPrinter implements TestListener {

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final String[] FILTERED_METHODS;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    FILTERED_METHODS = new String[] {
        "junit.framework.TestCase", //$NON-NLS-1$
        "junit.framework.TestResult", //$NON-NLS-1$
        "junit.framework.TestSuite", //$NON-NLS-1$
        "junit.framework.Assert.", //$NON-NLS-1$
        "junit.textui.TestRunner", //$NON-NLS-1$
        "java.lang.reflect.Method.invoke(" //$NON-NLS-1$
    };
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final PrintStream writer;
  private int passedCount;
  private volatile boolean passed;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  public ResultPrinter() {
    writer = System.out;
  }

/*^**************************************************************************^*/
/*- Pacakge-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  void beginTestSuite(TestSuite testSuite) {
    writer.print("Test: "); //$NON-NLS-1$
    writer.println(testSuite.getName());
    writer.print("Runs: "); //$NON-NLS-1$
    writer.println(testSuite.countTestCases());
    writer.println();
  }

  void endTest(TestResult result, long runTime) {
    writer.println();
    writer.print("Time:   "); //$NON-NLS-1$
    writer.print(NumberFormat.getInstance().format((double)runTime / 1000));
    writer.println(" s"); //$NON-NLS-1$
    writer.print("Passed: "); //$NON-NLS-1$
    writer.println(passedCount);
    writer.print("Error:  "); //$NON-NLS-1$
    int errorCount = result.errorCount();
    writer.println(errorCount);
    writer.print("Failed: "); //$NON-NLS-1$
    int failureCount = result.failureCount();
    writer.println(failureCount);
    writer.println();
    if (errorCount > 0) {
      printDefects(result.errors(), errorCount, "error"); //$NON-NLS-1$
    }
    if (failureCount > 0) {
      printDefects(result.failures(), failureCount, "failure"); //$NON-NLS-1$
    }
    writer.print("Status: "); //$NON-NLS-1$
    if (result.wasSuccessful()) {
      writer.println("OK"); //$NON-NLS-1$
    } else {
      writer.println("ERRORS !!!"); //$NON-NLS-1$
    }
  }

/*^**************************************************************************^*/
/*- TestListener Interface Implementation.                                   -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void startTest(Test test) {
    passed = true;
    writer.print(test);
  }

  /**
   * {@inheritDoc}
   */
  public void endTest(Test test) {
    if (passed) {
      writer.println(" PASSED");
      passedCount++;
    }
  }

  /**
   * {@inheritDoc}
   */
  public void addError(Test test, Throwable t) {
    writer.println(" ERROR");
    passed = false;
  }

  /**
   * {@inheritDoc}
   */
  public void addFailure(Test test, AssertionFailedError t) {
    writer.println(" FAILED");
    passed = false;
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void printDefects(Enumeration defects, int count, String type) {
    if (count == 1) {
      writer.print("There was "); //$NON-NLS-1$
      writer.print(count);
      writer.print(" "); //$NON-NLS-1$
      writer.print(type);
      writer.println(":"); //$NON-NLS-1$
    } else {
      writer.print("There were "); //$NON-NLS-1$
      writer.print(count);
      writer.print(" "); //$NON-NLS-1$
      writer.print(type);
      writer.println("s:"); //$NON-NLS-1$
    }
    for (int i = 1; defects.hasMoreElements(); i++) {
      printDefect((TestFailure)defects.nextElement(), i);
    }
    writer.println();
  }

  private void printDefect(TestFailure defect, int count) {
    writer.print(count);
    writer.print(") "); //$NON-NLS-1$
    writer.println(defect.failedTest());
    BufferedReader br = new BufferedReader(new StringReader(defect.trace()));
    try {
      for (String line = br.readLine(); line != null; line = br.readLine()) {
        if (!isPrintableLine(line)) {
          writer.print('\t');
          writer.println(line);
        }
      }
    } catch (IOException ioe) { // NOPMD
      // empty catch
    }
  }

  private boolean isPrintableLine(String line) {
    String[] filteredMethods = FILTERED_METHODS;
    for (int i = filteredMethods.length - 1; i >= 0; i--) {
      if (line.indexOf(filteredMethods[i]) > 0) {
        return true;
      }
    }
    return false;
  }

}
