/*
 * @(#)Utils.java  0.2.0 / 2007-02-28
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.tools.ng;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.channels.FileChannel;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import com.thoughtworks.qdox.model.AbstractJavaEntity;
import com.thoughtworks.qdox.model.DocletTag;
import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaField;
import com.thoughtworks.qdox.model.JavaMethod;
import com.thoughtworks.qdox.model.JavaParameter;
import com.thoughtworks.qdox.model.Type;

/**
 * Helper Class.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
public final class Utils {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final SimpleDateFormat DATE_TIME_FORMAT;
  private static final SimpleDateFormat DATE_FORMAT;
  private static final Type TYPE_CLASS;
  private static final Type TYPE_THROWABLE;
  private static final Type TYPE_STRING;


/*^**************************************************************************^*/
/*- Static Initialzation.                                                    -*/
/*^**************************************************************************^*/

  static {
    DATE_TIME_FORMAT = new SimpleDateFormat(
                                          "yyyy-MM-dd HH:mm:ss"); //$NON-NLS-1$
    DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd"); //$NON-NLS-1$
    TYPE_CLASS = new Type("java.lang.Class"); //$NON-NLS-1$
    TYPE_THROWABLE = new Type("java.lang.Throwable"); //$NON-NLS-1$
    TYPE_STRING = new Type("java.lang.String"); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private Utils() {
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns current date/time as ISO formatted string.
   *
   * @return current date/time as ISO formatted string
   */
  public static String currentDateTime() {
    return DATE_TIME_FORMAT.format(new Date());
  }

  /**
   * Returns current date as ISO formatted string.
   *
   * @return current date as ISO formatted string
   */
  public static String currentDate() {
    return DATE_FORMAT.format(new Date());
  }

  /**
   * Returns JNI type for the specified java type.
   *
   * @param javaType the java type
   *
   * @return JNI type for the specified java type
   */
  public static String getJniType(Type javaType) {
    if (javaType.isVoid()) {
      return "void"; //$NON-NLS-1$
    }
    switch (javaType.getDimensions()) {
      case 0:
        if (javaType.isPrimitive()) {
          return "j" + javaType.getValue(); //$NON-NLS-1$
        }
        if (javaType.isA(TYPE_STRING)) {
          return "jstring"; //$NON-NLS-1$
        }
        if (javaType.isA(TYPE_THROWABLE)) {
          return "jthrowable"; //$NON-NLS-1$
        }
        if (javaType.isA(TYPE_CLASS)) {
          return "jclass"; //$NON-NLS-1$
        }
        return "jobject"; //$NON-NLS-1$
      case 1:
        if (javaType.isPrimitive()) {
          return "j" + javaType.getValue() + "Array"; //$NON-NLS-1$ $NON-NLS-2$
        }
        return "jobjectArray"; //$NON-NLS-1$
      default:
        return "jobjectArray"; //$NON-NLS-1$
    }
  }

  /**
   * Returns the signature of the specified java type.
   *
   * @param javaType the java type
   * @param mangled if <tt>true</tt> the signature is mangled
   *
   * @return the signature of the specified java type
   */
  public static String getSignature(Type javaType, boolean mangled) {
    String name = javaType.getValue();
    if ("boolean".equals(name)) { //$NON-NLS-1$
      return "Z"; //$NON-NLS-1$
    }
    if ("byte".equals(name)) { //$NON-NLS-1$
      return "B"; //$NON-NLS-1$
    }
    if ("char".equals(name)) { //$NON-NLS-1$
      return "C"; //$NON-NLS-1$
    }
    if ("short".equals(name)) { //$NON-NLS-1$
      return "S"; //$NON-NLS-1$
    }
    if ("int".equals(name)) { //$NON-NLS-1$
      return "I"; //$NON-NLS-1$
    }
    if ("long".equals(name)) { //$NON-NLS-1$
      return "J"; //$NON-NLS-1$
    }
    if ("float".equals(name)) { //$NON-NLS-1$
      return "F"; //$NON-NLS-1$
    }
    if ("double".equals(name)) { //$NON-NLS-1$
      return "D"; //$NON-NLS-1$
    }
    if ("void".equals(name)) { //$NON-NLS-1$
      return "V"; //$NON-NLS-1$
    }
    if (mangled) {
      return "L" + name.replace('.', '_') + "_2"; //$NON-NLS-1$ $NON-NLS-2$
    }
    return "L" + name.replace('.', '/') + ";"; //$NON-NLS-1$ $NON-NLS-2$
  }

  /**
   * Returns the signature of the specified java type.
   *
   * @param javaType the java type
   *
   * @return the signature of the specified java type
   */
  public static String getSignature(Type javaType) {
    return getSignature(javaType, false);
  }

  /**
   * Returns signature of the specified method parameters.
   *
   * @param params the method parameters
   * @param mangled if <tt>true</tt> the signature is mangled
   *
   * @return signature of the specified method parameters
   */
  public static String getSignature(JavaParameter[] params, boolean mangled) {
    StringBuffer sb = new StringBuffer();
    if (mangled) {
      for (int i = 0; i < params.length; i++) {
        if (params[i].getType().getDimensions() == 1) {
          sb.append('_'); // NOPMD
          sb.append('3'); // NOPMD
        }
        sb.append(getSignature(params[i].getType(), true));
      }
    } else {
      for (int i = 0; i < params.length; i++) {
        if (params[i].getType().getDimensions() == 1) {
          sb.append('[');
        }
        sb.append(getSignature(params[i].getType(), false));
      }
    }
    return sb.toString();
  }

  /**
   * Returns signature of the specified method parameters.
   *
   * @param params the method parameters
   *
   * @return signature of the specified method parameters
   */
  public static String getSignature(JavaParameter[] params) {
    return getSignature(params, false);
  }

  /**
   * Returns signature of the specified java method.
   *
   * @param javaMethod the java method
   * @param mangled if <tt>true</tt> the signature is mangled
   *
   * @return signature of the specified java method
   */
  public static String getSignature(JavaMethod javaMethod, boolean mangled) {
    StringBuffer sb = new StringBuffer(80);
    sb.append('(');
    sb.append(getSignature(javaMethod.getParameters(), mangled));
    sb.append(')');
    sb.append(getSignature(javaMethod.getReturns(), mangled));
    return sb.toString();
  }

  /**
   * Returns signature of the specified java method.
   *
   * @param javaMethod the java method
   *
   * @return signature of the specified java method
   */
  public static String getSignature(JavaMethod javaMethod) {
    return getSignature(javaMethod, false);
  }

  /**
   * Returns full qualifed method name.
   *
   * @param javaClass the java class
   *
   * @return full qualifed method name
   */
  public static String getMangledClassName(JavaClass javaClass) {
    return javaClass.getFullyQualifiedName().replace('.', '_');
  }

  /**
   * Returns mangled method name.
   *
   * @param javaMethod the java method
   *
   * @return mangled method name
   */
  public static String getMangledMethodName(JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(100);
    sb.append(getMangledClassName(javaMethod.getParentClass()));
    sb.append('_');
    String methodName = javaMethod.getName();
    for (int i = 0; i < methodName.length(); i++) {
      char c = methodName.charAt(i);
      if (c == '_') {
        sb.append('_'); // NOPMD
        sb.append('1'); // NOPMD
      } else {
        sb.append(c);
      }
    }
    return sb.toString();
  }

  /**
   * Returns JNI method name.
   *
   * @param javaMethod the java method
   * @param overridden <tt>true</tt> if the method is overridden
   *
   * @return JNI method name
   */
  public static String getJniMethodName(JavaMethod javaMethod,
                                        boolean overridden) {
    StringBuffer sb = new StringBuffer("Java_"); //$NON-NLS-1$
    sb.append(getMangledMethodName(javaMethod));
    if (overridden) {
      sb.append('_'); // NOPMD
      sb.append('_'); // NOPMD
      sb.append(Utils.getSignature(javaMethod.getParameters(), true));
    }
    return sb.toString();
  }

  /**
   * Returns JNI constant name.
   *
   * @param javaField the java field
   *
   * @return JNI constant name
   */
  public static String getJniConstantName(JavaField javaField) {
    return getMangledClassName((JavaClass)javaField.getParent()) +
           "_" + javaField.getName(); //$NON-NLS-1$
  }

  /**
   * Returns the name of the bridge.
   *
   * @param javaClass the java class
   *
   * @return the name of the bridge
   */
  public static String getBridgeName(JavaClass javaClass) {
    String bridgeName = getNativeTagParameter(javaClass,
                                              "bridge"); //$NON-NLS-1$
    if (bridgeName != null) {
      return bridgeName;
    }
    return javaClass.getName() + "Bridge"; //$NON-NLS-1$
  }

  /**
   * Returns the name of the bridge object.
   *
   * @param javaClass the java class
   *
   * @return the name of the bridge object
   */
  public static String getBridgeObjectName(JavaClass javaClass) {
    String name = javaClass.getName();
    name = name.substring(0, 2).toLowerCase() + name.substring(2); //NOFB
    return name + "Bridge"; //$NON-NLS-1$
  }

  /**
   * Returns the name of the bridge method.
   *
   * @param javaMethod the java method
   *
   * @return the name of the bridge method
   */
  public static String getBridgeMethodName(JavaMethod javaMethod) {
    return javaMethod.getName().substring(1);
  }

  /**
   * Returns the parameters of the bridge method.
   *
   * @param javaMethod the java method
   *
   * @return the parameters of the bridge method
   */
  public static String getBridgeMethodParameters(JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(80);
    sb.append("JNIEnv *env"); //$NON-NLS-1$
    JavaParameter[] params = javaMethod.getParameters();
    if (javaMethod.isStatic()) {
      sb.append(", jclass cls"); //$NON-NLS-1$
    }
    for (int i = 1; i < params.length; i++) {
      sb.append(','); // NOPMD
      sb.append(' '); // NOPMD
      sb.append(Utils.getJniType(params[i].getType()));
      sb.append(' '); // NOPMD
      sb.append(params[i].getName());
    }
    return sb.toString();
  }

  /**
   * Returns the documentation of the bridge method.
   *
   * @param javaMethod the java method
   *
   * @return the documentation of the bridge method
   */
  public static String getBridgeConstructorComment(JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(160);
    sb.append("/*!"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" * \\brief Constructs new "); //$NON-NLS-1$
    sb.append(Utils.getBridgeName(javaMethod.getParentClass()));
    sb.append(" object."); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" *"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" * \\param env the JNIEnv pointer"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" * \\param obj "); //$NON-NLS-1$
    JavaParameter[] params = javaMethod.getParameters();
    for (int i = 0; i < params.length; i++) {
      sb.append(NativeGenerator.INSTANCE.getNewLine());
      sb.append(" * \\param "); //$NON-NLS-1$
      sb.append(params[i].getName());
    }
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" */"); //$NON-NLS-1$
    return sb.toString();
  }

  /**
   * Returns the declaration of the bridge constructor.
   *
   * @param javaMethod the java method
   *
   * @return the declaration of the bridge constructor
   */
  public static String getBridgeConstructorDeclaration(JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(80);
    sb.append("  "); //$NON-NLS-1$
    sb.append(getBridgeName(javaMethod.getParentClass()));
    sb.append("(JNIEnv *env, jobject obj"); //$NON-NLS-1$
    JavaParameter[] params = javaMethod.getParameters();
    for (int j = 0; j < params.length; j++) {
      sb.append(','); // NOPMD
      sb.append(' '); // NOPMD
      sb.append(Utils.getJniType(params[j].getType()));
      sb.append(' '); // NOPMD
      sb.append(params[j].getName());
    }
    sb.append(");"); //$NON-NLS-1$
    return sb.toString();
  }

  /**
   * Returns the declaration of the bridge constructor.
   *
   * @param javaMethod the java method
   *
   * @return the declaration of the bridge constructor
   */
  public static String getBridgeConstructorImplementation(
                                                       JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(80);
    String bridgeName = getBridgeName(javaMethod.getParentClass());
    sb.append(bridgeName);
    sb.append(':'); // NOPMD
    sb.append(':'); // NOPMD
    sb.append(bridgeName);
    sb.append("(JNIEnv *env, jobject obj"); //$NON-NLS-1$
    JavaParameter[] params = javaMethod.getParameters();
    for (int j = 0; j < params.length; j++) {
      sb.append(','); // NOPMD
      sb.append(' '); // NOPMD
      sb.append(Utils.getJniType(params[j].getType()));
      sb.append(' '); // NOPMD
      sb.append(params[j].getName());
    }
    sb.append(')');
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append('{');
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append("  // TODO: Implement"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append('}');
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    return sb.toString();
  }

  /**
   * Returns the documentation of the bridge method.
   *
   * @param javaMethod the java method
   *
   * @return the documentation of the bridge method
   */
  public static String getBridgeMethodComment(JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(256);
    sb.append("/*!"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" * \\brief "); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" *"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" * \\param env the JNIEnv pointer"); //$NON-NLS-1$
    int i = 1;
    if (javaMethod.isStatic()) {
      sb.append(NativeGenerator.INSTANCE.getNewLine());
      sb.append(" * \\param cls the java class"); //$NON-NLS-1$
      i = 0;
    }
    JavaParameter[] params = javaMethod.getParameters();
    for (; i < params.length; i++) {
      sb.append(NativeGenerator.INSTANCE.getNewLine());
      sb.append(" * \\param "); //$NON-NLS-1$
      sb.append(params[i].getName());
      sb.append(' ');
    }
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    if (!javaMethod.getReturns().isVoid()) {
      sb.append(" * "); //$NON-NLS-1$
      sb.append(NativeGenerator.INSTANCE.getNewLine());
      sb.append(" * \\return "); //$NON-NLS-1$
      sb.append(NativeGenerator.INSTANCE.getNewLine());
    }
    sb.append(" */"); //$NON-NLS-1$
    return sb.toString();
  }

  /**
   * Returns the declaration of the bridge method.
   *
   * @param javaMethod the java method
   *
   * @return the declaration of the bridge method
   */
  public static String getNativeCallMethodDeclaration(JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(80);
    sb.append("  "); //$NON-NLS-1$
    sb.append(getJniType(javaMethod.getReturns()));
    sb.append(' ');
    sb.append(getBridgeMethodName(javaMethod));
    sb.append('(');
    sb.append(getBridgeMethodParameters(javaMethod));
    sb.append(");"); //$NON-NLS-1$
    return formatMethodDeclaration(sb.toString());
  }

  /**
   * Returns the implementation of the bridge method.
   *
   * @param javaMethod the java method
   *
   * @return the implementation of the bridge method
   */
  public static String getBridgeMethodImplementation(JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(80);
    sb.append(getJniType(javaMethod.getReturns()));
    sb.append(' ');
    sb.append(Utils.getBridgeName(javaMethod.getParentClass()));
    sb.append("::"); //$NON-NLS-1$
    sb.append(getBridgeMethodName(javaMethod));
    sb.append('(');
    if (javaMethod.isStatic()) {
      sb.append(getNativeCallMethodParameters(javaMethod));
    } else {
      sb.append(getBridgeMethodParameters(javaMethod));
    }
    sb.append(')');
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append('{');
    if (!javaMethod.isStatic()) {
      sb.append(NativeGenerator.INSTANCE.getNewLine());
      sb.append("  Q_ASSERT(env);"); //$NON-NLS-1$
      JavaParameter[] params = javaMethod.getParameters();
      for (int i = 1; i < params.length; i++) {
        if (!params[i].getType().isPrimitive()) {
          sb.append(NativeGenerator.INSTANCE.getNewLine());
          sb.append("  Q_ASSERT("); //$NON-NLS-1$
          sb.append(params[i].getName());
          sb.append(");"); //$NON-NLS-1$
        }
      }
      sb.append(NativeGenerator.INSTANCE.getNewLine());
      if (javaMethod.getReturns().isVoid()) {
        sb.append("  QSWING_REENTRANT("); //$NON-NLS-1$
        sb.append(getBridgeMethodName(javaMethod));
      } else {
        sb.append("  QSWING_REENTRANT_RETURN("); //$NON-NLS-1$
        sb.append(getBridgeMethodName(javaMethod));
        sb.append(','); // NOPMD
        sb.append(' '); // NOPMD
        sb.append(Utils.getJniType(javaMethod.getReturns()));
      }
      sb.append(", QSWING_ENV_ARG"); //$NON-NLS-1$
      params = javaMethod.getParameters();
      for (int i = 1; i < params.length; i++) {
        sb.append(','); // NOPMD
        sb.append(' '); // NOPMD
        sb.append(Utils.getJniType(params[i].getType()).toUpperCase());
        sb.append("_ARG("); // NOPMD
        sb.append(params[i].getName());
        sb.append(')'); // NOPMD
      }
      sb.append(')');
    }
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append("  // TODO: Implement"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    if (!javaMethod.getReturns().isVoid()) {
      sb.append("  return 0;"); //$NON-NLS-1$
      sb.append(NativeGenerator.INSTANCE.getNewLine());
    }
    sb.append('}');
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    return formatMethodImplementation(sb.toString());
  }

  /**
   * Returns the parameters of the bridge callback method.
   *
   * @param javaMethod the java method
   *
   * @return the parameters of the bridge call-back method
   */
  public static String getCallbackMethodParameters(JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(80);
    JavaParameter[] params = javaMethod.getParameters();
    for (int i = 0; i < params.length; i++) {
      if (i > 0) {
        sb.append(','); // NOPMD
        sb.append(' '); // NOPMD
      }
      sb.append(Utils.getJniType(params[i].getType()));
      sb.append(' '); // NOPMD
      sb.append(params[i].getName());
    }
    return sb.toString();
  }

  /**
   * Returns the declaration of the bridge callback method.
   *
   * @param javaMethod the java method
   *
   * @return the declaration of the bridge method
   */
  public static String getCallbackMethodDeclaration(JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(80);
    sb.append("  "); //$NON-NLS-1$
    sb.append(getJniType(javaMethod.getReturns()));
    sb.append(' ');
    sb.append(getBridgeMethodName(javaMethod));
    sb.append('(');
    sb.append(getCallbackMethodParameters(javaMethod));
    sb.append(");"); //$NON-NLS-1$
    return formatMethodDeclaration(sb.toString());
  }

  /**
   * Returns the implementation of the bridge callback method.
   *
   * @param javaMethod the java method
   *
   * @return the implementation of the bridge method
   */
  public static String getCallbackMethodImplementation(JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(80);
    sb.append(getJniType(javaMethod.getReturns()));
    sb.append(' ');
    sb.append(Utils.getBridgeName(javaMethod.getParentClass()));
    sb.append("::"); //$NON-NLS-1$
    sb.append(getBridgeMethodName(javaMethod));
    sb.append('(');
    sb.append(getCallbackMethodParameters(javaMethod));
    sb.append(')');
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append('{');
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append("  // TODO: Implement"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    if (!javaMethod.getReturns().isVoid()) {
      sb.append("  return 0;"); //$NON-NLS-1$
      sb.append(NativeGenerator.INSTANCE.getNewLine());
    }
    sb.append('}');
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    // Utils.getNativeTagParameter(method, "code") /** @todo  ??? */
    return formatMethodImplementation(sb.toString());
  }

  /**
   * Returns the parameters of the bridge native call method.
   *
   * @param javaMethod the java method
   *
   * @return the parameters of the bridge call-back method
   */
  public static String getNativeCallMethodParameters(JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(80);
    int startParamIndex = 1;
    sb.append("JNIEnv *env"); //$NON-NLS-1$
    if (javaMethod.isStatic()) {
      startParamIndex = 0;
      sb.append(", jclass cls"); //$NON-NLS-1$
    } else if ("_create".equals(javaMethod.getName())) {
      sb.append(", jobject obj"); //$NON-NLS-1$
    }
    JavaParameter[] params = javaMethod.getParameters();
    for (int i = startParamIndex; i < params.length; i++) {
      sb.append(','); // NOPMD
      sb.append(' '); // NOPMD
      sb.append(Utils.getJniType(params[i].getType()));
      sb.append(' '); // NOPMD
      sb.append(params[i].getName());
    }
    return sb.toString();
  }

  /**
   * Returns the declaration of the bridge method.
   *
   * @param javaMethod the java method
   *
   * @return the declaration of the bridge method
   */
  public static String getNativeCallStaticMethodDeclaration(
                                                       JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(80);
    sb.append("  static "); //$NON-NLS-1$
    sb.append(getJniType(javaMethod.getReturns()));
    sb.append(' ');
    sb.append(getBridgeMethodName(javaMethod));
    sb.append('(');
    sb.append(getNativeCallMethodParameters(javaMethod));
    sb.append(");"); //$NON-NLS-1$
    return formatMethodDeclaration(sb.toString());
  }

  /**
   * Returns the documentation of the newInstance method.
   *
   * @param javaMethod the java method
   *
   * @return the documentation of the bridge method
   */
  public static String getNativeCallNewInstanceMethodComment(
                                                       JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(256);
    sb.append("/*!"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(
         " * \\brief Returns new instance of the bridge object"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" *"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" * \\param env the JNIEnv pointer"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" * \\param obj the QSwing object"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" * "); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" * \\return new instance of the bridge object"); //$NON-NLS-1$
    sb.append(NativeGenerator.INSTANCE.getNewLine());
    sb.append(" */"); //$NON-NLS-1$
    return sb.toString();
  }

  /**
   * Returns the declaration of the newInstance method.
   *
   * @param javaMethod the java method
   *
   * @return the declaration of the bridge method
   */
  public static String getNativeCallNewInstanceMethodDeclaration(
                                                       JavaMethod javaMethod) {
    StringBuffer sb = new StringBuffer(80);
    sb.append("  static "); //$NON-NLS-1$
    sb.append(getJniType(javaMethod.getReturns()));
    sb.append(" newInstance(JNIEnv *env, jobject obj);");
    return formatMethodDeclaration(sb.toString());
  }

  /**
   * Returns the implementation of the newInstance method.
   *
   * @param javaMethod the java method
   *
   * @return the implementation of the bridge method
   */
  public static String getNativeCallNewInstanceMethodImplementation(
                                                       JavaMethod javaMethod) {
     StringBuffer sb = new StringBuffer(256);
     sb.append(getJniType(javaMethod.getReturns()));
     sb.append(' ');
     sb.append(Utils.getBridgeName(javaMethod.getParentClass()));
     sb.append("::newInstance(JNIEnv *env, jobject obj)"); //$NON-NLS-1$
     sb.append(NativeGenerator.INSTANCE.getNewLine());
     sb.append('{');
     sb.append(NativeGenerator.INSTANCE.getNewLine());
     sb.append("  if (JNIQSwingWrapper::isQSwingThread(env)) {"); //$NON-NLS-1$
     sb.append(NativeGenerator.INSTANCE.getNewLine());
     sb.append("    return reinterpret_cast<jlong>(new "); //$NON-NLS-1$
     sb.append(Utils.getBridgeName(javaMethod.getParentClass()));
     sb.append("(env, obj));"); //$NON-NLS-1$
     sb.append(NativeGenerator.INSTANCE.getNewLine());
     sb.append("  }"); //$NON-NLS-1$
     sb.append(NativeGenerator.INSTANCE.getNewLine());
     sb.append("  return QSwingUtilitiesBridge::" + //$NON-NLS-1$
               "invokeLongAndWait(env, obj, createMID());"); //$NON-NLS-1$
     sb.append(NativeGenerator.INSTANCE.getNewLine());
     sb.append('}');
     sb.append(NativeGenerator.INSTANCE.getNewLine());
     return sb.toString();
  }

  /**
   * Returns the value of the specified parameter of the "javaqx.native" tag.
   *
   * @param javaEntity the abstract java entity
   * @param parameter the parameter
   *
   * @return the value of the specified parameter of the "javaqx.native" tag
   */
  public static String getNativeTagParameter(AbstractJavaEntity javaEntity,
                                             String parameter) {
    DocletTag tag = javaEntity.getTagByName("javaqx.native"); //$NON-NLS-1$
    if (tag != null) {
      String paramValue = tag.getNamedParameter(parameter);
      if (paramValue != null) {
        return paramValue;
      }
    }
    return null;
  }

  /**
   * Returns <tt>true</tt> if the code contains the specified parameter.
   *
   * @param code the code
   * @param param the parameter
   *
   * @return <tt>true</tt> if the code contains the specified parameter
   */
  public static boolean parameterUsed(String code, String param) {
    // TODO: implement better
    return ((code == null) || ("bridge".equals(param)) || //$NON-NLS-1$
            (code.indexOf(param) != -1));
  }

  /**
   * Print block comment.
   *
   * @param pw the print writer
   * @param comment the comment
   *
   * @throws IOException on an I/O error
   */
  public static void printBlockComment(PrintWriter pw, String comment)
                                                           throws IOException {
     pw.print("/*^*************************************"); //$NON-NLS-1$
     pw.println("*************************************^*/"); //$NON-NLS-1$
     pw.print("/*- "); //$NON-NLS-1$
     pw.print(comment);
     for (int i = 72 - comment.length(); i >= 0; i--) {
       pw.print(" "); //$NON-NLS-1$
     }
     pw.println("-*/"); //$NON-NLS-1$
     pw.print("/*^*************************************"); //$NON-NLS-1$
     pw.println("*************************************^*/"); //$NON-NLS-1$
     pw.println();
  }

  /**
   * Formats method declaration string.
   *
   * @param methodDeclaration the declaration string
   *
   * @return formatted method declaration string
   */
  public static String formatMethodDeclaration(String methodDeclaration) {
    if (methodDeclaration.length() > 79) {
      int bracketPosition = methodDeclaration.indexOf('(');
      String newLine = NativeGenerator.getInstance().getNewLine();
      char[] spaces = new char[bracketPosition + newLine.length()];
      for (int i = spaces.length - 1; i >= newLine.length(); i--) {
        spaces[i] = ' ';
      }
      for (int i = 0, j = 0; j < newLine.length(); i++, j++) {
        spaces[i] = newLine.charAt(j);
      }
      StringBuffer sb = new StringBuffer(methodDeclaration);
      int i = bracketPosition;
      int inLinePostion = i;
      int lastComma = i;
      while (i < sb.length()) {
        if (inLinePostion > 79) {
          sb.insert(lastComma + 1, spaces);
          i += spaces.length - newLine.length();
          inLinePostion = bracketPosition;
        } else {
          if (sb.charAt(i) == ',') {
            lastComma = i;
          }
          i++;
          inLinePostion++;
        }
      }
      return sb.toString();
    }
    return methodDeclaration;
  }

  /**
   * Formats method declaration string.
   *
   * @param methodDeclaration the declaration string
   *
   * @return formatted method declaration string
   */
  public static String formatMethodImplementation(String methodDeclaration) {
    String newLine = NativeGenerator.getInstance().getNewLine();
    int nli = methodDeclaration.indexOf(newLine);
    nli = nli + newLine.length();
    String methodName = methodDeclaration.substring(0, nli);
    String methodBody = methodDeclaration.substring(nli);
    return formatMethodDeclaration(methodName) + methodBody;
  }

  /**
   * Returns index of method implementation.
   *
   * @param methods the methods the methods
   * @param methodImplementation the method implementation
   *
   * @return index of method implementation
   */
  public static int getMethodImplementationIndex(ArrayList methods,
                                                 String methodImplementation) {
    String newLine = NativeGenerator.getInstance().getNewLine();
    String commMethodEnd = "*/" + newLine; //$NON-NLS-1$
    for (int i = 0; i < methods.size(); i++) {
      String methodText = (String)methods.get(i);
      if (!methodText.endsWith(commMethodEnd)) {
        int cpi = methodImplementation.indexOf(')') + 1;
        if (methodText.indexOf(methodImplementation.substring(0, cpi)) >= 0) {
          return i;
        }
      }
    }
    return -1;
  }

  /**
   * Back up the specified file.
   *
   * @param file the file to back up
   *
   * @throws IOException on an I/O error
   */
  public static void backupFile(File file) throws IOException {
    FileChannel fileChannel = null;
    FileChannel backupChannel = null;
    try {
      File backupFile = new File(file.getAbsolutePath() + "~"); //$NON-NLS-1$
      fileChannel = new FileInputStream(file).getChannel();
      backupChannel = new FileOutputStream(backupFile).getChannel();
      backupChannel.transferFrom(fileChannel, 0, fileChannel.size());
    } finally {
      if (fileChannel != null) {
        fileChannel.close();
      }
      if (backupChannel != null) {
        backupChannel.close();
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2007-02-28 16:54:21 Slobodan
   Initial version

*****************************************************************************^*/
