/*
 * @(#)JQBridgeClassDeclaration.java  0.2.0 / 2007-03-13
 *
 * Copyright (c) 2007, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.javaqx.tools.ng.bridge;

import java.io.IOException;
import java.io.PrintWriter;

import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaMethod;

import yu.ac.bg.etf.javaqx.tools.ng.AbstractNativeFile;
import yu.ac.bg.etf.javaqx.tools.ng.NativeFileText;
import yu.ac.bg.etf.javaqx.tools.ng.NativeGenerator;
import yu.ac.bg.etf.javaqx.tools.ng.Utils;

/**
 * Bridge Class Declaration.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 0.2.0
 */
class JQBridgeClassDeclaration extends NativeFileText {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final JavaClass javaClass;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JQBridgeClassDeclaration object.
   *
   * @param nativeFile the native file
   * @param javaClass the qdox java class
   */
  public JQBridgeClassDeclaration(AbstractNativeFile nativeFile,
                                  JavaClass javaClass) {
    super(nativeFile);
    this.javaClass = javaClass;
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void storeContent(PrintWriter pw) throws IOException {
    Utils.printBlockComment(pw, Utils.getBridgeName(javaClass) +
                            " Class."); //$NON-NLS-1$
    storeClassComment(pw);
    storeClassStart(pw);
    storeJniCalls(pw);
    storeCallBacks(pw);
    storeClassEnd(pw);
    storeMacroDefinitions(pw);
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void storeClassComment(PrintWriter pw) throws IOException {
    pw.println("/*!"); //$NON-NLS-1$
    pw.print(" * \\brief "); //$NON-NLS-1$
    pw.print(javaClass.getName());
    pw.println(" Bridge."); //$NON-NLS-1$
    pw.println(" *"); //$NON-NLS-1$
    pw.println(NativeGenerator.getInstance().getClassComment());
    pw.println(" */"); //$NON-NLS-1$
  }

  private void storeClassStart(PrintWriter pw) throws IOException {
    pw.print("class JAVAQX_EXPORT "); //$NON-NLS-1$
    pw.println(Utils.getBridgeName(javaClass));
    pw.println("{"); //$NON-NLS-1$
    pw.println("  Q_OBJECT"); //$NON-NLS-1$
  }

  private void storeClassEnd(PrintWriter pw) throws IOException {
    pw.println("};"); //$NON-NLS-1$
    pw.println();
  }

  private void storeJniCalls(PrintWriter pw) throws IOException {
    String bridgeObjectName = Utils.getBridgeObjectName(javaClass);
    JavaMethod[] methods = javaClass.getMethods();
    boolean[] implementedMethods = new boolean[methods.length];
    for (int i = 0; i < methods.length; i++) {
      JavaMethod method = methods[i];
      String bridgeObject = Utils.getNativeTagParameter(method,
                                                 "bridgeObject"); //$NON-NLS-1$
      implementedMethods[i] =
          ((Utils.getNativeTagParameter(method,
                                        "code") != null) || //$NON-NLS-1$
           ((bridgeObject != null) && !bridgeObjectName.equals(bridgeObject)));
    }
    boolean labeled = false;
    for (int i = 0; i < methods.length; i++) {
      JavaMethod method = methods[i];
      if (!implementedMethods[i] && method.isNative() && !method.isStatic() &&
          "_create".equals(method.getName())) { //$NON-NLS-1$
        if (!labeled) {
          pw.println("protected:"); //$NON-NLS-1$
          labeled = true;
        }
        pw.println(Utils.getBridgeConstructorDeclaration(method));
      }
    }
    labeled = false;
    for (int i = 0; i < methods.length; i++) {
      JavaMethod method = methods[i];
      if (!implementedMethods[i] && method.isNative() && !method.isStatic() &&
          !"_create".equals(method.getName())) { //$NON-NLS-1$
        if (!labeled) {
          pw.println("public slots: // native calls"); //$NON-NLS-1$
          labeled = true;
        }
        pw.println(Utils.getNativeCallMethodDeclaration(method));
      }
    }
    labeled = false;
    for (int i = 0; i < methods.length; i++) {
      JavaMethod method = methods[i];
      if (!implementedMethods[i] && method.isNative() && method.isStatic()) {
        if (!labeled) {
          pw.println("public: // native calls"); //$NON-NLS-1$
          labeled = true;
        }
        pw.println(Utils.getNativeCallStaticMethodDeclaration(method));
      }
    }
    for (int i = 0; i < methods.length; i++) {
      JavaMethod method = methods[i];
      if (!implementedMethods[i] && method.isNative() && !method.isStatic() &&
          "_create".equals(method.getName())) { //$NON-NLS-1$
        if (!labeled) {
          pw.println("public: // native calls"); //$NON-NLS-1$
          labeled = true;
        }
        pw.println(Utils.getNativeCallNewInstanceMethodDeclaration(method));
      }
    }
  }

  private void storeCallBacks(PrintWriter pw) throws IOException {
    JavaMethod[] methods = javaClass.getMethods();
    boolean labeled = false;
    for (int i = 0; i < methods.length; i++) {
      JavaMethod method = methods[i];
      String methodName = method.getName();
      if (!method.isNative() && methodName.charAt(0) == '_' &&
          !"true".equals(Utils.getNativeTagParameter(method, //$NON-NLS-1$
                                                 "overridden"))) { //$NON-NLS-1$
        if (!labeled) {
          pw.println("public: // native callbacks"); //$NON-NLS-1$
          labeled = true;
        }
        pw.println(Utils.getCallbackMethodDeclaration(method));
      }
    }
  }

  private void storeMacroDefinitions(PrintWriter pw) throws IOException {
    Utils.printBlockComment(pw, "Macro Definition."); //$NON-NLS-1$
    pw.print("#define "); //$NON-NLS-1$
    pw.print(Utils.getBridgeObjectName(javaClass));
    pw.print(" (reinterpret_cast<"); //$NON-NLS-1$
    pw.print(Utils.getBridgeName(javaClass));
    pw.println(" *>(bridge))"); //$NON-NLS-1$
    pw.println();
  }

}

/*^*****************************************************************************
  File History:

 - 2007-03-13 00:10:02 Slobodan
   Initial version

*****************************************************************************^*/
